package se.su.dsv.scipro.activityplan;

import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;

import javax.persistence.*;
import java.util.Date;
import java.util.Set;
import java.util.TreeSet;

@Data
@Entity
@Cacheable(true)
@EqualsAndHashCode(callSuper = false)
@SuppressWarnings("PMD.UnusedPrivateField")
public class ActivityPlan extends DomainObject {
	@Id
	@GeneratedValue
	private Long id;
	
	@OneToMany(mappedBy= "activityPlan",cascade=CascadeType.PERSIST, orphanRemoval = true)
	private Set<Activity> activities = new TreeSet<>(new Activity.ByDateComparator());
		
	@OneToOne(optional=false)
	private Project project;

	private Date startDate;

    private static class Builder implements IProject, IBuild {
        private Project project;

        @Override
        public ActivityPlan build() {
            ActivityPlan ps = new ActivityPlan();
            ps.setProject(project);
            return ps;
        }

        @Override
        public IBuild project(Project project) {
            this.project = project;
            return this;
        }
    }

    public static IProject builder() {
        return new Builder();
    }

    public interface IProject {
        IBuild project(Project project);
    }

    public interface IBuild {
        ActivityPlan build();
    }
}
