package se.su.dsv.scipro.checklist;


import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.*;

@Data
@EqualsAndHashCode(of = "id", callSuper = false)
@Entity
@Table(name = "checklist")
@Cacheable(true)

@SuppressWarnings("PMD.UnusedPrivateField")
public class Checklist extends DomainObject {

    @Id
    @GeneratedValue
    private Long id;

    @Column(nullable = false)
    private String name;

    @Basic(optional = true)
    private String description;

    /**
     * @deprecated Go via Project->Activity Plan->Activity->Checklist
     */
    @Deprecated
    @ManyToOne(optional = false)
    private Project project;

    @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true)
    private List<ChecklistQuestion> questions = new ArrayList<>();

    @ManyToMany
    private List<se.su.dsv.scipro.checklist.ChecklistCategory> categories = new ArrayList<>();

    @ElementCollection(fetch = FetchType.EAGER)
    private Map<User, Date> userLastOpenDate = new HashMap<>();

    protected Checklist() {
    }

    public int getNumberOfQuestions() {
        return questions.size();
    }

    public void addQuestion(ChecklistQuestion question) {
        questions.add(question);
    }

    public static IName builder() {
        return new Builder();
    }

    private static class Builder implements IName, IProject, IBuild {
        private Checklist instance = new Checklist();

        @Override
        public IProject name(String name) {
            instance.setName(name);
            return this;
        }

        @Override
        public IBuild project(Project project) {
            instance.setProject(project);
            return this;
        }

        @Override
        public IBuild questions(List<ChecklistQuestion> questions) {
            instance.setQuestions(questions);
            return this;
        }

        @Override
        public IBuild categories(se.su.dsv.scipro.checklist.ChecklistCategory... categories) {
            instance.setCategories(Arrays.asList(categories));
            return this;
        }

        @Override
        public IBuild description(String description) {
            instance.setDescription(description);
            return this;
        }

        @Override
        public IBuild userLastOpenDate(Map<User, Date> userLastOpenDate) {
            instance.setUserLastOpenDate(userLastOpenDate);
            return this;
        }

        @Override
        public Checklist build() {
            return instance;
        }
    }

    public interface IName {
        IProject name(String name);
    }

    public interface IProject {
        IBuild project(Project project);
    }


    public interface IBuild {
        IBuild questions(List<ChecklistQuestion> questions);

        IBuild categories(se.su.dsv.scipro.checklist.ChecklistCategory... categories);

        IBuild description(String description);

        IBuild userLastOpenDate(Map<User, Date> userLastOpenDate);

        Checklist build();
    }

}
