package se.su.dsv.scipro.checklist;

import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

@Data
@EqualsAndHashCode(of = "id", callSuper = false)
@Entity
@Table(name = "checklist_question")
@Cacheable(true)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class ChecklistQuestion extends DomainObject {
    @Id
    @GeneratedValue
    private Long id;

    @Lob
    @Column(nullable = false)
    private String question;

    @Column(nullable = false)
    private int questionNumber;

    @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true)
    private List<ChecklistAnswer> answers = new ArrayList<>();

    protected ChecklistQuestion() {
    }

    public ChecklistQuestion(String question, int questionNumber) {
        this.question = question;
        this.questionNumber = questionNumber;
    }

    public ChecklistAnswer getAnswerFromUser(User user) {
        if (user == null) {
            return null;
        }
        for (ChecklistAnswer answer : answers) {
            if (answer.getUser().equals(user)) {
                return answer;
            }
        }
        return new ChecklistAnswer(user);
    }

    public void addAnswer(ChecklistAnswer answer) {
        Iterator<ChecklistAnswer> iter = answers.iterator();
        while (iter.hasNext()) {
            if (answer.getUser().equals(iter.next().getUser())) {
                iter.remove();
            }
        }
        answers.add(answer);
    }
}