package se.su.dsv.scipro.checklist;

import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

@Data
@EqualsAndHashCode(of = "id", callSuper = false)
@Entity
@Table(name = "checklist_template")
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class ChecklistTemplate extends DomainObject {
    public static final int MIN_TITLE_LENGTH = 3;
    public static final int DEFAULT_TEMPLATE_NUMBER = 999;

    @Id
    @GeneratedValue
    private Long id;

    @Column(nullable = false)
    private String name;

    @Basic(optional = true)
    private String description;

    @Column
    private int templateNumber = DEFAULT_TEMPLATE_NUMBER;

    @Lob
    @ElementCollection
    private List<String> questions = new ArrayList<>(1);

    @ManyToOne(optional = false)
    private User creator;

    @ManyToMany
    private List<ChecklistCategory> categories = new ArrayList<>();

    @ManyToMany
    @JoinTable(name = "checklist_template_ProjectType",
            joinColumns = {@JoinColumn(name = "checklist_template_id")},
            inverseJoinColumns = {@JoinColumn(name = "projectType_id")})
    private Collection<ProjectType> projectTypes = new HashSet<>();

    public ChecklistTemplate() {
    }

    public ChecklistTemplate(String name, User creator) {
        this.name = name;
        this.creator = creator;
    }

    public void setQuestions(List<String> questions) {
        this.questions = questions;
    }

    public void addQuestion(String question) {
        questions.add(question);
    }

    @SuppressWarnings("UnusedDeclaration")
    public List<String> getCategoryNames() {
        List<String> names = new ArrayList<>();
        for (ChecklistCategory checklistCategory : getCategories()) {
            names.add(checklistCategory.getCategoryName());
        }
        return names;
    }

    public void clearQuestions(){
        questions.clear();
    }

}