package se.su.dsv.scipro.finalseminar

import java.util.Date
import javax.inject.{Inject, Provider}
import javax.persistence.EntityManager

import com.google.common.eventbus.EventBus
import com.google.inject.persist.Transactional
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.system.User

import scala.util.Right

class ExtendedFinalSeminarServiceImpl @Inject()(em: Provider[EntityManager], eventBus: EventBus) extends FinalSeminarServiceImpl(em, eventBus) {
  @Transactional
  def oppose(student: User, finalSeminar: FinalSeminar, project: Project): Either[OpposeError, FinalSeminarOpposition] = {
    if (!project.isParticipant(student)) Left(NotAuthor)
    else if (finalSeminar.getActiveParticipants.contains(student)) Left(AlreadyParticipant)
    else if (finalSeminar.getOpponents.contains(student)) Left(AlreadyOpponent)
    else {
      val opposition: FinalSeminarOpposition = new FinalSeminarOpposition
      opposition.setUser(student)
      opposition.setFinalSeminar(finalSeminar)
      opposition.setProject(project)
      finalSeminar.addOpposition(opposition)
      save(finalSeminar)
      Right(opposition)
    }
  }
}

object ExtendedFinalSeminarServiceImpl {
  def tryMove(finalSeminar: FinalSeminar, to: Date, earliest: ⇒ Date, isNonWorkDay: (Date) ⇒ Boolean): Either[SchedulingError, FinalSeminar] = {
    def move(finalSeminar: FinalSeminar): FinalSeminar = {
      finalSeminar.setStartDate(to)
      finalSeminar.setCancelled(false)
      finalSeminar
    }

    if (finalSeminar.getProject.isFinalSeminarRuleExempted) Right(move(finalSeminar))
    else if (isNonWorkDay(to)) Left(NonWorkDay(to))
    else if (to.before(earliest)) Left(NotEnoughWorkDays(to, earliest))
    else Right(move(finalSeminar))
  }
}