package se.su.dsv.scipro.finalthesis;

import lombok.Data;
import lombok.EqualsAndHashCode;
import org.joda.time.DateTime;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;

import javax.persistence.*;
import java.util.Date;

@Data
@EqualsAndHashCode(callSuper = false, of = {"fileDescription", "project"})
@Entity
@Table(uniqueConstraints = {
        @UniqueConstraint(name = "project_file_description", columnNames = {"project_id", "fileDescription_id"})
})
@SuppressWarnings("PMD.UnusedPrivateField")
public class FinalThesis extends DomainObject {

    public enum Status {
        APPROVED, REJECTED, NO_DECISION
    }

    @Id
    @GeneratedValue
    private Long id;

    @ManyToOne(optional = false)
    @JoinColumn(name = "fileDescription_id")
    private FileDescription fileDescription;

    @ManyToOne(optional = false)
    @JoinColumn(name = "project_id")
    private Project project;

    @Enumerated(EnumType.STRING)
    private Status status = Status.NO_DECISION;

    private Date dateApproved;

    private Date dateRejected;

    public boolean isRejected() {
        return getStatus() == Status.REJECTED;
    }

    public Date getDateRejected() {
        return new DateTime(dateRejected).toDate();
    }

    public Date getDateApproved() {
        return new DateTime(dateApproved).toDate();
    }

    public static IFileDescription builder() {
        return new Builder();
    }

    private static class Builder implements IFileDescription, IProject, IBuild {
        private FinalThesis instance = new FinalThesis();

        @Override
        public IProject fileDescription(FileDescription fileDescription) {
            instance.setFileDescription(fileDescription);
            return this;
        }

        @Override
        public IBuild project(Project project) {
            instance.setProject(project);
            return this;
        }

        @Override
        public IBuild status(Status status) {
            instance.setStatus(status);
            return this;
        }

        @Override
        public IBuild dateApproved(Date dateApproved) {
            instance.setDateApproved(dateApproved);
            return this;
        }

        @Override
        public IBuild dateRejected(Date dateRejected) {
            instance.setDateRejected(dateRejected);
            return this;
        }

        @Override
        public FinalThesis build() {
            return instance;
        }
    }

    public interface IFileDescription {
        IProject fileDescription(FileDescription fileDescription);
    }

    public interface IProject {
        IBuild project(Project project);
    }

    public interface IBuild {
        IBuild status(Status status);

        IBuild dateApproved(Date date);

        IBuild dateRejected(Date date);

        FinalThesis build();
    }
}
