package se.su.dsv.scipro.forummail;

import com.google.common.base.Function;
import com.google.inject.persist.Transactional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.forum.GroupForumService;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.io.InputStream;
import java.util.HashSet;
import java.util.Set;

class ReadForumMail implements MailHandler {
    private static final Logger LOGGER = LoggerFactory.getLogger(ReadForumMail.class);

    private final GroupForumService groupForumService;
    private final ProjectForumService projectForumService;
    private final ForumMailService forumMailService;
    private final FileDescriptionService fileDescriptionService;

    @Inject
    ReadForumMail(final GroupForumService groupForumService, final ProjectForumService projectForumService, final ForumMailService forumMailService, final FileDescriptionService fileDescriptionService) {
        this.groupForumService = groupForumService;
        this.projectForumService = projectForumService;
        this.forumMailService = forumMailService;
        this.fileDescriptionService = fileDescriptionService;
    }

    @Override
    @Transactional
    public void accept(final ForumMailMessage mail) {
        final ForumMail allowed = forumMailService.findByMessageID(mail.getInReplyTo());
        if (allowed != null) {

            Set<FileDescription> attachments = new HashSet<>();
            for (MailAttachment mailAttachment : mail.getAttachments()){
                attachments.add(fileDescriptionService.storeFile(new MailUpload(mailAttachment, allowed.getUser())));
            }

            if (allowed instanceof ForumMailGroup) {
                groupForumService.createReply(
                        ((ForumMailGroup) allowed).getGroupThread(),
                        allowed.getUser(),
                        mail.getMailBody(),
                        attachments);
            }
            else if (allowed instanceof ForumMailProject) {
                projectForumService.createReply(
                        ((ForumMailProject) allowed).getProjectThread(),
                        allowed.getUser(),
                        mail.getMailBody(),
                        attachments);
            }
            LOGGER.info("New reply through mail: {}", allowed);
        }
    }

    private static class MailUpload implements FileUpload {

        private final MailAttachment attachment;
        private final User uploader;

        public MailUpload(final MailAttachment attachment, final User uploader) {
            this.attachment = attachment;
            this.uploader = uploader;
        }

        @Override
        public String getFileName() {
            return attachment.getFileName();
        }

        @Override
        public String getContentType() {
            return attachment.getContentType();
        }

        @Override
        public User getUploader() {
            return uploader;
        }

        @Override
        public long getSize() {
            return attachment.getSize();
        }

        @Override
        public <T> T handleData(final Function<InputStream, T> handler) {
            return attachment.handleData(handler);
        }
    }
}
