package se.su.dsv.scipro.generalsystemsettings;

import lombok.AccessLevel;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.Setter;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.SystemModule;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;
import java.util.Set;


@Entity
@Table(name = "general_system_settings")
@Cacheable(true)
@Data
@EqualsAndHashCode(callSuper = true)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class GeneralSystemSettings extends DomainObject {
    public static final int DEFAULT_NUMER_OF_LATEST_REVIEWS_DISPLAYED = 3;
    @Id
    private Long id = null;

    @Basic
    private String daisyProfileLinkBaseURL;

    @ElementCollection
    @CollectionTable(name = "general_system_settings_alarm_recipients")
    @Column(name = "mail")
    private List<String> alarmMails = new ArrayList<>();

    @ElementCollection
    @CollectionTable(name = "general_system_settings_supervisor_change_recipients")
    @Column(name = "mail")
    private List<String> supervisorChangeMails = new ArrayList<>();

    @Basic(optional = false)
    private String remindMessage = "Not remind yet";

    @Basic(optional = true)
    private int projectPartnerDaysToLive;

    @Basic(optional = false)
    private boolean mailNotifications = true;

    @Basic(optional = false)
    private String mailFromName = "SciPro";

    @Basic(optional = false)
    private String systemFromMail = "noreply-scipro@dsv.su.se";

    @Basic(optional = false)
    private String smtpServer = "localhost";

    private boolean peerDisplayLatestReviews = true;

    @Basic(optional = false)
    private int numberOfLatestReviewsDisplayed = DEFAULT_NUMER_OF_LATEST_REVIEWS_DISPLAYED;

    @Basic(optional = false)
    private boolean publicReviewsActivated = true;

    @Basic(optional = false)
    private boolean peerDownloadEnabled = true;

    @Basic(optional = false)
    private String sciproURL = "http://localhost:8080/";

    @Basic(optional = false)
    private boolean showSingleSignOn = true;

    @ElementCollection
    @Enumerated(EnumType.STRING)
    @JoinTable(name = "general_system_settings_system_modules")
    @Setter(AccessLevel.NONE)
    private Set<SystemModule> systemModules = EnumSet.allOf(SystemModule.class);

    @Basic(optional = true)
    private String matchResponsibleMail = "";

    @Basic(optional = true)
    private String reviewerSupportMail;

    @Basic(optional = true)
    private String thesisSupportMail;

    @Basic(optional = true)
    private String externalRoomBookingURL;

    @Basic(optional = true)
    private String externalGettingStartedWithIdeaURL;

    public GeneralSystemSettings() {
    }

    /**
     * Not part of the public API, do NOT USE
     */
    public GeneralSystemSettings(Long id) {
        this.id = id;
    }

    public String getExternalRoomBookingURL() {
        return externalRoomBookingURL;
    }

    public String getSciproURL() {
        return sciproURL.endsWith("/") ? sciproURL : sciproURL + "/";
    }
}
