package se.su.dsv.scipro.mail;

import com.google.inject.Exposed;
import com.google.inject.PrivateModule;
import com.google.inject.Provides;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.profiles.CurrentProfile;

import javax.mail.Session;
import java.util.Properties;

public class MailModule extends PrivateModule {

    private static final String TEST_MAIL_SINK = "scipro-mailtest@dsv.su.se";

    @Override
    protected void configure() {
        requireBinding(CurrentProfile.class);
        requireBinding(GeneralSystemSettingsService.class);
        bind(MailEventService.class).to(MailEventServiceImpl.class);
        expose(MailEventService.class);
    }

    @Provides
    @Exposed
    public Mailer mailer(CurrentProfile currentProfile, Session session, FileDescriptionService fileDescriptionService) {
        switch (currentProfile.getCurrentProfile()) {
            case DEV:
                return new PrintingMailer();
            case PROD:
                return new Mail(session, fileDescriptionService);
            case TEST:
                return new RedirectingMailer(session, TEST_MAIL_SINK, fileDescriptionService);
            default:
                throw new Error("Yay for exhaustive matches");
        }
    }

    @Provides
    public Session session(GeneralSystemSettingsService generalSystemSettings) {
        String smtpHost = generalSystemSettings.getGeneralSystemSettingsInstance().getSmtpServer();
        Properties properties = new Properties();
        properties.setProperty("mail.smtp.host", smtpHost);
        return Session.getDefaultInstance(properties);
    }
}
