package se.su.dsv.scipro.mail;

import com.sun.mail.smtp.SMTPMessage;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.mail.*;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMultipart;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.Date;

abstract class SMTPMailer implements Mailer {

    private final Session session;
    private FileDescriptionService fileDescriptionService;

    SMTPMailer(final Session session, FileDescriptionService fileDescriptionService) {
        this.session = session;
        this.fileDescriptionService = fileDescriptionService;
    }

    @Override
    public MailResult mail(String fromName, String fromEmail, String recipients[], String subject, String message, final FileDescription attachment) throws MessagingException {

        SMTPMessage msg = new SMTPMessage(session);

        msg.setSendPartial(true);
        msg.setSentDate(new Date());
        try {
            InternetAddress addressFrom = new InternetAddress(fromEmail, fromName, "UTF-8");
            msg.setFrom(addressFrom);
        } catch (UnsupportedEncodingException e) {
            throw new Error(e); // UTF-8 is required by the Java standard to be available.
        }

        msg.setRecipients(Message.RecipientType.BCC, getRecipients(recipients));
        msg.setSubject(subject, "UTF-8");

        Multipart multipart = new MimeMultipart("mixed");
        MimeBodyPart textBodyPart = new MimeBodyPart();
        textBodyPart.setText(message, "UTF-8");
        multipart.addBodyPart(textBodyPart);

        if (attachment!=null){
            MimeBodyPart fileBodyPart = new MimeBodyPart();

            DataSource source = new DataSource() {
                @Override
                public InputStream getInputStream() throws IOException {
                    return fileDescriptionService.getData(attachment);
                }

                @Override
                public OutputStream getOutputStream() throws IOException {
                    throw new UnsupportedOperationException("No can do");
                }

                @Override
                public String getContentType() {
                    return attachment.getMimeType();
                }

                @Override
                public String getName() {
                    return attachment.getName();
                }
            };

            fileBodyPart.setDataHandler(new DataHandler(source));
            fileBodyPart.setFileName(attachment.getName());
            multipart.addBodyPart(fileBodyPart);
        }

        msg.setContent(multipart);

        try {
            Transport.send(msg);
            return new Success(msg);
        }
        catch (SendFailedException ex) {
            return new Failure(msg, ex);
        }
    }

    abstract InternetAddress[] getRecipients(final String[] recipients) throws AddressException;
}
