package se.su.dsv.scipro.match;

import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.match.Idea.Status;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.system.*;
import se.su.dsv.scipro.util.Pair;

import java.io.Serializable;
import java.util.*;

public interface IdeaService extends GenericService<Idea, Long>, FilteredService<Idea, Long, IdeaService.Filter> {
    Idea savePartnerProgram(Idea idea, User student, Program program);
    Long countMatched(ApplicationPeriod ap, User supervisor, ProjectType pc);
    Iterable<Idea> findAuthorIdeas(User author, Pageable pageable);
    Iterable<Idea> findAll(Filter params);
    Iterable<Idea> findAllInList(List<Idea> list, Pageable pageable);
    Iterable<Idea> findReadyToStart();
    Long countIdeas(Status status, ProjectType pc);
    Long countAuthorIdeas(User author);
    Long countAuthorsByApplicationPeriod(ApplicationPeriod ap, Filter params);
    Pair<Boolean, String> validateSupervisorAcceptance(Idea idea, boolean hasTargetsLeft);
    Pair<Boolean, String> validateSupervisorAcceptance(Idea idea, User supervisor);
    Pair<Boolean, String> validateStudentAcceptance(Idea idea, User creator, User coAuthor, final ApplicationPeriod applicationPeriod);
    Pair<Boolean, String> validateAdminAddAuthors(Idea idea, Set<User> authors);
    Idea saveSupervisorIdea(Idea idea, User supervisor, List<Keyword> keywords, boolean isNewIdea);
    Idea saveStudentIdea(Idea idea, User creator, Program creatorProgram, User coAuthor, List<Keyword> keywords, boolean isNewIdea);
    void confirmSupervision(Idea idea, User supervisor);
    void adminUnmatchIdea(Idea idea, User admin);
    void deleteUnmatchedIdea(Idea idea);
    Idea acceptIdea(Idea idea, User creator, Program accepteeProgram, User coAuthor, final ApplicationPeriod applicationPeriod);
    void changeSupervisor(Idea idea, User newSelection, User creator);
    void adminSetAuthors(Idea idea, Set<User> authors, User creator);
    String getStatusString(Idea idea);
    boolean authorParticipatingOnActiveIdea(User author, ApplicationPeriod ap);
    void inactivateIdea(Idea idea, boolean state, User creator);
    Long countByApplicationPeriodAndType(ApplicationPeriod applicationPeriod, Type type, Filter params);
    Long countByApplicationPeriodAndProjectType(ApplicationPeriod applicationPeriod, ProjectType projectType, Filter params);
    Long countByApplicationPeriod(ApplicationPeriod applicationPeriod, Filter params);
    SelectionState getSelectionState(Idea idea);

    @Getter
    @Setter
    @ToString
    @SuppressWarnings({"PMD.UnusedPrivateField", "UnusedDeclaration"})
    class Filter implements Serializable {
        private Collection<ProjectType> levels;
        private Collection<Status> statuses;
        private Collection<Type> types;
        private Set<Keyword> keywords;
        private User supervisor;
        private User author;
        private ResearchArea researchArea;
        private Unit unit;
        private Date createdAfter;
        private Date createdBefore;
        private Boolean noSupervisor;
        private String titleContains;
        private Program program;
        private ApplicationPeriod applicationPeriod;
        private Boolean published;
        private Boolean noFirstMeeting = false;

        public void setNoFirstMeeting(Boolean noFirstMeeting) {
            this.noFirstMeeting = noFirstMeeting;
        }

        public void setStatuses(Collection<Status> statuses) {
            this.statuses = statuses;
        }
    }
}
