package se.su.dsv.scipro.match;

import com.google.inject.persist.Transactional;
import com.mysema.query.types.expr.BooleanExpression;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.Program;
import se.su.dsv.scipro.system.QProgram;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.List;

class ProgramServiceImpl extends AbstractServiceImpl<Program,Long> implements ProgramService {
    @Inject
    ProgramServiceImpl(Provider<EntityManager> em) {
        super(em, Program.class, QProgram.program);
    }

    @Override
    @Transactional
    public Program updateExternalProgram(Long id, String name, String nameEn, String code) {
        Program program = findOne(QProgram.program.externalId.eq(id));
        if (program == null) {
            program = new Program();
            program.setExternalId(id);
        }
        program.setName(name);
        program.setNameEn(nameEn);
        program.setCode(code);
        return save(program);
    }

    @Override
    public List<Program> getAutoCompletePrograms(String searchString, int limit) {
        return findAll(nameContains(searchString), new PageRequest(0, limit, new Sort(Sort.Direction.ASC, "name")));
    }

    private BooleanExpression nameContains(String input) {
        return QProgram.program.name.contains(input).or(QProgram.program.code.contains(input));
    }
}