package se.su.dsv.scipro.milestones.dataobjects;

import lombok.AccessLevel;
import lombok.Getter;
import lombok.Setter;
import se.su.dsv.scipro.system.Event;
import se.su.dsv.scipro.system.LazyDeletableDomainObject;
import se.su.dsv.scipro.system.ProjectType;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Set;

@Getter
@Setter
@Entity
@Table(name = "milestone_activity_template")
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class MilestoneActivityTemplate extends LazyDeletableDomainObject {
    public static final String CREATE_SEMINAR = "CREATE_SEMINAR";
    public static final String THESIS_UPLOADED = "THESIS_UPLOADED";
    public static final String PEER_REVIEW_ONE = "PEER_REVIEW_ONE";
    public static final String PEER_REVIEW_TWO = "PEER_REVIEW_TWO";

    public static final String TITLE = "title";

    @Id
    @GeneratedValue
    @Setter(AccessLevel.NONE)
    private Long id;

    @Basic(optional = false)
    private String title;

    @Basic(optional = true)
    private String description;

    @Enumerated(EnumType.STRING)
    private Type type;

    @Column(unique = true)
    private String code;

    @ManyToMany
    @Setter
    @SuppressWarnings("MismatchedQueryAndUpdateOfCollection")
    private Set<ProjectType> projectTypes = new HashSet<>();

    @ManyToOne
    @JoinColumn(name = "phase", nullable = false)
    private MilestonePhaseTemplate milestonePhaseTemplate;

    @Column
    private int sortOrder;

    @Column
    private boolean editableByStudents = false;

    @OneToOne
    @JoinColumn(name = "activatedBy")
    private Event activatedBy;

    public MilestoneActivityTemplate() {
    }

    public MilestoneActivityTemplate(Type type) {
        this.type = type;
    }

    public MilestoneActivityTemplate(Type type, String title, String description) {
        this.type = type;
        this.title = title;
        this.description = description;
    }

    public void addProjectType(ProjectType projectType) {
        projectTypes.add(projectType);
    }

    public static class BySortOrderComparator implements Comparator<MilestoneActivityTemplate>, Serializable {
        @Override
        public int compare(MilestoneActivityTemplate o1, MilestoneActivityTemplate o2) {
            int sortOrderResult = o1.sortOrder - o2.sortOrder;
            int phaseSortOrderResult = o1.milestonePhaseTemplate.getSortOrder() - o2.getMilestonePhaseTemplate().getSortOrder();

            if (phaseSortOrderResult == 0) {
                return sortOrderResult;
            }
            return phaseSortOrderResult;
        }
    }

    public enum Type {
        PROJECT("Project"), STUDENT("Individual");

        private String asString;

        Type(String asString) {
            this.asString = asString;
        }

        @Override
        public String toString() {
            return asString;
        }
    }

    public boolean isAutomatic() {
        return code != null || activatedBy != null;
    }
}
