package se.su.dsv.scipro.notifications;

import com.google.inject.persist.Transactional;
import com.mysema.query.BooleanBuilder;
import com.mysema.query.types.Predicate;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.dataobject.NotificationEvent;
import se.su.dsv.scipro.notifications.dataobject.QNotification;
import se.su.dsv.scipro.notifications.dataobject.QProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class NotificationServiceImpl extends AbstractServiceImpl<Notification,Long> implements NotificationService {

    public static final int EMPTY = 0;

    @Inject
    public NotificationServiceImpl(Provider<EntityManager> em) {
        super(em, Notification.class, QNotification.notification);

    }

    @Override
    public long countUnread(User user) {
        if (user == null) {
            return EMPTY;
        }
        return count(QNotification.notification.user.eq(user).and(QNotification.notification.unread.isTrue()));
    }

    @Override
    @Transactional
    public void markRead(User user) {
        Iterable<Notification> unreadNotifications = findAll(
                QNotification.notification.unread.isTrue().and(QNotification.notification.user.eq(user)));

        for (Notification unreadNotification : unreadNotifications) {
            unreadNotification.setUnread(false);
            save(unreadNotification);
        }
    }

    public void setRead(User user, NotificationEvent notificationEvent, boolean read) {
        Iterable<Notification> notifications = findAll(
                QNotification.notification.notificationEvent.eq(notificationEvent)
                        .and(QNotification.notification.user.eq(user)));

        for (Notification notification : notifications) {
            notification.setUnread(!read);
            save(notification);
        }
    }

    @Override
    public void setRead(Project project, boolean read) {
        List<Collection<Notification>> notificationsList = from(QProjectEvent.projectEvent)
                .where(QProjectEvent.projectEvent.project.eq(project))
                .list(QProjectEvent.projectEvent.notifications);

        for (Collection<Notification> notifications: notificationsList){
            for (Notification notification : notifications) {
                notification.setUnread(!read);
                save(notification);
            }
        }
    }

    @Override
    public Collection<Notification> findUnreadByUsers(List<User> users) {
        if(users.isEmpty()) {
            return new ArrayList<>();
        }
        return findAll(QNotification.notification.unread.isTrue().and(QNotification.notification.user.in(users)));
    }

    @Override
    public void deleteAllForUser(User user) {
        for (Notification notification : findAll(QNotification.notification.user.eq(user))){
            delete(notification);
        }
    }

    @Override
    public Iterable<Notification> findAll(Filter filter, Pageable pageable) {
        return findAll(fromFilter(filter), pageable);
    }

    @Override
    public long count(Filter filter) {
        return count(fromFilter(filter));
    }

    private Predicate fromFilter(Filter filter) {
        BooleanBuilder bb = new BooleanBuilder();
        if (filter.getCurrentUser() != null) {
            bb.and(QNotification.notification.user.eq(filter.getCurrentUser()));
        }
        if (filter.getFromUser() != null) {
            bb.and(QNotification.notification.notificationEvent.causedBy.eq(filter.getFromUser()));
        }
        return bb;
    }
}
