package se.su.dsv.scipro.notifications.dataobject;

import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.Collection;

@Entity
@Inheritance(strategy = InheritanceType.SINGLE_TABLE)
@Table(name = "NotificationData")
public abstract class NotificationEvent extends DomainObject {


    @Id
    @GeneratedValue
    protected long id;

    @Enumerated(EnumType.STRING)
    protected Notification.Type type;

    @OneToMany(mappedBy = "notificationEvent", cascade = CascadeType.ALL, orphanRemoval = true)
    private Collection<Notification> notifications;

    @Basic(optional = true)
    private String source;

    @Basic(optional = true)
    private String additionalSource;


    @ManyToOne(optional = true)
    private User causedBy;

    public abstract Enum getEvent();

    /**
     * The title of the entity this event is about.
     *
     * @return a human readable title
     */
    protected abstract String getEntityTitle();

    /**
     * Return true if there is an entity backing this event.
     * Used to determine if this event was about a deletion, or has since been deleted.
     */
    public abstract boolean hasEntity();

    public abstract DomainObject getEntity();

    protected NotificationEvent() {

    }

    protected NotificationEvent(Notification.Type type) {
        this.type = type;
    }

    @Override
    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Notification.Type getType() {
        return type;
    }

    public void setType(Notification.Type type) {
        this.type = type;
    }

    public Collection<Notification> getNotifications() {
        return notifications;
    }

    public void setNotifications(Collection<Notification> notifications) {
        this.notifications = notifications;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public String getTitle() {
        return hasEntity() ? getEntityTitle() : "[Deleted entity]";
    }

    public User getCausedBy() {
        return causedBy;
    }

    public void setCausedBy(User user) {
        this.causedBy = user;
    }

    /**
     * Override this method if the event has an association with a {@link Project}.
     *
     * @return the associated project
     */
    public Project getProject() {
        return null;
    }

    public String getAdditionalSource() {
        return additionalSource;
    }

    public void setAdditionalSource(String additionalSource) {
        this.additionalSource = additionalSource;
    }
}
