package se.su.dsv.scipro.notifications.interfaces.impl;

import se.su.dsv.scipro.notifications.dataobject.NotificationEvent;
import se.su.dsv.scipro.system.User;

import java.text.MessageFormat;
import java.util.Collection;
import java.util.Iterator;

abstract class AbstractEventFormatter<E extends NotificationEvent> implements EventFormatter<E> {

    protected NotificationMailFormatterImpl formatter;

    AbstractEventFormatter(NotificationMailFormatterImpl formatter) {
        this.formatter = formatter;
    }

    protected String formatUser(User user) {
        return MessageFormat.format(getProperty("user"), user.getFirstName(), user.getLastName(), user.getEmailAddress());
    }

    protected <T> String formatUserList(Collection<T> objects, UserConverter<T> converter) {
        StringBuilder sb = new StringBuilder();
        for (Iterator<T> iterator = objects.iterator(); iterator.hasNext(); ) {
            User user = converter.toUser(iterator.next());
            sb.append(formatUser(user));
            if (iterator.hasNext()) {
                sb.append(", ");
            }
        }
        return sb.toString();
    }

    protected String getProperty(String property) {
        return formatter.getProperty(property);
    }

    interface UserConverter<T> {
        User toUser(T object);
    }
}
