package se.su.dsv.scipro.peer;

import lombok.AccessLevel;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.Setter;
import se.su.dsv.scipro.system.DomainObject;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.util.Set;
import java.util.TreeSet;

@Data
@EqualsAndHashCode(of = "id", callSuper = false)
@Entity
@Table(name = "comment_thread", uniqueConstraints = {@UniqueConstraint(columnNames = {"commentableKey", "commentableId"})})
@Cacheable(true)

@SuppressWarnings("PMD.UnusedPrivateField")
public class CommentThread extends DomainObject {

    public static final int COMMENT_MAX_LENGTH = 255;
    @Id
    @GeneratedValue
    @Setter(AccessLevel.NONE)
    private Long id;

    @Basic(optional = false)
    @Column(length = COMMENT_MAX_LENGTH)
    @Size(max = COMMENT_MAX_LENGTH)
    private String commentableKey;

    @Basic(optional = false)
    private Long commentableId;

    @OneToMany(mappedBy = "commentThread", orphanRemoval = true, cascade = CascadeType.ALL, targetEntity = Comment.class)
    private Set<Comment> comments = new TreeSet<>(new Comment.ByDateCreatedComparator());

    protected CommentThread() {
    }

    public CommentThread(Commentable commentable) {
        this.setCommentableKey(commentable.getCommentKey());
        this.setCommentableId(commentable.getId());
    }

}
