package se.su.dsv.scipro.peer;

import com.google.common.eventbus.EventBus;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.system.ProjectTypeSettings;
import se.su.dsv.scipro.workerthreads.AbstractWorker;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class ExpiredRequestWorker extends AbstractWorker {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExpiredRequestWorker.class);

    @Inject
    PeerRequestService peerRequestService;
    @Inject
    EventBus eventBus;

    @Override
    protected void doWork() {
        this.beginTransaction();
        try {
            List<PeerRequest> requests = peerRequestService.findByStatus(RequestStatus.WAITING);

            for (PeerRequest request : requests) {

                ProjectTypeSettings settings = request.getProject().getProjectType().getProjectTypeSettings();
                Date expirationDate = new DateTime().minusDays(settings.getNumDaysBeforePeerGetsCancelled()).toDate();

                if (request.getDateCreated().compareTo(expirationDate) < 0) {
                    LOGGER.info("Peer Request: " + request + " has now been waiting for " + settings.getNumDaysBeforePeerGetsCancelled() + " days and has been removed.");



                    request.setStatus(RequestStatus.CANCELED);
                    peerRequestService.save(request);

                    eventBus.post(new PeerRequestExpiredEvent(request));
                }

            }
            commitTransaction();
        } catch (Exception e) {
            this.rollbackTransaction();
            this.setSuccessfulWorker(false);
            throw new RuntimeException(e);
        }
    }

}
