package se.su.dsv.scipro.peer;

import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.system.ProjectTypeSettings;
import se.su.dsv.scipro.workerthreads.AbstractWorker;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class ExpiredReviewResetWorker extends AbstractWorker {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExpiredReviewResetWorker.class);

    @Inject
    private PeerRequestService peerRequestService;
    @Inject
    private PeerReviewService peerReviewService;

    @Override
    protected void doWork() {
        this.beginTransaction();
        try {
            List<PeerReview> reviews = peerReviewService.findAllReviewsInProgress();

            for (PeerReview peerReview : reviews) {
                PeerRequest peerRequest = peerReview.getPeerRequest();
                ProjectTypeSettings settings = peerReview.getProject().getProjectType().getProjectTypeSettings();

                Date expirationDate = new DateTime().minusDays(settings.getNumDaysToSubmitPeerReview()).toDate();

                if (peerReview.getDateCreated().compareTo(expirationDate) < 0) {
                    LOGGER.info("Expiring peerReview: " + peerReview + " and making " + peerRequest + " available for review again");

                    peerReview.expire();
                    peerReviewService.save(peerReview);

                    peerRequest.setStatus(RequestStatus.WAITING);
                    peerRequestService.save(peerRequest);
                }

            }
            commitTransaction();
        } catch (Exception e) {
            this.rollbackTransaction();
            this.setSuccessfulWorker(false);
            throw new RuntimeException(e);
        }
    }
}
