package se.su.dsv.scipro.report;

import com.google.common.base.Function;
import com.google.common.base.Joiner;
import com.google.common.collect.Iterables;
import lombok.ToString;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@ToString(exclude = {"gradingCriteria"})
@Entity
@SuppressWarnings("PMD.UnusedPrivateField")
public abstract class GradingReport extends Report {

    public enum Grade {
        A, B, C, D, E, F, FX
    }

    public enum State { INITIAL, REVIEWING, FINALIZED }

    @Enumerated(EnumType.STRING)
    private State state = State.INITIAL;

    @ManyToOne(optional = false)
    private Project project;

    @OneToMany(mappedBy = "gradingReport", cascade = {CascadeType.ALL})
    private List<GradingCriterion> gradingCriteria = new ArrayList<>();

    protected GradingReport() {
        // JPA
    }

    @Override
    public void submit() {
        super.submit();
        setState(State.FINALIZED);
    }

    public Project getProject() {
        return project;
    }

    GradingReport(Project project) {
        this.project = project;
    }

    void addCriterion(GradingCriterion criterion) {
        gradingCriteria.add(criterion);
    }

    public State getState() {
        return state;
    }

    public void setState(final State state) {
        this.state = state;
    }

    public List<GradingCriterion> getGradingCriteria() {
        Collections.sort(gradingCriteria, new AbstractCriterion.BySortOrderComparator());
        return Collections.unmodifiableList(gradingCriteria);
    }

    @SuppressWarnings("unused")
    public String getProjectTitle() {
        return project.getTitle();
    }

    @SuppressWarnings("unused")
    public String getAuthorNames() {
        return Joiner.on(", ")
                .join(Iterables.transform(getProject().getProjectParticipants(),
                        new Function<User, String>() {
                    @Override
                    public String apply(final User input) {
                        return input.getFullName();
                    }
                }));
    }

    public ProjectType getProjectType() {
        return project.getProjectType();
    }

    public Grade getGrade(GradeCalculator gradeCalculator) {
        return gradeCalculator.getGrade(this);
    }

    public long getPoints(GradeCalculator gradeCalculator) {
        return gradeCalculator.getPoints(this);
    }
}
