package se.su.dsv.scipro.report;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.ToString;
import se.su.dsv.scipro.finalseminar.FinalSeminarOpposition;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

@ToString(exclude = {"criteria"})
@EqualsAndHashCode(callSuper = false, of = "id")
@Entity
@Table(name = "grading_report_template")
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class GradingReportTemplate extends DomainObject {

    @Id
    @GeneratedValue
    @Getter
    private Long id;

    @OneToOne(optional = false)
    private ProjectType projectType;

    @OneToMany(mappedBy = "gradingReportTemplate", cascade = {CascadeType.ALL})
    private Collection<GradingCriterionTemplate> criteria = new HashSet<>();

    @Basic
    private int credits;

    protected GradingReportTemplate() {

    }

    public GradingReportTemplate(ProjectType projectType, int credits) {
        if (projectType == null) {
            throw new IllegalArgumentException("ProjectType may not be null");
        }
        this.projectType = projectType;
        this.credits = credits;
    }

    public SupervisorGradingReport createSupervisorReport(Project project, User student) {
        if (!this.projectType.equals(project.getProjectType())) {
            throw new IllegalArgumentException("Project has a different project class than this template");
        }
        return new SupervisorGradingReportFactory().using(criteria).create(project, student);
    }

    public ReviewerGradingReport createReviewerReport(Project project){
        if (!this.projectType.equals(project.getProjectType())) {
            throw new IllegalArgumentException("Project has a different project class than this template");
        }
        return new ReviewerGradingReportFactory().using(criteria).create(project);
    }

    public OppositionReport createOppositionReport(FinalSeminarOpposition finalSeminarOpposition) {
        return new OppositionReport(this, finalSeminarOpposition);
    }

    public GradingCriterionTemplate addProjectCriterion(String title, int pointsRequiredToPass, List<GradingCriterionPointTemplate> gradingCriterionPointTemplates) {
        GradingCriterionTemplate gradingCriterionTemplate = new ProjectGradingCriterionTemplate(this, criteria.size(), title, pointsRequiredToPass, gradingCriterionPointTemplates);
        criteria.add(gradingCriterionTemplate);
        return gradingCriterionTemplate;
    }

    public GradingCriterionTemplate addIndividualCriterion(String title, int pointsRequiredToPass, List<GradingCriterionPointTemplate> gradingCriterionPointTemplates) {
        GradingCriterionTemplate gradingCriterionTemplate = new IndividualGradingCriterionTemplate(this, criteria.size(), title, pointsRequiredToPass, gradingCriterionPointTemplates);
        criteria.add(gradingCriterionTemplate);
        return gradingCriterionTemplate;
    }

    public Iterable<GradingCriterionTemplate> getCriteria() {
        return criteria;
    }
}
