package se.su.dsv.scipro.report;

import com.google.common.base.Function;
import com.google.common.base.Joiner;
import com.google.common.collect.Iterables;
import lombok.Getter;
import lombok.Setter;
import se.su.dsv.scipro.finalseminar.FinalSeminarOpposition;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

@Entity
@Table(name = "opposition_report")
@SuppressWarnings("PMD.UnusedPrivateField")
public class OppositionReport extends AttachmentReport {

    public static final String UNUSED = "unused";

    @Getter
    @OneToOne(optional = false)
    private FinalSeminarOpposition finalSeminarOpposition;

    @SuppressWarnings("MismatchedQueryAndUpdateOfCollection")
    @OneToMany(mappedBy = "report", cascade = {CascadeType.ALL})
    private List<Criterion> oppositionCriteria = new ArrayList<>();

    @Basic
    @Column
    @Getter
    @Setter
    private String thesisSummary;


    protected OppositionReport() {

    }

    public OppositionReport(GradingReportTemplate gradingReportTemplate, FinalSeminarOpposition finalSeminarOpposition) {
        this.finalSeminarOpposition = finalSeminarOpposition;
        createCriteriaFromTemplate(gradingReportTemplate);
    }

    private void createCriteriaFromTemplate(GradingReportTemplate gradingReportTemplate) {
        for (GradingCriterionTemplate template : gradingReportTemplate.getCriteria()) {
            if (template.isProjectCriterion()) {
                oppositionCriteria.add(new Criterion(this, template));
            }
        }
    }

    public List<Criterion> getCriteria() {
        Collections.sort(oppositionCriteria, new AbstractCriterion.BySortOrderComparator());
        return Collections.unmodifiableList(oppositionCriteria);
    }

    @Override
    public boolean isFinished() {
        if (thesisSummaryIsEmpty()) {
            return false;
        }
        for (Criterion criterion : oppositionCriteria) {
            if (!criterion.isFilledOut()) {
                return false;
            }
        }
        return true;
    }

    private boolean thesisSummaryIsEmpty() {
        return thesisSummary == null || thesisSummary.isEmpty();
    }

    public User getUser() {
        return finalSeminarOpposition.getUser();
    }

    public String getProjectTitle() {
        return finalSeminarOpposition.getFinalSeminar().getProjectTitle();
    }

    @SuppressWarnings(UNUSED)
    public String getAuthorNames() {
        return Joiner.on(", ")
                .join(Iterables.transform(finalSeminarOpposition.getFinalSeminar().getProject().getProjectParticipants(),
                        new Function<User, String>() {
                    @Override
                    public String apply(final User input) {
                        return input.getFullName();
                    }
                }));
    }

    @SuppressWarnings(UNUSED)
    public String getSupervisorName() {
        return finalSeminarOpposition.getFinalSeminar().getProject().getSupervisorName();
    }

    @SuppressWarnings(UNUSED)
    public String getReviewerName() {
        return finalSeminarOpposition.getFinalSeminar().getProject().getReviewerName();
    }

    @SuppressWarnings(UNUSED)
    public Date getFinalSeminarDate() {
        return finalSeminarOpposition.getFinalSeminar().getStartDate();
    }

    @SuppressWarnings(UNUSED)
    public String getAuthorName() {
        return finalSeminarOpposition.getUser().getFullName();
    }

    public String getOpponentFirstName() {
        return finalSeminarOpposition.getUser().getFirstName();
    }

    public String getOpponentLastName() {
        return finalSeminarOpposition.getUser().getLastName();
    }
}
