package se.su.dsv.scipro.report;

import scala.Option;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

public class ReportServiceImpl extends AbstractServiceImpl<Report, Long> implements ReportService {

    private final FileDescriptionService fileDescriptionService;

    @Inject
    public ReportServiceImpl(Provider<EntityManager> em, final FileDescriptionService fileDescriptionService) {
        super(em, Report.class, QReport.report);
        this.fileDescriptionService = fileDescriptionService;
    }

    @Override
    public AttachmentReport submit(AttachmentReport report) {
        report.submit();
        return save(report);
    }

    @Override
    public void save(AttachmentReport report, Option<FileUpload> fileUpload) {
        storeReportFile(report, fileUpload);
        save(report);
    }

    @Override
    public void deleteAttachment(AttachmentReport report) {
        fileDescriptionService.delete(report.getAttachment().getId());
        report.setAttachment(null);
        save(report);
    }

    private void storeReportFile(AttachmentReport report, Option<FileUpload> fileUpload) {
        if (fileUpload.isDefined()) {
            FileDescription fileDescription = fileDescriptionService.storeFile(fileUpload.get());
            report.setAttachment(fileDescription);
        }
    }
}
