package se.su.dsv.scipro.report;

import lombok.Getter;
import lombok.Setter;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToOne;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

@Entity
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class SupervisorGradingReport extends GradingReport {
    @ManyToOne(optional = false)
    @Getter
    private User user;

    @Basic
    @Column(length = 2000)
    @Getter
    @Setter
    private String plagiarismFeedback;

    protected SupervisorGradingReport() {
        // JPA
    }

    SupervisorGradingReport(Project project, User user) {
        super(project);
        this.user = user;
    }

    public List<GradingCriterion> getProjectCriteria() {
        List<GradingCriterion> result = new ArrayList<>();
        for (GradingCriterion criterion : getGradingCriteria()) {
            if (criterion.isProjectCriterion()) {
                result.add(criterion);
            }
        }
        Collections.sort(result, new AbstractCriterion.BySortOrderComparator());
        return Collections.unmodifiableList(result);
    }

    public List<GradingCriterion> getIndividualCriteria() {
        List<GradingCriterion> result = new ArrayList<>();
        for (GradingCriterion criterion : getGradingCriteria()) {
            if (criterion.isIndividualCriterion()) {
                result.add(criterion);
            }
        }
        Collections.sort(result, new AbstractCriterion.BySortOrderComparator());
        return Collections.unmodifiableList(result);
    }

    public boolean isFinished() {
        return allCriteriaAssessed() && hasPlagiarismFeedback();
    }

    private boolean allCriteriaAssessed() {
        for (GradingCriterion gradingCriterion : getGradingCriteria()) {
            if (!gradingCriterion.isAssessed()) {
                return false;
            }
        }
        return true;
    }

    private boolean hasPlagiarismFeedback() {
        return plagiarismFeedback != null && !plagiarismFeedback.isEmpty();
    }

    public void copyProjectAssessmentFrom(SupervisorGradingReport from) {
        setPlagiarismFeedback(from.getPlagiarismFeedback());

        Iterator<GradingCriterion> copyTo = this.getProjectCriteria().iterator();
        Iterator<GradingCriterion> copyFrom = from.getProjectCriteria().iterator();

        while (copyTo.hasNext()) {
            copyAssessment(copyTo.next(), copyFrom.next());
        }
    }

    private void copyAssessment(GradingCriterion copyTo, GradingCriterion copyFrom) {
        copyTo.setPoints(copyFrom.getPoints());
        copyTo.setFeedback(copyFrom.getFeedback());
    }
}
