package se.su.dsv.scipro.reviewing;

import com.google.common.eventbus.EventBus;
import scala.Option;
import scala.runtime.AbstractFunction0;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.project.Project;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Date;

class RoughDraftApprovalServiceImpl extends AbstractReviewerApprovalServiceImpl<RoughDraftApproval> implements RoughDraftApprovalService {

    private final DaysService daysService;
    private final ReviewerDeadlineSettingsService reviewerDeadlineSettingsService;

    @Inject
    RoughDraftApprovalServiceImpl(
            final Provider<EntityManager> em,
            final EventBus eventBus,
            final FileDescriptionService fileDescriptionService,
            final DaysService daysService,
            final ReviewerDeadlineSettingsService reviewerDeadlineSettingsService)
    {
        super(em, RoughDraftApproval.class, QRoughDraftApproval.roughDraftApproval, eventBus, fileDescriptionService);
        this.daysService = daysService;
        this.reviewerDeadlineSettingsService = reviewerDeadlineSettingsService;
    }

    @Override
    public RoughDraftApproval requestApproval(final Project project, FileUpload thesis, final String comment) {
        final FileDescription storedThesis = fileDescriptionService.storeFile(thesis);
        final RoughDraftApproval roughDraftApproval = findBy(project).map(new AbstractFunction1<RoughDraftApproval, RoughDraftApproval>() {
            @Override
            public RoughDraftApproval apply(final RoughDraftApproval approval) {
                final Date deadline = getDeadline(new Date());
                approval.addNewThesis(storedThesis, comment, deadline);
                return approval;
            }
        }).getOrElse(new AbstractFunction0<RoughDraftApproval>() {
            @Override
            public RoughDraftApproval apply() {
                return newApproval(project, storedThesis, comment);
            }
        });
        final RoughDraftApproval persisted = save(roughDraftApproval);
        eventBus.post(new RoughDraftApprovalRequestedEvent(persisted));
        return persisted;
    }

    @Override
    protected RoughDraftApproval newApproval(Project project, FileDescription storedThesis, String comment) {
        final Date deadline = getDeadline(new Date());
        return new RoughDraftApproval(project, storedThesis, comment, deadline);
    }

    private Date getDeadline(final Date requested) {
        final ReviewerDeadlineSettings reviewerDeadlineSettings = reviewerDeadlineSettingsService.getInstance();
        return daysService.workDaysAfter(requested, reviewerDeadlineSettings.getRoughDraftApproval());
    }

    @Override
    public Option<RoughDraftApproval> findBy(Project project) {
        return Option.apply(findOne(QRoughDraftApproval.roughDraftApproval.project.eq(project)));
    }

}
