package se.su.dsv.scipro.springdata.serviceimpls;

import com.mysema.query.BooleanBuilder;
import com.mysema.query.jpa.JPASubQuery;
import com.mysema.query.types.expr.BooleanExpression;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.QTarget;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.springdata.services.SupervisorService;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Collection;

public class SupervisorServiceImpl extends AbstractServiceImpl<User,Long> implements SupervisorService {

    @Inject
    public SupervisorServiceImpl(Provider<EntityManager> em) {
        super(em, User.class, QUser.user);
    }

    @Override
    public Iterable<User> findAll(FilterParams params, Pageable pageable) {
        if (params != null) {
            return findAll(QUser.user.roles.any().eq(Roles.SUPERVISOR).and(fromParams(params)), pageable);
        } else {
            return findAll(QUser.user.roles.any().eq(Roles.SUPERVISOR), pageable);
        }
    }

    @Override
    public Iterable<User> findByResearchArea(ResearchArea area, Pageable pageable) {
        return findAll(areaFilter(area), pageable);
    }

    @Override
    public long count(FilterParams params) {
        return count(fromParams(params));
    }

    @Override
    public Long countByResearchArea(ResearchArea area) {
        return count(areaFilter(area));
    }

    private BooleanBuilder fromParams(FilterParams params) {
        BooleanBuilder bb = new BooleanBuilder();
        if (params.getUnit() != null) {
            bb.and(unitFilter(params.getUnit()));
        }
        if (params.getSupervisor() != null) {
            bb.and(supervisorIs(params.getSupervisor()));
        }
        if (params.getResearchArea() != null) {
            bb.and(areaFilter(params.getResearchArea()));
        }

        if (params.getLanguages()!=null){
            if (params.getLanguages().isEmpty()){
                bb.and(QUser.user.languages.any().isNull());
            } else {
                bb.and(languageFilter(params.getLanguages()));
            }
        }

        if (params.getHasTargetsInApplicationPeriod()!=null){
            bb.and(hasTargetsInApplicationPeriod(params.getHasTargetsInApplicationPeriod()));
        }

        return bb;
    }

    private BooleanExpression languageFilter(Collection<Language> languages) {
        return QUser.user.languages.any().in(languages);
    }

    private BooleanExpression unitFilter(Unit unit) {
            return QUser.user.unit.eq(unit);
    }

    private BooleanExpression areaFilter(ResearchArea area) {
        if (area != null) {
            return QUser.user.researchAreas.contains(area);
        } else {
            return QUser.user.isNotNull();
        }
    }

    private BooleanExpression supervisorIs(User supervisor) {
            return QUser.user.eq(supervisor);
    }

    private BooleanExpression hasTargetsInApplicationPeriod(ApplicationPeriod applicationPeriod){
        return QUser.user.in(new JPASubQuery().from(QTarget.target1).where(QTarget.target1.applicationPeriod.eq(applicationPeriod).and(QTarget.target1.target.notIn(0))).list(QTarget.target1.user));
    }

}
