package se.su.dsv.scipro.springdata.serviceimpls;

import com.google.inject.persist.Transactional;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.settings.dataobjects.QUserProfile;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Arrays;
import java.util.List;

public class UserProfileServiceImpl extends AbstractServiceImpl<UserProfile, Long> implements UserProfileService {
    @Inject
    public UserProfileServiceImpl(Provider<EntityManager> em) {
        super(em, UserProfile.class, QUserProfile.userProfile);

    }

    @Override
    @Transactional
    public UserProfile findByUser(User user) {
        UserProfile profile = findOne(QUserProfile.userProfile.user.eq(user));
        if (profile == null) {
            profile = new UserProfile();
            profile.setUser(user);
            profile.setDefaultProjectTypeFilter(getDefaultProjectTypes());
            profile = save(profile);
        }
        return profile;
    }

    @Override
    public boolean isReceiveForumMail(User user) {
        return findByUser(user).isReceiveForumMail();
    }

    @Override
    public void setReceiveForumMail(User user, boolean receive) {
        UserProfile profile = findByUser(user);
        profile.setReceiveForumMail(receive);
        save(profile);
    }

    @Override
    public List<User> findUsersWithMailCompilation() {
        return createQuery().where(QUserProfile.userProfile.mailCompilation.isTrue()).list(QUserProfile.userProfile.user);
    }

    @Override
    public Roles findSelectedRole(User user) {
        return findByUser(user).getSelectedRole();
    }

    @Override
    public void setSelectedRole(User user, Roles role) {
        UserProfile profile = findByUser(user);
        profile.setSelectedRole(role);
        save(profile);
    }

    @Override
    public boolean isExpandChecklistOverview(User user) {
        return findByUser(user).isExpandChecklistOverview();
    }

    @Override
    public void setExpandChecklistOverview(User user, boolean expand) {
        UserProfile profile = findByUser(user);
        profile.setExpandChecklistOverview(expand);
        save(profile);
    }

    private List<ProjectType> getDefaultProjectTypes() {
        QProjectType projectType = QProjectType.projectType;
        return from(projectType)
                .where(projectType.degreeLevel.in(Arrays.asList(DegreeLevel.FIRST_CYCLE, DegreeLevel.SECOND_CYCLE)))
                .list(projectType);
    }
}
