package se.su.dsv.scipro.system;

import se.su.dsv.scipro.reusable.SciProUtilities;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

public class HashServiceImpl extends AbstractServiceImpl<Hash, Long> implements HashService {
    @Inject
    public HashServiceImpl(Provider<EntityManager> em) {
        super(em, Hash.class, QHash.hash1);
    }

    @Override
    public void deleteAllHashesForUser(User user) {
        for (Hash hash : findAll(QHash.hash1.user.eq(user))){
            delete(hash);
        }
    }

    @Override
    public Hash createForUser(User user) {
        char[] input = user.getEmailAddress().toLowerCase().toCharArray();
        byte[] saltBytes = PasswordHandler.generateSalt();
        byte[] hashBytes = PasswordHandler.generateHash(input, saltBytes);
        Hash hash = Hash.builder().hash(SciProUtilities.convertToHex(hashBytes)).user(user).build();
        return save(hash);
    }

    @Override
    public Hash findByHash(String hashString) {
        return findOne(QHash.hash1.hash.eq(hashString));
    }
}