package se.su.dsv.scipro.system;

import com.mysema.query.jpa.JPASubQuery;
import com.mysema.query.types.expr.BooleanExpression;
import com.mysema.query.types.query.ListSubQuery;
import se.su.dsv.scipro.project.QProject;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Collection;
import java.util.List;

import static com.mysema.query.types.ExpressionUtils.allOf;
import static com.mysema.query.types.ExpressionUtils.anyOf;

public class ProjectTypeServiceImpl extends AbstractServiceImpl<ProjectType,Long> implements ProjectTypeService {

    @Inject
    public ProjectTypeServiceImpl(Provider<EntityManager> em) {
        super(em, ProjectType.class, QProjectType.projectType);
    }

    @Override
    public List<ProjectType> findAllActive() {
        return findAll(notDeleted());
    }

    @Override
    public List<ProjectType> findAllDeleted() {
        return findAll(QProjectType.projectType.deleted.isTrue());
    }

    @Override
    public List<ProjectType> findByDegreeLevels(Collection<DegreeLevel> degreeLevels) {
        return findAll(allOf(QProjectType.projectType.degreeLevel.in(degreeLevels), notDeleted()));
    }

    @Override
    public List<ProjectType> findWithModule(ProjectModule projectModule) {
        return findAll(allOf(QProjectType.projectType.projectModules.contains(projectModule), notDeleted()));
    }

    @Override
    public List<ProjectType> findBySupervisorProjects(User supervisor) {
        QProjectType projectType = QProjectType.projectType;
        return from(projectType).where(projectType.in(joinProjects(supervisor))).distinct().list(projectType);
    }

    @Override
    public boolean projectTypeWithModuleExists(ProjectModule module) {
        return !findWithModule(module).isEmpty();
    }

    private ListSubQuery<ProjectType> joinProjects(User supervisor) {
        QProject project = QProject.project;
        return new JPASubQuery()
                .from(project)
                .where(anyOf(project.headSupervisor.eq(supervisor),
                        project.reviewers.contains(supervisor),
                        project.coSupervisors.contains(supervisor)))
                .list(project.projectType);
    }

    private static BooleanExpression notDeleted() {
        return QProjectType.projectType.deleted.isFalse();
    }
}
