package se.su.dsv.scipro.thesislink;

import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;

@Data
@Entity
@Table(name = "externallink")
@EqualsAndHashCode(callSuper = false)
@SuppressWarnings("PMD.UnusedPrivateField")
public class ExternalLink extends DomainObject {

    public static final int MAX_CHARS = 255;

    @Id
    @GeneratedValue
    private Long id;

    @ManyToOne(optional = false)
    private Project project;

    @Column(nullable = false, length = MAX_CHARS)
    private String url = "";

    @ManyToOne(optional = false)
    private User user;

    @Column(nullable = true, length = MAX_CHARS)
    private String description = "";

    public static IProject builder() {
        return new Builder();
    }

    private static class Builder implements IProject, IURL, IUser, IBuild {
        private ExternalLink instance = new ExternalLink();

        @Override
        public ExternalLink build() {
            return instance;
        }

        @Override
        public IURL project(Project project) {
            instance.setProject(project);
            return this;
        }

        @Override
        public IUser url(String url) {
            instance.setUrl(url);
            return this;
        }

        @Override
        public IBuild user(User user) {
            instance.setUser(user);
            return this;
        }

        @Override
        public IBuild description(String description) {
            instance.setDescription(description);
            return this;
        }
    }

    public interface IProject {
        IURL project(Project project);
    }

    public interface IURL {
        IUser url(String url);
    }

    public interface IUser {
        IBuild user(User user);
    }

    public interface IBuild {
        IBuild description(String description);
        ExternalLink build();
    }
}
