package se.su.dsv.scipro.workerthreads;

import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.phd.AnnualReviewService;
import se.su.dsv.scipro.phd.StudyPlan;
import se.su.dsv.scipro.system.ProjectModule;

import javax.inject.Inject;

public class AnnualReviewWorker extends AbstractWorker {

    @Inject
    private AnnualReviewService annualReviewService;
    @Inject
    NotificationController notificationController;

    @Override
    protected void doWork() {
        this.beginTransaction();

        try {
            Iterable<StudyPlan> studyPlans = annualReviewService.findStudyPlansWithoutAnnualReviewThisYear();

            for (StudyPlan studyPlan : studyPlans) {
                if (studyPlan.getProject().getProjectType().getProjectModules().contains(ProjectModule.STUDY_PLAN)) {
                    notificationController.notifyProject(studyPlan.getProject(),
                            ProjectEvent.Event.ONE_YEAR_PASSED_FROM_LATEST_ANNUAL_REVIEW, new NotificationSource());
                }
            }
            commitTransaction();
        } catch (Exception e) {
            this.rollbackTransaction();
            this.setSuccessfulWorker(false);
            throw new RuntimeException(e);
        }

    }
}
