package se.su.dsv.scipro.workerthreads;

import javax.inject.Provider;
import java.io.Serializable;
import java.util.Comparator;

public class Task {
    private final Provider<? extends Worker> worker;
    private final String description;
    private final Schedule schedule;

    Task(final String description, final Provider<? extends Worker> worker, final Schedule schedule) {
        this.description = description;
        this.worker = worker;
        this.schedule = schedule;
    }

    void execute() {
        Runnable workerThread = worker.get();
        Thread thread = new Thread(workerThread);
        thread.start();
    }

    public String getDescription() {
        return description;
    }

    public String getWorkerName() {
        return worker.get().getName();
    }

    public long getPeriod() {
        return schedule.getTimeUnit().toMillis(schedule.getPeriod());
    }

    public static class ByDescriptionComparator implements Comparator<Task>, Serializable {
        @Override
        public int compare(Task o1, Task o2) {
            return o1.description.compareTo(o2.description);
        }
    }
}
