package se.su.dsv.scipro.workerthreads;

import com.google.inject.AbstractModule;
import com.google.inject.Scopes;
import com.google.inject.multibindings.Multibinder;
import se.su.dsv.scipro.mail.MailEventWorker;
import se.su.dsv.scipro.phd.AnnualReviewService;
import se.su.dsv.scipro.phd.AnnualReviewServiceImpl;
import se.su.dsv.scipro.system.Lifecycle;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

public class WorkerModule extends AbstractModule {
    private static final int NUMBER_OF_WORKER_THREADS = 4;

    @Override
    protected void configure() {
        Multibinder.newSetBinder(binder(), Lifecycle.class).addBinding().to(SchedulerImpl.class);
        bind(ScheduledExecutorService.class).toInstance(Executors.newScheduledThreadPool(NUMBER_OF_WORKER_THREADS));
        bind(Scheduler.class).to(SchedulerImpl.class).in(Scopes.SINGLETON);
        bind(TemporaryWorkerScheduler.class).asEagerSingleton();
        bind(WorkerDataService.class).to(WorkerDataServiceImpl.class);
        bind(AnnualReviewService.class).to(AnnualReviewServiceImpl.class);
        bind(ThesisUploadDeadlineWorker.class);
        bind(AnnualReviewWorker.class);
        bind(ManualMatchRemindWorker.class);
        bind(TurnItInCheckWorker.class);
        bind(MailEventWorker.class);
        bind(NotificationCompilationWorker.class);
        bind(ThesisUploadReminderWorker.class);
        bind(TurnItInExpirationWorker.class);
        bind(IdeaExportWorker.class);
    }
}
