package se.su.dsv.scipro.crosscutting

import javax.inject.{Inject, Singleton}

import com.google.common.eventbus.{EventBus, Subscribe}
import se.su.dsv.scipro.notifications.NotificationController
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent.Event
import se.su.dsv.scipro.notifications.dataobject.{NotificationSource, ProjectEvent}
import se.su.dsv.scipro.project.ReviewerAssignedEvent
import se.su.dsv.scipro.reviewing.{FinalSeminarApprovalService, ReviewerApproval, RoughDraftApprovalService}

@Singleton
class ReviewerAssignedNotifications(roughDraftApprovalService: RoughDraftApprovalService, finalSeminarApprovalService: FinalSeminarApprovalService, notificationController: NotificationController) {

  @Inject
  def this(roughDraftApprovalService: RoughDraftApprovalService, finalSeminarApprovalService: FinalSeminarApprovalService, notificationController: NotificationController, eventBus: EventBus) {
    this(roughDraftApprovalService, finalSeminarApprovalService, notificationController)
    eventBus.register(this)
  }

  @Subscribe
  def reviewerAssigned(event: ReviewerAssignedEvent): Unit = {
    roughDraftApprovalService.findBy(event.getProject)
      .filterNot(_.isDecided)
      .foreach(sendNotification(ProjectEvent.Event.ROUGH_DRAFT_APPROVAL_REQUESTED))

    finalSeminarApprovalService.findBy(event.getProject)
      .filterNot(_.isDecided)
      .foreach(sendNotification(ProjectEvent.Event.FINAL_SEMINAR_APPROVAL_REQUESTED))
  }

  private def sendNotification(event: Event): ReviewerApproval => Unit = approval => {
    notificationController.notifyProject(approval.getProject, event, new NotificationSource)
  }
}