package se.su.dsv.scipro.forum.notifications

import javax.inject.Inject

import com.google.common.eventbus.{EventBus, Subscribe}
import com.google.inject.persist.Transactional
import se.su.dsv.scipro.forum.dataobjects.ForumPost
import se.su.dsv.scipro.forum.{NewGroupForumReplyEvent, ForumPostReadEvent, NewProjectForumReplyEvent}
import se.su.dsv.scipro.notifications.dataobject.{Notification, GroupEvent, NotificationSource, ProjectForumEvent}
import se.su.dsv.scipro.notifications.{NotificationController, NotificationService}
import se.su.dsv.scipro.reviewing.NewReviewerInteractionMessage

import scala.collection.Set
import scala.collection.JavaConversions._

class ForumNotifications @Inject()(eventBus: EventBus,
                                   notificationController: NotificationController,
                                   forumNotificationRepository: ForumNotificationRepository,
                                   notificationService: NotificationService) {
  eventBus.register(this)

  @Subscribe
  @Transactional
  def newProjectForumPost(forumReplyEvent: NewProjectForumReplyEvent) = {
    val post = forumReplyEvent.getReply

    val event =
      if (post.getForumThread.getPostCount == 1) ProjectForumEvent.Event.NEW_FORUM_POST
      else ProjectForumEvent.Event.NEW_FORUM_POST_COMMENT

    sendAndConnect(post, source ⇒ {
      notificationController.notifyProjectForum(event, source, forumReplyEvent.getProjectThread.getProject, post.getPostedBy)
    })
  }

  @Subscribe
  @Transactional
  def newGroupForumPost(forumReplyEvent: NewGroupForumReplyEvent) = {
    val post = forumReplyEvent.getReply

    val event =
      if (post.getForumThread.getPostCount == 1) GroupEvent.Event.MESSAGE_THREAD_CREATED
      else GroupEvent.Event.MESSAGE_THREAD_REPLY

    sendAndConnect(post, source => {
      notificationController.notifyGroup(forumReplyEvent.getGroupThread.getGroup, event, source, post.getPostedBy)
    })
  }

  @Subscribe
  @Transactional
  def newReviewerInteractionMessage(event: NewReviewerInteractionMessage) = {
    val source: NotificationSource = new NotificationSource
    source.setMessage(event.getMessage.getContent)
    val notifications = notificationController.notifyProjectForum(ProjectForumEvent.Event.NEW_REVIEWER_INTERACTION, source, event.getProject)
    notifications.map(_.getNotificationEvent) foreach { notificationEvent ⇒
      val connection = new ForumNotification(event.getMessage, notificationEvent)
      forumNotificationRepository.save(connection)
    }
  }

  def sendAndConnect(post: ForumPost, send: NotificationSource ⇒ Set[Notification]) = {
    val notificationSource = new NotificationSource
    notificationSource.setMessage(s"Posted by ${post.getPostedBy.getFullName}\n\n${post.getContent}")
    notificationSource.setAdditionalMessage(post.getSubject)

    send(notificationSource).map(_.getNotificationEvent) foreach { notificationEvent ⇒
      val connection = new ForumNotification(post, notificationEvent)
      forumNotificationRepository.save(connection)
    }
  }

  @Subscribe
  @Transactional
  def forumPostRead(forumPostReadEvent: ForumPostReadEvent) = {
    forumNotificationRepository.findByForumPost(forumPostReadEvent.getPost) foreach { connection ⇒
      notificationService.setRead(forumPostReadEvent.getUser, connection.getNotificationEvent, true)
    }
  }
}
