package se.su.dsv.scipro.grading

import javax.inject.Provider
import javax.persistence.EntityManager

import com.mysema.query.jpa.impl.JPAQuery
import com.mysema.query.types.path.StringPath
import com.mysema.query.types.{EntityPath, PathMetadataFactory}
import org.springframework.data.domain.{Pageable, Sort}

trait FindAll[T] {
  
  val em: Provider[EntityManager]
  
  val path: EntityPath[T]

  private def createQuery: JPAQuery = {
    from(path)
  }

  private def from(paths: EntityPath[_]*): JPAQuery = {
    new JPAQuery(em.get).from(paths:_*)
  }
  
  private def pagedQuery(pageable: Pageable): JPAQuery = {
    applyPageable(createQuery, pageable)
  }

  private def applyPageable(query: JPAQuery, pageable: Pageable): JPAQuery = {
    val pagedQuery: JPAQuery = paged(query, pageable.getOffset, pageable.getLimit)
    if (pageable.getSort == null) {
      pagedQuery
    } else {
      sorted(pagedQuery, pageable.getSort)  
    }
  }

  private def paged(query: JPAQuery, offset: Long, limit: Long): JPAQuery = {
    query.limit(limit).offset(offset)
  }

  private def sorted(query: JPAQuery, sort: Sort): JPAQuery = {
    val property: StringPath = new StringPath(PathMetadataFactory.forProperty(path, sort.getProperty))
    query.orderBy(if (sort.isAscending) property.asc else property.desc)
  }

  def findAll(pageable: Pageable): Iterable[T] = {
    import scala.collection.JavaConverters._
    pagedQuery(pageable).list(path).asScala
  }
}
