package se.su.dsv.scipro.activityplan;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.checklist.*;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class ActivityPlanFacadeImplIntegrationTestChecklistMethods extends IntegrationTest {

    private static final String SOME_NAME = "checklist";

    @Inject
    private ActivityPlanFacade activityPlanFacade;

    private User employee1;
    private Checklist checklist;
    private ChecklistAnswer checklistAnswer;

    @Before
    public void startTransaction() throws Exception {

        employee1 = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        employee1 = save(employee1);

        ProjectType bachelor = new ProjectType(ProjectType.BACHELOR, "Bachelor", "Bachelor degree thesis project");
        bachelor = save(bachelor);

        Project project = Project.builder().
                title("project").
                projectType(bachelor).
                headSupervisor(employee1).
                build();
        project = save(project);

        List<ChecklistAnswer> checklistAnswers = new ArrayList<>();
        checklistAnswer = new ChecklistAnswer(employee1, ChecklistAnswerEnum.GREEN);
        checklistAnswer = save(checklistAnswer);
        checklistAnswers.add(checklistAnswer);

        List<ChecklistQuestion> checklistQuestions = new ArrayList<>();
        ChecklistQuestion checklistQuestion = new ChecklistQuestion("question?", 1);
        checklistQuestion.setAnswers(checklistAnswers);
        checklistQuestion = save(checklistQuestion);
        checklistQuestions.add(checklistQuestion);

        checklist = Checklist.builder().name(SOME_NAME).project(project).build();
        checklist.setQuestions(checklistQuestions);
        checklist = save(checklist);
    }

    @Test
    public void testStuff() {
        Assert.assertTrue(
                checklist.getQuestions() != null
                        && !checklist.getQuestions().isEmpty()
                        && checklist.getQuestions().get(0) != null
                        && checklist.getQuestions().get(0).getAnswers() != null
                        && !checklist.getQuestions().get(0).getAnswers().isEmpty()
                        && checklist.getQuestions().get(0).getAnswers().get(0) != null
                        && checklist.getQuestions().get(0).getAnswers().get(0).getAnswer() != null
                        && checklist.getQuestions().get(0).getAnswers().get(0).getAnswer().equals(ChecklistAnswerEnum.GREEN)
                        && checklist.getUserLastOpenDate().get(employee1) == null
        );
    }

    /**
     * If we run method, the date should get updated.
     */
    @Test
    public void testupdateUserLastOpenDate2() {

        checklist = activityPlanFacade.updateUserLastOpenDate(checklist, employee1);
        Date oldDate = checklist.getUserLastOpenDate().get(employee1);

        try {
            Thread.sleep(10);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        checklist = activityPlanFacade.updateUserLastOpenDate(checklist, employee1);
        Date newDate = checklist.getUserLastOpenDate().get(employee1);

        Assert.assertTrue(oldDate.before(newDate));
    }
}