package se.su.dsv.scipro.crosscutting;

import com.google.common.eventbus.EventBus;
import junit.framework.TestCase;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewing.FinalSeminarApproval;
import se.su.dsv.scipro.reviewing.FinalSeminarApprovalRequestEvent;
import se.su.dsv.scipro.reviewing.RoughDraftApproval;
import se.su.dsv.scipro.reviewing.RoughDraftApprovalRequestedEvent;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ReviewerSupportMailerTest extends TestCase {
    @Mock
    private EventBus eventBus;
    @Mock
    private MailEventService mailEventService;
    @Mock
    private GeneralSystemSettingsService generalSystemSettingsService;
    @InjectMocks
    private ReviewerSupportMailer reviewerSupportMailer;
    private FinalSeminarApproval finalSeminarApproval;
    private RoughDraftApproval roughDraftApproval;

    @Before
    public void setUp() throws Exception {
        ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        User supervisor = ObjectMother.SOME_USER;
        Set<ResearchArea> researchAreas = new HashSet<>();
        ResearchArea researchArea = new ResearchArea();
        researchArea.setTitle("IT");
        researchAreas.add(researchArea);
        supervisor.setResearchAreas(new HashSet<>(researchAreas));
        final Project project = Project.builder().title("Title").projectType(bachelor).headSupervisor(supervisor).build();
        finalSeminarApproval = new FinalSeminarApproval(project, new FileDescription(), "hello", new Date());
        roughDraftApproval = new RoughDraftApproval(project, new FileDescription(), "hello", new Date());
    }

    @Test
    public void registers_to_receive_events() throws Exception {
        verify(eventBus).register(reviewerSupportMailer);
    }

    @Test
    public void testOnFinalSeminarApprovalRequest() throws Exception {
        mockSystemSettings();
        reviewerSupportMailer.onFinalSeminarApprovalRequest(new FinalSeminarApprovalRequestEvent(finalSeminarApproval));
        verify(mailEventService).save(new MailEvent());
    }

    @Test
    public void testOnRoughDraftApprovalRequest() throws Exception {
        mockSystemSettings();
        reviewerSupportMailer.onRoughDraftApprovalRequest(new RoughDraftApprovalRequestedEvent(roughDraftApproval));
        verify(mailEventService).save(new MailEvent());
    }

    private void mockSystemSettings() {
        GeneralSystemSettings generalSystemSettings = new GeneralSystemSettings();
        generalSystemSettings.setReviewerSupportMail("support@test.se");
        when(generalSystemSettingsService.getGeneralSystemSettingsInstance()).thenReturn(generalSystemSettings);
    }
}