package se.su.dsv.scipro.crosscutting;

import com.google.common.eventbus.EventBus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewing.FinalSeminarApproval;
import se.su.dsv.scipro.reviewing.FinalSeminarApprovalRequestEvent;
import se.su.dsv.scipro.reviewing.RoughDraftApproval;
import se.su.dsv.scipro.reviewing.RoughDraftApprovalRequestedEvent;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import java.util.Date;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;

@RunWith(MockitoJUnitRunner.class)
public class ReviewingNotificationsTest {
    @Mock
    private EventBus eventBus;
    @Mock
    private NotificationController notificationController;
    @InjectMocks
    private ReviewingNotifications reviewingNotifications;
    private Project project;
    private FinalSeminarApproval finalSeminarApproval;
    private RoughDraftApproval roughDraftApproval;

    @Before
    public void setUp() throws Exception {
        ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        project = Project.builder().title("Title").projectType(bachelor).build();
        finalSeminarApproval = new FinalSeminarApproval(project, new FileDescription(), "hello", new Date());
        roughDraftApproval = new RoughDraftApproval(project, new FileDescription(), "hello", new Date());
    }

    @Test
    public void registers_to_receive_events() throws Exception {
        verify(eventBus).register(reviewingNotifications);
    }

    @Test
    public void sends_out_notification_when_a_final_seminar_approval_request_is_sent() throws Exception {
        reviewingNotifications.onFinalSeminarApprovalRequest(new FinalSeminarApprovalRequestEvent(finalSeminarApproval));

        verify(notificationController).notifyProject(eq(project), eq(ProjectEvent.Event.FINAL_SEMINAR_APPROVAL_REQUESTED), anyNotificationSource());
    }

    @Test
    public void sends_out_notification_when_a_rough_draft_approval_request_is_sent() throws Exception {
        reviewingNotifications.onRoughDraftApprovalRequest(new RoughDraftApprovalRequestedEvent(roughDraftApproval));

        verify(notificationController).notifyProject(eq(project), eq(ProjectEvent.Event.ROUGH_DRAFT_APPROVAL_REQUESTED), anyNotificationSource());
    }

    private static NotificationSource anyNotificationSource() {
        return any(NotificationSource.class);
    }
}
