package se.su.dsv.scipro.finalseminar;

import com.google.common.eventbus.EventBus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.plagiarism.TurnItInCheck;
import se.su.dsv.scipro.plagiarism.TurnItInCheckService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class FinalSeminarUploadControllerImplTest {

	@InjectMocks
	private FinalSeminarUploadControllerImpl finalSeminarUploadController;

	@Mock
	private ProjectService projectService;
	@Mock
	private FileDescriptionService fileDescriptionService;
	@Mock
	private FinalSeminarSettingsService finalSeminarSettingsService;
	@Mock
	private FinalSeminarService finalSeminarService;
	@Mock
	private TurnItInCheckService turnItInCheckService;
	@Mock
	private NotificationController notificationController;
	@Mock
	private FinalSeminarOppositionRepo finalSeminarOppositionRepo;
	@Mock
	private EventBus eventBus;
	public Project project;
	public FinalSeminar seminar;
	public User uploader;
	public FileDescription fileDescription;

	@Before
	public void setUp() throws Exception {
		project = Project.builder().title("title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).build();
		seminar = new FinalSeminar();
		seminar.setProject(project);
		uploader = new UserBuilder().create();
		project.addProjectParticipant(uploader);

		fileDescription = new FileDescription();
		prepareMocksForSeminarDocumentUpload(project, seminar, fileDescription);
	}

	@Test
	public void storeSeminarDocument() throws Exception {

        FileUpload fileUpload = createFileUpload();


		finalSeminarUploadController.storeSeminarDocument(fileUpload, seminar);

		verify(fileDescriptionService).storeFile(fileUpload);
		verify(finalSeminarService).save(seminar);
		verify(turnItInCheckService).save(isA(TurnItInCheck.class));

		assertEquals(seminar.getDocument(), fileDescription);
	}

    @Test
	public void sendsEventWhenFinalThesisUploaded() throws Exception {
		finalSeminarUploadController.storeSeminarDocument(createFileUpload(), seminar);

		verify(eventBus).post(isA(FinalSeminarThesisUploadedEvent.class));
	}

    private FileUpload createFileUpload() {
        FileUpload fileUpload = mock(FileUpload.class);
        when(fileUpload.getUploader()).thenReturn(uploader);
        when(fileUpload.getContentType()).thenReturn("application/pdf");
        return fileUpload;
    }

	private void prepareMocksForSeminarDocumentUpload(Project project, FinalSeminar seminar, FileDescription fileDescription) {
		when(projectService.findOne(anyLong())).thenReturn(project);
		when(finalSeminarSettingsService.getInstance()).thenReturn(new FinalSeminarSettings());
		when(finalSeminarService.findOne(anyLong())).thenReturn(seminar);
		when(fileDescriptionService.storeFile(any(FileUpload.class))).thenReturn(fileDescription);
		when(finalSeminarService.save(seminar)).thenReturn(seminar);
	}
}
