package se.su.dsv.scipro.match;


import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.system.ProjectType;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;

public class TestApplicationPeriod {

	private SimpleDateFormat date = new SimpleDateFormat("yyyy-MM-dd", Locale.ENGLISH);
	private ApplicationPeriod currentBachelor;

    @Before
    public void startTransaction() throws Exception {
        ProjectType bachelor = new ProjectType(ProjectType.BACHELOR, "Bachelor", "Bachelor degree thesis project");

		Set<ProjectType> bachelorSet = new HashSet<>();
		bachelorSet.add(bachelor);

		currentBachelor = createPeriod(bachelorSet, date("2012-07-15"), date("2012-08-15"), date("2012-09-01"), "Current bachelor period");
	}

    @Test
    public void testIfCourseStartHasPassed() {
    	Date currentDate = date("2012-07-30");
    	boolean shouldBeFalse = currentBachelor.getCourseStartDate().compareTo(currentDate)<0;
    	Assert.assertFalse("Not false", shouldBeFalse);
    	
    	currentDate = date("2012-09-02");
    	boolean shouldBeTrue = currentBachelor.getCourseStartDate().compareTo(currentDate)<0;
    	Assert.assertTrue("Not true", shouldBeTrue);
    }
    
    private Date date(String dateString) {
        try {
            return date.parse(dateString);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }
    
    private ApplicationPeriod createPeriod(final Set<ProjectType> projectTypeSet, final Date startDate, final Date endDate, final Date courseStartDate, final String name) {
        ApplicationPeriod appPeriod = new ApplicationPeriod();
        appPeriod.setStartDate(startDate);
        appPeriod.setEndDate(endDate);
        appPeriod.setCourseStartDate(courseStartDate);
        appPeriod.setProjectTypes(projectTypeSet);
        appPeriod.setName(name);
        return appPeriod;
    }
}
