package se.su.dsv.scipro.milestones.service.impl;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.junit.Assert.*;

public class MilestonePhaseTemplateServiceImplTest extends IntegrationTest {

    @Inject
    private MilestonePhaseTemplateServiceImpl service;
    private MilestonePhaseTemplate first;
    private MilestonePhaseTemplate second;

    @Before
    public void setUp() throws Exception {
        first = service.save(new MilestonePhaseTemplate("Title", "Description"));
        second = service.save(new MilestonePhaseTemplate("Title", "Description"));
        second.setDateCreated(new DateTime().plusDays(1).toDate());
    }

    @Test
    public void testFindAll() throws Exception {
        List<MilestonePhaseTemplate> servicePhases = service.findAll(new Sort(Sort.Direction.ASC, "sortOrder"));

        assertEquals(servicePhases.get(0), first);
        assertEquals(servicePhases.get(1), second);
    }

    @Test
    public void testFindAllActive() throws Exception {
        List<MilestonePhaseTemplate> servicePhases = service.findAllActive(new Date());

        assertEquals(servicePhases.size(), 1);
        assertEquals(servicePhases.get(0), first);
    }

    @Test
    public void testSavePhase() throws Exception {
        String title = "Title";
        String desc = "desc";
        MilestonePhaseTemplate phase = service.save(new MilestonePhaseTemplate(title, desc));

        List<MilestonePhaseTemplate> allActive = service.findAllActive(new Date());
        assertThat(allActive, hasItem(phase));
    }

    @Test
    public void testEditPhaseShouldNotCareAboutSortOrder() throws Exception {
        int before = first.getSortOrder();
        first.setTitle("New title");

        service.save(first);

        assertEquals(before, first.getSortOrder());
    }

    @Test
    public void testMoveUpPhase() throws Exception {
        service.moveUp(second);

        assertTrue(second.getSortOrder() < first.getSortOrder());
    }

    @Test
    public void testMoveUpPhaseOnTopDoesNothing() throws Exception {
        int before = first.getSortOrder();
        service.moveUp(first);
        assertEquals(before, first.getSortOrder());
    }

    @Test
    public void testMoveDownPhase() throws Exception {
        service.moveDown(first);

        assertTrue(second.getSortOrder() < first.getSortOrder());
    }

    @Test
    public void testMoveDownPhaseOnBottomDoesNothing() throws Exception {
        int before = second.getSortOrder();
        service.moveDown(second);
        assertEquals(before, second.getSortOrder());
    }

    @Test
    public void testCount() throws Exception {
        assertEquals(2, service.count());
    }
}
