package se.su.dsv.scipro.project;

import com.google.common.eventbus.EventBus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.runners.MockitoJUnitRunner;
import org.mockito.stubbing.Answer;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.TreeSet;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ProjectServiceImplTest {


    public static final ProjectType SOME_PROJECT_TYPE = new ProjectType(DegreeLevel.FIRST_CYCLE, "Some project type", "Some description");
    @Mock
    private ProjectRepo projectRepo;
    @Mock
    private EventBus eventBus;
    @Mock
    private Provider<EntityManager> em;
    @InjectMocks
    private ProjectServiceImpl projectService;
    private Project project;

    @Before
    public void setUp() throws Exception {
        when(projectRepo.save(isA(Project.class))).thenAnswer(new Answer<Object>() {
            @Override
            public Object answer(final InvocationOnMock invocation) throws Throwable {
                return invocation.getArguments()[0];
            }
        });

        initProjects();
    }

    private void initProjects() {
        User headSupervisorUser = new UserBuilder().create();
        project = Project.builder()
                .title("Some title")
                .projectType(SOME_PROJECT_TYPE)
                .headSupervisor(headSupervisorUser)
                .stateOfMind(StateOfMind.FINE)
                .build();
        project.addCoSupervisor(new UserBuilder().create());
        project.addReviewer(new UserBuilder().create());
    }

    @Test
    public void isSupervisorOrFollower() throws Exception {
        User user = new UserBuilder().create();
        Project project = new Project();
        project.addReviewer(user);

        assertTrue(projectService.isSupervisorOrFollower(project, user));
    }

    @Test
    public void isNotSupervisorOrFollower() throws Exception {
        User user = new UserBuilder().create();

        assertFalse(projectService.isSupervisorOrFollower(project, user));
    }

    @Test
    public void isNotPartOfProject() throws Exception {


        assertFalse(projectService.isPartOfProject(project, new UserBuilder().create()));
    }

    @Test
    public void isPartOfProject() throws Exception {
        User user = new UserBuilder().create();
        project.setHeadSupervisor(user);

        assertTrue(projectService.isPartOfProject(project, user));
    }

    @Test
    public void sends_out_event_when_activating_project() throws Exception {
        projectService.toggleActiveStatus(project, true);
        verify(eventBus).post(isA(ProjectActivatedEvent.class));
    }

    @Test
    public void sends_out_event_when_deactivating_project() throws Exception {
        projectService.toggleActiveStatus(project, false);
        verify(eventBus).post(isA(ProjectDeactivatedEvent.class));
    }
}
