package se.su.dsv.scipro.project;


import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.List;

public class TestProject extends IntegrationTest {

    @Inject
    private ProjectService projectService;

    private Project project1, project2;
    private User authorUser;

    @Before
    public void startTransaction() throws Exception {

        ProjectType bachelor = new ProjectType(ProjectType.BACHELOR, "Bachelor", "Bachelor degree thesis project");
        bachelor = save(bachelor);

        ProjectType master = new ProjectType(ProjectType.MASTER, "Master", "Master degree thesis project");
        master = save(master);

        User employeeUser = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        employeeUser = save(employeeUser);
        User reviewerUser = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        reviewerUser = save(reviewerUser);
        authorUser = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        authorUser = save(authorUser);

        project1 = Project.builder().
                title("Project 1").
                projectType(bachelor).
                headSupervisor(employeeUser).
                projectStatus(ProjectStatus.ACTIVE).
                build();
        project1.addProjectParticipant(authorUser);
        project1 = save(project1);

        project2 = Project.builder().title("Tester 2").projectType(master).headSupervisor(employeeUser).projectStatus(ProjectStatus.INACTIVE).build();
        project2.addProjectParticipant(authorUser);
        project2 = save(project2);

        project1.addReviewer(reviewerUser);
        project1 = save(project1);
    }

    @Test
    public void testGetProjectsByUser() {
        List<Project> projectsFromService = projectService.getProjectsByUser(authorUser);
        List<Project> projectsCreated = Arrays.asList(project1, project2);
        Assert.assertEquals(2, projectsFromService.size());
        Assert.assertEquals(projectsCreated, projectsFromService);
    }
}
