package se.su.dsv.scipro.projectpartner;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.hamcrest.CoreMatchers.not;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

public class ProjectPartnerServiceImplTest extends IntegrationTest {
    public static final PageRequest ALL = new PageRequest(0, Long.MAX_VALUE);

    @Inject
    private ProjectPartnerServiceImpl projectPartnerService;
    private User user;
    private ProjectType projectType;

    @Before
    public void setUp() throws Exception {
        user = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        projectType = save(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
    }

    @Test
    public void getProjectPartnerInSpan() throws Exception {
        ProjectPartner projectPartner = newProjectPartner();
        ProjectPartner pastProjectPartner = newProjectPartner(10);
        assertThat(projectPartnerService.getProjectPartnerInSpan(5, projectType, ALL), hasItem(projectPartner));
        assertThat(projectPartnerService.getProjectPartnerInSpan(5, projectType, ALL), not(hasItem(pastProjectPartner)));
    }

    @Test
    public void getProjectPartnerInSpanDaysAreZero() throws Exception {
        ProjectPartner projectPartner = newProjectPartner(365);
        assertThat(projectPartnerService.getProjectPartnerInSpan(0, projectType, ALL), hasItem(projectPartner));
    }

    @Test
    public void countProjectPartnerInSpan() throws Exception {
        newProjectPartner();
        newProjectPartner(10);
        assertEquals(1, projectPartnerService.countProjectPartnerInSpan(5, projectType));
    }

    @Test
    public void countProjectPartnerInSpanDaysAreZero() throws Exception {
        newProjectPartner();
        newProjectPartner(10);
        assertEquals(2, projectPartnerService.countProjectPartnerInSpan(0, projectType));
    }

    private ProjectPartner newProjectPartner() {
        return newProjectPartner(0);
    }

    private ProjectPartner newProjectPartner(int daysOld) {
        ProjectPartner projectPartner = new ProjectPartner(user);
        projectPartner.setInfotext("Hi I need a thesis writing partner");
        projectPartner.setDateCreated(new DateTime().minusDays(daysOld).toDate());
        projectPartner.setProjectType(projectType);
        return save(projectPartner);
    }
}
