package se.su.dsv.scipro.report;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.finalseminar.FinalSeminarOpposition;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class OppositionReportTest {

    private OppositionReport oppositionReport;

    @Before
    public void prepareTemplate() throws Exception {
        ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor");

        GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(bachelor, 30);
        addCriteria(gradingReportTemplate);

        oppositionReport = gradingReportTemplate.createOppositionReport(new FinalSeminarOpposition());
    }

    @Test
    public void opposition_report_is_not_finished_if_any_criteria_is_not_filled_out() throws Exception {
        assertFalse(oppositionReport.isFinished());
    }

    @Test
    public void opposition_report_is_not_finished_if_criteria_are_filled_out_but_thesis_summary_is_missing() throws Exception {
        fillOutAllCriteria();
        assertFalse(oppositionReport.isFinished());
    }

    @Test
    public void opposition_report_is_not_finished_if_only_summary_is_filled_in() throws Exception {
        oppositionReport.setThesisSummary("summary");
        assertFalse(oppositionReport.isFinished());
    }

    @Test
    public void opposition_report_is_finished() throws Exception {
        oppositionReport.setThesisSummary("summary");
        fillOutAllCriteria();
        assertTrue(oppositionReport.isFinished());
    }

    private void fillOutAllCriteria() {
        for (Criterion criterion : oppositionReport.getCriteria()) {
            criterion.setFeedback("filled out");
        }
    }

    private void addCriteria(GradingReportTemplate gradingReportTemplate) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(1).description("For 1 point: Be nice to your supervisor").build());
        gradingReportTemplate.addProjectCriterion("U1 Sammanfattning", 1,gradingCriterionPointTemplates);
        gradingReportTemplate.addProjectCriterion("U2 Introduktion", 1,gradingCriterionPointTemplates);
        gradingReportTemplate.addProjectCriterion("U3 Problem", 1,gradingCriterionPointTemplates);
        gradingReportTemplate.addProjectCriterion("U4 Frågeställning", 1, gradingCriterionPointTemplates);
        gradingReportTemplate.addProjectCriterion("U5 Vetenskaplig förankring", 1, gradingCriterionPointTemplates);
    }

}
