package se.su.dsv.scipro.report;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.mockito.Mockito.mock;

public class SupervisorBachelorGradeCalculatorTest extends GradeCalculatorTest {
    private GradingReport gradingReport;
    private SupervisorBachelorGradeCalculator gradeCalculator;

    @Before
    public void setUp() throws Exception {
        GradingReportTemplate gradingReportTemplate = prepareTemplate();

        gradingReport = gradingReportTemplate.createSupervisorReport(ObjectMother.SOME_PROJECT, mock(User.class));
        gradeCalculator = new SupervisorBachelorGradeCalculator();
    }

    @Test
    public void a_report_with_24_points_gets_B() throws Exception {
        assessAllCriteriaWithMinimumAndTotal(24);
        assertEquals(GradingReport.Grade.B, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void a_report_with_22_points_gets_C() throws Exception {
        assessAllCriteriaWithMinimumAndTotal(22);
        assertEquals(GradingReport.Grade.C, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void a_report_with_18_points_gets_D() throws Exception {
        assessAllCriteriaWithMinimumAndTotal(18);
        assertEquals(GradingReport.Grade.D, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void a_report_with_16_points_gets_E() throws Exception {
        assessAllCriteriaWithMinimumAndTotal(16);
        assertEquals(GradingReport.Grade.E, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void a_report_with_9_points_gets_F() throws Exception {
        assessAllCriteriaWithMinimumAndTotal(9);
        assertEquals(GradingReport.Grade.F, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void a_report_with_less_than_14_points_but_8_completed_project_criteria_with_fx_flag_and_2_completed_individual_criteria_with_fx_flag_gets_Fx() throws Exception {
        assessAllCriteria(0);
        assignPointsToProjectCriteriaWithFxFlag(8);
        assignPointsToTwoIndividualCriteriaWithFxFlag(2);

        assertEquals(GradingReport.Grade.FX, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void a_report_with_points_but_no_feedback_has_grade() throws Exception {
        for (GradingCriterion gradingCriterion : gradingReport.getGradingCriteria()) {
            gradingCriterion.setPoints(gradingCriterion.getMaxPoints());
        }
        assertNotNull(gradingReport.getGrade(gradeCalculator));
    }

    @Override
    protected GradingReport getGradingReport() {
        return gradingReport;
    }

    @Override
    protected GradeCalculator getGradeCalculator() {
        return gradeCalculator;
    }


    protected GradingReportTemplate prepareTemplate() {
        GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(ObjectMother.BACHELOR, 30);

        gradingReportTemplate.addProjectCriterion("U1 Sammanfattning", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U2 Introduktion", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U3 Problem", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U4 Frågeställning", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U5 Vetenskaplig förankring", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U6 Metodval", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U7 Metodtillämpning", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U8 Resultat", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U9 Slutsatser och diskussion", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U10 Form, struktur och språk", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U11 Argumentation", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U12 Källhänvisningar och dokumentation", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U13 Originalitet och signifikans", 0, getPointTemplates(3)).setFx(false);

        gradingReportTemplate.addIndividualCriterion("Ö1 Oppositionsrapport", 1, getPointTemplates(2));
        gradingReportTemplate.addIndividualCriterion("Ö2 Presentationer", 1, getPointTemplates(1));
        gradingReportTemplate.addIndividualCriterion("Ö3 Aktivitet vid seminarier och möten", 1, getPointTemplates(1));
        gradingReportTemplate.addIndividualCriterion("Ö4 Deadlines", 0, getPointTemplates(1)).setFx(false);
        gradingReportTemplate.addIndividualCriterion("Ö5 Revision efter slutseminarium", 0, getPointTemplates(1)).setFx(false);

        return gradingReportTemplate;
    }

    private List<GradingCriterionPointTemplate> getPointTemplates(int maxPoint) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        for (int i = 0; i <= maxPoint; i++) {
            gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(i).description("Description").build());
        }
        return gradingCriterionPointTemplates;
    }
}
