package se.su.dsv.scipro.system;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.test.UserBuilder;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class PasswordServiceImplTest {

    public static final String EMPTY_STRING = "";
    @Mock
    private PasswordRepo passwordRepo;
    @InjectMocks
    private PasswordServiceImpl passwordService;

    @Test
    public void update_password() throws Exception {
        String newInput = "password-changed";
        User user = new UserBuilder().create();
        Password oldPassword = new Password();
        oldPassword.setUser(user);

        when(passwordRepo.findByUser(user)).thenReturn(oldPassword);

        passwordService.updatePassword(user, newInput);

        ArgumentCaptor<Password> captor = ArgumentCaptor.forClass(Password.class);
        verify(passwordRepo).save(captor.capture());

        assertTrue(PasswordHandler.isMatch(newInput.toCharArray(), captor.getValue()));
    }

    @Test
    public void does_not_update_password_if_not_provided() throws Exception {

        passwordService.updatePassword(new User(), EMPTY_STRING);

        verify(passwordRepo, never()).save(any(Password.class));
    }

    @Test
    public void find_by_user() throws Exception {
        User user = new UserBuilder().create();
        Password password = new Password();

        when(passwordRepo.findByUser(user)).thenReturn(password);

        assertEquals(password, passwordService.findByUser(user));
    }
}
