package se.su.dsv.scipro.workerthreads;

import com.google.inject.util.Providers;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;

@RunWith(MockitoJUnitRunner.class)
public class SchedulerImplTest {

    @Mock
    private ScheduledExecutorService scheduledExecutorService;
    @Mock
    private Worker abstractWorker;
    @Mock
    private Task task;
    @InjectMocks
    private SchedulerImpl scheduler;

    @Test
    public void schedules_correctly() throws Exception {
        long period = 1;
        TimeUnit timeUnit = TimeUnit.SECONDS;
        scheduler.schedule("My worker").runBy(Providers.of(abstractWorker)).every(period, timeUnit);
        verify(scheduledExecutorService).scheduleAtFixedRate(any(Runnable.class), eq(period), eq(period), eq(timeUnit));
    }

    @Test
    public void execute_task_now() throws Exception {
        scheduler.execute(task);
        ArgumentCaptor<Runnable> captor = ArgumentCaptor.forClass(Runnable.class);
        verify(scheduledExecutorService).execute(captor.capture());
        captor.getValue().run();
        verify(task).execute();
    }

    @Test
    public void stop() throws Exception {
        scheduler.stop();
        verify(scheduledExecutorService).shutdownNow();
    }
}
