package se.su.dsv.scipro.reviewing

import java.util.Date

import org.hamcrest.CoreMatchers._
import org.junit.Assert._
import org.junit.Test
import se.su.dsv.scipro.file.FileDescription
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.reviewing.{ReviewerDecisionReminderWorker => Worker}
import se.su.dsv.scipro.system.{DegreeLevel, ProjectType, Unit, User}

import scala.collection.JavaConverters._

class ReviewerDecisionReminderWorkerTest {

  @Test def should_remind_if_is_same_date() = {
    val today = new Date
    val roughDraftApproval = newRoughDraftApproval

    val needReminding = Worker.needReminding(_ => today, _ => Set(today))(today)(Set(roughDraftApproval))

    assertThat(needReminding.asJava, hasItem((roughDraftApproval,today)))
  }

  @Test def should_not_remind_if_is_different_date() = {
    val date = new Date(987452359L)
    val roughDraftApproval = newRoughDraftApproval

    val needReminding = Worker.needReminding(_ => new Date(0), _ => Set(date))(new Date(0))(Set(roughDraftApproval))

    assertThat(needReminding.asJava, not(hasItem((roughDraftApproval,new Date(0)))))
  }

  @Test def reminder_should_include_title_of_every_idea() = {
    val roughDraftApproval = newRoughDraftApproval

    val reminder = Worker.generateRoughDraftReminder(roughDraftApproval.getProject.getReviewer, Set((roughDraftApproval,new Date(0))))
    assertThat(reminder.getMessageBody, containsString(roughDraftApproval.getProject.getTitle))
  }

  @Test def generated_reminder_should_be_sent_to_the_reviewer() = {
    val roughDraftApproval = newRoughDraftApproval
    val reviewer = roughDraftApproval.getProject.getReviewer

    val reminder = Worker.generateRoughDraftReminder(reviewer, Set((roughDraftApproval,new Date)))
    assertThat(reminder.getRecipients, hasItem(reviewer))
  }


  def newUnit: Unit = {
    val unit = new Unit
    unit.setMatchResponsible("test@test,test")
    unit
  }

  def newUser: User = {
    val user = User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build()
    user.setId(234L)
    user.setUnit(newUnit)
    user
  }

  def newProject: Project = {
    val projectType: ProjectType = new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING)
    val project = Project.builder().title("This is a title").projectType(projectType).build()
    project.addReviewer(newUser)
    project.setHeadSupervisor(newUser)
    project.setId(555L)
    project
  }

  def newFileDescription : FileDescription = {
   val fileDescription = new FileDescription
    fileDescription
  }

  def newRoughDraftApproval: RoughDraftApproval = {
    val roughDraftApproval = new RoughDraftApproval(newProject,newFileDescription,"", new Date())
    roughDraftApproval
  }

}
