package se.su.dsv.scipro.io.facade;

import com.google.inject.persist.Transactional;
import se.su.dsv.scipro.io.ExternalExporter;
import se.su.dsv.scipro.io.dto.Role;
import se.su.dsv.scipro.io.exceptions.ExternalExportException;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.Unit;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;

public class ExporterFacade {

    private final ExternalExporter exporter;

    @Inject
    public ExporterFacade(ExternalExporter exporter) {
        this.exporter = exporter;
    }

    /**
     * Exports the project to the specified unit. The export is done in four
     * steps; the project, authors, supervisors, and reviewers. Does not
     * persist the project it will only update the
     * {@link se.su.dsv.scipro.project.Project#getIdentifier() identifier}.
     * If the export fails at any step an attempt to delete the remote project
     * will be performed.
     *
     * @param project project to export
     * @param unit    organisation unit the project belongs to
     * @throws ExternalExportException if the process failed at any step
     */
    @Transactional
    public void exportProject(Project project, Unit unit)
            throws ExternalExportException {

        try {
            exporter.createProject(project, unit);

            for (User student : project.getProjectParticipants()) {
                exporter.addStudentToProject(project, student);
            }

            exporter.addContributorToProject(project, project.getHeadSupervisor(), Role.SUPERVISOR);

            for (User user : project.getReviewers()) {
                exporter.addContributorToProject(project, user, Role.EXAMINER);
            }
        } catch (ExternalExportException eee) {
            exporter.deleteProject(project);
            throw eee;
        }
    }

}
