package se.su.dsv.scipro.io.http;

import org.glassfish.jersey.client.authentication.HttpAuthenticationFeature;
import scala.Option;
import se.su.dsv.scipro.io.dto.*;

import javax.inject.Inject;
import javax.inject.Named;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.GenericType;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.util.List;
import java.util.Set;

import static javax.ws.rs.client.Entity.json;
import static javax.ws.rs.client.Entity.xml;

public class DaisyAPIImpl implements DaisyAPI {

    public static final String STUDENT = "student";
    public static final String COURSE = "course";
    public static final String CONTRIBUTOR = "contributor";
    public static final String THESES = "theses";
    public static final String SUBUNITS = "subunits";
    public static final String USERNAMES = "usernames";
    public static final String RESEARCH_AREAS = "researchAreas";
    public static final String SUPERVISORS = "supervisors";
    public static final String PROGRAM = "program";
    public static final String THESIS = "thesis";
    public static final String ORGUNIT = "orgunit";
    public static final String PERSON = "person";
    public static final String USERNAME = "username";
    private String baseUrl;
    private String user;
    private String password;

    public DaisyAPIImpl() {
    }

    @Inject
    public void setBaseUrl(@Named("url") String baseUrl) {
        this.baseUrl = baseUrl;
    }

    @Inject
    public void setUser(@Named("user") String user) {
        this.user = user;
    }

    @Inject
    public void setPassword(@Named("password") String password) {
        this.password = password;
    }

    @Override
    public Set<ProjectParticipant> getContributors(Long projectId) {
        return thesis()
                .path(String.valueOf(projectId))
                .path(CONTRIBUTOR)
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(new GenericType<Set<ProjectParticipant>>() {
                });
    }


    @Override
    public Set<StudentProjectParticipant> getAuthors(Long projectId) {
        return thesis()
                .path(String.valueOf(projectId))
                .path(STUDENT)
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(new GenericType<Set<StudentProjectParticipant>>() {
                });
    }

    @Override
    public Set<Thesis> getProjects(Long personId) {
        return person()
                .path(String.valueOf(personId))
                .path(THESES)
                .request(MediaType.APPLICATION_XML_TYPE)
                .get(new GenericType<Set<Thesis>>() {
                });
    }

    @Override
    public Set<Unit> getSubUnits(Long unitId) {
        return units()
                .path(String.valueOf(unitId))
                .path(SUBUNITS)
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(new GenericType<Set<Unit>>() {
                });
    }


    @Override
    public Set<UserName> getUsernames(Long personId) {
        return person()
                .path(String.valueOf(personId))
                .path(USERNAMES)
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(new GenericType<Set<UserName>>() {
                });
    }

    @Override
    public List<ResearchArea> getResearchAreas(Long personId) {
        return person()
                .path(String.valueOf(personId))
                .path(RESEARCH_AREAS)
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(new GenericType<List<ResearchArea>>() {
                });
    }

    @Override
    public List<Person> getSupervisors(Long unitId) {
        return units()
                .path(String.valueOf(unitId))
                .path(SUPERVISORS)
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(new GenericType<List<Person>>() {
                });
    }

    @Override
    public Option<Person> findPersonById(Long id) {
        Response response = person()
                .path(String.valueOf(id))
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(Response.class);
        return response.getStatus() == 200
                ? Option.apply(response.readEntity(Person.class))
                : Option.<Person>empty();
    }

    @Override
    public Option<Person> findByUsername(String userName) {
        Response response = person()
                .path(USERNAME)
                .path(userName)
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(Response.class);
        return response.getStatus() == 200
                ? Option.apply(response.readEntity(Person.class))
                : Option.<Person>empty();
    }

    @Override
    public Response addContributor(Long projectIdentifier, ProjectParticipant contributor) {
        return thesis()
                .path(String.valueOf(projectIdentifier))
                .path(CONTRIBUTOR)
                .request(MediaType.APPLICATION_JSON_TYPE)
                .post(json(contributor));
    }

    @Override
    public Response deleteContributor(Long projectId, Long personId) {
        return thesis()
                .path(String.valueOf(projectId))
                .path(CONTRIBUTOR)
                .path(String.valueOf(personId))
                .request(MediaType.APPLICATION_JSON_TYPE)
                .delete();
    }

    @Override
    public Response createProject(Thesis project) {
        return thesis()
                .request(MediaType.APPLICATION_JSON_TYPE)
                .post(json(project));
    }

    @Override
    public Response addAuthor(Long projectId, StudentProjectParticipant author) {
        return thesis()
                .path(String.valueOf(projectId))
                .path(STUDENT)
                .request(MediaType.APPLICATION_XML_TYPE)
                .post(xml(author));
    }

    @Override
    public Response deleteProject(Long projectId) {
        return thesis()
                .path(String.valueOf(projectId))
                .request(MediaType.APPLICATION_JSON_TYPE)
                .delete();
    }

    @Override
    public Response getStudent(Long id) {
        return student()
                .path(String.valueOf(id))
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get();
    }

    @Override
    public Program getProgram(Long id) {
        return program()
                .path(String.valueOf(id))
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(Program.class);
    }

    @Override
    public Option<Thesis> getThesis(Long projectIdentifier) {
        Response response = thesis()
                .path(String.valueOf(projectIdentifier))
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(Response.class);
        return response.getStatus() == 200
                ? Option.apply(response.readEntity(Thesis.class))
                : Option.<Thesis>empty();
     }

    @Override
    public void updateThesis(Thesis thesis) {
        thesis()
                .path(String.valueOf(thesis.getId()))
                .request()
                .put(xml(thesis));
    }

    @Override
    public void deleteAuthor(Long projectId, Long authorId) {
        thesis()
                .path(String.valueOf(projectId))
                .path(STUDENT)
                .path(String.valueOf(authorId))
                .request()
                .delete();
    }

    @Override
    public Option<String> getGradeForStudentInCourse(Long authorId, Long courseId) {
        Response response = course()
                .path(String.valueOf(courseId))
                .path(STUDENT)
                .path(String.valueOf(authorId))
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get();
        return response.getStatus() == 200
                ? Option.apply(response.readEntity(String.class))
                : Option.<String>empty();
    }

    @Override
    public List<Person> findByPersonnummer(final String personnummer) {
        return person()
                .queryParam("personnummer", personnummer)
                .request(MediaType.APPLICATION_JSON_TYPE)
                .get(new GenericType<List<Person>>(){});
    }

    private WebTarget program() {
        return target()
                .path(PROGRAM);
    }

    private WebTarget student() {
        return target()
                .path(STUDENT);
    }

    private WebTarget thesis() {
        return target()
                .path(THESIS);
    }

    private WebTarget units() {
        return target()
                .path(ORGUNIT);
    }

    private WebTarget person() {
        return target()
                .path(PERSON);
    }

    private WebTarget course() {
        return target()
                .path(COURSE);
    }

    private WebTarget target() {
        return ClientBuilder.newClient()
                .register(HttpAuthenticationFeature.basic(user, password))
                .target(baseUrl);
    }
}
