package se.su.dsv.scipro.integration.daisy.workers;

import com.google.common.eventbus.EventBus;
import com.google.inject.persist.UnitOfWork;
import com.google.inject.util.Providers;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import scala.Option;
import se.su.dsv.scipro.grading.GradingCompletedEvent;
import se.su.dsv.scipro.io.dto.*;
import se.su.dsv.scipro.io.http.DaisyAPI;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.QProject;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;
import se.su.dsv.scipro.workerthreads.WorkerData;
import se.su.dsv.scipro.workerthreads.WorkerDataService;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;

import java.util.*;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class GradingCompletedMilestoneActivatorTest {

    private GradingCompletedMilestoneActivator gradingCompletedMilestoneActivator;

    private Project project = new Project();
    private User student = new User();

    @Mock
    private ProjectService projectService;
    @Mock
    private UserService userService;
    @Mock
    private DaisyAPI daisyAPI;
    @Mock
    private EventBus eventBus;
    @Mock
    private UnitOfWork unitOfWork;
    @Mock
    private EntityManager entityManager;
    @Mock
    private EntityTransaction entityTransaction;
    @Mock
    private WorkerDataService workerDataService;

    @Before
    public void setup() {
        gradingCompletedMilestoneActivator = new GradingCompletedMilestoneActivator(projectService, daisyAPI, eventBus, userService);
        gradingCompletedMilestoneActivator.setUnitOfWork(unitOfWork);
        gradingCompletedMilestoneActivator.setTxManager(Providers.of(entityManager));
        gradingCompletedMilestoneActivator.setWorkerDataService(workerDataService);
        when(workerDataService.save(any(WorkerData.class))).thenReturn(new WorkerData());
        when(entityManager.getTransaction()).thenReturn(entityTransaction);

        project.setId(3493L);
        project.setIdentifier(234L);
        project.setProjectStatus(ProjectStatus.ACTIVE);
        List<Project> projects = Collections.singletonList(project);

        when(projectService.findAll(QProject.project.projectStatus.eq(ProjectStatus.ACTIVE).and(QProject.project.identifier.isNotNull()))).thenReturn(projects);


        Set<StudentProjectParticipant> participants = new HashSet<>();
        StudentProjectParticipant studentProjectParticipant = new StudentProjectParticipant();
        studentProjectParticipant.setCourse(new Course());
        studentProjectParticipant.setPerson(new Person());
        participants.add(studentProjectParticipant);
        when(daisyAPI.getAuthors(anyLong())).thenReturn(participants);

        when(daisyAPI.getGradeForStudentInCourse(anyLong(), anyLong())).thenReturn(Option.apply("grade"));

        when(userService.findByExternalIdentifier(anyLong())).thenReturn(student);
    }

    @Test
    public void testDoWork() throws Exception {
        gradingCompletedMilestoneActivator.run();

        verify(eventBus).post(isA(GradingCompletedEvent.class));
    }
}