package se.su.dsv.scipro;

import org.apache.wicket.Page;
import org.apache.wicket.RestartResponseException;
import se.su.dsv.scipro.admin.pages.AdminStartPage;
import se.su.dsv.scipro.basepages.MenuPage;
import se.su.dsv.scipro.examiner.pages.ExaminerStartPage;
import se.su.dsv.scipro.loginlogout.pages.LoginPage;
import se.su.dsv.scipro.project.pages.ProjectStartPage;
import se.su.dsv.scipro.reviewer.ReviewerStartPage;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.supervisor.pages.SupervisorStartPage;

import javax.inject.Inject;

public class HomePage extends MenuPage {

    @Inject
    private UserProfileService userProfileService;

    public HomePage() {
        Roles selectedRole = userProfileService.findSelectedRole(SciProSession.get().getUser());
        if (selectedRole != null) {
            switch (selectedRole) {
                case ADMIN:
                    show(AdminStartPage.class);
                    break;
                case SUPERVISOR:
                    show(SupervisorStartPage.class);
                    break;
                case AUTHOR:
                    show(ProjectStartPage.class);
                    break;
                case REVIEWER:
                    show(ReviewerStartPage.class);
                    break;
                case EXAMINER:
                    show(ExaminerStartPage.class);
                    break;
                default:
                    show(LoginPage.class);
                    break;
            }
        } else {
            if (isSysAdmin() || isAdmin()) {
                show(AdminStartPage.class);
            } else if (isSupervisor()) {
                show(SupervisorStartPage.class);
            } else if (isAuthor()) {
                show(ProjectStartPage.class);
            } else if (isReviewer()) {
                show(ReviewerStartPage.class);
            } else if (isExaminer()) {
                show(ExaminerStartPage.class);
            }
        }
    }


    private void show(Class<? extends Page> page) {
        throw new RestartResponseException(page);
    }

    private boolean isSysAdmin() {
        return SciProSession.get().authorizedForRole(Roles.SYSADMIN) && SciProSession.get().hasActualRole(Roles.SYSADMIN);
    }

    private boolean isAdmin() {
        return SciProSession.get().authorizedForRole(Roles.ADMIN) && SciProSession.get().hasActualRole(Roles.ADMIN);
    }

    private boolean isSupervisor() {
        return SciProSession.get().authorizedForRole(Roles.SUPERVISOR) && SciProSession.get().hasActualRole(Roles.SUPERVISOR);
    }

    private boolean isAuthor() {
        return SciProSession.get().authorizedForRole(Roles.AUTHOR) && SciProSession.get().hasActualRole(Roles.AUTHOR);
    }

    private boolean isReviewer() {
        return SciProSession.get().authorizedForRole(Roles.REVIEWER) && SciProSession.get().hasActualRole(Roles.REVIEWER);
    }

    private boolean isExaminer() {
        return SciProSession.get().authorizedForRole(Roles.EXAMINER) && SciProSession.get().hasActualRole(Roles.EXAMINER);
    }
}
