package se.su.dsv.scipro.activityplan;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.BootstrapDateTimeComponent;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

public class AddActivitiesFromTemplatePanel extends GenericPanel<Project> {

    private final IModel<Project> model;
    @Inject
    private NotificationController notificationController;
    @Inject
    private ActivityPlanFacade activityPlanFacade;
    @Inject
    private ActivityPlanTemplateService activityPlanTemplateService;

    private ActivityPlanTemplateService.Filter filter;

    public AddActivitiesFromTemplatePanel(final String id, final IModel<Project> model){
		super(id, model);
        filter = new ActivityPlanTemplateService.Filter();
        this.model = model;
        filter.setCreator(this.model.getObject().getHeadSupervisor());
        add(new ListFragment("content"));
	}

    private void notifyChangeActivityPlan(String name, ProjectEvent.Event event) {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(getModelObject().getId()));

        NotificationSource source = new NotificationSource();
        source.setMessage(name);

        notificationController.notifyProject(getModelObject(), event, source);
    }

    private class ListFragment extends Fragment {

        public ListFragment(final String id) {
            super(id, "list", AddActivitiesFromTemplatePanel.this);
            setOutputMarkupId(true);
            final WebMarkupContainer container = new WebMarkupContainer("stupidContainerSinceWicketCantRefreshListViews");
            container.setOutputMarkupId(true);
            add(container);
            add(new TextField<String>("filter", new PropertyModel<String>(filter, "filterString")) {
                {
                    add(new OnChangeAjaxBehavior() {
                        @Override
                        protected void onUpdate(final AjaxRequestTarget target) {
                            target.add(container);
                        }
                    });
                }
            });
            add(new AjaxCheckBox("onlyRelevantTemplates", new Model<>(true)) {
                @Override
                protected void onUpdate(final AjaxRequestTarget target) {
                    if (getModelObject()){
                        filter.setCreator(model.getObject().getHeadSupervisor());
                    } else {
                        filter.setCreator(null);
                    }
                    target.add(container);
                }
            });
            container.add(new ListView<ActivityPlanTemplate>("templates", getActivityPlanTemplates()) {
                @Override
                protected void populateItem(final ListItem<ActivityPlanTemplate> item) {
                    item.add(new AjaxLink<ActivityPlanTemplate>("title", item.getModel()) {
                        {
                            setBody(new PropertyModel<String>(item.getModel(), "title"));
                        }

                        @Override
                        public void onClick(final AjaxRequestTarget target) {
                            DetailsFragment content = new DetailsFragment("content", getModel());
                            AddActivitiesFromTemplatePanel.this.replace(content);
                            target.add(content);
                        }
                    });
                }
            });
        }

        private IModel<List<ActivityPlanTemplate>> getActivityPlanTemplates() {
            return new LoadableDetachableModel<List<ActivityPlanTemplate>>() {
                @Override
                protected List<ActivityPlanTemplate> load() {
                    return activityPlanTemplateService.findAll(filter);
                }
            };
        }
    }

    private class DetailsFragment extends Fragment {

        private Model<Date> dateModel = Model.of(getDate());

        private Date getDate() {
            Calendar cal = Calendar.getInstance();
            cal.set(Calendar.HOUR, 8);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.AM_PM, Calendar.AM);
            return cal.getTime();
        }

        public DetailsFragment(final String id, final IModel<ActivityPlanTemplate> model) {
            super(id, "details", AddActivitiesFromTemplatePanel.this);
            setOutputMarkupId(true);
            add(new ActivityPlanTemplateDetailsPanel("viewpanel", model));
            add(new AjaxLink<Void>("backlink") {
                @Override
                public void onClick(AjaxRequestTarget target) {
                    ListFragment content = new ListFragment("content");
                    AddActivitiesFromTemplatePanel.this.replace(content);
                    target.add(content);
                }
            });

            Form<ActivityPlanTemplate> form = new Form<ActivityPlanTemplate>("form", model) {
                @Override
                protected void onSubmit() {
                    activityPlanFacade.addActivitiesFromTemplate(AddActivitiesFromTemplatePanel.this.getModelObject(), getModelObject(), dateModel.getObject());
                    notifyChangeActivityPlan(getModelObject().getTitle(), ProjectEvent.Event.ACTIVITY_ADDED);
                }
            };
            form.add(new BootstrapDateTimeComponent("fromDate", dateModel));
            add(form);
        }
    }
}
