package se.su.dsv.scipro.activityplan;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.checklist.ChecklistTemplateService;
import se.su.dsv.scipro.reusable.SciProUtilities;

import javax.inject.Inject;
import java.util.Arrays;

public abstract class AdminEditActivityPanel extends Panel {
    public static final String FORM = "form";
    public static final String TITLE_FIELD = "titleField";
    public static final String DESCRIPTION_AREA = "descriptionArea";
    public static final String DAYS_FIELD = "daysOffsetField";
    public static final String CHECKLIST_DROP_DOWN = "checklistDropDown";
    public static final String HAND_IN_REQUIRED = "handInRequired";
    public static final String ADD_BUTTON = "addButton";

    @Inject
    private ChecklistTemplateService checklistTemplateService;

    protected abstract void onSubmit(AjaxRequestTarget target, IModel<ActivityTemplate> activity);

    public AdminEditActivityPanel(String id, IModel<ActivityTemplate> model) {
        super(id, model);
        add(new ActivityForm(FORM, model));
    }

    protected class ActivityForm extends Form<ActivityTemplate> {
        public ActivityForm(String id, final IModel<ActivityTemplate> model) {
            super(id, model);

            TextField<String> title = new TextField<>(TITLE_FIELD, new PropertyModel<String>(model, "title"));
            title.setRequired(true);
            add(title);

            add(new TextArea<>(DESCRIPTION_AREA, new PropertyModel<>(model, "description")));

            TextField<Integer> daysOffset = new TextField<>(DAYS_FIELD, new PropertyModel<Integer>(model, "daysOffset"));
            daysOffset.setRequired(true);
            add(daysOffset);

            DropDownChoice<ChecklistTemplate> checklistTemplate = new DropDownChoice<>(CHECKLIST_DROP_DOWN, new PropertyModel<ChecklistTemplate>(model, "checklistTemplate"), SciProUtilities.copyIterator(checklistTemplateService.findAll().iterator()), new IChoiceRenderer<ChecklistTemplate>() {
                @Override
                public Object getDisplayValue(ChecklistTemplate checklistTemplate) {
                        return checklistTemplate.getName();
                }

                @Override
                public String getIdValue(ChecklistTemplate checklistTemplate, int i) {
                    return Long.toString(checklistTemplate.getId());
                }
            });
            checklistTemplate.setNullValid(true);
            add(checklistTemplate);

            DropDownChoice<Action> action = new DropDownChoice<>("action", new PropertyModel<Action>(model, "action"), Arrays.asList(Action.values()), new EnumChoiceRenderer<Action>(this));
            action.setRequired(true);
            add(action);

            add(new AjaxButton(ADD_BUTTON) {
                @Override
                protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                    super.onSubmit(target, form);
                    AdminEditActivityPanel.this.onSubmit(target, model);
                }
            });
        }
    }
}
