package se.su.dsv.scipro.admin.pages.settings;

import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.basic.EnumLabel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.admin.pages.AbstractAdminSystemPage;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.List;

public class AdminProjectModuleSettingsPage extends AbstractAdminSystemPage {

    public static final String PROJECT_TYPE_LIST = "projectTypeList";
    public static final String MODULE_LIST = "moduleList";
    public static final String NAME = "name";
    public static final String FORM = "form";
    public static final String CHECKBOX = "checkBox";
    public static final String HEADER_LIST = "headerList";
    public static final String HEADER_NAME = "headerName";

    @Inject
    private ProjectTypeService projectTypeService;

    public AdminProjectModuleSettingsPage() {
        final Form<List<ProjectType>> form = new Form<List<ProjectType>>(FORM, getProjectTypes()) {
            @Override
            protected void onSubmit() {
                for (ProjectType projectType : getModelObject()) {
                    projectTypeService.save(projectType);
                }
                success(getString("submitted"));
            }
        };
        add(form);
        form.add(new ListView<ProjectType>(HEADER_LIST, form.getModel()) {

            @Override
            protected void populateItem(ListItem<ProjectType> item) {
                item.add(new Label(HEADER_NAME, new PropertyModel<>(item.getModel(), "name")));
            }
        });
        form.add(new PropertyListView<ProjectModule>(MODULE_LIST, Arrays.asList(ProjectModule.values())) {
            @Override
            protected void populateItem(final ListItem<ProjectModule> moduleItem) {
                moduleItem.add(new EnumLabel<>(NAME, moduleItem.getModel()));
                moduleItem.add(new ListView<ProjectType>(PROJECT_TYPE_LIST, form.getModel()) {
                    @Override
                    protected void populateItem(ListItem<ProjectType> item) {
                        item.add(new CheckBox(CHECKBOX, new ModuleModel(item.getModel(), moduleItem.getModelObject())));
                    }
                });
            }
        });
    }

    private LoadableDetachableModel<List<ProjectType>> getProjectTypes() {
        return new LoadableDetachableModel<List<ProjectType>>() {
            @Override
            protected List<ProjectType> load() {
                return projectTypeService.findAllActive();
            }
        };
    }

    private static final class ModuleModel extends AbstractCheckBoxModel {

        private final IModel<ProjectType> projectTypeModel;
        private final ProjectModule projectModule;

        private ModuleModel(IModel<ProjectType> projectTypeModel, ProjectModule projectModule) {
            this.projectTypeModel = projectTypeModel;
            this.projectModule = projectModule;
        }

        @Override
        public boolean isSelected() {
            return projectTypeModel.getObject().getProjectModules().contains(projectModule);
        }

        @Override
        public void select() {
            projectTypeModel.getObject().addModule(projectModule);
        }

        @Override
        public void unselect() {
            projectTypeModel.getObject().removeModule(projectModule);
        }
    }


}
