package se.su.dsv.scipro.admin.pages.settings;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.admin.pages.AbstractAdminSystemPage;
import se.su.dsv.scipro.admin.panels.AdminProjectTypePanel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.List;

public class AdminProjectTypeSettingsPage extends AbstractAdminSystemPage {

    @Inject
    private ProjectTypeService projectTypeService;

    public AdminProjectTypeSettingsPage() {
        final ModalWindowPlus modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);

        IModel<List<ProjectType>> activeProjectType = new LoadableDetachableModel<List<ProjectType>>() {

            @Override
            protected List<ProjectType> load() {
                return projectTypeService.findAllActive();
            }
        };

        final WebMarkupContainer activeWmc = new WebMarkupContainer("activeProjectTypeContainer");
        activeWmc.setOutputMarkupId(true);
        add(activeWmc);

        final ListView<ProjectType> listView = new ListView<ProjectType>("projectTypeList", activeProjectType) {
            @Override
            protected void populateItem(ListItem<ProjectType> item) {
                final IModel<ProjectType> model = item.getModel();

                item.add(new Label("name", model.getObject().getName()));
                item.add(new MultiLineLabel("desc", model.getObject().getDescription()));

                AjaxLink<Void> editLink = new AjaxLink<Void>("editLink") {

                    @Override
                    public void onClick(AjaxRequestTarget target) {
                        modalWindowPlus.setContent(new AdminProjectTypePanel(modalWindowPlus.getContentId(), model));
                        modalWindowPlus.setTitle("Edit");
                        modalWindowPlus.show(target);
                    }
                };
                item.add(editLink);
            }

        };
        activeWmc.add(listView);

        final IModel<List<ProjectType>> deletedProjectType = new LoadableDetachableModel<List<ProjectType>>() {

            @Override
            protected List<ProjectType> load() {
                return projectTypeService.findAllDeleted();
            }
        };

        final WebMarkupContainer deletedWmc = new WebMarkupContainer("deletedProjectTypeContainer");
        deletedWmc.setOutputMarkupId(true);
        add(deletedWmc);

        final ListView<ProjectType> listViewDeleted = new ListView<ProjectType>("projectTypeListDel", deletedProjectType) {
            @Override
            protected void populateItem(ListItem<ProjectType> item) {
                final IModel<ProjectType> model = item.getModel();

                item.add(new Label("nameDel", model.getObject().getName()));
                item.add(new MultiLineLabel("descDel", model.getObject().getDescription()));

                AjaxLink<Void> undeleteLink = new AjaxLink<Void>("undeleteLink") {

                    @Override
                    public void onClick(AjaxRequestTarget target) {
                        model.getObject().setDeleted(false);
                        model.setObject(projectTypeService.save(model.getObject()));
                        deletedProjectType.detach();
                        target.add(activeWmc);
                        target.add(deletedWmc);
                        info("Type activated");
                    }
                };
                item.add(undeleteLink);
            }

        };
        deletedWmc.add(listViewDeleted);

        AjaxLink<Void> createLink = new AjaxLink<Void>("createLink") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                modalWindowPlus.setContent(new AdminProjectTypePanel(modalWindowPlus.getContentId()));
                modalWindowPlus.setTitle("Create new type");
                modalWindowPlus.show(target);
            }
        };
        add(createLink);
    }

}
