package se.su.dsv.scipro.admin.pages.settings;

import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.admin.pages.AbstractAdminSystemPage;
import se.su.dsv.scipro.components.BootstrapDateField;
import se.su.dsv.scipro.components.BootstrapDatePicker;
import se.su.dsv.scipro.components.ListField;
import se.su.dsv.scipro.plagiarism.TurnitinSettings;
import se.su.dsv.scipro.plagiarism.TurnitinSettingsService;

import javax.inject.Inject;
import java.util.Date;

public class TurnitinSettingsPage extends AbstractAdminSystemPage {

    static final String SETTINGS = "settings";
    static final String USERNAME = "username";
    static final String PASSWORD = "password";
    static final String FIRST_NAME = "firstName";
    static final String LAST_NAME = "lastName";
    static final String CLASS_NO = "classNo";
    static final String CLASS_NAME = "className";
    static final String ASSIGNMENT_NAME = "assignmentName";
    static final String SHARED_KEY = "sharedKey";
    static final String ACCOUNT_NO = "accountNo";
    static final String EXPIRATION_DATE = "expirationDate";
    static final String EXPIRATION_MAILS = "expirationMails";
    static final String EXPIRATION_CONTENT = "expirationContent";
    public static final String MAX_FILE_SIZE = "maxFileSize";

    @Inject
    private TurnitinSettingsService turnitinSettingsService;

    public TurnitinSettingsPage() {
        add(new TurnitinSettingsForm(SETTINGS));
    }

    class TurnitinSettingsForm extends Form<TurnitinSettings> {
        public TurnitinSettingsForm(String id) {
            super(id);
            setModel(new CompoundPropertyModel<>(getSettings()));
            add(new CheckBox("enabled"));
            add(new RequiredTextField<>(USERNAME, String.class));
            add(new RequiredTextField<>(PASSWORD, String.class));
            add(new RequiredTextField<>(FIRST_NAME, String.class));
            add(new RequiredTextField<>(LAST_NAME, String.class));
            add(new RequiredTextField<>(CLASS_NO, Integer.class));
            add(new RequiredTextField<>(CLASS_NAME, String.class));
            add(new RequiredTextField<>(ASSIGNMENT_NAME, String.class));
            add(new RequiredTextField<>(SHARED_KEY, String.class));
            add(new RequiredTextField<>(ACCOUNT_NO, Integer.class));
            add(new RequiredTextField<>(MAX_FILE_SIZE, Integer.class));
            add(new ListField<String>(EXPIRATION_MAILS));
            add(new TextArea<String>(EXPIRATION_CONTENT));
            add(new RequiredTextField<>("integrationId", Integer.class));
            add(new RequiredTextField<>("integrationURL", String.class));
            add(newDateField());
        }

        private LoadableDetachableModel<TurnitinSettings> getSettings() {
            return new LoadableDetachableModel<TurnitinSettings>() {
                @Override
                protected TurnitinSettings load() {
                    return turnitinSettingsService.getSettings();
                }
            };
        }

        @Override
        protected void onSubmit() {
            turnitinSettingsService.save(getModelObject());
            info("Settings saved");
        }
    }

    private FormComponent<Date> newDateField() {
        FormComponent<Date> dateTextField = new BootstrapDateField(EXPIRATION_DATE);
        dateTextField.add(new BootstrapDatePicker());
        return dateTextField;
    }
}
