package se.su.dsv.scipro.basepanels;

import org.apache.wicket.Page;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.util.io.IClusterable;
import se.su.dsv.scipro.admin.pages.AbstractAdminPage;
import se.su.dsv.scipro.admin.pages.AdminStartPage;
import se.su.dsv.scipro.examiner.pages.ExaminerStartPage;
import se.su.dsv.scipro.project.pages.ProjectPage;
import se.su.dsv.scipro.project.pages.ProjectStartPage;
import se.su.dsv.scipro.reviewer.ReviewerStartPage;
import se.su.dsv.scipro.reviewer.ReviewerPage;
import se.su.dsv.scipro.supervisor.pages.AbstractSupervisorPage;
import se.su.dsv.scipro.supervisor.pages.SupervisorStartPage;

import java.util.ArrayList;
import java.util.List;

public class RolesMenuPanel extends Panel {

    private final Class<? extends Page> containerClass;


    public RolesMenuPanel(String id, Class<? extends Page> containerClass) {
        super(id);
        this.containerClass = containerClass;

        Label currentRole = new Label("currentRole", getSelectedRole());
        add(currentRole);

        WebMarkupContainer wmc = new WebMarkupContainer("rolesContainer");
        add(wmc.setOutputMarkupId(true));

        final ListView<MenuItem> rolesListView = new ListView<MenuItem>("rolesList", getItemList()) {
            @Override
            protected void populateItem(ListItem<MenuItem> item) {
                BookmarkablePageLink<Object> roleItem = new BookmarkablePageLink<>("roleItem", item.getModelObject().getTargetClass());
                roleItem.add(new Label("linkText", item.getModelObject().getName()));
                item.add(roleItem);
            }
        };

        wmc.add(rolesListView);
    }

    private String getSelectedRole() {
        if (ProjectPage.class.isAssignableFrom(containerClass)) {
            return ProjectStartPage.MAIN_MENU_LABEL;
        }
        if (AbstractSupervisorPage.class.isAssignableFrom(containerClass)) {
            return SupervisorStartPage.MAIN_MENU_LABEL;
        }
        if (ReviewerPage.class.isAssignableFrom(containerClass)) {
            return ReviewerStartPage.MAIN_MENU_LABEL;
        }
        if (ExaminerStartPage.class.isAssignableFrom(containerClass)) {
            return ExaminerStartPage.MAIN_MENU_LABEL;
        }
        if (AbstractAdminPage.class.isAssignableFrom(containerClass)) {
            return AdminStartPage.MAIN_MENU_LABEL;
        }
        return "Choose";
    }

    private List<MenuItem> getItemList() {
        List<MenuItem> list = new ArrayList<>();
        list.add(new MenuItem(ProjectStartPage.MAIN_MENU_LABEL, ProjectStartPage.class));
        list.add(new MenuItem(SupervisorStartPage.MAIN_MENU_LABEL, SupervisorStartPage.class));
        list.add(new MenuItem(ReviewerStartPage.MAIN_MENU_LABEL, ReviewerStartPage.class));
        list.add(new MenuItem(ExaminerStartPage.MAIN_MENU_LABEL, ExaminerStartPage.class));
        list.add(new MenuItem(AdminStartPage.MAIN_MENU_LABEL, AdminStartPage.class));
        return list;
    }

    protected static class MenuItem implements IClusterable {
        private String name;
        private Class<? extends Page> targetClass;

        public MenuItem(String name, Class<? extends Page> targetClass) {
            this.name = name;
            this.targetClass = targetClass;
        }

        public String getName() {
            return name;
        }

        public Class<? extends Page> getTargetClass() {
            return targetClass;
        }
    }
}