package se.su.dsv.scipro.checklists;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.activityplan.ProjectActivityPlanPage;
import se.su.dsv.scipro.checklist.Checklist;
import se.su.dsv.scipro.checklist.ChecklistService;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightActivityPlan;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.project.pages.AbstractProjectDetailsPage;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;

@Authorization(authorizedRoles = {Roles.AUTHOR})
public class ProjectViewChecklistPage extends AbstractProjectDetailsPage implements MenuHighlightActivityPlan {

    @Inject
    ChecklistService checklistService;

    public ProjectViewChecklistPage(PageParameters pp) {
        super(pp);

        final Long checklistId = pp.get(PageParameterKeys.MAP.get(Checklist.class)).toOptionalLong();
        if (checklistId == null) {
            noSuchChecklist();
        }

        final Checklist checklist = checklistService.findOne(checklistId);
        if (checklist == null) {
            noSuchChecklist();
        }

        final IModel<Checklist> checklistModel = new DetachableServiceModel<>(checklistService, checklist);
        add(new ChecklistQuestionPanel("viewChecklistPanel", new CompoundPropertyModel<>(checklistModel)));
    }

    private void noSuchChecklist() {
        throw new RestartResponseException(ProjectActivityPlanPage.class, getPageParameters());
    }
}
