package se.su.dsv.scipro.components;

import org.apache.wicket.extensions.ajax.markup.html.repeater.data.table.AjaxFallbackHeadersToolbar;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.ISortableDataProvider;
import org.apache.wicket.extensions.markup.html.repeater.data.table.NoRecordsToolbar;
import org.apache.wicket.markup.ComponentTag;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class DefaultDataPanel<T, S> extends DataTable<T, S> {
    private static final long DEFAULT_ROWS_PER_PAGE = 20L;
    private static final List<Long> ROWS_PER_PAGE_OPTIONS = Collections.unmodifiableList(Arrays.asList(DEFAULT_ROWS_PER_PAGE, 100L, 500L, 1000L));

    public DefaultDataPanel(final String id, final List<? extends IColumn<T, S>> columns, final ISortableDataProvider<T, S> provider) {
        this(id, columns, provider, DEFAULT_ROWS_PER_PAGE);
    }

    public DefaultDataPanel(final String id, final List<? extends IColumn<T, S>> columns, final ISortableDataProvider<T, S> dataProvider, final long rowsPerPage) {
        super(id, columns, dataProvider, rowsPerPage);

        addTopToolbar(new ResizeableNavigationToolbar(this, ROWS_PER_PAGE_OPTIONS));
        addTopToolbar(new AjaxFallbackHeadersToolbar<>(this, dataProvider));
        addBottomToolbar(new NoRecordsToolbar(this));
    }

    @Override
    protected void onComponentTag(ComponentTag tag) {
        checkComponentTag(tag, "table");
        super.onComponentTag(tag);
    }
}
