package se.su.dsv.scipro.data;

import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.GenericService;

public class DetachableServiceModel<DO extends DomainObject> implements IModel<DO> {

    private static final long serialVersionUID = -7932592734004059133L;

    private transient boolean attached = false;
    private GenericService<DO, Long> service;
    private Long id;
    private DO object;

    public DetachableServiceModel(GenericService<DO, Long> service) {
        this.service = service;
    }

    public DetachableServiceModel(GenericService<DO, Long> service, DO object) {
        this(service);

        setObject(object);
    }

    public DetachableServiceModel(GenericService<DO, Long> service, Long id) {
        this(service);

        this.id = id;
    }

    @Override
    public DO getObject() {
        if (!attached) {
            reload();
        }
        return object;
    }

    @Override
    public void setObject(DO object) {
        attached = true;
        id = object != null ? object.getId() : null;
        this.object = object;
    }

    @Override
    public void detach() {
        if (object != null) {
            // Workaround for Wicket only calling setObject if they are not
            // equal but since Hibernate injects id into the same reference
            // they are always equal so setObject will never get called.
            id = object.getId();
        }
        if (id != null) {
            attached = false;
            object = null;
        }
    }

    private void reload() {
        if (id != null) {
            setObject(service.findOne(id));
        }
    }
}
