package se.su.dsv.scipro.datatables;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.IExportableColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;

/**
 * Author: fred-fri
 * Date: 5/4/12
 */
public abstract class ClickableOrderColumn<T> extends AbstractColumn<T, String> implements IExportableColumn<T, String, String> {

    private final String property;

    public ClickableOrderColumn(IModel<String> displayModel,
                                String property, String sort) {
        super(displayModel, sort);
        this.property = property;
    }

    public void populateItem(Item<ICellPopulator<T>> cellItem,
                             String componentId, IModel<T> rowModel) {
        cellItem.add(new LinkPanel(componentId, rowModel,
                new PropertyModel<>(rowModel, property)));
    }

    protected abstract void onClick(IModel<T> clicked, AjaxRequestTarget target, boolean up);

    private class LinkPanel extends Panel {
        public LinkPanel(String id, IModel<T> rowModel, IModel<?>
                labelModel) {
            super(id);

            add(new Label("label", labelModel));

            AjaxLink<T> uplink = new AjaxLink<T>("uplink", rowModel) {
                @Override
                public void onClick(AjaxRequestTarget target) {
                    ClickableOrderColumn.this.onClick(getModel(), target, true);
                }
            };
            add(uplink);

            AjaxLink<T> downlink = new AjaxLink<T>("downlink", rowModel) {
                @Override
                public void onClick(AjaxRequestTarget target) {
                    ClickableOrderColumn.this.onClick(getModel(), target, false);
                }
            };
            add(downlink);
        }
    }
}