package se.su.dsv.scipro.datatables;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.util.AjaxConfirmationLink;


public abstract class LabelAndLinkColumn<T> extends Panel {
	private LabelAndLinkColumn<T> ajaxLinkColumn;

    public abstract void onClick(AjaxRequestTarget target, IModel<T> clickedModel);
    public abstract String labelString();
    
    /**
     * Datatable column with a label and a link next to it. If a confirmationMessage string is available, 
     * a confirmation dialog pops up before confirming link click
     * 
     */
    public LabelAndLinkColumn(String id, final IModel<T> model, String linkString) {
    	this(id, model, linkString, null, true);
    }

    public LabelAndLinkColumn(String id, final IModel<T> model, String linkString, String confirmationMessage, boolean showLinks) {
    	 super(id, model);
         ajaxLinkColumn = this;

         AjaxLink<T> link;
         if(confirmationMessage!=null){
        	 link = new AjaxConfirmationLink<T>("link", model, confirmationMessage) {


				@Override
				public void onClick(AjaxRequestTarget target) {
					  ajaxLinkColumn.onClick(target, model);					
				}
			};
         } else {
        	 link = new AjaxLink<T>("link") {

      			@Override
                  public void onClick(AjaxRequestTarget target) {
                      ajaxLinkColumn.onClick(target, model);
                  }
              };
         }
         

         link.add(new Label("linkLabel", linkString));
         link.setVisible(showLinks);
         add(new Label("label", labelString()));
         add(link);

    }
    
    /**
     * Use this constructor if you don't want the link to have a confirmation dialog when clicking
     * 
     */
    public LabelAndLinkColumn(String id, final IModel<T> model, String linkString, boolean showLinks) {
       this(id, model, linkString, null, showLinks);
    }
}
