package se.su.dsv.scipro.datatables.target;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.Target;
import se.su.dsv.scipro.match.TargetService;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;

public abstract class EditTargetPanel extends Panel {

    private final IModel<ApplicationPeriod> apModel;
    private final IModel<User> eModel;
    private final IModel<ProjectType> pcModel;

    @Inject
    private TargetService targetService;

    public abstract void update(AjaxRequestTarget ajaxTarget, User e);

    public EditTargetPanel(String id, final IModel<ApplicationPeriod> apModel, final IModel<User> eModel, final IModel<ProjectType> pcModel, final boolean isEditable) {
        super(id);
        this.apModel = apModel;
        this.eModel = eModel;
        this.pcModel = pcModel;
        final IModel<Target> target = findTarget(apModel, eModel, pcModel);

        final TextField<Integer> inputField = new TextField<Integer>("input", new PropertyModel<Integer>(target, "target")) {
            @Override
            protected void onConfigure() {
                setEnabled(isEditable);
            }
        };
        inputField.add(new AjaxFormComponentUpdatingBehavior("onchange") {
            @Override
            protected void onUpdate(AjaxRequestTarget ajaxTarget) {
                targetService.save(target.getObject());
                update(ajaxTarget, eModel.getObject());
            }
        });
        add(inputField);
    }

    @Override
    protected void onDetach() {
        super.onDetach();
        apModel.detach();
        eModel.detach();
        pcModel.detach();
    }

    private LoadableDetachableModel<Target> findTarget(final IModel<ApplicationPeriod> apModelObject, final IModel<User> eModelObject, final IModel<ProjectType> projectType) {
        return new LoadableDetachableModel<Target>() {
            @Override
            protected Target load() {
                return targetService.findOne(apModelObject.getObject(), eModelObject.getObject(), projectType.getObject());
            }
        };
    }
}
