package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.AjaxConfirmationLink;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class AttendingPanel extends GenericPanel<Project> {

    private ListView<FinalSeminar> participations;

    @Inject
    private FinalSeminarActiveParticipationService finalSeminarActiveParticipationService;

    @Inject
    private FinalSeminarService finalSeminarService;

    public AttendingPanel(String id, final IModel<Project> model) {
        super(id, model);
        add(new AutoHidingListView<FinalSeminar>("oppositions", getOppositions()) {
            @Override
            protected void populateItem(ListItem<FinalSeminar> item) {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(item.getModel().getObject().getProject().getId()));
                BookmarkablePageLink<Void> link = new BookmarkablePageLink<>("seminarLink", ProjectFinalSeminarDetailsPage.class, pp);
                item.add(link);
                link.add(new Label("seminarLabel", new PropertyModel<>(item.getModel(), "project.title")));
                item.add(new DateLabel("date", new PropertyModel<Date>(item.getModel(), "startDate"), DateStyle.DATETIME));
            }
        });
        add(new Label("noOppositions", "No oppositions"){
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getOppositions().getObject().isEmpty());
            }
        });
        final WebMarkupContainer wmc = new WebMarkupContainer("wmc");
        wmc.setOutputMarkupId(true);
        add(wmc);

        participations = new AutoHidingListView<FinalSeminar>("participations", getParticipations()) {
            @Override
            protected void populateItem( final ListItem<FinalSeminar> item) {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(item.getModel().getObject().getProject().getId()));
                BookmarkablePageLink<Void> link = new BookmarkablePageLink<>("seminarLink", ProjectFinalSeminarDetailsPage.class, pp);
                item.add(link);
                link.add(new Label("seminarLabel", new PropertyModel<>(item.getModel(), "project.title")));
                item.add(new DateLabel("date", new PropertyModel<Date>(item.getModel(), "startDate"), DateStyle.DATETIME));
                Component removeParticipation = new  AjaxConfirmationLink<FinalSeminar>("remove", item.getModel(), "Are you sure you want to delete this active participation?") {
                    @Override
                    public void onClick(AjaxRequestTarget target) {
                        item.getModelObject().removeActiveParticipant(SciProSession.get().getUser());
                        finalSeminarService.save(item.getModelObject());
                        refresh();
                        target.add(wmc);
                    }
                };
                removeParticipation.setVisible(!finalSeminarActiveParticipationService.findByFinalSeminarUser(
                        item.getModelObject(),SciProSession.get().getUser()).hasGrade());
                item.add(removeParticipation);
            }
        };
        wmc.add(participations);
        add(new Label("noParticipations", "No participations"){
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getParticipations().getObject().isEmpty());
            }
        });
    }

    private void refresh(){
        participations.detach();
    }

    private LoadableDetachableModel<List<FinalSeminar>> getOppositions(){
        return new LoadableDetachableModel<List<FinalSeminar>>() {
            @Override
            protected List<FinalSeminar> load() {
                return finalSeminarService.findUserOpposing(getModelObject(), SciProSession.get().getUser());
            }
        };
    }

    private LoadableDetachableModel<List<FinalSeminar>> getParticipations(){
        return new LoadableDetachableModel<List<FinalSeminar>>() {
            @Override
            protected List<FinalSeminar> load() {
                return finalSeminarService.findUserParticipating(getModelObject(), SciProSession.get().getUser());
            }
        };
    }
}
