package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormChoiceComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.form.Radio;
import org.apache.wicket.markup.html.form.RadioGroup;
import org.apache.wicket.markup.html.link.AbstractLink;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.basepages.MenuPage;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.LinkWrapper;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class FinalSeminarDataPanel extends Panel {

    @Inject
    private FinalSeminarService finalSeminarService;

    private FinalSeminarService.Filter filter;
    private boolean supervisorView;
    private RadioGroup<Boolean> group;
    private ExportableDataPanel<FinalSeminar,String> dataTable;

    public FinalSeminarDataPanel(String id, boolean isSupervisorView) {
        super(id);

        supervisorView = isSupervisorView;
        addModalWindow();
        addFiltering();
        addDataTable();
    }

    private void addModalWindow() {
        ModalWindowPlus modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);
    }

    private void addFiltering() {
        filter = new FinalSeminarService.Filter();
        if (!supervisorView) {
            filter.setOnlyActiveOrCompletedProjects(true);
        }
        filter.setFromDate(new Date());

        group = new RadioGroup<>("dateChoice", new Model<>(true));
        add(group);

        group.add(new AjaxFormChoiceComponentUpdatingBehavior() {
            @Override
            protected void onUpdate(final AjaxRequestTarget target) {
                target.add(dataTable);
                if (group.getModelObject()) {
                    filter.setFromDate(new Date());
                    filter.setToDate(null);
                } else {
                    filter.setToDate(new Date());
                    filter.setFromDate(null);
                }
            }
        });

        group.add(new Radio<>("future", Model.of(true)));
        group.add(new Radio<>("past", Model.of(false)));

        add(new AjaxCheckBox("onlyMy", new Model<Boolean>()) {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                if (getModelObject()) {
                    filter.setHeadSupervisor(SciProSession.get().getUser());
                } else {
                    filter.setHeadSupervisor(null);
                }
                target.add(dataTable);
            }

            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(supervisorView);
            }
        });
    }

    private void addDataTable() {
        SortableDataProvider<FinalSeminar, String> provider = new FilteredDataProvider<>(finalSeminarService, Model.of(filter));
        provider.setSort("startDate", SortOrder.DESCENDING);
        dataTable = new ExportableDataPanel<>("dp", createColumns(), provider);
        add(dataTable);
    }

    private List<IColumn<FinalSeminar, String>> createColumns() {
        List<IColumn<FinalSeminar, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<FinalSeminar>(Model.of("Date"), "startDate", "startDate", DateStyle.DATETIME));
        columns.add(new PropertyColumn<FinalSeminar, String>(Model.of("Type"), "project.projectType.name", "project.projectType.name"));
        columns.add(new AbstractColumn<FinalSeminar, String>(Model.of("Title"), "project.title") {
            @Override
            public void populateItem(final Item<ICellPopulator<FinalSeminar>> cellItem, final String componentId, final IModel<FinalSeminar> rowModel) {
                cellItem.add(LinkWrapper.apply(componentId, new AbstractFunction1<String, AbstractLink>() {
                    @Override
                    public AbstractLink apply(final String id) {
                        final Class<? extends MenuPage> pageClass = getRedirectPage();
                        final PageParameters pp = new PageParameters();
                        pp.set(PageParameterKeys.MAP.get(Project.class), rowModel.getObject().getProject().getId());
                        final BookmarkablePageLink<Void> link = new BookmarkablePageLink<>(id, pageClass, pp);
                        link.setBody(new PropertyModel<>(rowModel, "project.title"));
                        return link;
                    }

                    private Class<? extends MenuPage> getRedirectPage() {
                        return iSupervisorView() ?
                                SupervisorFinalSeminarDetailsPage.class :
                                ProjectFinalSeminarDetailsPage.class;
                    }
                }));
            }
        });
        return columns;
    }

    private boolean iSupervisorView() {
        return supervisorView;
    }
}