package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import scala.util.Either;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.date.DateService;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.SeminarEvent;

import javax.inject.Inject;
import java.util.Date;

public class MoveFinalSeminarPanel extends GenericPanel<FinalSeminar> {
    public static final String DATE = "startDate";
    public static final String MOVE_FORM = "moveForm";

    @Inject
    private FinalSeminarService finalSeminarService;
    @Inject
    private DateService dateService;
    @Inject
    private NotificationController notificationController;

    public MoveFinalSeminarPanel(String id, IModel<FinalSeminar> model) {
        super(id, model);
        add(new MoveForm(MOVE_FORM, model));
    }

    private class MoveForm extends Form<FinalSeminar> {

        public MoveForm(String form, final IModel<FinalSeminar> model) {
            super(form, model);
            FormComponent<Date> startDate = new FinalSeminarDateTimeComponent(DATE, new PropertyModel<Date>(model, "startDate"));
            startDate.add(new SeminarDateValidator(getModelObject(), dateService.getFormat(DateStyle.DATE)));
            add(startDate);
        }

        @Override
        protected void onSubmit() {
            Either<SchedulingError, FinalSeminar> move =
                    finalSeminarService.move(getModelObject(), getModelObject().getStartDate());
            if (move.isRight()) {
                notificationController.notifySeminar(getModelObject(), SeminarEvent.Event.DATE_CHANGED, new NotificationSource());

                MoveForm.this.success("Final seminar moved to the new date");
            }
            else {
                error("Could not move the final seminar to that date");
            }
        }
    }
}
