package se.su.dsv.scipro.finalseminar;

import com.google.common.base.Function;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.link.StatelessLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

import javax.inject.Inject;
import java.text.SimpleDateFormat;
import java.util.Date;

public abstract class OpposeColumnPanel extends Panel {

    public static final String LINK = "link";
    public static final String OPPOSE_CONFIRMATION = "Are you sure you want to oppose on this seminar?";

    @Inject
    private FinalSeminarService finalSeminarService;

    public abstract void onClick(AjaxRequestTarget target, IModel<FinalSeminar> clickedModel);

    public OpposeColumnPanel(String id, final IModel<FinalSeminar> finalSeminarModel, final IModel<Project> projectModel) {
        super(id, finalSeminarModel);

        final OppositionRegistrationStatus registrationStatus = finalSeminarService.canOppose(SciProSession.get().getUser(), finalSeminarModel.getObject(), projectModel.getObject());
        final Component opposeLink = registrationStatus.fold(
                new Function<UngradedOpposition, Component>() {
                    @Override
                    public Component apply(final UngradedOpposition ungradedOpposition) {
                        return getLabel("You have ungraded oppositions");
                    }
                },
                new Function<FinalSeminarCancelled, Component>() {
                    @Override
                    public Component apply(final FinalSeminarCancelled finalSeminarCancelled) {
                        return getLabel("The final seminar has been cancelled");
                    }
                },
                new Function<AlreadyParticipating, Component>() {
                    @Override
                    public Component apply(final AlreadyParticipating alreadyParticipating) {
                        return getLabel("You are already participating on the final seminar");
                    }
                },
                new Function<AlreadyHappened, Component>() {
                    @Override
                    public Component apply(final AlreadyHappened alreadyHappened) {
                        return getLabel("The seminar has already happened");
                    }
                },
                new Function<TooManyOpponents, Component>() {
                    @Override
                    public Component apply(final TooManyOpponents tooManyOpponents) {
                        return getLabel("The seminar is already full on opponents");
                    }
                },
                new Function<PriorityForSeminarAuthors, Component>() {
                    @Override
                    public Component apply(final PriorityForSeminarAuthors priorityForSeminarAuthors) {
                        return getLabel("Authors with their own final seminar have " +
                                "priority to register for " + priorityForSeminarAuthors.getPriorityDays() +
                                " day(s). This priority will end at " + asDateTime(priorityForSeminarAuthors.getPriorityEnd().toDate()));
                    }
                },
                new Function<AlreadyOpposed, Component>() {
                    @Override
                    public Component apply(final AlreadyOpposed alreadyOpposed) {
                        return getLabel("You have already completed an opposition and can not perform a second one.");
                    }
                }, new Function<CanRegister, Component>() {
                    @Override
                    public Component apply(final CanRegister input) {
                        return new OpposeLink(LINK, finalSeminarModel);
                    }
                }
        );

        add(opposeLink);
    }

    public String asDateTime(Date date) {
        SimpleDateFormat dateFormat = new SimpleDateFormat(DateStyle.DATETIME.getFormat());
        return dateFormat.format(date);
    }

    private Component getLabel(final String label) {
        // Dumb workaround to get the <a> tag transformed into a <span>
        return new StatelessLink<Void>(LINK){
            @Override
            public void onClick() {}
        }
                .setBody(Model.of(label))
                .setEnabled(false);
    }

    private class OpposeLink extends AjaxConfirmationLink {

        private final IModel<FinalSeminar> finalSeminarModel;

        public OpposeLink(final String id, final IModel<FinalSeminar> finalSeminarModel) {
            super(id, OpposeColumnPanel.OPPOSE_CONFIRMATION);
            this.finalSeminarModel = finalSeminarModel;
            setOutputMarkupId(true);
        }

        @Override
        public void onClick(AjaxRequestTarget target) {
            OpposeColumnPanel.this.onClick(target, finalSeminarModel);
            setEnabled(false);
            target.add(this);
        }
    }
}
