package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.panel.EmptyPanel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.project.pages.AbstractProjectDetailsPage;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.ProjectModule;

import javax.inject.Inject;

@Authorization(authorizedRoles = {Roles.AUTHOR})
@ProjectModuleComponent(ProjectModule.FINAL_SEMINAR)
public class ProjectFinalSeminarPage extends AbstractProjectDetailsPage {

    @Inject
    private FinalSeminarService finalSeminarService;

    public ProjectFinalSeminarPage(PageParameters pp) {
        super(pp);

        final FinalSeminar seminar = finalSeminarService.findByProject(getActiveProject());

        add(new WebMarkupContainer(NOSEMINAR) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(seminar == null);
            }
        });

        if (seminar != null) {
            DetachableServiceModel<FinalSeminar> seminarModel = new DetachableServiceModel<>(finalSeminarService, seminar);
            add(new SeminarPanel(SEMINAR, seminarModel));
        } else {
            add(new EmptyPanel(SEMINAR));
        }

    }

    static final String NOSEMINAR = "noSeminar";
    static final String SEMINAR = "seminar";

}
