package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.*;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.OppositeVisibility;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.grading.OppositionReportPdfResource;
import se.su.dsv.scipro.report.OppositionReport;
import se.su.dsv.scipro.report.OppositionReportService;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.AjaxConfirmationLink;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;

@ProjectModuleComponent(ProjectModule.GRADING)
public class SeminarOppositionReportPanel extends GenericPanel<FinalSeminarOpposition> {

    @Inject
    private OppositionReportService oppositionReportService;

    private final WebMarkupContainer wmc;
    private final FeedbackPanel feedbackPanel;

    private IModel<FinalSeminar> seminar;

    public SeminarOppositionReportPanel(String id, IModel<FinalSeminarOpposition> model) {
        super(id, model);

        FinalSeminarOpposition opposition = getModelObject();
        seminar = Model.of(opposition.getFinalSeminar());

        feedbackPanel = new FencedFeedbackPanel("feedback", this);
        feedbackPanel.setOutputMarkupId(true);
        add(feedbackPanel);

        wmc = new WebMarkupContainer("wmc");
        wmc.setOutputMarkupId(true);
        add(wmc);

        ViewAttachmentPanel oldReport = getOldReport(opposition);
        wmc.add(oldReport);
        wmc.add(getNewReportContainer(oldReport));

        wmc.add(getOppositionReportLabel(opposition));

        wmc.add(getNoOppositionReportYet(opposition));

        PageParameters parameters = new PageParameters();
        parameters.add(PageParameterKeys.MAP.get(FinalSeminarOpposition.class), opposition.getId());
        wmc.add(getOppositionReportLink(opposition, parameters));

        final IModel<OppositionReport> report = getReport(opposition);
        wmc.add(getDownloadPdfReportPanel(opposition, report));

        wmc.add(getDownloadAttachment(opposition, report));

        wmc.add(getDeleteOpponentReportLink(opposition));

        wmc.add(getDeleteOppositionReportLink(opposition));
    }

    private Component getNewReportContainer(ViewAttachmentPanel oldReport) {
        return new WebMarkupContainer("newReport").add(new OppositeVisibility(oldReport));
    }

    private ViewAttachmentPanel getOldReport(final FinalSeminarOpposition opposition) {
        return new ViewAttachmentPanel("reportDownload", new PropertyModel<FileDescription>(opposition, "opponentReport"), DateStyle.DATETIME) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(getDefaultModelObject() != null && allowedToSeeOppositionReport(opposition));
            }
        };
    }

    private Label getOppositionReportLabel(final FinalSeminarOpposition opposition) {
        return new Label("oppositionReportLabel", new ResourceModel("opposition.report")) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(allowedToSeeOppositionReport(opposition));
            }
        };
    }

    private Label getNoOppositionReportYet(final FinalSeminarOpposition opposition) {
        return new Label("noOppositionReportYet", new ResourceModel("noOppositionReportYet")){
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(allowedToSeeOppositionReport(opposition) && (opposition.getOppositionReport() == null || !opposition.getOppositionReport().isSubmitted()));
            }
        };
    }

    private BookmarkablePageLink<Void> getOppositionReportLink(final FinalSeminarOpposition opposition, final PageParameters parameters) {
        return new BookmarkablePageLink<Void>("oppositionReportLink", OppositionReportPage.class, parameters) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(hasThesis() && isOpponentAndNotSubmitted(opposition));
            }
        };
    }

    private DownloadPdfReportPanel getDownloadPdfReportPanel(final FinalSeminarOpposition opposition, final IModel<OppositionReport> report) {
        return new DownloadPdfReportPanel("downloadPdfPanel", seminar, report, new OppositionReportPdfResource(report)) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(allowedToSeeOppositionReport(opposition));
            }
        };
    }

    private IModel<OppositionReport> getReport(final FinalSeminarOpposition opposition) {
        return new LoadableDetachableModel<OppositionReport>() {
            @Override
            protected OppositionReport load() {
                return oppositionReportService.findOrCreateReport(opposition);
            }
        };
    }

    private ViewAttachmentPanel getDownloadAttachment(final FinalSeminarOpposition opposition, final IModel<OppositionReport> report) {
        return new ViewAttachmentPanel("downloadAttachment", new PropertyModel<FileDescription>(report, "attachment")) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getDefaultModelObject() != null && allowedToSeeOppositionReport(opposition) && report.getObject().isSubmitted());
            }
        };
    }

    private AjaxConfirmationLink<OppositionReport> getDeleteOpponentReportLink(final FinalSeminarOpposition opposition) {
        return new AjaxConfirmationLink<OppositionReport>("removeReportLink", new StringResourceModel("are.you.sure", this, null)) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                oppositionReportService.deleteOpponentReport(opposition);
                info(getString("opposition.report.removed"));
                target.add(feedbackPanel);
                target.add(wmc);
            }

            @Override
            protected void onConfigure() {
                setVisible(isAdmin() && hasUploadedOpponentReport(opposition));
            }
        };
    }

    private AjaxConfirmationLink<OppositionReport> getDeleteOppositionReportLink(final FinalSeminarOpposition opposition) {
        return new AjaxConfirmationLink<OppositionReport>("removeOppositionLink", new StringResourceModel("are.you.sure", this, null)) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                oppositionReportService.deleteOppositionReport(opposition);
                info(getString("opposition.report.removed"));
                target.add(feedbackPanel);
                target.add(wmc);
            }

            @Override
            protected void onConfigure() {
                setVisible(isAdmin() && hasSubmittedOppositionReport(opposition));
            }
        };
    }

    private boolean isOpponentAndNotSubmitted(FinalSeminarOpposition opposition) {
        return isOppositionAuthor(opposition) && !oppositionReportService.findOrCreateReport(opposition).isSubmitted();
    }

    private boolean hasThesis() {
        return seminar.getObject().getDocument() != null;
    }

    private boolean allowedToSeeOppositionReport(FinalSeminarOpposition opposition) {
        return isHeadSupervisor() || isCoSupervisor() || isRespondent() || isOppositionAuthor(opposition) || isReviewer() || isAdmin();
    }

    private boolean hasUploadedOpponentReport(FinalSeminarOpposition opposition) {
        return opposition.getOpponentReport() != null;
    }

    private boolean isCoSupervisor() {
        return seminar.getObject().getProject().isCoSupervisor(SciProSession.get().getUser());
    }

    private boolean isRespondent() {
        return seminar.getObject().getProject().isParticipant(SciProSession.get().getUser());
    }

    private boolean isOppositionAuthor(FinalSeminarOpposition opposition) {
        return opposition.getUser().equals(SciProSession.get().getUser());
    }

    private boolean isReviewer() {
        User reviewer = seminar.getObject().getProject().getReviewer();
        return SciProSession.get().getUser().equals(reviewer);
    }

    private boolean hasSubmittedOppositionReport(FinalSeminarOpposition opposition) {
        return oppositionReportService.findOrCreateReport(opposition).isSubmitted();
    }

    private boolean isHeadSupervisor() {
        return SciProSession.get().getUser().equals(seminar.getObject().getProject().getHeadSupervisor());
    }

    private boolean isAdmin() {
        return SciProSession.get().authorizedForRole(Roles.SYSADMIN) && SciProSession.get().hasActualRole(Roles.SYSADMIN);
    }
}
