package se.su.dsv.scipro.finalthesis;

import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.upload.FileUpload;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.validation.IValidatable;
import org.apache.wicket.validation.IValidator;
import org.apache.wicket.validation.ValidationError;
import se.su.dsv.scipro.files.WicketFileUpload;
import se.su.dsv.scipro.finalseminar.FinalSeminarService;
import se.su.dsv.scipro.project.Project;

import javax.inject.Inject;
import java.util.List;

public class FinalThesisUploadFormPanel extends GenericPanel<Project> {

    public static final String FILE = "file";
    public static final String FORM = "form";

    @Inject
    private FinalThesisService finalThesisService;
    @Inject
    private FinalSeminarService finalSeminarService;

    public FinalThesisUploadFormPanel(String id, IModel<Project> project) {
        super(id, project);
        add(new FinalThesisUploadForm(FORM, project));
    }

    private final class FinalThesisUploadForm extends Form<Project> {

        private final FileUploadField uploadField;

        private FinalThesisUploadForm(String id, IModel<Project> project) {
            super(id, project);
            add(new FeedbackPanel("feedback"));
            uploadField = new FileUploadField(FILE);
            uploadField.setRequired(true);
            uploadField.add(new PDFValidator());
            add(uploadField);
        }

        @Override
        protected void onSubmit() {

            FileUpload fileUpload = uploadField.getFileUpload();
            finalThesisService.upload(WicketFileUpload.from(fileUpload), getModelObject());
        }
    }

    private static final class PDFValidator implements IValidator<List<FileUpload>> {
        @Override
        public void validate(IValidatable<List<FileUpload>> validatable) {
            for (FileUpload fileUpload : validatable.getValue()) {
                if (!fileUpload.getContentType().equals("application/pdf")) {
                    validatable.error(new ValidationError(this));
                }
            }
        }
    }

    @Override
    protected void onConfigure() {
        setVisibilityAllowed(hasHadFinalSeminar() && uploadAllowed());
    }

    private boolean uploadAllowed() {
        return finalThesisService.isUploadAllowed(getModelObject());
    }

    private boolean hasHadFinalSeminar() {
        return finalSeminarService.hasHadFinalSeminar(getModelObject());
    }
}
