package se.su.dsv.scipro.finalthesis;

import com.google.common.base.Function;
import com.google.common.collect.FluentIterable;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxFallbackButton;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.files.WicketFileUpload;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Set;

public class RejectFinalThesisPanel extends GenericPanel<Project> {

    private final IModel<Project> project;
    private final FileUploadField upload;
    private final FeedbackPanel feedbackPanel;
    private FormComponent<String> contentArea;

    protected final Form form;
    protected RequiredTextField<String> subjectField;

    public static final String FEEDBACK = "feedback";
    public static final String FORM = "form";
    public static final String REJECTED_SUBJECT = "Final thesis revision needed";
    public static final String CANCEL_REJECTION = "cancelRejection";
    public static final String ATTACHMENT = "attachment";
    public static final String MESSAGE_LABEL = "messageLabel";
    public static final String CONTENT = "content";
    public static final String CREATE = "create";
    public static final String SUBJECT = "subject";

    @Inject
    private FinalThesisService finalThesisService;

    @Inject
    ProjectForumService projectForumService;

    public RejectFinalThesisPanel(final String id, final IModel<Project> model) {
        super(id, model);
        this.project = model;
        form = new Form<Void>(FORM) {
            @Override
            protected void onSubmit() {
                RejectFinalThesisPanel.this.onSubmit();
            }
        };
        add(form);
        feedbackPanel = new FeedbackPanel(FEEDBACK);
        form.add(feedbackPanel);
        feedbackPanel.setOutputMarkupId(true);
        subjectField = newSubjectField(Model.of(""));
        form.add(subjectField);
        form.add(new Label(MESSAGE_LABEL,new ResourceModel("why.description.needed")));
        contentArea = new TextArea<>(CONTENT, Model.of("")).setRequired(true);
        form.add(contentArea);
        upload = new FileUploadField(ATTACHMENT);
        form.add(upload);
        form.add(new Link<Void>(CANCEL_REJECTION) {
            @Override
            public void onClick() {
                cancel();
            }
        });
        form.add(new AjaxFallbackButton(CREATE, form) {
            @Override
            protected void onError(AjaxRequestTarget target, Form<?> form) {
                target.add(feedbackPanel);
            }
        });
    }

    protected RequiredTextField<String> newSubjectField(IModel<String> subject) {
        RequiredTextField<String> field = new RequiredTextField<>(SUBJECT, subject);
        field.setVisible(false);
        return field;
    }

    protected void cancel(){}

    protected void onCreate() {
        finalThesisService.reject(getModelObject());
    }

    protected void onSubmit() {
        User user = SciProSession.get().getUser();
        Set<FileUpload> uploads = FluentIterable.from(upload.getFileUploads())
                .transform(new Function<org.apache.wicket.markup.html.form.upload.FileUpload, FileUpload>() {
                    @Override
                    public FileUpload apply(org.apache.wicket.markup.html.form.upload.FileUpload fileUpload) {
                        return WicketFileUpload.from(fileUpload);
                    }
                })
                .toSet();

        projectForumService.createThread(project.getObject(), user,
                REJECTED_SUBJECT, contentArea.getInput(), uploads);

        onCreate();
        setResponsePage(getPage());
    }

}
