package se.su.dsv.scipro.forum.panels.threaded;

import com.google.common.base.Function;
import com.google.common.collect.FluentIterable;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxFallbackButton;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.files.WicketFileUpload;
import se.su.dsv.scipro.forum.Forum;
import se.su.dsv.scipro.session.SciProSession;

import java.util.Set;

public class SubmitForumThreadPanel<A> extends Panel {

    private final Forum<A> forum;

    public void onThreadCreated(final A thread) { }

    public SubmitForumThreadPanel(String id, final Forum<A> forum) {
        super(id);
        this.forum = forum;
        add(new ForumForm("form"));
    }

    private class ForumForm extends Form<Void> {

        public ForumForm(String id) {
            super(id);

            final FeedbackPanel feedbackPanel = new FeedbackPanel("feedback");
            feedbackPanel.setOutputMarkupId(true);
            add(feedbackPanel);

            final IModel<String> subject =  new Model<>();

            final FormComponent<String> contentArea = new TextArea<>("content", new Model<String>()).setRequired(true);
            add(contentArea);

            final FileUploadField upload = new FileUploadField("attachment");
            add(upload);

            add(new AjaxFallbackButton("postLink", this) {
                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    target.add(feedbackPanel);
                }
                @Override
                protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                    String subjectString = subject.getObject();
                    Set<FileUpload> attachments = FluentIterable.from(upload.getFileUploads())
                            .transform(new Function<org.apache.wicket.markup.html.form.upload.FileUpload, FileUpload>() {
                                @Override
                                public FileUpload apply(org.apache.wicket.markup.html.form.upload.FileUpload fileUpload) {
                                    return WicketFileUpload.from(fileUpload);
                                }
                            })
                            .toSet();
                    A thread = forum.createThread(SciProSession.get().getUser(), subjectString, contentArea.getModelObject(), attachments);
                    onThreadCreated(thread);
                }
            });

            add(new RequiredTextField<>("subject", subject));
        }
    }
}
