package se.su.dsv.scipro.forum.panels.threaded;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.forum.ForumThread;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.springdata.services.UserProfileService;

import javax.inject.Inject;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class ViewForumThreadPanel<A> extends GenericPanel<A> {

    @Inject
    ProjectForumService projectForumService;
    @Inject
    UserProfileService profileService;

    private final ForumThread<A> forumThread;
    private final WebMarkupContainer wrapper;
    private SubmitForumReplyPanel topReplyPanel;
    private AjaxLink<Void> topReplyLink;

    @SuppressWarnings("unchecked")
    public ViewForumThreadPanel(String id, final IModel<A> model, final ForumThread<A> forumThread) {
        super(id, model);
        this.forumThread = forumThread;

        forumThread.setRead(SciProSession.get().getUser(), model.getObject());

        wrapper = new WebMarkupContainer(WRAPPER);
        add(wrapper.setOutputMarkupId(true));

        addPostList();
        addReplyButtons();
        addReplyPanels();
    }

    @Override
    protected void onInitialize() {
        super.onInitialize();
        addHeader();
    }

    private void addHeader() {
        wrapper.add(newBackLink(BACK_LINK));
        wrapper.add(new Label(SUBJECT, forumThread.getSubject(getModelObject())));
    }

    protected Component newBackLink(final String id) {
        return new WebMarkupContainer(id).setVisible(false);
    }

    private void addPostList() {

        wrapper.add(new ListView<ForumPost>(POST_LIST, new PostProvider()) {

            @Override
            protected void populateItem(ListItem<ForumPost> item) {
                item.add(new ForumPostPanel(POST, item.getModel()));
            }
        });
    }

    private void addReplyButtons() {
        topReplyLink = new AjaxLink<Void>(TOP_REPLY_LINK) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                topReplyVisibility(false, true);
                target.add(wrapper);
            }
        };
        wrapper.add(topReplyLink);
    }

    private void addReplyPanels() {
        topReplyPanel = new SubmitForumReplyPanel<A>(TOP_REPLY_PANEL, getModel(), forumThread) {
            @Override
            protected void updateOnSubmit() {
                topReplyVisibility(true, false);
            }
        };
        wrapper.add(topReplyPanel);

        topReplyPanel.setOutputMarkupPlaceholderTag(true).setVisible(false);
    }

    private void topReplyVisibility(boolean showLink, boolean showPanel) {
        topReplyLink.setVisible(showLink);
        topReplyPanel.setVisible(showPanel);
    }

    static final String WRAPPER = "wrapper";
    static final String POST_LIST = "postList";
    static final String POST = "post";
    static final String TOP_REPLY_LINK = "topReplyLink";
    static final String TOP_REPLY_PANEL = "topReplyPanel";
    static final String BACK_LINK = "backLink";
    static final String SUBJECT = "subject";

    private class PostProvider extends LoadableDetachableModel<List<ForumPost>> {

        @Override
        protected List<ForumPost> load() {
            List<ForumPost> posts = forumThread.getPosts(getModelObject());
            Collections.sort(posts, new Comparator<ForumPost>() {
                @Override
                public int compare(ForumPost o1, ForumPost o2) {
                    return o2.getDateCreated().compareTo(o1.getDateCreated());
                }
            });
            return posts;
        }
    }
}
