package se.su.dsv.scipro.generalsystemsettings;

import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.validation.validator.EmailAddressValidator;
import se.su.dsv.scipro.admin.pages.AbstractAdminSystemPage;
import se.su.dsv.scipro.components.ListField;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.SystemModule;

import java.util.Arrays;

@Authorization(authorizedRoles = { Roles.SYSADMIN})
public class AdminGeneralSettingsPage extends AbstractAdminSystemPage {

    static final String SETTINGS_FORM = "generalSettings";
    static final String ALARM_MAILS = "alarmMails";
    static final String SUPERVISOR_CHANGE_MAILS = "supervisorChangeMails";

    public AdminGeneralSettingsPage() {
		add(new GeneralSystemSettingsForm(SETTINGS_FORM){
			@Override
			protected void addFields() {
                add(new RequiredTextField<>("mailFromName", String.class));
                add(new RequiredTextField<>("systemFromMail", String.class));
                add(new RequiredTextField<>("smtpServer", String.class));
                add(new RequiredTextField<>("sciproURL", String.class));
                add(new TextField<String>("thesisSupportMail")
                        .add(EmailAddressValidator.getInstance()));
                add(new TextField<String>("reviewerSupportMail")
                        .add(EmailAddressValidator.getInstance()));
                add(new TextField<String>("matchResponsibleMail")
                        .add(EmailAddressValidator.getInstance()));
                add(new CheckBox("mailNotifications"));
                add(new CheckBox("showSingleSignOn"));
                add(new ListField<String>(ALARM_MAILS));
                add(new ListField<String>(SUPERVISOR_CHANGE_MAILS));
                add(new TextField<>("daisyProfileLinkBaseURL", String.class));
                add(new TextField<String>("externalRoomBookingURL"));
                add(new TextField<String>("externalGettingStartedWithIdeaURL"));
                add(new CheckBoxMultipleChoice<>("systemModules", Arrays.asList(SystemModule.values()), new EnumChoiceRenderer<SystemModule>(this))
                        .setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER));
            }
		});
	}
}
