package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.report.GradingCriterion;
import se.su.dsv.scipro.report.GradingCriterionPoint;
import se.su.dsv.scipro.report.GradingReport;

import java.util.List;

public class CriteriaPanel extends GenericPanel<GradingReport> {

    public static final String CRITERIA = "criteria";
    public static final String TITLE = "title";
    public static final String MAX_POINTS = "maxPoints";
    public static final String POINTS = "points";
    public static final String FEEDBACK = "feedback";

    @SuppressWarnings("unchecked")
    public CriteriaPanel(final String id, final IModel<? extends GradingReport> model) {
        super(id, (IModel<GradingReport>) model);
        add(new PropertyListView<GradingCriterion>(CRITERIA, new PropertyModel<List<? extends GradingCriterion>>(model, "gradingCriteria")) {
            {
                setReuseItems(true);
            }

            @Override
            protected void populateItem(final ListItem<GradingCriterion> item) {
                item.add(new Label(TITLE));
                item.add(new Label(MAX_POINTS, getPointsString(item.getModelObject())));

                PropertyModel<List<GradingCriterionPoint>> gradingCriterionPoints = new PropertyModel<>(item.getModel(), "gradingCriterionPoints");
                final RadioChoice<GradingCriterionPoint> pointsRadioChoice = new RadioChoice<>(POINTS, new GradingCriterionPointsModelConverter(item.getModel(), gradingCriterionPoints), gradingCriterionPoints, getPointsRenderer());
                item.add(pointsRadioChoice.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER));

                TextArea<String> textArea = new TextArea<>(FEEDBACK);
                item.add(textArea);
            }

            private IChoiceRenderer<? super GradingCriterionPoint> getPointsRenderer() {
                return new ChoiceRenderer<GradingCriterionPoint>() {
                    @Override
                    public Object getDisplayValue(GradingCriterionPoint object) {
                        return object.getPoint().toString() + " p"
                                + (object.getDescription() != null && !object.getDescription().equals("") ? " - " + object.getDescription() : "");
                    }

                    @Override
                    public String getIdValue(GradingCriterionPoint object, int index) {
                        return object.getPoint().toString();
                    }
                };
            }
        });
    }

    private String getPointsString(GradingCriterion criterion) {
        return "(*" + criterion.getPointsRequiredToPass() + "* is required to pass)";
    }

    private static final class GradingCriterionPointsModelConverter implements IModel<GradingCriterionPoint> {
        private final IModel<GradingCriterion> criterionIModel;
        private final PropertyModel<List<GradingCriterionPoint>> gradingCriterionPoints;

        public GradingCriterionPointsModelConverter(IModel<GradingCriterion> criterionIModel, PropertyModel<List<GradingCriterionPoint>> gradingCriterionPoints) {
            this.criterionIModel = criterionIModel;
            this.gradingCriterionPoints = gradingCriterionPoints;
        }

        @Override
        public void detach() {
            criterionIModel.detach();
            gradingCriterionPoints.detach();
        }

        @Override
        public GradingCriterionPoint getObject() {
            Integer points = criterionIModel.getObject().getPoints();
            for (GradingCriterionPoint gradingCriterionPoint : gradingCriterionPoints.getObject()) {
                if (gradingCriterionPoint.getPoint().equals(points)) {
                    return gradingCriterionPoint;
                }
            }
            return null;
        }

        @Override
        public void setObject(GradingCriterionPoint object) {
            criterionIModel.getObject().setPoints(object.getPoint());
        }
    }
}
