package se.su.dsv.scipro.grading;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.border.Border;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.SubmitLink;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import scala.Option;
import se.su.dsv.scipro.components.AutoSavingBehaviour;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.files.WicketFileUpload;
import se.su.dsv.scipro.report.AttachmentReport;
import se.su.dsv.scipro.report.Criterion;
import se.su.dsv.scipro.report.ReportService;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.util.JavascriptEventConfirmation;

import javax.inject.Inject;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

public class FillOutReportPanel<T extends AttachmentReport> extends Border {

    public static final String FORM = "form";
    public static final String GRADING_CRITERIA = "criteria";
    public static final String TITLE = "title";
    public static final String DESCRIPTION = "description";
    public static final String FEEDBACK = "feedback";
    public static final String SAVE = "save";
    public static final String SUBMIT = "submit";
    public static final String FEEDBACK_PANEL = "feedbackPanel";

    @Inject
    private ReportService reportService;

    public FillOutReportPanel(String id, final IModel<T> model) {
        super(id, model);

        ReportForm form = new ReportForm(FORM, model);
        addToBorder(new ScrollingSaveButtonPanel(SAVE, form));
        addToBorder(form);
    }

    public void onSubmit() {
        //override where suitable
    }

    private class ReportForm extends Form<T> {

        private final FileUploadField attachment;

        public ReportForm(String id, final IModel<T> model) {
            super(id, model);

            add(getBodyContainer());
            add(new ComponentFeedbackPanel(FEEDBACK_PANEL, this));


            add(new PropertyListView<Criterion>(GRADING_CRITERIA, new PropertyModel<List<Criterion>>(model, GRADING_CRITERIA)) {
                @Override
                protected void populateItem(ListItem<Criterion> item) {
                    item.add(new Label(TITLE));
                    item.add(new Label(DESCRIPTION));
                    TextArea<Object> textArea = new TextArea<>(FEEDBACK);
                    textArea.add(new AutoSavingBehaviour() {
                        @Override
                        protected void onUpdate(AjaxRequestTarget target) {
                            reportService.save(model.getObject());
                        }
                    });
                    item.add(textArea);
                }
            });

            attachment = new FileUploadField("uploadAttachment") {
                @Override
                protected void onConfigure() {
                    setVisible(ReportForm.this.getModelObject().getAttachment() == null);
                }
            };
            add(attachment);
            add(new ViewAttachmentPanel("viewAttachment", new PropertyModel<FileDescription>(getModel(), "attachment")));
            add(new DeleteAttachmentLink("deleteAttachment", getModel()));

            SubmitLink finishLink = new SubmitLink(SUBMIT) {
                @Override
                public void onAfterSubmit() {
                    try {
                        reportService.submit(getModelObject());
                        getForm().success(getString("submitted"));
                        FillOutReportPanel.this.onSubmit();
                    } catch (IllegalStateException ignored) {
                        getForm().error(getString("notFilledOut"));
                    }
                }
            };
            finishLink.add(new JavascriptEventConfirmation("onclick", new ResourceModel("confirmFinish")));
            add(finishLink);
        }

        @Override
        protected void onSubmit() {
            Option<FileUpload> fileUploadOptional = WicketFileUpload.ofOptional(attachment.getFileUpload());
            reportService.save(getModelObject(), fileUploadOptional);
            String dateString = new SimpleDateFormat("HH:mm:ss").format(new Date());
            getSession().success(getString("saved") + " " + dateString);
        }

        @Override
        protected void onConfigure() {
            super.onConfigure();
            setEnabled(!getModelObject().isSubmitted());
        }
    }

    private class DeleteAttachmentLink extends Link<T> {

        public DeleteAttachmentLink(String id, IModel<T> model) {
            super(id, model);
            add(new JavascriptEventConfirmation("onclick", new ResourceModel("delete.attachment")));
        }

        @Override
        public void onClick() {
            reportService.deleteAttachment(getModelObject());
        }

        @Override
        protected void onConfigure() {
            setVisible(getModelObject().getAttachment() != null);
        }
    }
}
