package se.su.dsv.scipro.grading;

import com.google.common.base.Joiner;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.report.*;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class ReviewerGradingReportPdf implements GradingReportPdf {

    private final IModel<ReviewerGradingReport> reviewerGradingReport;
    private final IModel<SupervisorGradingReport> supervisorGradingReport;
    private GradeCalculator gradeCalculator;

    public ReviewerGradingReportPdf(IModel<ReviewerGradingReport> reviewerGradingReport, IModel<SupervisorGradingReport> supervisorGradingReport, GradeCalculator gradeCalculator) {
        this.reviewerGradingReport = reviewerGradingReport;
        this.supervisorGradingReport = supervisorGradingReport;
        this.gradeCalculator = gradeCalculator;
    }

    @Override
    public String getProjectTitle() {
        return reviewerGradingReport.getObject().getProject().getTitle();
    }

    @Override
    public String getProjectType() {
        return reviewerGradingReport.getObject().getProject().getProjectType().getName();
    }

    @Override
    public String getAuthorName() {
        return supervisorGradingReport.getObject().getUser().getFullName();
    }

    @Override
    public String getAuthorNames() {
        List<String> names = new ArrayList<>();
        for (User student : reviewerGradingReport.getObject().getProject().getProjectParticipants()) {
            names.add(student.getFullName());
        }
	    return Joiner.on(", ").join(names);
    }

    @Override
    public String getSupervisorName() {
        return reviewerGradingReport.getObject().getProject().getSupervisorName();
    }

    @Override
    public String getReviewerName() {
        return reviewerGradingReport.getObject().getProject().getReviewerName();
    }

    @Override
    public String getPlagiarismFeedback() {
        return supervisorGradingReport.getObject().getPlagiarismFeedback();
    }

    @Override
    public String getGrade() {
        GradingReport.Grade grade = reviewerGradingReport.getObject().getGrade(gradeCalculator);
        if (grade == null){
            return "Betyget räknas ut när alla kriterier har poängsatts";
        }
        return grade.name();
    }

    @Override
    public Date getLastModified() {
        return reviewerGradingReport.getObject().getLastModified();
    }

    @Override
    public List<GradingCriterion> getGradingCriteria() {
        List<GradingCriterion> criteria = new ArrayList<>(reviewerGradingReport.getObject().getGradingCriteria());
        criteria.addAll(supervisorGradingReport.getObject().getIndividualCriteria());
        return criteria;
    }

    @Override
    public String getProjectTypeName() {
        return reviewerGradingReport.getObject().getProjectType().getName();
    }
}
