package se.su.dsv.scipro.grading;

import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.OppositeVisibility;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.finalthesis.ApproveFinalThesisPanel;
import se.su.dsv.scipro.forum.panels.threaded.ViewForumThreadPanel;
import se.su.dsv.scipro.report.GradingReport;
import se.su.dsv.scipro.report.GradingReportService;
import se.su.dsv.scipro.report.SupervisorGradingReport;
import se.su.dsv.scipro.reviewer.ReviewerInteractionForumThread;
import se.su.dsv.scipro.reviewing.ReviewerInteractionService;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.supervisor.pages.AbstractSupervisorProjectDetailsPage;
import se.su.dsv.scipro.system.ProjectModule;

import javax.inject.Inject;

@ProjectModuleComponent(ProjectModule.GRADING)
public class SupervisorGradingPage extends AbstractSupervisorProjectDetailsPage {

    public static final String FILL_OUT_PANEL = "fillOutPanel";

    @Inject
    private GradingReportService gradingReportService;
    @Inject
    private ReviewerInteractionService reviewerInteractionService;
    @Inject
    private FileDescriptionService fileDescriptionService;

    public SupervisorGradingPage(PageParameters pp) {
        super(pp);
        add(new ApproveFinalThesisPanel("finalThesis", projectModel));

        add(new FillOutGradingReportPanel(FILL_OUT_PANEL, projectModel) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(projectModel.getObject().getHeadSupervisor().equals(SciProSession.get().getUser()));
            }
        });

        final IModel<GradingReport> reviewerGradingReport = new LoadableDetachableModel<GradingReport>() {
            @Override
            protected GradingReport load() {
                return gradingReportService.getReviewerGradingReport(projectModel.getObject());
            }
        };
        add(new CriteriaPanel("viewReviewerReport", reviewerGradingReport) {
            @Override
            protected void onConfigure() {
                setVisible(reviewerGradingReport.getObject().getState() != GradingReport.State.INITIAL &&
                           supervisorHasDoneInitialAssessment());
            }
        });
        add(new Label("initialAssessmentNotDone", new ResourceModel("initial.assessment.not.done")).add(new OppositeVisibility(get("viewReviewerReport"))));

        add(new ViewForumThreadPanel<>("discussion", projectModel, new ReviewerInteractionForumThread(reviewerInteractionService, fileDescriptionService)));
    }

    private boolean supervisorHasDoneInitialAssessment() {
        return Iterables.all(gradingReportService.getSupervisorGradingReports(projectModel.getObject()), new Predicate<SupervisorGradingReport>() {
            @Override
            public boolean apply(final SupervisorGradingReport input) {
                return input.getState() != GradingReport.State.INITIAL;
            }
        });
    }
}
