package se.su.dsv.scipro.group;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.LargeModalWindow;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.supervisor.pages.SupervisorEditGroupPage;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Date;

public class GroupDetailsPanel extends Panel {

    private final ModalWindowPlus modalWindowPlus;

    public GroupDetailsPanel(String id, final IModel<Group> model) {
        super(id, model);

        modalWindowPlus = new LargeModalWindow("dialog");
        add(modalWindowPlus);

        add(new Label("title", new PropertyModel<>(model, "title")));

        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Group.class), String.valueOf(model.getObject().getId()));
        add(new BookmarkablePageLink<Group>("edit", SupervisorEditGroupPage.class, pp){
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(model.getObject().getUser().equals(SciProSession.get().getUser()));
            }
        });

        add(new DateLabel("created", new PropertyModel<Date>(model, "dateCreated"), DateStyle.DATE));
        add(new Label("active", model.getObject().isActive() ? "(Active)" : "(Inactive)"));
        add(new AjaxLink<Group>("viewProjects", model) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                modalWindowPlus.setContent(new ViewGroupProjectsPanel(modalWindowPlus.getContentId(), model));
                modalWindowPlus.setTitle("Group: " + model.getObject().getTitle());
                modalWindowPlus.show(target);
            }
        });
        add(new Label("description", new PropertyModel<>(model, "description")));
    }
}

