package se.su.dsv.scipro.group;

import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.forum.ForumThread;
import se.su.dsv.scipro.forum.GroupForumService;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.GroupThread;
import se.su.dsv.scipro.system.User;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class GroupForumThread implements ForumThread<GroupThread> {
    private final GroupForumService groupForumService;
    private final FileDescriptionService fileDescriptionService;

    public GroupForumThread(final GroupForumService groupForumService, final FileDescriptionService fileDescriptionService) {
        this.groupForumService = groupForumService;
        this.fileDescriptionService = fileDescriptionService;
    }

    @Override
    public List<ForumPost> getPosts(final GroupThread groupThread) {
        return groupForumService.getPosts(groupThread);
    }

    @Override
    public ForumPost reply(final GroupThread groupThread, final User poster, final String content, final Set<FileUpload> attachments) {
        Set<FileDescription> storedAttachments = store(attachments);
        return groupForumService.createReply(groupThread, poster, content, storedAttachments);
    }

    private Set<FileDescription> store(final Set<FileUpload> attachments) {
        Set<FileDescription> storedAttachments = new HashSet<>(attachments.size());
        for (FileUpload attachment : attachments) {
            storedAttachments.add(fileDescriptionService.storeFile(attachment));
        }
        return storedAttachments;
    }

    @Override
    public String getSubject(final GroupThread groupThread) {
        return groupThread.getForumThread().getSubject();
    }

    @Override
    public void setRead(User user, GroupThread groupThread) {
        groupForumService.markRead(user, groupThread);
    }
}
