package se.su.dsv.scipro.loginlogout.pages;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.basepages.PublicPage;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.Collections;
import java.util.HashSet;

public class FirstTimeRegistrationPage extends PublicPage {

    @Inject
    private UserService userService;
    @Inject
    private UserNameService usernameService;
    @Inject
    private PasswordRepo passwordRepo;

    public FirstTimeRegistrationPage() {
        super();

        if (userService.count() > 0) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        add(new FeedbackPanel("feedback"));

        add(new FirstTimeRegistrationForm("form"));

        add(new BookmarkablePageLink<Void>("back", getApplication().getHomePage()));
    }

    private class FirstTimeRegistrationForm extends StatelessForm<Void> {

        private IModel<String> firstName = new Model<>();
        private IModel<String> lastName = new Model<>();
        private IModel<String> email = new Model<>();
        private IModel<String> passwordModel = new Model<>();

        public FirstTimeRegistrationForm(final String id) {
            super(id);

            add(new RequiredTextField<>("firstName", firstName));
            add(new RequiredTextField<>("lastName", lastName));
            add(new RequiredTextField<>("email", email));
            add(new PasswordTextField("password", passwordModel));
        }

        @Override
        protected void onSubmit() {
            User user = new User();
            user.setFirstName(firstName.getObject());
            user.setLastName(lastName.getObject());
            user.setEmailAddress(email.getObject());
            Username username = createUsernameObject(user);
            user.setUsernames(new HashSet<>(Collections.singleton(username)));
            user.setEmailAddress(email.getObject());
            user.addRole(Roles.SYSADMIN);
            Password password = createPassword(user, passwordModel.getObject());
            user = userService.save(user);
            password = passwordRepo.save(password);
            user.setPassword(password);
            userService.save(user);
            usernameService.save(username);
            success(getString("success"));
        }

        private Username createUsernameObject(final User user) {
            Username username = new Username();
            username.setUsername(email.getObject());
            username.setUser(user);

            return username;
        }

        public Password createPassword(final User user, final String passwordString) {
            char[] passwordInput = passwordString.toCharArray();

            Password password = new Password();
            password.setSalt(PasswordHandler.generateSalt());
            password.setHash(PasswordHandler.generateHash(passwordInput, password.getSalt()));
            password.setUser(user);

            return password;
        }
    }
}
