package se.su.dsv.scipro.match;

import lombok.Getter;
import lombok.Setter;
import org.apache.commons.collections.IteratorUtils;
import org.apache.wicket.Component;
import org.apache.wicket.MetaDataKey;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.util.ListModel;
import org.joda.time.DateTime;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.date.DatePickerPanel;
import se.su.dsv.scipro.profile.StudentProfileProgramPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.springdata.services.UnitService;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.*;

import static java.util.Arrays.asList;

public abstract class AbstractAdminIdeaPanel extends Panel {

    private static final MetaDataKey<Long> PRESELECTED_APPLICATION_PERIOD = new MetaDataKey<Long>() {};

    public static final String DIALOG = "dialog";
    public static final String DATA_TABLE = "dataTable";
    public static final String HEADER_DATE_CREATED = "Created";
    public static final String HEADER_PROJECT_TYPE = "Type";
    public static final String HEADER_TITLE = "Title";
    public static final String HEADER_STATUS = "Status";
    public static final String HEADER_SUPERVISOR = "Supervisor";
    public static final String HEADER_AUTHORS = "Authors";
    public static final int ONE_YEAR_AGO = 12;
    public static final int MIN_TITLE_LENGTH = 3;
    public static final String TITLE = "title";
    public static final String FEEDBACK = "feedback";
    protected final FeedbackPanel feedback;

    @Inject
    private IdeaService ideaService;
    @Inject
    private TargetService targetService;
    @Inject
    private KeywordService keywordService;
    @Inject
    private ApplicationPeriodService applicationPeriodService;
    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private UnitService unitService;
    @Inject
    private ResearchAreaService researchAreaService;

    private ModalWindowPlus modalWindowPlus;
    protected ExportableDataPanel<Idea, String> dataTable;
    private final IdeaService.Filter filter = new IdeaService.Filter();
    private DatePickerPanel datePickerPanel;
    private KeywordFilterPanel keywordPanel;

    @Getter
    @Setter
    @SuppressWarnings("UnusedDeclaration")
    private String titleString;
    @Getter
    @Setter
    @SuppressWarnings("UnusedDeclaration")
    private boolean dateFiltersActive;

    public AbstractAdminIdeaPanel(String id) {
        super(id);
        feedback = new FeedbackPanel(FEEDBACK);
        feedback.setOutputMarkupId(true);
        add(feedback);
        addDialog();
        addFiltering();
        addDataTable();
    }

    private void addDialog() {
        modalWindowPlus = new ModalWindowPlus(DIALOG);
        add(modalWindowPlus);
    }

    private void addDataTable() {
        SortableDataProvider<Idea, String> provider = new FilteredDataProvider<>(ideaService, Model.of(filter));
        dataTable = new ExportableDataPanel<>(DATA_TABLE, createColumns(), provider);
        dataTable.setOutputMarkupId(true);
        add(dataTable);
    }

    private List<IColumn<Idea,String>> createColumns() {
        List<IColumn<Idea,String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Idea>(Model.of(HEADER_DATE_CREATED), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Idea, String>(Model.of(HEADER_PROJECT_TYPE), "projectType.name", "projectType.name"));

        columns.add(new PropertyColumn<Idea, String>(Model.of(HEADER_TITLE), TITLE) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> components, String s, final IModel<Idea> model) {
                components.add(new AjaxLinkPanel<Idea>(s, model, new PropertyModel(model, TITLE)) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Idea> model) {
                        openDialog(model, target);
                    }
                });
            }
        });

        columns.add(new AbstractExportableColumn<Idea, String, String>(Model.of(HEADER_STATUS), "match.status") {
            @Override
            public IModel<String> getDataModel(IModel<Idea> rowModel) {
                return Model.of(ideaService.getStatusString(rowModel.getObject()));
            }
        });

        columns.add(new AbstractExportableColumn<Idea, String, String>(Model.of(HEADER_SUPERVISOR)) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> item, String id, IModel<Idea> ideaModel) {
                addSupervisorField(item, id, ideaModel);
            }

            @Override
            public IModel<String> getDataModel(IModel<Idea> rowModel) {
                return new PropertyModel<>(rowModel, "match.supervisor.fullName");
            }
        });

        columns.add(new AbstractExportableColumn<Idea, String, String>(Model.of(HEADER_AUTHORS)) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, IModel<Idea> rowModel) {
                cellItem.add(new DisplayMultiplesPanel<IdeaParticipation>(componentId, new ListAdapterModel<>(new PropertyModel<Iterable<IdeaParticipation>>(rowModel, "ideaParticipations"))) {
                    @Override
                    public Component getComponent(String componentId, IModel<IdeaParticipation> t) {
                        return new StudentProfileProgramPanel(componentId, t, true);
                    }
                });
            }

            @Override
            public IModel<String> getDataModel(IModel<Idea> rowModel) {
                StringBuilder participants = new StringBuilder();
                Set<IdeaParticipation> participantsSet = rowModel.getObject().getIdeaParticipations();
                for (IdeaParticipation participation: participantsSet) {
                    participants.append(participation.getUser().getFullName()).append("; ");
                }
                return Model.of(participants.toString());
            }
        });
        columns.addAll(addPanelSpecificColumns());
        return columns;
    }

    private void openDialog(IModel<Idea> model, AjaxRequestTarget target) {
        modalWindowPlus.setContent(new AdminIdeaDetailsPanel(modalWindowPlus.getContentId(), model, dataTable) {

            @Override
            public void update(AjaxRequestTarget target) {
                target.add(dataTable);
                modalWindowPlus.close(target);
                target.appendJavaScript("window.scrollTo(0,0);");
            }

        });
        modalWindowPlus.setTitle(model.getObject().getTitle());
        modalWindowPlus.show(target);
    }

    private void addSupervisorField(Item<ICellPopulator<Idea>> item, String id, IModel<Idea> ideaModel) {
        final Idea idea = ideaModel.getObject();
        switch (idea.getType()) {
            case SUPERVISOR:
                item.add(new Label(id, ideaModel.getObject().getMatch().getSupervisor().getFullName()));
                break;
            case STUDENT:
                item.add(new EmployeeAutoCompleteDivPanel(id, new Model<>(idea.getMatch().getSupervisor())) {
                    @Override
                    public void onNewEmployeeSelection(AjaxRequestTarget target, User newSelection) {
                        if (targetService.hasTargetsLeft(idea.getApplicationPeriod(), newSelection, idea.getProjectType())) {
                            ideaService.changeSupervisor(idea, newSelection, SciProSession.get().getUser());
                            info("Supervisor changed");
                        } else {
                            error("The supervisor have reached current target numbers");
                        }
                        target.addListener(new AjaxFeedbackPanelUpdater());
                    }
                }.setEnabled(idea.getProject() == null && idea.getApplicationPeriod() != null && idea.getProjectType() != null));
                break;
            default:
                throw new IllegalStateException("idea needs a type");
        }
    }

    private void addFiltering() {
        statusFilter();
        ideaTypeFilter();
        projectTypeFilter();
        unitFilter();
        newKeywordsFilter(getKeywords());
        researchAreaFilter();
        add(newSupervisorFilter());
        add(newAuthorFilter());
        programFilter();
        dateFilter();
        add(newTitleFilter());
        add(newApplicationPeriodFilter());
        addFirstMeetingFilter();
    }

    private void addFirstMeetingFilter() {
        add(new AjaxCheckBox("noFirstMeeting", new PropertyModel<Boolean>(filter, "noFirstMeeting")) {
            @Override
            protected void onUpdate(final AjaxRequestTarget target) {
                target.add(dataTable);
            }
        });
    }

    @SuppressWarnings("unchecked")
    private Component newApplicationPeriodFilter() {
        List<ApplicationPeriod> list = IteratorUtils.toList(applicationPeriodService.findAll().iterator());
        Collections.sort(list, new Comparator<ApplicationPeriod>() {
            @Override
            public int compare(ApplicationPeriod o1, ApplicationPeriod o2) {
                return o2.getStartDate().compareTo(o1.getStartDate());
            }
        });
        IModel<ApplicationPeriod> applicationPeriod = new DetachableServiceModel<>(applicationPeriodService, getSession().getMetaData(PRESELECTED_APPLICATION_PERIOD));
        filter.setApplicationPeriod(applicationPeriod.getObject());
        return new AjaxDropDownChoice<ApplicationPeriod>("applicationPeriodFilter",
                applicationPeriod,
                new ListModel(list),
                new ChoiceRenderer<ApplicationPeriod>("name", "id")) {
            @Override
            public void onNewSelection(AjaxRequestTarget target, ApplicationPeriod objectSelected) {
                filter.setApplicationPeriod(objectSelected);
                target.add(dataTable);
                getSession().setMetaData(PRESELECTED_APPLICATION_PERIOD, objectSelected != null ? objectSelected.getId() : null);
            }
        }.setNullValid(true);
    }

    private IModel<Set<Keyword>> getKeywords() {
        return new LoadableDetachableModel<Set<Keyword>>() {
            @Override
            protected Set<Keyword> load() {
                if (filter.getResearchArea() != null) {
                    return new HashSet<>(keywordService.findByResearchArea(filter.getResearchArea()));
                }
                return new HashSet<>();
            }
        };
    }

    private void ideaTypeFilter() {
        filter.setTypes(new ArrayList<>(asList(Idea.Type.STUDENT, Idea.Type.SUPERVISOR)));
        AjaxCheckBoxMultipleChoice<Idea.Type> ideaType = new AjaxCheckBoxMultipleChoice<Idea.Type>("typeFilter",
                new PropertyModel<List<Idea.Type>>(filter, "types"),
                asList(Idea.Type.STUDENT, Idea.Type.SUPERVISOR), new EnumChoiceRenderer<Idea.Type>(this)) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        };
        add(ideaType);
    }

    private void statusFilter() {
        AjaxCheckBoxMultipleChoice<Idea.Status> status = new AjaxCheckBoxMultipleChoice<Idea.Status>("statusFilter",
                getDefaultStatus(), getStatusChoices(), new EnumChoiceRenderer<Idea.Status>(this)) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        };
        add(status);
    }

    private PropertyModel<List<Idea.Status>> getDefaultStatus() {
        List<Idea.Status> defaultSelection = new ArrayList<>();
        defaultSelection.addAll(asList(Idea.Status.UNMATCHED, Idea.Status.MATCHED));
        filter.setStatuses(defaultSelection);
        return new PropertyModel<>(filter, "statuses");
    }

    private List<Idea.Status> getStatusChoices() {
        List<Idea.Status> choices = new ArrayList<>();
        choices.addAll(asList(Idea.Status.UNMATCHED, Idea.Status.MATCHED, Idea.Status.COMPLETED, Idea.Status.INACTIVE));
        return choices;
    }

    private void projectTypeFilter() {
        AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                getDefaultLevels(), projectTypeService.findWithModule(ProjectModule.MATCH), new ChoiceRenderer<ProjectType>("name", "id")) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        };
        add(projectType);
    }

    private PropertyModel<List<ProjectType>> getDefaultLevels() {
        filter.setLevels(projectTypeService.findWithModule(ProjectModule.MATCH));
        return new PropertyModel<>(filter, "levels");
    }

    private void unitFilter() {
        add(new AjaxDropDownChoice<Unit>("unitFilter", new Model<Unit>(), new ListModel<>(unitService.findAllAsList()),
                new ChoiceRenderer<Unit>(TITLE, "id")) {
            @Override
            public void onNewSelection(AjaxRequestTarget target, Unit objectSelected) {
                filter.setUnit(getModelObject());
                target.add(dataTable);
            }
        });
    }

    private void researchAreaFilter() {
        add(new AjaxDropDownChoice<ResearchArea>("areaFilter", new Model<ResearchArea>(),
                new ListModel<>(researchAreaService.findNotDeleted()),
                new ChoiceRenderer<ResearchArea>(TITLE, "id")) {
            @Override
            public void onNewSelection(AjaxRequestTarget target, ResearchArea objectSelected) {
                filter.setResearchArea(getModelObject());
                target.add(dataTable);
                keywordPanel.setVisible(objectSelected != null && !keywordService.findByResearchArea(objectSelected).isEmpty());
                target.add(keywordPanel);
            }
        });
    }

    private void newKeywordsFilter(IModel<Set<Keyword>> availableKeywords) {
        keywordPanel = new KeywordFilterPanel("keywordsFilter", availableKeywords) {
            @Override
            public void onKeywordsSelected(AjaxRequestTarget target, Set<Keyword> keywords) {
                filter.setKeywords(keywords);
                target.add(dataTable);
            }
        };
        keywordPanel.setVisible(false);
        keywordPanel.setOutputMarkupPlaceholderTag(true);
        add(keywordPanel);
    }

    private void dateFilter() {
        final WebMarkupContainer dateContainer = new WebMarkupContainer("dateContainer");
        dateContainer.setOutputMarkupId(true);

        Date startDate = new DateTime().minusMonths(ONE_YEAR_AGO).toDate();
        datePickerPanel = new DatePickerPanel("dateFilter", startDate) {
            @Override
            public void updateStartDate(AjaxRequestTarget target) {
                filter.setCreatedAfter(getStartDate());
                target.add(dataTable);
            }

            @Override
            public void updateEndDate(AjaxRequestTarget target) {
                filter.setCreatedBefore(getEndDate());
                target.add(dataTable);
            }

            @Override
            protected void onConfigure() {
                setEnabled(dateFiltersActive);
            }
        };
        dateContainer.add(datePickerPanel);

        dateContainer.add(new AjaxCheckBox("dateFiltersActive", new PropertyModel<Boolean>(this, "dateFiltersActive")) {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                if (!dateFiltersActive) {
                    filter.setCreatedAfter(null);
                    filter.setCreatedBefore(null);
                } else {
                    filter.setCreatedAfter(datePickerPanel.getStartDate());
                    filter.setCreatedBefore(datePickerPanel.getEndDate());
                }
                target.add(dateContainer);
                target.add(dataTable);
            }
        });
        add(dateContainer);
    }

    private EmployeeAutoComplete newSupervisorFilter() {
        return new EmployeeAutoComplete("supervisorFilter") {

            @Override
            protected void action(AjaxRequestTarget target, User employee) {
                filter.setSupervisor(employee);
                target.add(dataTable);
            }
        };
    }

    private AuthorAutoComplete newAuthorFilter() {
        return new AuthorAutoComplete("authorFilter") {

            @Override
            protected void action(AjaxRequestTarget target, User newSelection) {
                filter.setAuthor(newSelection);
                target.add(dataTable);
            }
        };
    }

    private void programFilter() {
        add(new ProgramAutoComplete("programFilter") {
            @Override
            protected void action(AjaxRequestTarget target, Program object) {
                filter.setProgram(object);
                target.add(dataTable);
            }
        });
    }

    private TextField<String> newTitleFilter() {
        TextField<String> titleField = new TextField<>("titleFilter", new PropertyModel<String>(this, "titleString"));
        titleField.add(new AjaxFormComponentUpdatingBehavior("onkeyup") {

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                if (titleString != null && titleString.length() >= MIN_TITLE_LENGTH) {
                    filter.setTitleContains(titleString);
                } else {
                    filter.setTitleContains(null);
                }
                target.add(dataTable);

            }
        });
        return titleField;
    }

    protected abstract List<IColumn<Idea, String>> addPanelSpecificColumns();
}
