package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.EnclosureContainer;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.IChoiceRenderer;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.*;
import se.su.dsv.scipro.components.CheckBoxRequiredValidator;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.project.panels.PartnerAdditionPanel;
import se.su.dsv.scipro.projectpartner.ProjectPartnerPage;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.Program;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import java.util.Set;

public class AcceptIdeaDialogPanel extends Panel {

    @Inject
    private IdeaService ideaService;
    @Inject
    private UserService userService;

    private ApplicationPeriod applicationPeriod;

    public AcceptIdeaDialogPanel(String id, IModel<Idea> model, final ApplicationPeriod applicationPeriod) {
        super(id, model);
        this.applicationPeriod = applicationPeriod;
        FeedbackPanel feedback = new FeedbackPanel("feedback");
        feedback.setOutputMarkupId(true);
        add(feedback);
        add(new AcceptIdeaForm("form", model, feedback));
    }


    private class AcceptIdeaForm extends Form<Idea> {

        private PartnerAdditionPanel coAuthorPanel;
        private DropDownChoice<Program> programDropDownChoice;

        public AcceptIdeaForm(String id, final IModel<Idea> model, FeedbackPanel feedback) {
            super(id, model);
            add(new IdeaDetailsPanel("details", model, model.getObject().getType().equals(Type.SUPERVISOR)));
            addPartnerSelection(model);
            addProgramSelection();
            addSelectButton(model, feedback);
            addContract();
        }

        private void addProgramSelection() {
            final LoadableDetachableModel<User> userModel = getUserModel();
            WebMarkupContainer program = new WebMarkupContainer("program") {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(userModel.getObject().hasPrograms());
                }
            };
            add(program);

            programDropDownChoice = new DropDownChoice<>("programDropDown", new Model<Program>(), new ListAdapterModel<>(new PropertyModel<Set<Program>>(userModel, "programs")), new IChoiceRenderer<Program>() {
                @Override
                public Object getDisplayValue(Program object) {
                    return object.getName();
                }

                @Override
                public String getIdValue(Program object, int index) {
                    return Long.toString(object.getId());
                }
            });
            program.add(programDropDownChoice);

            program.add(new InfoPanel("programInfo", new ResourceModel("programInfo")));
        }

        private void addPartnerSelection(IModel<Idea> model) {
            coAuthorPanel = new PartnerAdditionPanel("author", true);
            coAuthorPanel.setVisible(model.getObject().getMaxAuthors() > 1);
            EnclosureContainer partnerEnclosure = new EnclosureContainer("partnerEnclosure", coAuthorPanel);
            partnerEnclosure.add(coAuthorPanel);
            add(partnerEnclosure);
            addInfoLink(partnerEnclosure);
        }

        private void addInfoLink(EnclosureContainer container) {
            container.add(new Link<Void>("partnerLink") {

                @Override
                public void onClick() {
                    setResponsePage(ProjectPartnerPage.class);
                }
            });
        }

        private void addSelectButton(final IModel<Idea> model, final FeedbackPanel feedback) {
            AjaxButton selectLink = new AjaxButton("selectLink") {

                @Override
                public void onSubmit(AjaxRequestTarget target, Form form) {
                    Pair<Boolean, String> acceptance = ideaService.validateStudentAcceptance(model.getObject(), getCurrentUser(), coAuthorPanel.getCoAuthor(), applicationPeriod);
                    if (acceptance.getHead()) {
                        ideaService.acceptIdea(model.getObject(), getCurrentUser(), programDropDownChoice.getModelObject(), coAuthorPanel.getCoAuthor(), applicationPeriod);
                        info(acceptance.getTail());
                    } else {
                        error(acceptance.getTail());

                    }
                    target.add(feedback);
                }

                @Override
                protected void updateAjaxAttributes(AjaxRequestAttributes attributes) {
                    super.updateAjaxAttributes(attributes);
                    AjaxCallListener ajaxCallListener = new AjaxCallListener();
                    ajaxCallListener.onPrecondition("return confirm('Are you sure you want to select this idea?');");
                    attributes.getAjaxCallListeners().add(ajaxCallListener);
                }

                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    target.add(feedback);
                }
            };
            add(selectLink);
        }

        private void addContract() {
            CheckBox contract = new CheckBox("contract", new Model<Boolean>());
            contract.add(new CheckBoxRequiredValidator());
            add(contract);
        }

    }

    private LoadableDetachableModel<User> getUserModel() {
        return new LoadableDetachableModel<User>() {
            @Override
            protected User load() {
                return userService.findOne(getCurrentUser().getId());
            }
        };
    }

    private User getCurrentUser() {
        return SciProSession.get().getUser();
    }
}
