package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;

public abstract class AdminIdeaDetailsPanel extends Panel {

    private Panel overviewPanel, historyPanel, currentPanel;
    private AjaxFallbackLink<Void> overviewLink, historyLink;

    public abstract void update(AjaxRequestTarget target);

    public AdminIdeaDetailsPanel(String id, IModel<Idea> ideaModel, WebMarkupContainer wmc) {
        super(id, ideaModel);
        addPanels(ideaModel, wmc);
        addOverviewLink();
        addHistoryLink();
    }

    private void addPanels(IModel<Idea> ideaModel, final WebMarkupContainer wmc) {
        add(new IdeaExportPanel("exportStatus", ideaModel));
        overviewPanel = new AdminIdeaDetailsCrudPanel("panel", ideaModel) {

            @Override
            public void updateAndClose(AjaxRequestTarget target) {
                update(target);
            }

            @Override
            public void updateTarget(AjaxRequestTarget target) {
                target.add(wmc);
            }
        };
        overviewPanel.setOutputMarkupId(true);
        historyPanel = new AdminIdeaHistoryPanel("panel", ideaModel);
        historyPanel.setOutputMarkupId(true);
        currentPanel = overviewPanel;
        currentPanel.setOutputMarkupId(true);
        add(currentPanel);
    }

    private void addOverviewLink() {
        overviewLink = new AjaxFallbackLink<Void>("overviewLink") {

            @Override
            protected void onConfigure() {
                super.onConfigure();
                setEnabled(!currentPanel.equals(overviewPanel));
            }

            @Override
            public void onClick(AjaxRequestTarget target) {
                switchPanel(target, overviewPanel, historyLink);
            }
        };
        overviewLink.setOutputMarkupId(true);
        add(overviewLink);
    }

    private void addHistoryLink() {
        historyLink = new AjaxFallbackLink<Void>("historyLink") {

            @Override
            protected void onConfigure() {
                super.onConfigure();
                setEnabled(!currentPanel.equals(historyPanel));
            }

            @Override
            public void onClick(AjaxRequestTarget target) {
                switchPanel(target, historyPanel, overviewLink);
            }
        };
        historyLink.setOutputMarkupId(true);
        add(historyLink);
    }

    private void switchPanel(AjaxRequestTarget target, Panel replacePanel, AjaxFallbackLink<Void> linkToUpdate) {
        currentPanel.replaceWith(replacePanel);
        currentPanel = replacePanel;
        target.add(currentPanel);
        target.add(linkToUpdate);
        target.add(this);
    }
}
