package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.datatables.AjaxCheckboxWrapper;
import se.su.dsv.scipro.datatables.ClickableIconColumn;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class AdminIdeaOverviewPanel extends AbstractAdminIdeaPanel {

    public static final String HEADER_MEETING = "First meeting";

    @Inject
    private IdeaService ideaService;

    public AdminIdeaOverviewPanel(String id) {
        super(id);
    }

    @Override
    protected List<IColumn<Idea, String>> addPanelSpecificColumns() {
        List<IColumn<Idea, String>> list = new ArrayList<>();
        list.add(new AbstractExportableColumn<Idea, String, String>(Model.of(HEADER_MEETING)) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> item, String componentId, IModel<Idea> ideaModel) {
                Idea idea = ideaModel.getObject();
                if (idea.getFirstMeeting() != null) {
                    item.add(new DateLabel(componentId, new PropertyModel<Date>(idea, "firstMeeting.firstMeetingDate")));
                } else {
                    item.add(new Label(componentId, "-"));
                }
            }

            @Override
            public IModel<String> getDataModel(IModel<Idea> rowModel) {
                Idea idea = rowModel.getObject();
                if (idea.getFirstMeeting() != null) {
                    return Model.of(idea.getFirstMeeting().getFirstMeetingDate().toString());
                } else {
                    return Model.of("-");
                }
            }
        });
        list.add(new AbstractExportableColumn<Idea, String, Boolean>(Model.of("Inactive")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, final IModel<Idea> rowModel) {

                cellItem.add(new AjaxCheckboxWrapper(componentId, new PropertyModel<Boolean>(rowModel, "inactive")) {
                    @Override
                    public void onChange(AjaxRequestTarget target, boolean selected) {
                        ideaService.inactivateIdea(rowModel.getObject(), selected, SciProSession.get().getUser());
                        success(getString("inactive", rowModel));
                        target.add(feedback);
                        target.add(dataTable);
                    }
                });
            }

            @Override
            public IModel<Boolean> getDataModel(IModel<Idea> rowModel) {
                return Model.of(!rowModel.getObject().isInactive());
            }
        });
        list.add(new ClickableIconColumn<Idea>(Model.of("Delete"), null, ClickableIconColumn.DELETE, "Are you sure you want to delete this idea?") {
            @Override
            public boolean shouldBeVisible(IModel<Idea> rowModel) {
                return rowModel.getObject().getMatchStatus().equals(Idea.Status.UNMATCHED);
            }

            @Override
            protected void onClick(IModel<Idea> clicked, AjaxRequestTarget target) {
                ideaService.deleteUnmatchedIdea(clicked.getObject());
                info("Idea '" + clicked.getObject().getTitle() + "' deleted successfully");
                target.add(feedback);
                target.add(dataTable);
            }
        });
        return list;
    }
}
