package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.GenericDataProvider;
import se.su.dsv.scipro.datatables.AjaxCheckboxWrapper;
import se.su.dsv.scipro.datatables.ClickableIconColumn;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public class AdminKeywordCrudPanel extends Panel {

    private static final Logger LOGGER = LoggerFactory.getLogger(AdminKeywordCrudPanel.class);
    private final FeedbackPanel feedback;
    @Inject
    private KeywordService keywordService;

    private ModalWindowPlus modalWindowPlus;

    static final String DIALOG = "modalWindowPlus";
    static final String CREATE_NEW_LINK = "createNewLink";
    static final String DATA_TABLE = "dataTable";
    static final String DATE_CREATED = "Created";
    static final String ACTIVE = "Active";
    static final String KEYWORD = "Keyword";
    static final String DELETE = "Delete";
    static final String EDIT = "Edit";
    private ExportableDataPanel<Keyword, String> dataTable;

    public AdminKeywordCrudPanel(String id) {
        super(id);
        feedback = new FeedbackPanel("feedback");
        feedback.setOutputMarkupId(true);
        add(feedback);
        addDialog();
        addCreateNewLink();
        addDataTable();
    }

    private void addDialog() {
        modalWindowPlus = new ModalWindowPlus(DIALOG);
        add(modalWindowPlus);
    }

    private void addCreateNewLink() {
        AjaxLink<Void> createNewLink = new AjaxLink<Void>(CREATE_NEW_LINK) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Create new keyword");
                Keyword k = new Keyword();
                k.setKeyword("");
                modalWindowPlus.setContent(new EditKeywordPanel(modalWindowPlus.getContentId(), new DetachableServiceModel<>(keywordService, k)) {
                    @Override
                    public void onSave(AjaxRequestTarget target) {
                        target.add(dataTable);
                    }
                });
                modalWindowPlus.show(target);
            }
        };
        add(createNewLink);
    }

    private void addDataTable() {
        SortableDataProvider<Keyword, String> provider = new GenericDataProvider<>(keywordService);
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        dataTable = new ExportableDataPanel<>(DATA_TABLE, createColumns(), provider);
        dataTable.setOutputMarkupId(true);
        add(dataTable);
    }

    private List<IColumn<Keyword, String>> createColumns() {
        List<IColumn<Keyword, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Keyword>(Model.of(DATE_CREATED), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Keyword, String>(Model.of(KEYWORD), "keyword", "keyword"));
        columns.add(new AbstractExportableColumn<Keyword, String, Boolean>(Model.of(ACTIVE)) {
            @Override
            public void populateItem(Item<ICellPopulator<Keyword>> item, String id, final IModel<Keyword> model) {
                final Keyword keyword = model.getObject();
                final boolean state = !keyword.isDeleted();
                item.add(new AjaxCheckboxWrapper(id, Model.of(state)) {
                    @Override
                    public void onChange(AjaxRequestTarget target, boolean selected) {
                        model.getObject().setDeleted(state);
                        keywordService.save(model.getObject());
                        target.add(dataTable);
                    }
                });
            }
            @Override
            public IModel<Boolean> getDataModel(IModel<Keyword> rowModel) {
                return Model.of(!rowModel.getObject().isDeleted());
            }
        });
        columns.add(new ClickableIconColumn<Keyword>(Model.of(EDIT), null, ClickableIconColumn.EDIT) {
            @Override
            protected void onClick(IModel<Keyword> clicked, AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Edit keyword: " + clicked.getObject().getKeyword());
                modalWindowPlus.setContent(new EditKeywordPanel(modalWindowPlus.getContentId(), clicked) {
                    @Override
                    public void onSave(AjaxRequestTarget target) {
                        target.add(dataTable);
                    }
                });
                modalWindowPlus.show(target);
                target.add(dataTable);
            }
        });
        columns.add(new ClickableIconColumn<Keyword>(Model.of(DELETE), null, ClickableIconColumn.DELETE) {
            @Override
            protected void onClick(IModel<Keyword> clicked, AjaxRequestTarget target) {
                try {
                    keywordService.delete(clicked.getObject().getId());
                    info("Keyword deleted");
                } catch (Exception e) {
                    String message = "Could not delete keyword, it is being used";
                    error(message);
                    LOGGER.info(message, e);
                }
                target.add(feedback);
                target.add(dataTable);
            }
        });
        return columns;
    }
}
