package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.components.AjaxIconLinkPanel;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.GenericDataProvider;
import se.su.dsv.scipro.datatables.ClickableIconColumn;
import se.su.dsv.scipro.datatables.LabelAndLinkColumn;
import se.su.dsv.scipro.springdata.services.SupervisorService;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.ResearchAreaService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public class AdminResearchAreaCrudPanel extends Panel {

    public static final String DATA_TABLE = "dataTable";

    @Inject
    private ResearchAreaService researchAreaService;
    @Inject
    private KeywordService keywordService;
    @Inject
    private SupervisorService supervisorService;

    private ModalWindowPlus modalWindowPlus;
    private ExportableDataPanel<ResearchArea, String> dataTable;
    private FeedbackPanel feedbackPanel;

    public AdminResearchAreaCrudPanel(String id) {
        super(id);
        feedbackPanel = new FeedbackPanel("feedback");
        feedbackPanel.setOutputMarkupId(true);
        add(feedbackPanel);

        addDialog();
        addDataTable();
        addCreateNewLink();
    }

    private void addDialog() {
        modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);
    }

    private void addCreateNewLink() {
        AjaxLink<Void> createNewLink = new AjaxLink<Void>("createNewLink") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Create new research area");
                ResearchArea researchArea = new ResearchArea();
                modalWindowPlus.setContent(new EditResearchAreaPanel(modalWindowPlus.getContentId(), new DetachableServiceModel<>(researchAreaService, researchArea)) {
                    @Override
                    public void onSave(AjaxRequestTarget target) {
                        target.add(dataTable);
                    }
                });
                modalWindowPlus.show(target);
            }
        };
        add(createNewLink);
    }

    private void addDataTable() {
        SortableDataProvider<ResearchArea, String> provider = new GenericDataProvider<>(researchAreaService);
        provider.setSort("title", SortOrder.DESCENDING);
        dataTable = new ExportableDataPanel<>(DATA_TABLE, createColumns(), provider);
        dataTable.setOutputMarkupId(true);
        add(dataTable);
    }

    private List<IColumn<ResearchArea, String>> createColumns() {
        List<IColumn<ResearchArea, String>> columns = new ArrayList<>();
        columns.add(getResearchArea());
        columns.add(getExternalIdentifier());
        columns.add(getSupervisorRelationships());
        columns.add(getKeywordRelationships());
        columns.add(getEditKeywords());
        columns.add(getEdit());
        columns.add(getDelete());
        return columns;
    }

    private PropertyColumn<ResearchArea, String> getResearchArea() {
        return new PropertyColumn<>(Model.of("Research area"), "title", "title");
    }

    private AbstractColumn<ResearchArea, String> getExternalIdentifier() {
        return new AbstractColumn<ResearchArea, String>(Model.of("Has external system identifier")) {
            @Override
            public void populateItem(Item<ICellPopulator<ResearchArea>> cellItem, String componentId, IModel<ResearchArea> rowModel) {
                String hasExternalIdentifier = (rowModel.getObject().getIdentifier() == null) ? "No" : "Yes";
                cellItem.add(new Label(componentId, Model.of(hasExternalIdentifier)));
            }
        };
    }

    private AbstractExportableColumn<ResearchArea, String, Long> getSupervisorRelationships() {
        return new AbstractExportableColumn<ResearchArea, String, Long>(Model.of("Supervisor relationships")) {
            @Override
            public void populateItem(Item<ICellPopulator<ResearchArea>> item, String id, IModel<ResearchArea> model) {
                final Long supervisorCount = supervisorService.countByResearchArea(model.getObject());
                item.add(new LabelAndLinkColumn<ResearchArea>(id, model, "Details") {

                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<ResearchArea> model) {
                        modalWindowPlus.setContent(new AdminResearchAreaSupervisorPanel(modalWindowPlus.getContentId(), model));
                        modalWindowPlus.setTitle("Research area " + model.getObject().getTitle() + " supervisors relationships");
                        modalWindowPlus.show(target);
                    }

                    @Override
                    public String labelString() {
                        return supervisorCount + " ";
                    }
                });
            }

            @Override
            public IModel<Long> getDataModel(IModel<ResearchArea> rowModel) {
                return Model.of(supervisorService.countByResearchArea(rowModel.getObject()));
            }
        };
    }

    private AbstractExportableColumn<ResearchArea, String, Long> getKeywordRelationships() {
        return new AbstractExportableColumn<ResearchArea, String, Long>(Model.of("Keyword relationships")) {
            @Override
            public void populateItem(Item<ICellPopulator<ResearchArea>> item, String id, IModel<ResearchArea> model) {
                final Long keywordCount = keywordService.countByResearchArea(model.getObject());
                item.add(new LabelAndLinkColumn<ResearchArea>(id, model, "Details") {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<ResearchArea> model) {
                        modalWindowPlus.setContent(new AdminResearchAreaKeywordPanel(modalWindowPlus.getContentId(), model));
                        modalWindowPlus.setTitle("Research area " + model.getObject().getTitle() + " keyword relationships");
                        modalWindowPlus.show(target);
                    }

                    @Override
                    public String labelString() {
                        return keywordCount + " ";
                    }
                });
            }

            @Override
            public IModel<Long> getDataModel(IModel<ResearchArea> rowModel) {
                return Model.of(keywordService.countByResearchArea(rowModel.getObject()));
            }
        };
    }

    private ClickableIconColumn<ResearchArea> getEditKeywords() {
        return new ClickableIconColumn<ResearchArea>(Model.of("Edit keyword relationships"), null, ClickableIconColumn.EDIT) {

            @Override
            protected void onClick(IModel<ResearchArea> clicked, AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Edit keyword relationships: " + clicked.getObject().getTitle());
                modalWindowPlus.setContent(new AdminResearchAreaEditPanel(modalWindowPlus.getContentId(), clicked) {
                    @Override
                    protected void updateTarget(AjaxRequestTarget target) {
                        target.add(dataTable);
                    }
                });
                modalWindowPlus.show(target);
            }
        };
    }

    private ClickableIconColumn<ResearchArea> getEdit() {
        return new ClickableIconColumn<ResearchArea>(Model.of("Edit research area"), null, ClickableIconColumn.EDIT) {
            @Override
            protected void onClick(IModel<ResearchArea> clicked, AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Edit research area: " + clicked.getObject().getTitle());
                modalWindowPlus.setContent(new EditResearchAreaPanel(modalWindowPlus.getContentId(), clicked) {
                    @Override
                    public void onSave(AjaxRequestTarget target) {
                        target.add(dataTable);
                    }
                });
                modalWindowPlus.show(target);
            }
        };
    }

    private AbstractColumn<ResearchArea, String> getDelete() {
        return new AbstractColumn<ResearchArea, String>(Model.of("Delete")) {
            @Override
            public void populateItem(Item<ICellPopulator<ResearchArea>> cellItem, final String componentId, final IModel<ResearchArea> rowModel) {
                cellItem.add(new AjaxIconLinkPanel<ResearchArea>(componentId, rowModel, Model.of(AjaxIconLinkPanel.DELETE), "Are you sure?") {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<ResearchArea> model) {
                        researchAreaService.delete(model.getObject());
                        info("Deleted research area");
                        target.add(feedbackPanel);
                        target.add(dataTable);
                    }

                    @Override
                    protected void onConfigure() {
                        setVisibilityAllowed(keywordService.countByResearchArea(rowModel.getObject())<1);
                    }
                });
            }
        };
    }
}
