package se.su.dsv.scipro.match;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.springdata.services.SupervisorService;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class AdminResearchAreaSupervisorPanel extends Panel {

    public static final String DATA_TABLE = "dataTable";
    @Inject
    private SupervisorService supervisorService;

    public AdminResearchAreaSupervisorPanel(String id, IModel<ResearchArea> areaModel) {
        super(id);
        addTable(areaModel);
    }

    private void addTable(final IModel<ResearchArea> areaModel) {
        SortableDataProvider<User, String> provider = new SortableDataProvider<User, String>() {
            @Override
            public Iterator<? extends User> iterator(long first, long count) {
                return supervisorService.findByResearchArea(areaModel.getObject(), new PageAdapter(first, count, getSort())).iterator();
            }

            @Override
            public long size() {
                return supervisorService.countByResearchArea(areaModel.getObject());
            }

            @Override
            public IModel<User> model(User object) {
                return new DetachableServiceModel<>(supervisorService, object);
            }
        };
        provider.setSort("firstName", SortOrder.DESCENDING);
        add(new ExportableDataPanel<>(DATA_TABLE, createColumns(), provider));
    }

    private List<IColumn<User, String>> createColumns() {
        List<IColumn<User, String>> columns = new ArrayList<>();
        columns.add(new PropertyColumn<User, String>(Model.of("First name"), "firstName", "firstName"));
        columns.add(new PropertyColumn<User, String>(Model.of("Last name"), "lastName", "lastName"));
        columns.add(new PropertyColumn<User, String>(Model.of("E-mail address"), "emailAddress", "emailAddress"));
        return columns;
    }
}
