package se.su.dsv.scipro.match;

import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.IChoiceRenderer;
import org.apache.wicket.markup.html.form.StatelessForm;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.Program;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;
import java.util.Set;

public class CompleteIdeaDialogPanel extends Panel {

    @Inject
    private IdeaService ideaService;
    @Inject
    private UserService userService;

    private DropDownChoice<Program> programDropDownChoice;
    private IModel<Idea> model;

    public CompleteIdeaDialogPanel(String id, IModel<Idea> model) {
        super(id, model);
        this.model = model;

        add(new FeedbackPanel("feedback"));
        add(new IdeaAuthorInfoPanel("authors", model));

        add(new IdeaDetailsPanel("details", model, model.getObject().getType().equals(Type.SUPERVISOR)));
        add(new IdeaWatsonCompletionPanel("watsonCompletion", model));

        addProgramSelection();
    }


    private void addProgramSelection() {
        final LoadableDetachableModel<User> userModel = getUserModel();

        Form<Void> form = new StatelessForm<Void>("form") {
            @Override
            protected void onSubmit() {
                ideaService.savePartnerProgram(model.getObject(), getCurrentUser(), programDropDownChoice.getModelObject());
                info("Saved program");
            }

            @Override
            protected void onConfigure() {
                setVisibilityAllowed(userModel.getObject().hasPrograms());
            }
        };
        add(form);

        programDropDownChoice = new DropDownChoice<>("programDropDown", new Model<Program>(), new ListAdapterModel<>(new PropertyModel<Set<Program>>(userModel, "programs")), new IChoiceRenderer<Program>() {
            @Override
            public Object getDisplayValue(Program object) {
                return object.getName();
            }

            @Override
            public String getIdValue(Program object, int index) {
                return Long.toString(object.getId());
            }
        });
        programDropDownChoice.setNullValid(true);
        form.add(programDropDownChoice);
    }

    private LoadableDetachableModel<User> getUserModel() {
        return new LoadableDetachableModel<User>() {
            @Override
            protected User load() {
                return userService.findOne(getCurrentUser().getId());
            }
        };
    }

    private User getCurrentUser() {
        return SciProSession.get().getUser();
    }
}
