package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.feedback.IFeedbackMessageFilter;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.util.ListModel;
import se.su.dsv.scipro.components.AjaxDropDownChoice;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.ResearchAreaService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

public abstract class EditKeywordPanel extends Panel {

    @Inject
    private KeywordService keywordService;
    @Inject
    private ResearchAreaService researchAreaService;

    public EditKeywordPanel(String id, final IModel<Keyword> model) {
        super(id, model);
        add(new KeywordForm("form", model));
    }

    public abstract void onSave(AjaxRequestTarget target);

    private class KeywordForm extends Form<Keyword> {
        private final AjaxDropDownChoice<ResearchArea> addResearchAreas;
        private final ListView<ResearchArea> researchAreas;
        private final List<ResearchArea> currentAreas;

        public KeywordForm(String id, IModel<Keyword> model) {
            super(id, model);

            currentAreas = new ArrayList<>(getModelObject().getResearchAreas());

            final WebMarkupContainer wmc = new WebMarkupContainer("wmc");
            wmc.setOutputMarkupId(true);

            RequiredTextField<String> keyword = new RequiredTextField<>("keyword", new PropertyModel<String>(model, "keyword"));
            wmc.add(keyword);
            wmc.add(new ComponentFeedbackPanel("keywordFeedback", keyword));


            final WebMarkupContainer raWMC = new WebMarkupContainer("raWMC");
            raWMC.setOutputMarkupId(true);
            wmc.add(raWMC);

            addResearchAreas = new AjaxDropDownChoice<ResearchArea>("addResearchAreas", new Model<ResearchArea>(null), getSelectableResearchAreas(currentAreas), new ChoiceRenderer<ResearchArea>("title", "id")) {
                @Override
                public void onNewSelection(AjaxRequestTarget target, ResearchArea objectSelected) {
                    if (objectSelected!= null && !currentAreas.contains(objectSelected)){
                        currentAreas.add(objectSelected);
                        researchAreas.setList(currentAreas);
                        addResearchAreas.setChoices(getSelectableResearchAreas(currentAreas));
                        target.add(raWMC);
                    }
                }
            };
            addResearchAreas.setNullValid(true);
            raWMC.add(addResearchAreas);

            researchAreas = new ListView<ResearchArea>("researchAreas", new ArrayList<>(currentAreas)) {
                @Override
                protected void populateItem(final ListItem<ResearchArea> item) {
                    item.add(new Label("title", new PropertyModel(item.getModel(), "title")));
                    item.add(new AjaxLink<ResearchArea>("remove") {
                        @Override
                        public void onClick(AjaxRequestTarget target) {
                            currentAreas.remove(item.getModelObject());
                            researchAreas.setList(currentAreas);
                            addResearchAreas.setChoices(getSelectableResearchAreas(currentAreas));
                            target.add(raWMC);
                        }
                    });
                }
            };
            raWMC.add(researchAreas);

            raWMC.add(new Label("noAreas", "No research areas"){
                @Override
                protected void onConfigure() {
                    super.onConfigure();
                    setVisibilityAllowed(currentAreas.isEmpty());
                }
            });

            FeedbackPanel feedback = new FeedbackPanel("feedback", new IFeedbackMessageFilter() {
                @Override
                public boolean accept(FeedbackMessage message) {
                    return !message.isRendered();
                }
            });
            feedback.setOutputMarkupId(true);
            wmc.add(feedback);

            add(wmc);

            add(new AjaxButton("save") {
                @Override
                protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                    super.onSubmit(target, form);

                    Keyword keyword = KeywordForm.this.getModelObject();
                    keyword.setResearchAreas(new HashSet<>(currentAreas));
                    KeywordForm.this.setModelObject(keywordService.save(keyword));

                    info("Saved");
                    target.add(wmc);
                    onSave(target);
                }

                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    super.onError(target, form);
                    target.add(wmc);
                }
            });
        }

        private ListModel<ResearchArea> getSelectableResearchAreas(List<ResearchArea> currentAreas){
            List<ResearchArea> notDeleted = researchAreaService.findNotDeleted();
            notDeleted.removeAll(currentAreas);
            notDeleted.remove(null);
            return new ListModel<>(notDeleted);
        }
    }
}