package se.su.dsv.scipro.match;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.applicationperiod.ApplicationPeriodInfoPanel;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.datatables.ClickableIconColumn;
import se.su.dsv.scipro.date.DeactivatableDatePickerPanel;
import se.su.dsv.scipro.firstmeeting.FirstMeetingColumnPanel;
import se.su.dsv.scipro.firstmeeting.FirstMeetingPanel;
import se.su.dsv.scipro.match.Idea.Status;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.profile.StudentProfileProgramPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static java.util.Arrays.asList;

public class SupervisorMyIdeasPanel extends Panel {

    @Inject
    private IdeaService ideaService;

    private final IdeaService.Filter params = new IdeaService.Filter();
    private ExportableDataPanel<Idea, String> dataTable;
    private ModalWindowPlus modalWindowPlus;
    private final User supervisor;
    private IModel<Idea> idea;

    public SupervisorMyIdeasPanel(String id, User user, IModel<Idea> idea) {
        super(id, idea);
        this.idea = idea;
        supervisor = user;
        setup();
    }

    public SupervisorMyIdeasPanel(String id, User user) {
        this(id, user, new Model<Idea>());
    }

    private void setup() {
        add(new BookmarkablePageLink<>("create", SupervisorIdeaSubmissionPage.class));
        addDialog();
        addFiltering();
        addInfoPanel();
        addDataTable();
        addHiddenDialogOpener();
    }


    private void addDialog() {
        modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);
    }

    private void addHiddenDialogOpener() {
        add(new AjaxLink("openPopup") {
            @Override
            public void onClick(AjaxRequestTarget ajaxRequestTarget) {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(Idea.class), idea.getObject().getId());
                setResponsePage(SupervisorIdeaDetailsPage.class, pp);
            }
        });
    }

    @Override
    public void renderHead(IHeaderResponse response) {
        String jQueryString = "";
        if (idea.getObject() != null) {
            jQueryString += "jQuery('#openPopup').trigger('click');";
        }
        response.render(JavaScriptHeaderItem.forScript("window.onload = function () {"+jQueryString+"}", "1"));
    }

    private void addFiltering() {
        params.setSupervisor(supervisor);
        ideaTypeFilter();
        ideaStatusFilter();
        dateFilter();
    }

    private void dateFilter() {
        add(new DeactivatableDatePickerPanel("createdBetween",
                new PropertyModel<Date>(params, "createdAfter"),
                new PropertyModel<Date>(params, "createdBefore")) {
            @Override
            protected void datesChanged(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        });
    }

    private void ideaStatusFilter() {
        add(new AjaxCheckBoxMultipleChoice<Status>("statusFilter", getStatusModel(),
                getStatuses(), new EnumChoiceRenderer<Status>(this)) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        });
    }

    private void ideaTypeFilter() {
        add(new AjaxCheckBoxMultipleChoice<Type>("typeFilter", getIdeaTypeModel(),
                getIdeaTypes(), new EnumChoiceRenderer<Type>(this)) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        });
    }

    private void addInfoPanel() {
        add(new ApplicationPeriodInfoPanel("applicationPeriodInfo", true));
    }

    private void addDataTable() {
        List<IColumn<Idea, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Idea>(Model.of("Created"), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Idea, String>(Model.of("Type"), "projectType.name", "projectType.name"));
        columns.add(newTitleColumn());
        columns.add(newAuthorsColumn());
        columns.add(new AbstractExportableColumn<Idea, String, String>(Model.of("Student selection")) {
            @Override
            public IModel<String> getDataModel(final IModel<Idea> rowModel) {
                return new EnumModel<>(SupervisorMyIdeasPanel.this, new LoadableDetachableModel<SelectionState>() {
                    @Override
                    protected SelectionState load() {
                        return ideaService.getSelectionState(rowModel.getObject());
                    }
                });
            }
        });
        columns.add(newStatusColumn());
        columns.add(newFirstMeetingColumn());
        columns.add(newEditColumn());

        columns.add(new AbstractColumn<Idea, String>(Model.of("Copy")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, IModel<Idea> rowModel) {
                cellItem.add(new AjaxLinkPanel<Idea>(componentId, rowModel, Model.of("Copy")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Idea> model) {
                        User currentSupervisor = SciProSession.get().getUser();
                        Idea copy = model.getObject().copy(currentSupervisor);
                        copy = ideaService.save(copy);
                        PageParameters pp = new PageParameters();
                        pp.add(PageParameterKeys.MAP.get(Idea.class), copy.getId());
                        setResponsePage(SupervisorIdeaSubmissionPage.class, pp);
                    }
                });
            }
        });

        columns.add(newDeleteColumn());
        FilteredDataProvider<Idea, IdeaService.Filter> provider = new FilteredDataProvider<>(ideaService, Model.of(params));
        provider.setSort("dateCreated", SortOrder.ASCENDING);
        dataTable = new ExportableDataPanel<>("dataTable", columns, provider);
        dataTable.setOutputMarkupId(true);
        add(dataTable);
    }

    private IColumn<Idea, String> newAuthorsColumn() {
        return new AbstractColumn<Idea, String>(Model.of("Authors")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, IModel<Idea> rowModel) {
                cellItem.add(new DisplayMultiplesPanel<IdeaParticipation>(componentId, new ListAdapterModel<>(new PropertyModel<Iterable<IdeaParticipation>>(rowModel, "ideaParticipations"))) {
                    @Override
                    public Component getComponent(String componentId, IModel<IdeaParticipation> t) {
                        return new StudentProfileProgramPanel(componentId, t, false);
                    }
                });
            }
        };
    }

    private ClickableIconColumn<Idea> newEditColumn() {
        return new ClickableIconColumn<Idea>(Model.of("Edit"), null, ClickableIconColumn.EDIT) {

            @Override
            protected void onClick(IModel<Idea> model, AjaxRequestTarget target) {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(Idea.class), model.getObject().getId());
                setResponsePage(SupervisorIdeaSubmissionPage.class, pp);
            }

            @Override
            public boolean shouldBeVisible(IModel<Idea> model) {
                return model.getObject().getMatchStatus() == Status.UNMATCHED;
            }
        };
    }

    private ClickableIconColumn<Idea> newDeleteColumn() {
        return new ClickableIconColumn<Idea>(Model.of("Delete"), null, ClickableIconColumn.DELETE, "Are you sure you want to delete this idea?") {

            @Override
            protected void onClick(IModel<Idea> model, AjaxRequestTarget target) {
                ideaService.deleteUnmatchedIdea(model.getObject());
                target.add(dataTable);
            }

            @Override
            public boolean shouldBeVisible(IModel<Idea> model) {
                return model.getObject().getMatchStatus() == Status.UNMATCHED;
            }
        };
    }

    private AbstractColumn<Idea, String> newFirstMeetingColumn() {
        return new AbstractColumn<Idea, String>(Model.of("First meeting")) {

            @Override
            public void populateItem(Item<ICellPopulator<Idea>> item, String id, final IModel<Idea> model) {
                item.add(new FirstMeetingColumnPanel(id, model) {

                    @Override
                    protected void onColumnClick(AjaxRequestTarget target, IModel<Idea> model) {
                        openFirstMeetingDialog(target, model);
                    }
                });
            }
        };
    }

    private AbstractColumn<Idea, String> newTitleColumn() {
        return new AbstractExportableColumn<Idea, String, String>(Model.of("Title"), "title") {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> components, String s, IModel<Idea> model) {
                components.add(new AjaxLinkPanel<Idea>(s, model, new PropertyModel(model, "title")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Idea> model) {
                        PageParameters pp = new PageParameters();
                        pp.add(PageParameterKeys.MAP.get(Idea.class), model.getObject().getId());
                        setResponsePage(SupervisorIdeaDetailsPage.class, pp);
                    }
                });
            }

            @Override
            public IModel<String> getDataModel(final IModel<Idea> rowModel) {
                return new PropertyModel<>(rowModel, "title");
            }
        };
    }

    private AbstractColumn<Idea, String> newStatusColumn() {
        return new AbstractColumn<Idea, String>(Model.of("Status"), "match.status") {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> item, String id, IModel<Idea> model) {
                item.add(new Label(id, ideaService.getStatusString(model.getObject())));
            }
        };
    }

    private void openFirstMeetingDialog(AjaxRequestTarget target, IModel<Idea> model) {
        modalWindowPlus.setContent(new FirstMeetingPanel(modalWindowPlus.getContentId(), model) {
            @Override
            public void onSaved(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        });
        modalWindowPlus.setTitle("Idea title: " + model.getObject().getTitle());
        modalWindowPlus.show(target);
    }

    private PropertyModel<List<Status>> getStatusModel() {
        params.setStatuses(new ArrayList<>(asList(Status.MATCHED, Status.UNMATCHED)));
        return new PropertyModel<>(params, "statuses");
    }

    private List<Status> getStatuses() {
        return asList(Status.MATCHED, Status.COMPLETED, Status.UNMATCHED, Status.INACTIVE);
    }

    private PropertyModel<List<Type>> getIdeaTypeModel() {
        params.setTypes(new ArrayList<>(getIdeaTypes()));
        return new PropertyModel<>(params, "types");
    }

    private List<Type> getIdeaTypes() {
        return asList(Type.SUPERVISOR, Type.STUDENT);
    }

}
