package se.su.dsv.scipro.milestones.panels;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.finalseminar.SupervisorFinalSeminarPage;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;

public class MileStoneCheckBoxPanel extends Panel {

    @Inject
    private MileStoneService mileStoneService;

    private final IModel<Project> projectModel;
    private final IModel<MilestoneActivityTemplate> activityModel;
    private final MilestoneActivityTemplate.Type activityType;

    public MileStoneCheckBoxPanel(String id, IModel<Project> projectModel, IModel<MilestoneActivityTemplate> activityModel, MilestoneActivityTemplate.Type type) {
        this(id, projectModel, activityModel, null, type);
    }

    public MileStoneCheckBoxPanel(String id, IModel<Project> projectModel, IModel<MilestoneActivityTemplate> activityModel, IModel<User> studentModel, MilestoneActivityTemplate.Type type) {
        super(id);
        this.projectModel = projectModel;
        this.activityModel = activityModel;
        this.activityType = type;

        add(newCheckBox(studentModel));
        add(newSeminarLink());
    }

    private AjaxCheckBox newCheckBox(final IModel<User> studentModel) {
        final boolean isProjectMileStone = studentModel == null;
        final boolean isConfirmed = isProjectMileStone ? isProjectMileStoneConfirmed(activityModel) : isStudentMileStoneConfirmed(activityModel, studentModel);

        return new AjaxCheckBox(CHECKBOX, Model.of(isConfirmed)) {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                if (isProjectMileStone) {
                    updateProjectMileStone(activityModel, isConfirmed);
                } else {
                    updateStudentMileStone(activityModel, studentModel, isConfirmed);
                }
                setResponsePage(getPage());
            }

            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(isCorrectActivityType(activityModel));
            }

            @Override
            protected void updateAjaxAttributes(AjaxRequestAttributes attributes) {
                super.updateAjaxAttributes(attributes);
                AjaxCallListener ajaxCallListener = new AjaxCallListener();
                ajaxCallListener.onPrecondition("if (!confirm('" + getConfirmationMessage(isConfirmed) + "')) {window.location.reload(); return false;}");
                attributes.getAjaxCallListeners().add(ajaxCallListener);
            }
        };
    }

    private BookmarkablePageLink<Void> newSeminarLink() {
        return new BookmarkablePageLink<Void>(CREATE_SEMINAR, SupervisorFinalSeminarPage.class) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(isCorrectActivityType(activityModel) &&
                        isSeminarActivity(activityModel) &&
                        !isProjectMileStoneConfirmed(activityModel));
            }
        };
    }

    private boolean isSeminarActivity(IModel<MilestoneActivityTemplate> activityModel) {
        MilestoneActivityTemplate activity = activityModel.getObject();
        return activity.isAutomatic() && MilestoneActivityTemplate.CREATE_SEMINAR.equals(activity.getCode());
    }

    private boolean isCorrectActivityType(IModel<MilestoneActivityTemplate> activityModel) {
        return activityType.equals(activityModel.getObject().getType());
    }

    private boolean isProjectMileStoneConfirmed(IModel<MilestoneActivityTemplate> activityModel) {
        return mileStoneService.isConfirmed(projectModel.getObject(), activityModel.getObject());
    }

    private boolean isStudentMileStoneConfirmed(IModel<MilestoneActivityTemplate> activityModel, IModel<User> studentModel) {
        return mileStoneService.isConfirmed(projectModel.getObject(), studentModel.getObject(), activityModel.getObject());
    }

    private void updateStudentMileStone(IModel<MilestoneActivityTemplate> activityModel, IModel<User> studentModel, boolean isConfirmed) {
        mileStoneService.setConfirmed(projectModel.getObject(), studentModel.getObject(), activityModel.getObject(), !isConfirmed);
    }

    private void updateProjectMileStone(IModel<MilestoneActivityTemplate> activityModel, boolean isConfirmed) {
        mileStoneService.setConfirmed(projectModel.getObject(), activityModel.getObject(), !isConfirmed);
    }

    private String getConfirmationMessage(boolean isConfirmed) {
        return isConfirmed ? getString("revoke") : getString("confirm");
    }

    static final String CHECKBOX = "checkBox";
    static final String CREATE_SEMINAR = "createSeminar";
}
