package se.su.dsv.scipro.milestones.panels;

import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.project.Project;

import javax.inject.Inject;
import java.util.List;

public class MileStoneProgressPanel extends Panel {

    @Inject
    private MilestoneActivityTemplateService activityService;

    private final IModel<Long> completed;
    private final MilestoneActivityTemplate.Type type;
    private final IModel<Project> projectIModel;

    public MileStoneProgressPanel(String id, IModel<Long> completed, MilestoneActivityTemplate.Type type, IModel<Project> projectIModel) {
        super(id);
        this.completed = completed;
        this.type = type;
        this.projectIModel = projectIModel;

        addProgressBar();
    }

    private void addProgressBar() {
        add(new ListView<MilestoneActivityTemplate>(LIST, getActivities()) {

            @Override
            protected void populateItem(ListItem<MilestoneActivityTemplate> item) {
                item.add(new Label(LIST_BOX, NON_BREAKABLE_SPACE).setEscapeModelStrings(false));
            }

            @Override
            protected ListItem<MilestoneActivityTemplate> newItem(int index, IModel<MilestoneActivityTemplate> itemModel) {
                return new ListItem<MilestoneActivityTemplate>(index, getListItemModel(getModel(), index)) {
                    @Override
                    protected void onComponentTag(final ComponentTag tag) {
                        tag.put("class", (getIndex() > completed.getObject() - 1) ? "progress-white" : "progress-green");
                        super.onComponentTag(tag);
                    }
                };
            }

        });

    }

    private LoadableDetachableModel<List<MilestoneActivityTemplate>> getActivities() {
        return new LoadableDetachableModel<List<MilestoneActivityTemplate>>() {
            @Override
            protected List<MilestoneActivityTemplate> load() {
                return activityService.getActivities(type, projectIModel.getObject().getProjectType(), projectIModel.getObject().getDateCreated());
            }
        };
    }

    static final String LIST = "list";
    static final String LIST_BOX = "box";
    static final String NON_BREAKABLE_SPACE = "&nbsp;";
}
