package se.su.dsv.scipro.nonworkdays;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.components.BootstrapDateField;
import se.su.dsv.scipro.components.BootstrapDatePicker;
import se.su.dsv.scipro.nonworkperiod.NonWorkDayPeriod;
import se.su.dsv.scipro.nonworkperiod.NonWorkDayPeriodService;

import javax.inject.Inject;
import java.util.Date;

public abstract class NonWorkDaysCRUDPanel extends Panel {

    public static final String FEEDBACK = "feedback";
    public static final String FORM = "form";
    public static final String START_DATE = "startDate";
    public static final String END_DATE = "endDate";
    public static final String COMMENT = "comment";
    public static final String SAVE = "save";
    private static final Logger LOGGER = LoggerFactory.getLogger(NonWorkDaysCRUDPanel.class);

    @Inject
    private NonWorkDayPeriodService nonWorkDayPeriodService;

    private FeedbackPanel feedback;

    public abstract void handleSave(AjaxRequestTarget target);

    public NonWorkDaysCRUDPanel(String id, final IModel<NonWorkDayPeriod> model) {
        super(id, model);

        feedback = new FeedbackPanel(FEEDBACK);
        feedback.setOutputMarkupId(true);
        add(feedback);

        Form form = new Form<>(FORM, model);
        form.add(new BootstrapDateField(START_DATE, new PropertyModel<Date>(model, "startDate"))
                .add(new BootstrapDatePicker()));
        form.add(new BootstrapDateField(END_DATE, new PropertyModel<Date>(model, "endDate"))
                .add(new BootstrapDatePicker()));
        form.add(new TextArea<>(COMMENT, new PropertyModel<String>(model, "comment")));

        form.add(new AjaxSubmitLink(SAVE) {
            @Override
            protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                target.add(feedback);
                if (model.getObject().getStartDate().after(model.getObject().getEndDate())) {
                    error("Start date has to be before end date");
                    return;
                }
                if (model.getObject().getComment() == null || model.getObject().getComment().length() < 1) {
                    error("Enter a comment");
                    return;
                }
                try {
                    model.setObject(nonWorkDayPeriodService.save(model.getObject()));
                    handleSave(target);
                } catch (RuntimeException rte) {
                    String message = "Non work day periods can not have overlapping start and end dates";
                    error(message);
                    LOGGER.info(message, rte);
                }
            }
        });
        add(form);
    }
}
