package se.su.dsv.scipro.notifications.panels;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormChoiceComponentUpdatingBehavior;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.Radio;
import org.apache.wicket.markup.html.form.RadioGroup;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.components.AjaxFeedbackPanelUpdater;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.springdata.services.UserProfileService;

import javax.inject.Inject;


public class NotificationSettingsPanel extends Panel {


    static final String FORM = "form";
    static final String MAIL_COMPILATION = "mailCompilation";
    static final String TRUE = "compile";
    static final String FALSE = "noCompile";

    @Inject
    UserProfileService profileService;

    private IModel<UserProfile> profile;

    public NotificationSettingsPanel(String id) {
        super(id);
        profile = new CompoundPropertyModel<>(getProfile());
        Form<UserProfile> form = new Form<>(FORM, profile);

        RadioGroup<Boolean> group = new RadioGroup<Boolean>(MAIL_COMPILATION) {
            @Override
            protected void onSelectionChanged(final Object newSelection) {
                updateProfile(null);
            }
        };

        group.add(new AjaxFormChoiceComponentUpdatingBehavior() {
            @Override
            protected void onUpdate(final AjaxRequestTarget target) {
                updateProfile(target);
            }
        });

        group.add(new Radio<>(TRUE, Model.of(true)));
        group.add(new Radio<>(FALSE, Model.of(false)));
        form.add(group);

        add(form);
    }

    private IModel<UserProfile> getProfile() {
        return new LoadableDetachableModel<UserProfile>() {
            @Override
            protected UserProfile load() {
                return profileService.findByUser(SciProSession.get().getUser());
            }
        };
    }

    private void updateProfile(final AjaxRequestTarget target) {
        profile.setObject(profileService.save(profile.getObject()));

        if (target != null) {
            info("Notification settings saved");
            target.addListener(new AjaxFeedbackPanelUpdater());
        }
    }
}
