package se.su.dsv.scipro.notifications.settings.panels;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.StringResourceModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.settings.service.ReceiverConfigurationService;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.List;

public abstract class AbstractReceiverConfigurationPanel extends Panel {

    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractReceiverConfigurationPanel.class);

    static final String EVENT_LIST = "events";
    static final String EVENT = "event";
    static final String CONFIGURATIONS = "configurations";
    static final String CONFIGURATION = "configuration";
    static final String MEMBER_LIST = "members";
    static final String MEMBER = "member";
    static final String DESCRIPTION = "description";

    @Inject ReceiverConfigurationService configurationService;

    private final Notification.Type group;

    private final List<? extends Enum> events;

    public AbstractReceiverConfigurationPanel(final String id, final Notification.Type group, final Enum[] events) {
        super(id);

        this.group = group;
        this.events = Arrays.asList(events);
    }

    protected boolean updateSetting(Enum event, Member.Type member, boolean setting) {
        LOGGER.info("{} is changing {}.{} for {} to {}", SciProSession.get().getUser(), group, event, member, setting);
        return configurationService.setReceiving(group, event, member, setting);
    }

    protected boolean getSetting(Enum event, Member.Type member) {
        return configurationService.isReceiving(group, event, member);
    }

    protected abstract List<Member.Type> getMemberTypes();

    @Override
    protected void onInitialize() {
        super.onInitialize();

        add(new ListView<Member.Type>(MEMBER_LIST, getMemberTypes()) {
            @Override
            protected void populateItem(ListItem<Member.Type> item) {
                item.add(new Label(MEMBER, new StringResourceModel("${name}", this, item.getModel())));
            }
        });

        add(new ListView<Enum>(EVENT_LIST, events) {
            @Override
            protected void populateItem(final ListItem<Enum> eventItem) {
                eventItem.add(new Label(EVENT, eventItem.getModelObject().name()));
                eventItem.add(new Label(DESCRIPTION, getString(eventItem.getModelObject().getClass().getEnclosingClass().getSimpleName() + "." + eventItem.getModelObject().name())));
                eventItem.add(new ConfigurationList(CONFIGURATIONS, getMemberTypes(), eventItem.getModelObject()));
            }
        });
    }

    class ConfigurationList extends ListView<Member.Type> {

        private final Enum event;

        public ConfigurationList(String id, List<? extends Member.Type> model, Enum event) {
            super(id, model);

            this.event = event;
        }

        @Override
        protected void populateItem(ListItem<Member.Type> item) {
            boolean setting = getSetting(event, item.getModelObject());

            item.add(new ReceiverSettingCheckbox(CONFIGURATION, Model.of(setting), event, item.getModelObject()));
        }


    }

    public class ReceiverSettingCheckbox extends AjaxCheckBox {
        private final Enum event;

        private final Member.Type member;

        public ReceiverSettingCheckbox(String id, IModel<Boolean> model, Enum event, Member.Type member) {
            super(id, model);

            this.event = event;
            this.member = member;

            setOutputMarkupId(true);
        }
        @Override
        protected void onUpdate(AjaxRequestTarget target) {
            boolean newSetting = updateSetting(event, member, getModelObject());
            setModelObject(newSetting);
            target.add(this);
        }

        @Override
        protected void updateAjaxAttributes(final AjaxRequestAttributes attributes) {
            super.updateAjaxAttributes(attributes);
            AjaxCallListener ajaxCallListener = new AjaxCallListener();
            ajaxCallListener.onPrecondition("if (!confirm('Are you sure?')) {window.location.reload(); return false;}");
            attributes.getAjaxCallListeners().add(ajaxCallListener);
        }
    }
}
