package se.su.dsv.scipro.peer;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.admin.pages.AbstractAdminSystemPage;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.ProjectTypeSettings;

import javax.inject.Inject;
import java.util.List;

public class AdminPeerSettingsPage extends AbstractAdminSystemPage {

    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private GeneralSystemSettingsService generalSystemSettingsService;

    public AdminPeerSettingsPage() {
        final Form<Void> peerSettingsForm = new Form<Void>("peerSettingsForm") {

            @Override
            public void onSubmit() {
                info(getString("saved"));
            }
        };

        peerSettingsForm.add(new ListView<ProjectType>("projectTypeList", getProjectTypes()) {

            @Override
            protected void populateItem(ListItem<ProjectType> item) {
                item.add(new Label("projectTypeName", item.getModelObject().getName()));
                item.add(new PeerProjectTypeSettingsForm("projectTypeSettingsForm", item.getModel()));
                add(item);
            }

        });

        PeerRatingsSettingsForm peerRatingsSettingsForm = new PeerRatingsSettingsForm("peerRatingsSettingsForm", getSettingsModel());
        peerSettingsForm.add(peerRatingsSettingsForm);

        peerSettingsForm.add(new Button("submit"));
        add(peerSettingsForm);
    }

    private LoadableDetachableModel<List<? extends ProjectType>> getProjectTypes() {
        return new LoadableDetachableModel<List<? extends ProjectType>>() {
            @Override
            protected List<? extends ProjectType> load() {
                return projectTypeService.findWithModule(ProjectModule.PEER);
            }
        };
    }

    private CompoundPropertyModel<GeneralSystemSettings> getSettingsModel() {
        return new CompoundPropertyModel<>(new LoadableDetachableModel<GeneralSystemSettings>() {
            @Override
            protected GeneralSystemSettings load() {
                return generalSystemSettingsService.getGeneralSystemSettingsInstance();
            }
        });
    }

    private class PeerRatingsSettingsForm extends Form<GeneralSystemSettings> {

        public PeerRatingsSettingsForm(String id, IModel<GeneralSystemSettings> model) {
            super(id, model);

            CheckBox peerDisplayLatestReviews = new CheckBox("peerDisplayLatestReviews");
            add(peerDisplayLatestReviews);

            TextField<Integer> numberOfLatestReviewsDisplayed =
                    new TextField<>("numberOfLatestReviewsDisplayed");
            numberOfLatestReviewsDisplayed.setRequired(true);
            add(numberOfLatestReviewsDisplayed);

            CheckBox publicReviewsActivated = new CheckBox("publicReviewsActivated");
            add(publicReviewsActivated);
            CheckBox peerDownloadEnabled = new CheckBox("peerDownloadEnabled");
            add(peerDownloadEnabled);
        }

        @Override
        public void onSubmit() {
            if (getModelObject().getNumberOfLatestReviewsDisplayed() == 0) {
                warn("Number of reviews to show has been changed to 1. If you don't want to show any reviews, please hide the panel.");
                getModelObject().setNumberOfLatestReviewsDisplayed(1);
            }
            generalSystemSettingsService.save(getModelObject());
        }
    }

    private class PeerProjectTypeSettingsForm extends Form<ProjectTypeSettings> {

        public PeerProjectTypeSettingsForm(final String id, IModel<ProjectType> projectType) {
            super(id, new CompoundPropertyModel<>(new PropertyModel<ProjectTypeSettings>(projectType, "projectTypeSettings")));

            TextField<Integer> numDaysBetweenPeerReviewsOnSameProject =
                    new TextField<>("numDaysBetweenPeerReviewsOnSameProject");
            numDaysBetweenPeerReviewsOnSameProject.setRequired(true);
            add(numDaysBetweenPeerReviewsOnSameProject);

            TextField<Integer> numDaysToSubmitPeerReview = new TextField<>("numDaysToSubmitPeerReview");
            numDaysToSubmitPeerReview.setRequired(true);
            add(numDaysToSubmitPeerReview);

            TextField<Integer> numDaysBeforePeerGetsCancelled = new TextField<>("numDaysBeforePeerGetsCancelled");
            numDaysBeforePeerGetsCancelled.setRequired(true);
            add(numDaysBeforePeerGetsCancelled);
        }

        @Override
        public void onSubmit() {
            projectTypeService.save(getModelObject().getProjectType());
        }
    }

}
