package se.su.dsv.scipro.peer;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class LatestReviewPanel extends Panel {

    public static final String NO_REVIEWERS = "noReviewers";
    public static final String LIST_VIEW = "listView";
    public static final String REVIEW_LINK = "reviewLink";
    @Inject
	private PeerReviewService peerReviewService;
    @Inject
    private GeneralSystemSettingsService settingsService;

    private final GeneralSystemSettings settings;

    @Override
    protected void onConfigure() {
        setVisibilityAllowed(settings.isPeerDisplayLatestReviews());
    }

    public LatestReviewPanel(String id) {
		super(id);
		setOutputMarkupId(true);

        settings = settingsService.getGeneralSystemSettingsInstance();

		add(new ListView<PeerReview>(LIST_VIEW, getReviews()) {
			@Override
			protected void populateItem(ListItem<PeerReview> item) {
                item.add(new Label("reviewer", new PropertyModel<>(item.getModel(), "reviewer.fullName")));
				PageParameters pp = new PageParameters();
				pp.add(PageParameterKeys.MAP.get(PeerReview.class), item.getModelObject().getId());
				BookmarkablePageLink<Void> link = new BookmarkablePageLink<>(REVIEW_LINK, ViewPeerReviewPage.class, pp);
				link.add(new DateLabel("reviewDate", new PropertyModel<Date>(item.getModel(), "lastModified"), DateStyle.DATETIME));
				link.setEnabled(settings.isPublicReviewsActivated());
				item.add(link);
				add(item);
			}
		});

		add(new Label(NO_REVIEWERS, new ResourceModel("noReviewers")) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getReviews().getObject().isEmpty());
            }
        });

	}

    private LoadableDetachableModel<List<PeerReview>> getReviews() {
        return new LoadableDetachableModel<List<PeerReview>>() {

			@Override
			protected List<PeerReview> load() {
				return peerReviewService.findAllCompleted(new PageRequest(0, settings.getNumberOfLatestReviewsDisplayed(), new Sort(Sort.Direction.DESC, "lastModified")));
			}
		};
    }

}
