package se.su.dsv.scipro.peer;

import com.google.inject.Inject;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.AjaxEnumLinkPanel;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.ArrayList;
import java.util.List;

public class PeerReviewStatusStatisticsPanel extends Panel {

    @Inject
    private PeerReviewService peerReviewService;

    private PeerReviewService.Filter filter;
    private boolean isSupervisorView;

    public PeerReviewStatusStatisticsPanel(String id, PeerReviewService.Filter filter, boolean isSupervisorView) {
        super(id);
        this.filter = filter;
        this.isSupervisorView = isSupervisorView;
        add(new PeerReviewFilterDetailsPanel("filterDetails", filter));

        setupDataTable();

    }

    private void setupDataTable() {
        SortableDataProvider<PeerReview, String> dataProvider = new FilteredDataProvider<>(peerReviewService, Model.of(filter), "dateCreated", SortOrder.DESCENDING);
        add(new ExportableDataPanel<>("dataTable", createColumns(), dataProvider));
    }

    private List<IColumn<PeerReview, String>> createColumns() {
        List<IColumn<PeerReview, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<PeerReview>(Model.of("Commenced"), "dateCreated", "dateCreated", DateStyle.DATETIME));

        columns.add(new AbstractColumn<PeerReview, String>(Model.of("Reviewing author"), "reviewer.lastName") {
            @Override
            public void populateItem(Item<ICellPopulator<PeerReview>> cellItem, String componentId, IModel<PeerReview> rowModel) {
                cellItem.add(new UserLinkPanel(componentId, new PropertyModel<User>(rowModel, "reviewer")));
            }
        });

        columns.add(new AbstractColumn<PeerReview, String>(Model.of("Requesting author"), "peerRequest.requester.lastName") {
            @Override
            public void populateItem(Item<ICellPopulator<PeerReview>> cellItem, String componentId, IModel<PeerReview> rowModel) {
                cellItem.add(new UserLinkPanel(componentId, new PropertyModel<User>(rowModel, "peerRequest.requester")));
            }
        });

        columns.add(new PropertyColumn<PeerReview, String>(Model.of("Title"), "peerRequest.project.title"));

        columns.add(new AbstractColumn<PeerReview, String>(Model.of("Status"), "status") {
            @Override
            public void populateItem(Item<ICellPopulator<PeerReview>> cellItem, String componentId, IModel<PeerReview> rowModel) {
                cellItem.add(new AjaxEnumLinkPanel<PeerReview, PeerReview.ReviewStatus>(componentId, rowModel, new PropertyModel<PeerReview.ReviewStatus>(rowModel, "status")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<PeerReview> model) {
                        PageParameters pp = new PageParameters();
                        pp.add(PageParameterKeys.MAP.get(PeerReview.class), getModelObject().getId());
                        if (isSupervisorView) {
                            setResponsePage(SupervisorViewPeerReviewAllPeersPage.class, pp);
                        } else {
                            setResponsePage(AdminPeerReviewPage.class, pp);
                        }
                    }

                    @Override
                    protected void onConfigure() {
                        super.onConfigure();
                        setEnabled(getModelObject().isCompleted());
                    }
                });
            }
        });
        if (filter.getStatuses().contains(PeerReview.ReviewStatus.COMPLETED)) {
            columns.add(new AbstractColumn<PeerReview, String>(Model.of("Number of answered checklists questions")) {
                @Override
                public void populateItem(Item<ICellPopulator<PeerReview>> cellItem, String componentId, IModel<PeerReview> rowModel){
                    cellItem.add(new Label(componentId, rowModel.getObject().getAnswers().size()));
                }
            });
        }
        return columns;
    }
}
