package se.su.dsv.scipro.peer

import javax.inject.Inject

import org.apache.wicket.RestartResponseException
import org.apache.wicket.authorization.UnauthorizedInstantiationException
import org.apache.wicket.model.PropertyModel
import org.apache.wicket.request.mapper.parameter.PageParameters
import se.su.dsv.scipro._
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyProjects
import se.su.dsv.scipro.data.DetachableServiceModel
import se.su.dsv.scipro.exceptions.PageNotFoundException
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService
import se.su.dsv.scipro.panels.CommentThreadPanel
import se.su.dsv.scipro.profile.UserLinkPanel
import se.su.dsv.scipro.session.SciProSession
import se.su.dsv.scipro.system.User
import se.su.dsv.scipro.util.PageParameterKeys

class ViewPeerReviewPage(pp: PageParameters) extends AbstractPeerPortalPage(pp) with MenuHighlightAuthorMyProjects {
  @Inject private var peerReviewService: PeerReviewService = _
  @Inject private var generalSystemSettingsService: GeneralSystemSettingsService = _

  val optReviewModel = for {
    reviewId ← pp.get(PageParameterKeys.MAP.get(classOf[PeerReview])).asLong
    review ← Option(peerReviewService.findOne(reviewId))
  } yield new DetachableServiceModel(peerReviewService, review)

  optReviewModel.fold(throw new PageNotFoundException) { reviewModel ⇒
    add(new PeerPortalHeader("header", projectModel) {
      protected override def submissionStartsCollapsed: Boolean = true
    })

    if (!checkAccess(SciProSession.get.getUser, reviewModel.getObject)) {
      throw new RestartResponseException(classOf[AccessDeniedPage])
    }

    add(new DisplayReviewPanel("review", reviewModel))
    add(new ReviewPageRequestDetailsPanel("request", new PropertyModel[PeerRequest](reviewModel, "peerRequest")))
    add(new UserLinkPanel("reviewedBy", new PropertyModel[User](reviewModel, "reviewer")))
    add(new CommentThreadPanel("comment", reviewModel))
  }

  private def checkAccess(user: User, peerReview: PeerReview): Boolean = {
    if (generalSystemSettingsService.getGeneralSystemSettingsInstance.isPublicReviewsActivated) {
      true
    }
    else {
      import scala.collection.JavaConversions._
      peerReview.getMembers.exists(_.getUser == user)
    }
  }
}
