package se.su.dsv.scipro.profile;

import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.WebComponent;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;

public class UserProfileImage extends WebComponent {
    private static final String YES = "";
    private static final String NO = "f";

    public enum Size {
        TINY(16), MEDIUM(64), LARGE(128);

        private final int pixels;

        Size(final int pixels) {
            this.pixels = pixels;
        }

        int getPixels() {
            return pixels;
        }
    }

    @Inject
    private UserProfileService userProfileService;

    private final IModel<User> user;
    private final Size size;

    public UserProfileImage(final String id, final IModel<User> user, final Size size) {
        super(id, user);
        this.user = user;
        this.size = size;
    }

    @Override
    protected void onComponentTag(final ComponentTag tag) {
        checkComponentTag(tag, "img");
        super.onComponentTag(tag);
        UserProfile profile = userProfileService.findByUser(user.getObject());
        CharSequence imageSrc = getImageSrc(profile);
        tag.put("src", imageSrc);
        tag.put("width", size.getPixels());
        tag.put("height", size.getPixels());
        tag.put("alt", user.getObject().getFullName());
    }

    private CharSequence getImageSrc(final UserProfile profile) {
        switch (profile.getPicture()) {
            case NONE:
                return gravatarUrl("", NO);
            case FILE:
                if (profile.getProfilePicture() != null) {
                    final PageParameters pp = new PageParameters();
                    pp.set("id", user.getObject().getId());
                    pp.set("h", Long.toHexString(profile.getProfilePicture().getId()));
                    return urlFor(new ProfilePictureResource(), pp);
                }
                else {
                    return gravatarUrl("", NO);
                }
            case GRAVATAR:
                final String md5 = SciProUtilities.md5(profile.getUser().getEmailAddress().trim().toLowerCase());
                return gravatarUrl(md5, showGravatar());
            default:
                throw new Error();
        }
    }

    private String gravatarUrl(final String md5, final String showGravatar) {
        return String.format("https://www.gravatar.com/avatar/%s?r=g&d=mm&s=%d&%s", md5, size.getPixels(), showGravatar);
    }

    private String showGravatar() {
        return userProfileService.findByUser(user.getObject()).getPicture() == UserProfile.Picture.GRAVATAR ? YES : NO;
    }
}
