package se.su.dsv.scipro.project.pages;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.activityplan.UpcomingActivitiesPanel;
import se.su.dsv.scipro.checklists.ChecklistOverviewPanel;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyProjects;
import se.su.dsv.scipro.finalseminar.OverviewSeminarPanel;
import se.su.dsv.scipro.finalthesis.FinalThesisPanel;
import se.su.dsv.scipro.group.AuthorGroupPage;
import se.su.dsv.scipro.group.GroupsOverviewPanel;
import se.su.dsv.scipro.latestevents.LatestEventsPanel;
import se.su.dsv.scipro.milestones.panels.MilestoneSummaryPanel;
import se.su.dsv.scipro.peer.PeerSummaryPanel;
import se.su.dsv.scipro.project.panels.ExternalResourcesPanel;
import se.su.dsv.scipro.projectstate.ProjectStatePanel;
import se.su.dsv.scipro.rwandagrading.DisplayGradePanel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.thesislink.ExternalLinkPanel;

public class ProjectDetailsPage extends AbstractProjectDetailsPage implements MenuHighlightAuthorMyProjects {

    public static final String FINAL_THESIS_PANEL = "finalThesisPanel";
    public static final String LATEST_EVENTS = "latestEvents";
    public static final String PEER_SUMMARY = "peerSummary";
    public static final String FINAL_SEMINAR_NEW = "finalSeminarNew";
    public static final String UPCOMING_ACTIVITIES = "upcomingActivities";
    public static final String MILESTONES = "milestones";
    public static final String PROJECT_STATE_PANEL = "projectStatePanel";
    public static final String EXTERNAL_ORGANIZATION = "externalOrganization";
    public static final String EXTERNAL_RESOURCES = "externalResources";
    public static final String EXTERNAL_LINK = "externalLink";

    public ProjectDetailsPage(PageParameters pp) {
        super(pp);

        add(new ExternalResourcesPanel(EXTERNAL_RESOURCES, new PropertyModel<ProjectType>(projectModel, "projectType")));

        add(new ExternalLinkPanel(EXTERNAL_LINK, projectModel));
        add(new PeerSummaryPanel(PEER_SUMMARY, projectModel));

        add(new LatestEventsPanel(LATEST_EVENTS, projectModel));
        add(new FinalThesisPanel(FINAL_THESIS_PANEL, projectModel));
        add(new ProjectStatePanel(PROJECT_STATE_PANEL, projectModel) {
            @Override
            public boolean isAuthorView() {
                return true;
            }
        });
        add(new OverviewSeminarPanel(FINAL_SEMINAR_NEW, projectModel));
        add(new Label(EXTERNAL_ORGANIZATION, new PropertyModel<>(projectModel, "externalOrganization.name")) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getDefaultModelObject() != null);
            }
        });

        add(new UpcomingActivitiesPanel(UPCOMING_ACTIVITIES, projectModel) {
            @Override
            public boolean isAuthorView() {
                return true;
            }
        });

        add(new MilestoneSummaryPanel(MILESTONES, projectModel));
        add(new GroupsOverviewPanel("groups", projectModel, new GroupsOverviewPanel.GroupLinkSupplier() {
            @Override
            public BookmarkablePageLink<Void> newGroupLink(String id, PageParameters pp) {
                return new BookmarkablePageLink<>(id, AuthorGroupPage.class, pp);
            }
        }));

        add(new DisplayGradePanel("rwandaGrade", projectModel));
        add(new ChecklistOverviewPanel("checklistOverview", projectModel));
    }

}