package se.su.dsv.scipro.project.pages;

import com.google.inject.Inject;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.ISortableDataProvider;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.basepages.PublicPage;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.DefaultDataPanel;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import static java.util.Arrays.asList;

public class PublicActiveThesisProjectListingPage extends PublicPage {
    @Inject
    ProjectService projectService;

    public PublicActiveThesisProjectListingPage() {
        add(new DefaultDataPanel<>("projects", createColumns(), dataProvider()));
    }

    private ISortableDataProvider<Project, String> dataProvider() {
        SortableDataProvider<Project, String> provider = new SortableDataProvider<Project, String>() {
            @Override
            public Iterator<? extends Project> iterator(long first, long count) {
                return projectService.findAll(filterForActive(), new PageAdapter(first, count, getSort())).iterator();
            }

            @Override
            public long size() {
                return projectService.count(filterForActive());
            }

            @Override
            public IModel<Project> model(Project project) {
                return new DetachableServiceModel<>(projectService, project);
            }
        };
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        return provider;
    }

    private List<? extends IColumn<Project, String>> createColumns() {
        List<IColumn<Project, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Project>(Model.of("Start date"), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Title"), "title"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Type"), "projectType.name"));
        columns.add(new AbstractColumn<Project, String>(Model.of("Authors")) {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, IModel<Project> rowModel) {
                boolean first = true;
                StringBuilder authors = new StringBuilder("");
                for (User user : rowModel.getObject().getProjectParticipants()) {
                    if (first) {
                        first = false;
                        authors.append(user.getFirstName()).append(" ").append(user.getLastName());
                    } else {
                        authors.append(", ").append(user.getFirstName()).append(" ").append(user.getLastName());
                    }
                }
                cellItem.add(new Label(componentId, authors));
            }
        });
        columns.add(new PropertyColumn<Project, String>(Model.of("Supervisor"), "headSupervisor.fullName"));
        return columns;
    }

    private ProjectService.Filter filterForActive() {
        ProjectService.Filter filter = new ProjectService.Filter();
        filter.setStatuses(new ArrayList<>(asList(ProjectStatus.ACTIVE)));
        return filter;
    }
}
