package se.su.dsv.scipro.project.panels;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.activityplan.ProjectActivityPlanPage;
import se.su.dsv.scipro.components.AbstractMenuPanel;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightActivityPlan;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorOpposition;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightForum;
import se.su.dsv.scipro.finalseminar.FinalSeminarService;
import se.su.dsv.scipro.finalseminar.ProjectFinalSeminarPage;
import se.su.dsv.scipro.finalseminar.ProjectOppositionPage;
import se.su.dsv.scipro.forum.pages.ProjectForumBasePage;
import se.su.dsv.scipro.milestones.pages.ProjectMileStonePage;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.peer.PeerPortalPage;
import se.su.dsv.scipro.phd.ProjectAdminPage;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.pages.AbstractProjectDetailsPage;
import se.su.dsv.scipro.project.pages.ProjectDetailsPage;
import se.su.dsv.scipro.project.pages.ProjectFilePage;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public class ProjectSubTabMenuPanel extends AbstractMenuPanel {

    public static final String FINAL_SEMINAR = "Final seminar";
    public static final String OPPOSITION_ACTIVE_PARTICIPATION = "Opposition & Active participation";
    public static final String PEER = "Peer";
    public static final String FORUM = "Forum";
    public static final String FILES = "Files";
    public static final String ACTIVITY_PLAN = "Activity plan";
    public static final String MILESTONES = "Milestones";
    public static final String OVERVIEW = "Overview";
    public static final String ADMIN = "PhD Admin";

    @Inject
    private FinalSeminarService finalSeminarService;
    @Inject
    private MilestoneActivityTemplateService milestoneActivityTemplateService;

    final PageParameters projectParams;
    final Project active;

    public ProjectSubTabMenuPanel(String id, Class<? extends Page> containerClass, PageParameters projectParams, Project active) {
        super(id, AbstractProjectDetailsPage.class, containerClass);
        this.projectParams = projectParams;
        this.active = active;
    }

    /**
     * Add items to the submenu here
     */

    @Override
    protected List<MenuItem> getItemList() {
        List<MenuItem> itemList = new ArrayList<>();
        itemList.add(new MenuItem(OVERVIEW, ProjectDetailsPage.class, projectParams));
        if (milestoneActivityTemplateService.countActivities(active.getProjectType(), active.getDateCreated()) != 0) {
            itemList.add(new MenuItem(MILESTONES, ProjectMileStonePage.class));
        }
        itemList.add(new MenuItem(ACTIVITY_PLAN, ProjectActivityPlanPage.class, MenuHighlightActivityPlan.class, projectParams));
        itemList.add(new MenuItem(FILES, ProjectFilePage.class, projectParams));
        itemList.add(new MenuItem(FORUM, ProjectForumBasePage.class, MenuHighlightForum.class, projectParams));
        itemList.add(new MenuItem(PEER, PeerPortalPage.class, projectParams));
        if (finalSeminarService.findByProject(active) != null) {
            itemList.add(new MenuItem(FINAL_SEMINAR, ProjectFinalSeminarPage.class, projectParams));
        }
        itemList.add(new MenuItem(OPPOSITION_ACTIVE_PARTICIPATION, ProjectOppositionPage.class, MenuHighlightAuthorOpposition.class, projectParams));
        itemList.add(new MenuItem(ADMIN, ProjectAdminPage.class, projectParams));
        return itemList;
    }

    @Override
    protected MenuType getMenuType() {
        return MenuType.TAB_MULTIPLE_ROWS;
    }

}
