package se.su.dsv.scipro.projectpartner;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.markup.html.navigation.paging.AjaxPagingNavigator;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.navigation.paging.PagingNavigator;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.model.*;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightProjectIdeas;
import se.su.dsv.scipro.data.FilteredListModel;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.match.AbstractIdeaProjectPage;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class ProjectPartnerPage extends AbstractIdeaProjectPage implements MenuHighlightProjectIdeas {

    public static final int ITEMS_PER_PAGE = 5;
    @Inject
    private ProjectPartnerService projectPartnerService;
    @Inject
    private GeneralSystemSettingsService generalSystemSettingsService;
    @Inject
    private ProjectTypeService projectTypeService;

    public ProjectPartnerPage(PageParameters pp) {
        super(pp);

        final GeneralSystemSettings settings = generalSystemSettingsService.getGeneralSystemSettingsInstance();
        add(new FeedbackPanel("feedback"));
        final IModel<? extends List<? extends ProjectType>> matchableTypes = getMatchableTypes();
        add(new ListView<ProjectType>("ads", matchableTypes) {
            @Override
            protected void populateItem(final ListItem<ProjectType> item) {
                WebMarkupContainer container = new WebMarkupContainer("listContainer");
                ProjectPartnerDataProvider projectPartnerDataProvider =
                        new ProjectPartnerDataProvider(projectPartnerService, settings.getProjectPartnerDaysToLive(), item.getModel());
                final DataView<ProjectPartner> dataView = new DataView<ProjectPartner>("list", projectPartnerDataProvider, ITEMS_PER_PAGE) {
                    @Override
                    protected void populateItem(Item<ProjectPartner> item) {
                        item.add(new Label("exname", new PropertyModel<String>(item.getModel(), "user.fullName")));
                        item.add(new Label("exmail", new PropertyModel<String>(item.getModel(), "user.emailAddress")));
                        item.add(new DateLabel("exdate", new PropertyModel<Date>(item.getModel(), "dateCreated")));
                        item.add(new MultiLineLabel("exinfoText", new PropertyModel<String>(item.getModel(), "infotext")));
                    }
                };
                container.add(dataView);
                container.setOutputMarkupId(true);
                PagingNavigator pagingNavigator = new AjaxPagingNavigator("navigator", dataView);
                container.add(pagingNavigator);
                item.add(container);
                final String href = item.getModelObject().getName().toLowerCase().replaceAll("[^a-z]", "");
                item.add(AttributeModifier.replace("id", href));
                if (item.getIndex() == 0) {
                    item.add(AttributeAppender.append("class", "active"));
                }
            }
        });

        add(new InfoPanel("infoPanel", new ResourceModel("info.text")));

        Form<ProjectPartner> form = new Form<ProjectPartner>("projectPartnerForm", new CompoundPropertyModel<>(new ProjectPartner(SciProSession.get().getUser()))){
            @Override
            protected void onSubmit() {
                getSession().info(getString("saved"));
                projectPartnerService.save(getModelObject());
                setResponsePage(ProjectPartnerPage.class);
            }
        };
        TextArea<String> infoText = new TextArea<>("infotext");
        infoText.setRequired(true);
        form.add(infoText);
        final RadioChoice<ProjectType> projectTypeChoice =
                new RadioChoice<>("projectType", matchableTypes, new ChoiceRenderer<>("name", "id"));
        projectTypeChoice.setRequired(true);
        projectTypeChoice.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER);
        form.add(projectTypeChoice);
        add(form);

        add(new ListView<ProjectType>("adChoice", matchableTypes) {
            @Override
            protected void populateItem(final ListItem<ProjectType> item) {
                final String href = "#" + item.getModelObject().getName().toLowerCase().replaceAll("[^a-z]", "");
                final ExternalLink tab = new ExternalLink("tabLink", Model.of(href), new PropertyModel<>(item.getModel(), "name"));
                if (item.getIndex() == 0) {
                    item.add(AttributeAppender.append("class", "active"));
                }
                item.add(tab);
            }
        });
    }

    private IModel<? extends List<? extends ProjectType>> getMatchableTypes() {
        final IModel<List<? extends ProjectType>> matchableTypes = new LoadableDetachableModel<List<? extends ProjectType>>() {
            @Override
            protected List<? extends ProjectType> load() {
                return projectTypeService.findWithModule(ProjectModule.MATCH);
            }
        };
        return new FilteredListModel<ProjectType>(matchableTypes) {
            @Override
            protected boolean accept(final ProjectType projectType) {
                return projectType.getMaxAuthors() > 1;
            }
        };
    }
}