package se.su.dsv.scipro.rwandagrading;

import org.apache.commons.lang3.StringUtils;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import scala.Option;
import scala.runtime.AbstractFunction0;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.NullableOptionModel;
import se.su.dsv.scipro.grading.Grade;
import se.su.dsv.scipro.grading.GradeService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;

import javax.inject.Inject;
import java.util.Date;

@ProjectModuleComponent(ProjectModule.FINAL_GRADING)
public class GradePanel extends GenericPanel<Project> {

    @Inject
    private GradeService gradeService;

    public GradePanel(String id, IModel<Project> model) {
        super(id, model);
        add(new GradeForm("form", model));
    }

    private class GradeForm extends Form<Project> {
        private final IModel<String> gradeValueModel = new Model<>();

        public GradeForm(String id, final IModel<Project> model) {
            super(id, model);
            final IModel<Option<Grade>> gradeModel = new LoadableDetachableModel<Option<Grade>>() {
                @Override
                protected Option<Grade> load() {
                    return gradeService.getGrade(model.getObject());
                }
            };
            final String currentGrade = gradeModel.getObject().map(new AbstractFunction1<Grade, String>() {
                @Override
                public String apply(final Grade v1) {
                    return v1.getValue();
                }
            }).getOrElse(new AbstractFunction0<String>() {
                @Override
                public String apply() {
                    return "";
                }
            });
            gradeValueModel.setObject(currentGrade);

            add(new TextField<>("grade", gradeValueModel));
            final FeedbackPanel feedbackPanel = new FencedFeedbackPanel("feedback", this);
            feedbackPanel.setMaxMessages(1);
            add(feedbackPanel);
            add(new DateLabel("reportedOn", new PropertyModel<Date>(new NullableOptionModel<>(gradeModel), "reportedOn")) {
                @Override
                protected void onConfigure() {
                    setVisible(getDefaultModelObject() != null);
                }
            });
        }

        @Override
        protected void onSubmit() {
            final String gradeValue = gradeValueModel.getObject();
            if (StringUtils.isEmpty(gradeValue)) {
                gradeService.removeGrade(getModelObject());
                success("Grade removed");
            }
            else {
                gradeService.reportGrade(getModelObject(), SciProSession.get().getUser(), new Date(), gradeValue);
                success("Grade saved");
            }
        }
    }
}
