package se.su.dsv.scipro.statistics;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.EmployeeAutoComplete;
import se.su.dsv.scipro.date.DeactivatableDatePickerPanel;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.milestones.service.MilestoneStatisticsService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import static java.util.Arrays.asList;

public class AdminMedianMilestonePage extends AbstractAdminStatisticsPage {

    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private MilestoneActivityTemplateService milestoneActivityTemplateService;
    @Inject
    private MilestoneStatisticsService milestoneStatisticsService;

    private MilestoneStatisticsService.Filter params;

    public AdminMedianMilestonePage() {

        params = new MilestoneStatisticsService.Filter();
        
        add(new FilterForm("selection"));

        WebMarkupContainer wmc = new WebMarkupContainer("wmc") {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(params.getFirstMilestone()!=null &&
                        params.getSecondMilestone()!=null &&
                        getLoadedCount().getObject() != null &&
                        getLoadedCount().getObject() > 0);
            }
        };
        add(wmc);

        wmc.add(new Label("median", getLoadedMedian()));
        wmc.add(new Label("count", getLoadedCount()));
        wmc.add(new Label("notCompletedBoth", getLoadedNotCompletedCount()));

    }

    private LoadableDetachableModel<Long> getLoadedNotCompletedCount(){
        return new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return milestoneStatisticsService.countNotCompletedBoth(params);
            }
        };
    }

    private LoadableDetachableModel<Long> getLoadedMedian(){
        return new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return milestoneStatisticsService.median(params);
            }
        };
    }

    private LoadableDetachableModel<Long> getLoadedCount(){
        return new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return milestoneStatisticsService.countCompletedBoth(params);
            }
        };
    }

    private class FilterForm extends Form {

        public FilterForm(String id) {
            super(id);

            add(new EmployeeAutoComplete("supervisorFilter") {
                @Override
                protected void action(AjaxRequestTarget pTarget, User newSelection) {
                    params.setHeadSupervisor(newSelection);
                }
            });

            add(new DeactivatableDatePickerPanel("dates",
                    new PropertyModel<Date>(params, "createdAfter"),
                    new PropertyModel<Date>(params, "createdBefore")));

            params.setStatuses(asList(ProjectStatus.values()));
            CheckBoxMultipleChoice<ProjectStatus> projectStatus = new CheckBoxMultipleChoice<>("statusFilter",
                    new PropertyModel<List<ProjectStatus>>(params, "statuses"),
                    asList(ProjectStatus.values()),
                    new EnumChoiceRenderer<ProjectStatus>(this)
                    );
            projectStatus.setRequired(true);
            projectStatus.setLabelPosition(AbstractChoice.LabelPosition.WRAP_BEFORE);
            add(projectStatus);

            params.setProjectTypes(projectTypeService.findByDegreeLevels(Arrays.asList(DegreeLevel.FIRST_CYCLE, DegreeLevel.SECOND_CYCLE)));
            CheckBoxMultipleChoice<ProjectType> projectType = new CheckBoxMultipleChoice<>("projectTypeFilter",
                    new PropertyModel<List<ProjectType>>(params, "projectTypes"),
                    projectTypeService.findAllActive(),
                    new ChoiceRenderer<ProjectType>("name", "id")
            );
            projectType.setRequired(true);
            projectType.setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER);
            add(projectType);

            IModel<? extends List<? extends MilestoneActivityTemplate>> templates = getTemplates();

            DropDownChoice<MilestoneActivityTemplate> first = new DropDownChoice<>("first",
                    new PropertyModel<MilestoneActivityTemplate>(params, "firstMilestone"),
                    templates,
                    new MilestoneActivityTemplateChoiceRenderer());
            first.setRequired(true);
            add(first);

            DropDownChoice<MilestoneActivityTemplate> second = new DropDownChoice<>("second",
                    new PropertyModel<MilestoneActivityTemplate>(params, "secondMilestone"),
                    templates,
                    new MilestoneActivityTemplateChoiceRenderer());
            second.setRequired(true);
            add(second);

        }
    }

    private IModel<? extends List<? extends MilestoneActivityTemplate>> getTemplates() {
        return new LoadableDetachableModel<List<? extends MilestoneActivityTemplate>>() {
            @Override
            protected List<? extends MilestoneActivityTemplate> load() {
                List<MilestoneActivityTemplate> all = milestoneActivityTemplateService.findAll();
                Collections.sort(all, new MilestoneActivityTemplate.BySortOrderComparator());
                return all;
            }
        };
    }

    private static class MilestoneActivityTemplateChoiceRenderer extends ChoiceRenderer<MilestoneActivityTemplate> {
        public MilestoneActivityTemplateChoiceRenderer() {
            super("title", "id");
        }

        @Override
        public Object getDisplayValue(final MilestoneActivityTemplate object) {
            return String.format("[%s] %s", object.getMilestonePhaseTemplate().getTitle(), object.getTitle());
        }
    }

}