package se.su.dsv.scipro.statistics;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.EnumLabel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.joda.time.DateTime;
import org.joda.time.LocalDate;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.date.DatePickerPanel;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.reviewing.*;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class AdminReviewerDeadlinesPage extends AbstractAdminStatisticsPage {
    private DatePickerPanel datePicker;
    @Inject
    private ReviewerDeadlineFollowupService reviewerDeadlineFollowupService;
    @Inject
    private DaysService daysService;

    private ReviewerApproval.Step step = ReviewerApproval.Step.ROUGH_DRAFT_APPROVAL;

    private WebMarkupContainer wmc;

    public AdminReviewerDeadlinesPage() {

        addTabs();

        addDatePickerForm();

        addLateApprovalsList();
    }

    private void addTabs() {
        final WebMarkupContainer rdContainer = new WebMarkupContainer("roughDraftLinkContainer");
        rdContainer.setOutputMarkupId(true);
        add(rdContainer);
        final WebMarkupContainer fsContainer = new WebMarkupContainer("finalSeminarLinkContainer");
        fsContainer.setOutputMarkupId(true);
        add(fsContainer);

        rdContainer.add(new AjaxFallbackLink<Object>("roughDraftLink") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                step = ReviewerApproval.Step.ROUGH_DRAFT_APPROVAL;
                rdContainer.add(new AttributeModifier("class", new Model<>("active")));
                fsContainer.add(new AttributeModifier("class", new Model<>("")));
                target.add(rdContainer);
                target.add(fsContainer);
                target.add(wmc);
            }
        });

        fsContainer.add(new AjaxFallbackLink<Object>("finalSeminarLink") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                step = ReviewerApproval.Step.FINAL_SEMINAR_APPROVAL;
                rdContainer.add(new AttributeModifier("class", new Model<>("")));
                fsContainer.add(new AttributeModifier("class", new Model<>("active")));
                target.add(rdContainer);
                target.add(fsContainer);
                target.add(wmc);
            }
        });
    }

    private void addDatePickerForm() {
        Date startDate = new DateTime().minusMonths(1).toDate();
        Date endDate = new DateTime().minusDays(1).toDate();
        datePicker = new DatePickerPanel("filter", startDate, endDate);
        add(new Form<Integer>("form") {
            {
                add(datePicker);
            }
        });
    }

    private void addLateApprovalsList() {
        wmc = new WebMarkupContainer("container");
        wmc.setOutputMarkupId(true);
        add(wmc);
        PropertyListView<Pair<? extends ReviewerApproval, Decision>> listView = new PropertyListView<Pair<? extends ReviewerApproval, Decision>>("reviewerApprovals", getLateReviewerApprovals()) {
            @Override
            protected void populateItem(ListItem<Pair<? extends ReviewerApproval, Decision>> item) {
                item.add(new Label("head.project.title"));
                item.add(new Label("head.project.reviewer.fullName"));
                IModel<Date> deadline = new PropertyModel<>(item.getModel(), "tail.deadline");
                item.add(new DateLabel("deadline", deadline));
                Date decisionDate = item.getModelObject().getTail().getDecisionDate();
                item.add(new Label("late", getLateLabel(deadline, decisionDate)));
                item.add(new DateLabel("tail.requested"));
                item.add(new EnumLabel<Status>("tail.status"));
                item.add(new Label("head.project.reviewer.unit.title"));
            }
        };
        wmc.add(listView);
    }

    private int getLateLabel(IModel<Date> deadline, Date decisionDate) {
        Date endDate = (decisionDate == null) ? LocalDate.now().toDate() : LocalDate.fromDateFields(decisionDate).toDate();
        return daysService.workDaysBetween(deadline.getObject(), endDate) + 1;
    }

    private LoadableDetachableModel<List<Pair<? extends ReviewerApproval, Decision>>> getLateReviewerApprovals() {
        return new LoadableDetachableModel<List<Pair<? extends ReviewerApproval, Decision>>>() {
            @Override
            protected List<Pair<? extends ReviewerApproval, Decision>> load() {
                if (step.equals(ReviewerApproval.Step.ROUGH_DRAFT_APPROVAL)) {
                    return reviewerDeadlineFollowupService.getLateReviewerApprovals(ReviewerApproval.Step.ROUGH_DRAFT_APPROVAL, datePicker.getStartDate(), datePicker.getEndDate(), new Date());
                } else {
                    return reviewerDeadlineFollowupService.getLateReviewerApprovals(ReviewerApproval.Step.FINAL_SEMINAR_APPROVAL, datePicker.getStartDate(), datePicker.getEndDate(), new Date());
                }
            }
        };
    }

    @Override
    public void renderHead(IHeaderResponse response) {
        super.renderHead(response);
        response.render(JavaScriptHeaderItem.forUrl("//cdn.datatables.net/1.10.0/js/jquery.dataTables.js"));
    }

}
