package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorMyProjects;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.repository.panels.ProjectFilePanel;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.util.PageParameterKeys;

@Authorization(authorizedRoles={Roles.SUPERVISOR})
@ProjectModuleComponent(ProjectModule.FILES)
public class SupervisorFilePage extends AbstractSupervisorProjectDetailsPage implements MenuHighlightSupervisorMyProjects {

    private static final Logger LOGGER = LoggerFactory.getLogger(SupervisorFilePage.class);

	public SupervisorFilePage(PageParameters pp) {
		super(pp);

        Long projectId = pp.get(PageParameterKeys.MAP.get(Project.class)).toLong();

        Project project = projectService.findOne(projectId);
        if (!projectService.isPartOfProject(project, SciProSession.get().getUser())) {
            failAndRedirect();
        }
        add(new ProjectFilePanel("filePanel", new DetachableServiceModel<>(projectService, project)));
	}

    private void failAndRedirect() {
        LOGGER.warn("User not part of project tried to access the file panel");
        throw new RestartResponseException(getApplication().getApplicationSettings().getAccessDeniedPage());
    }
}
