package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.AjaxLinkPanel;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.DefaultDataPanel;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorMyGroups;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.group.Group;
import se.su.dsv.scipro.group.GroupService;
import se.su.dsv.scipro.security.auth.SystemModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.SystemModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

@SystemModuleComponent(SystemModule.GROUP)
public class SupervisorMyGroupsPage extends AbstractSupervisorPage implements MenuHighlightSupervisorMyGroups {

    @Inject
    private GroupService groupService;

    public SupervisorMyGroupsPage(PageParameters pp) {
        super(pp);

        add(new FeedbackPanel("feedback"));

        add(new BookmarkablePageLink<>("createGroupLink", SupervisorEditGroupPage.class));

        GroupService.Filter filter = new GroupService.Filter();
        filter.setNonAuthorInvolvee(SciProSession.get().getUser());
        SortableDataProvider<Group, String> provider = new FilteredDataProvider<>(groupService, Model.of(filter));
        provider.setSort("dateCreated", SortOrder.DESCENDING);

        add(new DefaultDataPanel<>("myGroups", createColumns(), provider));
    }

    private List<IColumn<Group, String>> createColumns() {
        List<IColumn<Group, String>> columns = new ArrayList<>();

        columns.add(new DateColumn<Group>(Model.of("Date created"), "dateCreated", "dateCreated", DateStyle.DATE));

        columns.add(new AbstractColumn<Group, String>(Model.of("Title"), "title") {
            @Override
            public void populateItem(Item<ICellPopulator<Group>> cellItem, String componentId, IModel<Group> rowModel) {
                cellItem.add(new AjaxLinkPanel<Group>(componentId, rowModel, new PropertyModel<>(rowModel, "title")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Group> model) {
                        PageParameters pp = new PageParameters();
                        pp.add(PageParameterKeys.MAP.get(Group.class), model.getObject().getId());
                        setResponsePage(SupervisorGroupPage.class, pp);
                    }
                });
            }
        });

        return columns;
    }
}
