package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.checklists.ChecklistOverviewPanel;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorMyProjects;
import se.su.dsv.scipro.finalseminar.OverviewSeminarPanel;
import se.su.dsv.scipro.finalseminar.ProjectActiveParticipationListPanel;
import se.su.dsv.scipro.finalseminar.ProjectOppositionsListPanel;
import se.su.dsv.scipro.group.GroupsOverviewPanel;
import se.su.dsv.scipro.latestevents.LatestEventsPanel;
import se.su.dsv.scipro.milestones.panels.MilestoneSummaryPanel;
import se.su.dsv.scipro.peer.PeerSummaryPanel;
import se.su.dsv.scipro.project.panels.ExternalResourcesPanel;
import se.su.dsv.scipro.projectstate.ProjectStatePanel;
import se.su.dsv.scipro.rwandagrading.GradePanel;
import se.su.dsv.scipro.supervisor.panels.SupervisorExternalProjectPanel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.thesislink.ExternalLinkPanel;

public class SupervisorProjectDetailsPage extends AbstractSupervisorProjectDetailsPage implements MenuHighlightSupervisorMyProjects {

    public static final String FINAL_SEMINAR_NEW = "finalSeminarNew";
    public static final String EXTERNAL_ORGANIZATION = "externalOrganization";
    public static final String MILESTONE = "milestone";
    public static final String PEER_SUMMARY = "peerSummary";
    public static final String LATEST_UPLOAD = "latestUpload";
    public static final String STATE_OF_MIND_PANEL = "stateOfMindPanel";
    public static final String EXTERNAL_RESOURCES = "externalResources";
    public static final String EXTERNAL_LINK = "externalLink";
    public static final String OPPOSITION = "opposition";
    public static final String ACTIVE_PARTICIPATION = "activeParticipation";

    public SupervisorProjectDetailsPage(PageParameters pp) {
        super(pp);

        add(new ExternalLinkPanel(EXTERNAL_LINK, projectModel));

        add(new ExternalResourcesPanel(EXTERNAL_RESOURCES, new PropertyModel<ProjectType>(projectModel, "projectType")));

        add(new ProjectStatePanel(STATE_OF_MIND_PANEL, projectModel) {
            @Override
            public boolean isAuthorView() {
                return false;
            }
        });
        add(new LatestEventsPanel(LATEST_UPLOAD, projectModel));

        add(new PeerSummaryPanel(PEER_SUMMARY, projectModel));

        add(new ProjectOppositionsListPanel(OPPOSITION, projectModel));

        add(new ProjectActiveParticipationListPanel(ACTIVE_PARTICIPATION, projectModel));

        add(new SupervisorExternalProjectPanel(EXTERNAL_ORGANIZATION, projectModel));

        add(new OverviewSeminarPanel(FINAL_SEMINAR_NEW, projectModel));

        add(new MilestoneSummaryPanel(MILESTONE, projectModel));
        add(new GroupsOverviewPanel("groups", projectModel, new GroupsOverviewPanel.GroupLinkSupplier() {
            @Override
            public BookmarkablePageLink<Void> newGroupLink(String id, PageParameters pp) {
                return new BookmarkablePageLink<>(id, SupervisorGroupPage.class, pp);
            }
        }));

        add(new GradePanel("rwandaGrade", projectModel));

        add(new ChecklistOverviewPanel("checklistOverview", projectModel));

    }

}
