package se.su.dsv.scipro.supervisor.panels;

import org.apache.wicket.markup.html.basic.EnumLabel;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import scala.Option;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.NullableOptionModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.reviewing.Decision;
import se.su.dsv.scipro.reviewing.ReviewerApproval;
import se.su.dsv.scipro.reviewing.Status;

public class FinalSeminarApprovalProcessPanel extends GenericPanel<ReviewerApproval> {
    public FinalSeminarApprovalProcessPanel(final String id, final IModel<ReviewerApproval> process) {
        super(id, new CompoundPropertyModel<>(process));

        add(new ViewAttachmentPanel("currentThesis", DateStyle.DATE));
        add(new EnumLabel<Status>("currentStatus"));
        add(new MultiLineLabel("currentDecision.comment"){
            @Override
            protected void onConfigure() {
                setVisible(getDefaultModelObject() != null);
            }
        });
        add(new MultiLineLabel("currentReason") {
            @Override
            protected void onConfigure() {
                setVisible(getModelObject().isDecided());
            }
        });
        add(new ViewAttachmentPanel("currentAttachment", new NullableOptionModel<>(new PropertyModel<Option<FileDescription>>(process, "currentAttachment"))));
        add(new ViewAttachmentPanel("currentGradingReport", new NullableOptionModel<>(new PropertyModel<Option<FileDescription>>(process, "currentGradingReport"))));
        add(new PropertyListView<Decision>("history") {
            @Override
            protected void populateItem(final ListItem<Decision> item) {
                item.add(new MultiLineLabel("comment") {
                    @Override
                    protected void onConfigure() {
                        setVisible(getDefaultModelObject() != null);
                    }
                });
                item.add(new MultiLineLabel("reason") {
                    @Override
                    protected void onConfigure() {
                        setVisible(getDefaultModelObject() != null);
                    }
                });
                item.add(new ViewAttachmentPanel("thesis", DateStyle.DATE));
                item.add(new EnumLabel<Status>("status"));
                item.add(new DateLabel("decisionDate", DateStyle.DATE));
                item.add(new ViewAttachmentPanel("attachment", new NullableOptionModel<>(new PropertyModel<Option<FileDescription>>(item.getModel(), "Attachment"))));
                item.add(new ViewAttachmentPanel("gradingReport", new NullableOptionModel<>(new PropertyModel<Option<FileDescription>>(item.getModel(), "gradingReport"))));
            }
        });
    }
}
