package se.su.dsv.scipro.thesislink;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.*;
import org.apache.wicket.validation.validator.StringValidator;
import org.apache.wicket.validation.validator.UrlValidator;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

import javax.inject.Inject;
import java.util.List;

public class ExternalLinkPanel extends GenericPanel<Project> {

    public static final String CONTAINER = "container";
    public static final String FORM = "form";
    public static final String LINKS = "links";
    public static final String USER = "user";
    public static final String LINK = "link";
    public static final String DESCRIPTION = "description";
    public static final String DELETE = "delete";
    public static final String URL = "url";
    public static final String FEEDBACK = "feedback";
    public static final String INFO = "info";

    @Inject
    private ExternalLinkService externalLinkService;

    private final WebMarkupContainer container;
    private final ListView<ExternalLink> links;

    public ExternalLinkPanel(String id, IModel<Project> projectModel) {
        super(id, projectModel);

        container = new WebMarkupContainer(CONTAINER);
        container.setOutputMarkupId(true);
        add(container);

        container.add(new FencedFeedbackPanel(FEEDBACK, this));
        container.add(new InfoPanel(INFO, new ResourceModel("info")));

        container.add(new LinkForm(FORM, Model.of(newLink())));

        links = new ListView<ExternalLink>(LINKS, getLoadedList()) {
            @Override
            protected void populateItem(final ListItem<ExternalLink> item) {
                item.add(new UserLinkPanel(USER, new PropertyModel<User>(item.getModel(), "user")));
                item.add(new org.apache.wicket.markup.html.link.ExternalLink(LINK, new PropertyModel<String>(item.getModel(), "url"), new PropertyModel<String>(item.getModel(), "description")));
                item.add(new DeleteLink(DELETE, item.getModel()));
            }
        };
        container.add(links);
    }

    private ExternalLink newLink() {
        return ExternalLink.builder().project(getModelObject()).url("").user(SciProSession.get().getUser()).description("").build();
    }

    private class DeleteLink extends AjaxConfirmationLink<ExternalLink> {
        public DeleteLink(String id, IModel<ExternalLink> model) {
            super(id, model, "Are you sure you want to delete the link?");
        }

        @Override
        public void onClick(AjaxRequestTarget target) {
            target.add(container);
            externalLinkService.delete(getModelObject());
            links.setModel(getLoadedList());
            ExternalLinkPanel.this.success(getString("deleted"));
        }

        @Override
        protected void onConfigure() {
            setVisibilityAllowed(getModelObject().getUser().equals(SciProSession.get().getUser()));
        }
    }

    private IModel<List<ExternalLink>> getLoadedList() {
        return new LoadableDetachableModel<List<ExternalLink>>() {
            @Override
            protected List<ExternalLink> load() {
                return externalLinkService.findByProject(getModelObject());
            }
        };
    }

    private class LinkForm extends Form<ExternalLink> {

        public LinkForm(String id, final IModel<ExternalLink> externalLinkModel) {
            super(id, externalLinkModel);

            TextField<String> url = new RequiredTextField<>(URL, new PropertyModel<String>(externalLinkModel, "url"));
            url.add(StringValidator.lengthBetween(0, ExternalLink.MAX_CHARS));
            url.add(new UrlValidator(new String[]{"http", "https"}));
            add(url);

            TextField<String> description = new RequiredTextField<>(DESCRIPTION, new PropertyModel<String>(externalLinkModel, "description"));
            description.add(StringValidator.lengthBetween(0, ExternalLink.MAX_CHARS));
            add(description);
        }

        @Override
        protected void onSubmit() {
            super.onSubmit();
            externalLinkService.save(getModelObject());
            setModelObject(newLink());
            success(getString("saved"));
        }

        @Override
        protected void onComponentTag(ComponentTag tag) {
            super.onComponentTag(tag);
            if(!hasError()) {
                tag.append("style", "display: none;", ";");
            }
        }
    }
}