package se.su.dsv.scipro.components

import org.apache.wicket.extensions.markup.html.form.DateTextField
import org.apache.wicket.markup.head.{CssHeaderItem, IHeaderResponse, JavaScriptHeaderItem}
import org.apache.wicket.request.resource.PackageResourceReference
import org.apache.wicket.resource.JQueryPluginResourceReference
import org.joda.time.{LocalDate, LocalTime}

class BootstrapTimeField(id: String) extends DateTextField(id, "HH:mm") {
  setOutputMarkupId(true)

  override def renderHead(response: IHeaderResponse) = {
    super.renderHead(response)
    response.render(CssHeaderItem.forReference(new PackageResourceReference(classOf[BootstrapTimeField], "bootstrap-clockpicker.css")))
    response.render(JavaScriptHeaderItem.forReference(new JQueryPluginResourceReference(classOf[BootstrapTimeField], "bootstrap-clockpicker.js")))
  }

  override protected def onRender() = {
    getResponse.write(
      s"""
         |<div class="input-group">
       """.stripMargin)
    super.onRender()
  }

  override protected def onAfterRender() = {
    getResponse.write(
      s"""
         |<span class="input-group-addon">
         |  <span class="glyphicon glyphicon-time" style="cursor: pointer;">
         |</span>
         |</div>
       """.stripMargin)

    getResponse.write(
      s"""
         |<script type="text/javascript">
         |  $$(function() {
         |    var $$input = $$('#$getMarkupId');
         |    $$input.parent().clockpicker({
         |      "donetext":"",
         |      "autoclose":true,
         |      "afterHide":function() {
         |        $$input.trigger('inputchange');
         |      }
         |    });
         |  });
         |</script>
       """.stripMargin)
    super.onAfterRender()
  }

  override def updateModel() = {
    Option(getModelObject).fold(super.updateModel()) { currentDate =>
      val date = new LocalDate(currentDate)
      val time = new LocalTime(getConvertedInput)
      val dateTime = date.toLocalDateTime(time)
      setModelObject(dateTime.toDate)
    }
  }
}
