package se.su.dsv.scipro.notifications.pages

import javax.inject.Inject

import org.apache.wicket.markup.html.WebPage
import org.apache.wicket.request.mapper.parameter.PageParameters
import se.su.dsv.scipro._
import se.su.dsv.scipro.notifications.dataobject._
import se.su.dsv.scipro.notifications.{NotificationEventService, NotificationService}
import se.su.dsv.scipro.security.auth.Authorization
import se.su.dsv.scipro.security.auth.roles.Roles
import se.su.dsv.scipro.session.SciProSession
import se.su.dsv.scipro.system.User

@Authorization(requiresLoggedInUser = true,
  authorizedRoles = Array(Roles.AUTHOR, Roles.SUPERVISOR, Roles.REVIEWER, Roles.EXAMINER, Roles.ADMIN, Roles.SYSADMIN)
)
class NotificationLandingPage(pp: PageParameters) extends WebPage(pp) {
  @Inject private var notificationEventService: NotificationEventService = _
  @Inject private var notificationService: NotificationService = _

  val currentUser = SciProSession.get().getUser

  val maybeLocation = for {
    id <- pp.get("id").asLong
    notification <- notificationEventService.findOne(id)
    _ <- markRead(notification, currentUser)
    location <- getLocation(notification)
  } yield location

  maybeLocation map {
    case (page, parameters) => setResponsePage(page, parameters)
  } getOrElse {
    setResponsePage(getApplication.getHomePage)
  }

  private def getLocation(event: NotificationEvent) = {
    import NotificationLandingPage._
    val locationFor = event match {
      case event: GroupEvent => groupLocation(event)
      case event: IdeaEvent => ideaLocation(event)
      case event: ProjectForumEvent => projectForumLocation(event)
      case event: PeerRequestEvent => peerRequestLocation(event)
      case event: ProjectEvent => projectLocation(event)
      case event: MileStoneEvent => milestoneLocation(event)
      case event: PeerEvent => peerReviewLocation(event)
      case event: SeminarEvent => finalSeminarLocation(event)
    }
    locationFor.lift(currentUser)
  }

  def markRead(notification: NotificationEvent, user: User): Option[Unit] =
    Some(notificationService.setRead(currentUser, notification, true))
}

object NotificationLandingPage extends GroupRedirection with IdeaRedirection with ProjectForumRedirection
with PeerRequestRedirection with PeerReviewRedirection with ProjectRedirection with MilestoneRedirection
with FinalSeminarRedirection