package se.su.dsv.scipro.repository.components

import org.apache.wicket.request.resource.AbstractResource.{WriteCallback, ResourceResponse}
import org.apache.wicket.request.resource.ContentDisposition._
import org.apache.wicket.request.resource.IResource.Attributes
import org.apache.wicket.request.resource.{AbstractResource, IResource, ResourceReference}
import org.apache.wicket.util.time.Time
import se.su.dsv.scipro.file.FileDescription

abstract class FileDescriptionResourceReference(name: String) extends ResourceReference(name) {
  import FileDescriptionResourceReference._

  def fileDescription(attributes: Attributes): Option[FileDescription]

  def getResource: IResource = {
    new AbstractResource {
      protected def newResourceResponse(attributes: Attributes): ResourceResponse = {
        val file = fileDescription(attributes)
        val response = file.fold(notFound)(serveFile)
        response(attributes)
      }

      private def notFound = (attributes: Attributes) => {
        val response = new ResourceResponse
        response.setError(NotFound)
        response
      }

      private def serveFile(fileDescription: FileDescription) = (attributes: Attributes) => {
        val resourceResponse = new ResourceResponse
        resourceResponse.setContentType(fileDescription.getMimeType)
        resourceResponse.setContentLength(fileDescription.getSize)
        resourceResponse.setFileName(fileDescription.getName)
        resourceResponse.setLastModified(Time.valueOf(fileDescription.getLastModified))
        resourceResponse.setCacheDurationToMaximum()
        resourceResponse.setStatusCode(Ok)
        resourceResponse.setContentDisposition(ATTACHMENT)

        resourceResponse.setWriteCallback(new WriteCallback {
          def writeData(attributes: Attributes) = {
            val data = fileDescription.getData
            try { writeStream(attributes, fileDescription.getData) }
            finally { if (data != null) data.close() }
          }
        })

        resourceResponse
      }
    }
  }
}

object FileDescriptionResourceReference {
  val Ok = 200
  val NotFound = 404
}