package se.su.dsv.scipro;

import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;
import org.apache.wicket.request.Request;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.SystemModule;
import se.su.dsv.scipro.system.User;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

public class SciProSessionTest extends SciProSession {

    Collection<Roles> currentRoles = new HashSet<>();
    Set<ProjectModule> currentProjectModules = new HashSet<>();
    Set<SystemModule> currentSystemModules = new HashSet<>();
    boolean loggedIn = true;
    User user;

    public SciProSessionTest(Request request) {
        super(request);
        setUser(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
    }

    @Override
    public synchronized boolean isLoggedIn() {
        return loggedIn;
    }

    @Override
    public boolean hasActualRole(Roles role) {
        return currentRoles.contains(role);
    }

    @Override
    public synchronized boolean authorizedForRole(final Roles role) {
        return Iterables.any(currentRoles, new Predicate<Roles>() {
            @Override
            public boolean apply(final Roles input) {
                return input.authorizedForRole(role);
            }
        });
    }

    @Override
    public synchronized void setUser(User user) {
        this.user = user;
    }

    @Override
    public User getUser() {
        return user;
    }

    @Override
    public Set<ProjectModule> getProjectModules() {
        return currentProjectModules;
    }

    @Override
    public Set<SystemModule> getSystemModules() {
        return currentSystemModules;
    }
}