package se.su.dsv.scipro;

import com.google.common.eventbus.EventBus;
import com.google.inject.AbstractModule;
import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.multibindings.Multibinder;
import com.google.inject.name.Names;
import org.apache.wicket.Component;
import org.apache.wicket.Page;
import org.apache.wicket.Session;
import org.apache.wicket.guice.GuiceComponentInjector;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.Response;
import org.apache.wicket.util.tester.WicketTester;
import org.junit.Before;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.activityplan.*;
import se.su.dsv.scipro.admin.pages.AdminStartPage;
import se.su.dsv.scipro.checklist.*;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.date.DateService;
import se.su.dsv.scipro.economy.TransactionService;
import se.su.dsv.scipro.examiner.pages.ExaminerStartPage;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.finalseminar.*;
import se.su.dsv.scipro.finalthesis.FinalThesisRepo;
import se.su.dsv.scipro.finalthesis.FinalThesisService;
import se.su.dsv.scipro.forum.BasicForumService;
import se.su.dsv.scipro.forum.GroupForumService;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.forummail.ForumMailSettingsService;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.grading.GradeService;
import se.su.dsv.scipro.group.GroupFacade;
import se.su.dsv.scipro.group.GroupService;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.match.*;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.milestones.service.MilestonePhaseTemplateService;
import se.su.dsv.scipro.milestones.service.MilestoneStatisticsService;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.nonworkperiod.NonWorkDayPeriodService;
import se.su.dsv.scipro.notes.NoteService;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.NotificationService;
import se.su.dsv.scipro.peer.*;
import se.su.dsv.scipro.phd.DepartmentServiceService;
import se.su.dsv.scipro.phd.StudyPlanService;
import se.su.dsv.scipro.plagiarism.PlagiarismControl;
import se.su.dsv.scipro.plagiarism.TurnItInCheckRepo;
import se.su.dsv.scipro.plagiarism.TurnItInCheckService;
import se.su.dsv.scipro.plagiarism.TurnitinSettingsService;
import se.su.dsv.scipro.profiles.CurrentProfile;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.pages.ProjectStartPage;
import se.su.dsv.scipro.projectpartner.ProjectPartnerService;
import se.su.dsv.scipro.report.GradingReportService;
import se.su.dsv.scipro.report.OppositionReportService;
import se.su.dsv.scipro.reviewer.ReviewerStartPage;
import se.su.dsv.scipro.reviewing.*;
import se.su.dsv.scipro.security.auth.AuthenticationService;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.springdata.services.SupervisorService;
import se.su.dsv.scipro.springdata.services.UnitService;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.supervisor.pages.SupervisorStartPage;
import se.su.dsv.scipro.system.*;
import se.su.dsv.scipro.thesislink.ExternalLinkService;
import se.su.dsv.scipro.workerthreads.Scheduler;
import se.su.dsv.scipro.workerthreads.WorkerDataService;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.Collection;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public abstract class SciProTest {

    @Mock
    protected EventBus eventBus;
    @Mock
    protected UserRepo userRepo;
    @Mock
    protected ActivityPlanService activityPlanService;

    /**
     * Mock services
     */
    @Mock
    protected GradeService gradeService;
    @Mock
    protected ProjectFinalSeminarStatisticsService projectFinalSeminarStatisticsService;
    @Mock
    protected NoteService noteService;
    @Mock
    protected UserFacade userFacade;
    @Mock
    protected PasswordFacade passwordFacade;
    @Mock
    protected GroupService groupService;
    @Mock
    protected HashService hashService;
    @Mock
    protected LanguageService languageService;
    @Mock
    protected ReviewerDeadlineFollowupService reviewerDeadlineFollowupService;
    @Mock
    protected ProgramService programService;
    @Mock
    protected ReviewerDeadlineSettingsService reviewerDeadlineSettingsService;
    @Mock
    protected AuthenticationService authenticationService;
    @Mock
    protected StudyPlanService studyPlanService;
    @Mock
    protected ActivityService activityService;
    @Mock
    protected ApplicationPeriodService applicationPeriodService;
    @Mock
    protected ChecklistService checklistService;
    @Mock
    protected ChecklistTemplateService checklistTemplateService;
    @Mock
    protected CommentService commentService;
    @Mock
    protected CommentThreadService commentThreadService;
    @Mock
    protected DateService dateService;
    @Mock
    protected DaysService daysService;
    @Mock
    protected FileDescriptionService fileDescriptionService;
    @Mock
    protected FinalSeminarService finalSeminarService;
    @Mock
    protected FinalThesisService finalThesisService;
    @Mock
    protected TransactionService transactionService;
    @Mock
    protected BasicForumService basicForumService;
    @Mock
    protected ProjectForumService projectForumService;
    @Mock
    protected GroupForumService groupForumService;
    @Mock
    protected GeneralSystemSettingsService generalSystemSettingsService;
    @Mock
    protected GenericService genericService;
    @Mock
    protected KeywordService keywordService;
    @Mock
    protected IdeaService ideaService;
    @Mock
    protected MatchService matchService;
    @Mock
    protected NotificationService notificationService;
    @Mock
    protected PeerRequestService peerRequestService;
    @Mock
    protected PeerReviewService peerReviewService;
    @Mock
    protected ProjectTypeService projectTypeService;
    @Mock
    protected ProjectPartnerService projectPartnerService;
    @Mock
    protected ActivityTemplateService activityTemplateService;
    @Mock
    protected ProjectService projectService;
    @Mock
    protected ResearchAreaService researchAreaService;
    @Mock
    protected ActivityPlanTemplateService activityPlanTemplateService;
    @Mock
    protected SupervisorService supervisorService;
    @Mock
    protected TurnItInCheckService turnItInCheckService;
    @Mock
    protected TurnItInCheckRepo turnItInCheckRepo;
    @Mock
    protected UnitService unitService;
    @Mock
    protected UserNameService userNameService;
    @Mock
    protected UserService userService;
    @Mock
    protected DepartmentServiceService departmentServiceService;
    @Mock
    protected MyReviewService myReviewService;
    @Mock
    protected UserProfileService userProfileService;
    @Mock
    protected NonWorkDayPeriodService nonWorkDayPeriodService;
    @Mock
    protected MileStoneService mileStoneService;
    @Mock
    protected MilestoneStatisticsService milestoneStatisticsService;
    @Mock
    protected MilestoneActivityTemplateService milestoneActivityTemplateService;
    @Mock
    protected MilestonePhaseTemplateService mileStonePhaseTemplateService;
    @Mock
    protected TurnitinSettingsService turnitinSettingsService;
    @Mock
    protected FinalSeminarSettingsService finalSeminarSettingsService;
    @Mock
    protected FinalSeminarRespondentService finalSeminarRespondentService;
    @Mock
    protected FinalSeminarOppositionService finalSeminarOppositionService;
    @Mock
    protected FinalSeminarActiveParticipationService finalSeminarActiveParticipationService;
    @Mock
    protected WorkerDataService workerDataService;
    @Mock
    protected GradingReportService gradingReportService;
    @Mock
    protected OppositionReportService oppositionReportService;
    @Mock
    protected MailEventService mailEventService;
    /**
     * Other
     */
    @Mock
    protected NotificationController notificationController;
    @Mock
    protected FinalSeminarUploadController finalSeminarUploadController;
    @Mock
    protected FinalSeminarOppositionRepo finalSeminarOppositionRepo;
    @Mock
    protected PlagiarismControl plagiarismControl;
    @Mock
    protected TargetService targetService;
    @Mock
    protected IdeaFacade ideaFacade;
    @Mock
    protected PeerPortal peerPortal;
    @Mock
    protected ActivityPlanFacade activityPlanFacade;
    @Mock
    protected GroupFacade groupFacade;
    @Mock
    protected ChecklistQuestionRepo checklistQuestionRepo;
    @Mock
    protected ChecklistCategoryRepo checklistCategoryRepo;
    @Mock
    protected ExternalLinkService externalLinkService;
    @Mock
    protected ApplicationPeriodProjectTypeService applicationPeriodProjectTypeService;
    @Mock
    protected PasswordRepo passwordRepo;
    @Mock
    protected PasswordService passwordService;
    @Mock
    protected PerformReviewService performReviewService;
    @Mock
    protected ChecklistAnswerService checklistAnswerService;
    @Mock
    protected CurrentUser currentUser;
    @Mock
    private Scheduler scheduler;
    @Mock
    protected ReviewerInteractionService reviewerInteractionService;
    @Mock
    protected ReviewerApprovalService reviewerApprovalService;
    @Mock
    protected FinalThesisRepo finalThesisRepo;
    @Mock
    protected FinalSeminarApprovalService finalSeminarApprovalService;
    @Mock
    protected ReviewerDecisionService reviewerDecisionService;
    @Mock
    protected RoughDraftApprovalService roughDraftApprovalService;
    @Mock
    protected ExternalResourceService externalResourceService;
    @Mock
    protected FooterLinkService footerLinkService;
    @Mock
    protected FooterAddressRepo footerAddressRepo;
    @Mock
    protected ForumMailSettingsService forumMailSettingsService;
    @Mock
    protected FinalSeminarCreationService finalSeminarCreationService;

    protected WicketTester tester;

    @Before
    public void setUpApplication() throws Exception {
        final Class<? extends SciProTest> clazz = getClass();
        final Injector injector = Guice.createInjector(new MockModule(clazz));

        SciProApplication application = new SciProApplication(injector.getInstance(CurrentProfile.class)) {

            @Override
            public Session newSession(Request request, Response response) {
                return new SciProSessionTest(request);
            }
        };
        application.getComponentInstantiationListeners().add(
                new GuiceComponentInjector(application, injector));
        tester = new WicketTester(application);

        setLoggedIn(true);
        setRoles(Roles.values());
        setModules(ProjectModule.values());
        setSystemModules(SystemModule.values());
    }

    @Before
    public void configureDateFormats() throws Exception {
        when(dateService.getFormat(DateStyle.DATETIME)).thenReturn("yyyy-MM-dd HH:mm");
        when(dateService.getFormat(DateStyle.DATE)).thenReturn("yyyy-MM-dd");
        when(dateService.getFormat(DateStyle.TIME)).thenReturn("HH:mm");
    }

    @Before
    public void createGeneralSystemSettings() throws Exception {
        when(generalSystemSettingsService.getGeneralSystemSettingsInstance()).thenReturn(new GeneralSystemSettings());
    }

    @Before
    public void createUserProfile() throws Exception {
        when(userProfileService.findByUser(any(User.class))).thenReturn(new UserProfile());
    }


    public Class<? extends Page> getHomePage() {
        Collection<Roles> roles = ((SciProSessionTest) tester.getSession()).currentRoles;
        if (roles.contains(Roles.ADMIN)) return AdminStartPage.class;
        if (roles.contains(Roles.SUPERVISOR)) return SupervisorStartPage.class;
        if (roles.contains(Roles.AUTHOR)) return ProjectStartPage.class;
        if (roles.contains(Roles.REVIEWER)) return ReviewerStartPage.class;
        if (roles.contains(Roles.EXAMINER)) return ExaminerStartPage.class;

        return HomePage.class;
    }

    protected void setRoles(Roles... roles) {
        SciProSessionTest session = (SciProSessionTest) tester.getSession();

        session.currentRoles.clear();
        session.currentRoles.addAll(Arrays.asList(roles));
    }

    protected void setLoggedIn(boolean loggedIn) {
        SciProSessionTest session = (SciProSessionTest) tester.getSession();
        session.loggedIn = loggedIn;
    }

    protected void setLoggedInAs(User user) {
        setLoggedIn(user != null);
        SciProSessionTest session = (SciProSessionTest) tester.getSession();
        session.setUser(user);
    }

    protected void setModules(ProjectModule... projectModules) {
        SciProSessionTest session = (SciProSessionTest) tester.getSession();
        session.currentProjectModules.clear();
        session.currentProjectModules.addAll(Arrays.asList(projectModules));
    }

    protected void setSystemModules(SystemModule... modules) {
        SciProSessionTest session = (SciProSessionTest) tester.getSession();
        session.currentSystemModules.clear();
        session.currentSystemModules.addAll(Arrays.asList(modules));
    }

    protected String path(Object... parts) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < parts.length; i++) {
            if (parts[i] instanceof Component) {
                sb.append(((Component) parts[i]).getId());
            } else {
                sb.append(parts[i]);
            }
            if (i + 1 < parts.length) {
                sb.append(Component.PATH_SEPARATOR);
            }
        }
        return sb.toString();
    }

    private class MockModule extends AbstractModule {
        private final Class<? extends SciProTest> clazz;

        public MockModule(Class<? extends SciProTest> clazz) {
            this.clazz = clazz;
        }

        @SuppressWarnings("unchecked")
        @Override
        protected void configure() {
            Multibinder.newSetBinder(binder(), Lifecycle.class);
            bindConstant().annotatedWith(Names.named("profile")).to("TEST");
            for (Class<?> c = clazz; c != null; c = c.getSuperclass()) {
                for (Field f : c.getDeclaredFields()) {
                    boolean accessible = f.isAccessible();
                    f.setAccessible(true);
                    if (f.getAnnotation(Mock.class) != null) {
                        try {
                            Class<Object> type = (Class<Object>) f.getType();
                            bind(type).toInstance(f.get(SciProTest.this));
                        } catch (IllegalAccessException e) {
                            throw new RuntimeException(e);
                        }
                    }
                    f.setAccessible(accessible);
                }
            }
        }
    }
}
