package se.su.dsv.scipro;

import org.junit.Test;
import se.su.dsv.scipro.reusable.SciProUtilities;

import java.util.Calendar;
import java.util.Date;
import java.util.Random;

import static org.junit.Assert.assertEquals;

/**
 * Created by: fred-fri
 * Date: 2013 03 20
 */
public class SciProUtilitiesTest {

    @Test
    public void testSetTimeOfDayTo0000() throws Exception {
        Date before = new Date();
        Date after = SciProUtilities.setTimeOfDayTo0000(before);

        Calendar c = Calendar.getInstance();
        c.setTime(after);

        assert c.get(Calendar.HOUR_OF_DAY)==0;
        assert c.get(Calendar.MINUTE)==0;
        assert c.get(Calendar.SECOND)==0;
        assert c.get(Calendar.MILLISECOND)==0;
    }

    @Test
    public void testSetTimeOfDayTo2359() throws Exception {
        Date before = new Date();
        Date after = SciProUtilities.setTimeOfDayTo2359(before);

        Calendar c = Calendar.getInstance();
        c.setTime(after);

        assert c.get(Calendar.HOUR_OF_DAY)==23;
        assert c.get(Calendar.MINUTE)==59;
        assert c.get(Calendar.SECOND)==59;
        assert c.get(Calendar.MILLISECOND)==999;
    }

    @Test
    public void md5_same() throws Exception {
        Random random = new Random();
        byte[] data = new byte[4096];
        for (int i = 0; i < 1000; i++) {
            random.nextBytes(data);

            assertEquals(convertToHex(data), SciProUtilities.convertToHex(data));
        }
    }

    private static String convertToHex(byte[] data) {
        StringBuilder buf = new StringBuilder();
        for (byte aData : data) {
            int halfByte = (aData >>> 4) & 0x0F;
            int twoHalfs = 0;
            do {
                if ((0 <= halfByte) && (halfByte <= 9)) {
                    buf.append((char) ('0' + halfByte));
                } else {
                    buf.append((char) ('a' + (halfByte - 10)));
                }
                halfByte = aData & 0x0F;
            } while (twoHalfs++ < 1);
        }
        return buf.toString();
    }
}
