package se.su.dsv.scipro.activityplan;

import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;

import static se.su.dsv.scipro.activityplan.EditActivityTemplatePanel.*;

public class EditActivityTemplatePanelTest extends SciProTest {

    private EditActivityTemplatePanel panel;

    private ActivityPlanTemplate activityPlanTemplate;

    private ActivityTemplate pet1;
    private ActivityTemplate pet2;
    private ActivityTemplate pet3;

    @Before
    public void setUp() throws Exception {
        activityPlanTemplate = new ActivityPlanTemplate();
        activityPlanTemplate.setId(1L);
        activityPlanTemplate.setCreator(new UserBuilder().setId(1L).setEmail("e@mail.com").setFirstName("Nisse").setLastName("Karlsson").create());
        activityPlanTemplate.setIsSysAdminTemplate(true);
        activityPlanTemplate.setDescription("naizz");

        pet1 = new ActivityTemplate();
        pet1.setTitle("1");
        activityPlanTemplate.addActivity(pet1);

        pet2 = new ActivityTemplate();
        pet2.setTitle("2");
        activityPlanTemplate.addActivity(pet2);

        pet3 = new ActivityTemplate();
        pet3.setTitle("3");
        activityPlanTemplate.addActivity(pet3);

        Mockito.when(activityPlanTemplateService.findOne(Matchers.isA(Long.class))).thenReturn(activityPlanTemplate);
        Mockito.when(activityPlanTemplateService.save(Matchers.isA(ActivityPlanTemplate.class))).thenReturn(activityPlanTemplate);
        startPanel();
    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void contents() throws Exception {
        startPanel();
        tester.assertComponent(path(panel.getId(), FORM), Form.class);
        tester.assertComponent(path(panel.getId(), FORM, TITLE_FIELD), TextField.class);
        tester.assertComponent(path(panel.getId(), FORM, DESCRIPTION_AREA), TextArea.class);
        tester.assertComponent(path(panel.getId(), FORM, ADD_LINK), AjaxLink.class);
        tester.assertComponent(path(panel.getId(), FORM), WebMarkupContainer.class);
        tester.assertComponent(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST), ListView.class);
        tester.assertComponent(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 1, ACTIVITY_LINK), AjaxLink.class);

        tester.assertComponent(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 1, UP), AjaxLink.class);
        tester.assertComponent(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 1, DOWN), AjaxLink.class);
        tester.assertComponent(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 1, REMOVE), AjaxLink.class);

        tester.assertInvisible(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 0, UP));
        tester.assertComponent(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 0, DOWN), AjaxLink.class);
        tester.assertComponent(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 0, REMOVE), AjaxLink.class);

        tester.assertComponent(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 2, UP), AjaxLink.class);
        tester.assertInvisible(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 2, DOWN));
        tester.assertComponent(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 2, REMOVE), AjaxLink.class);
    }

    @Test
    public void up() throws Exception {
        startPanel();
        tester.assertListView(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST), Arrays.asList(pet1, pet2, pet3));
        tester.clickLink(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 1, UP));
        tester.assertListView(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST), Arrays.asList(pet2, pet1, pet3));
    }

    @Test
    public void down() throws Exception {
        startPanel();
        tester.assertListView(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST), Arrays.asList(pet1, pet2, pet3));
        tester.clickLink(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 1, DOWN));
        tester.assertListView(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST), Arrays.asList(pet1, pet3, pet2));
    }

    @Test
    public void remove() throws Exception {
        startPanel();
        tester.assertListView(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST), Arrays.asList(pet1, pet2, pet3));
        tester.clickLink(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST, 1, REMOVE));
        tester.assertListView(path(panel.getId(), FORM, CONTAINER, ACTIVITY_LIST), Arrays.asList(pet1, pet3));
    }

    @Test
    public void title_required() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel.getId(), FORM));
        formTester.submit();
        tester.assertErrorMessages("Title cannot be left blank");
    }

    @Test
    public void saves() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel.getId(), FORM));
        formTester.setValue(TITLE_FIELD, "a nice title");
        formTester.submit();
        Mockito.verify(activityPlanTemplateService).save(activityPlanTemplate);
        tester.assertNoErrorMessage();
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new EditActivityTemplatePanel("panel", Model.of(activityPlanTemplate)));
    }
}
