package se.su.dsv.scipro.activityplan;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

public class ProjectActivityPlanPageTest extends PageTest {

    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        setLoggedInAs(user);
        User supervisor = new UserBuilder().setFirstName("Super").setLastName("Supersson").create();
        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        Mockito.when(projectService.findOne(Matchers.anyLong())).thenReturn(project);
        Mockito.when(projectService.isPartOfProject(project, user)).thenReturn(true);
    }

    @Override
    protected Class<? extends Page> getPage() {
        return ProjectActivityPlanPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.set(PageParameterKeys.MAP.get(Project.class), "1");
        return pageParameters;
    }
}
