package se.su.dsv.scipro.activityplan;

import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.User;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyZeroInteractions;

public class SaveAsTemplatePanelTest extends SciProTest {

    private ActivityPlan activityPlan;
    private User user;
    private SaveAsTemplatePanel panel;

    @Before
    public void setUp() throws Exception {
        activityPlan = new ActivityPlan();
        user = User.builder().firstName("Kalle").lastName("Tester").emailAddress("kalle@example.com").build();
        setLoggedInAs(user);

        panel = tester.startComponentInPage(new SaveAsTemplatePanel("panel", Model.of(activityPlan)));
    }

    @Test
    public void save_plan_as_template() throws Exception {
        final String name = "My awesome activity plan";
        final String description = "Using this activity plan will guarantee huge success.";

        FormTester formTester = tester.newFormTester(path(panel, "form"));
        formTester.setValue("title", name);
        formTester.setValue("description", description);
        tester.clickLink(path(panel, "form", "save"));

        verify(activityPlanFacade).createTemplateFromSchedule(activityPlan, user, name, description);
    }

    @Test
    public void title_is_required() throws Exception {
        tester.clickLink(path(panel, "form", "save"));

        tester.assertFeedback(path(panel, "feedback"), "'Title' is required.");

        verifyZeroInteractions(activityPlanFacade);
    }
}