package se.su.dsv.scipro.admin.pages;

import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.forummail.ForumMailSettings;

import java.io.Serializable;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsCollectionContaining.hasItem;
import static org.hamcrest.core.StringContains.containsString;
import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class AdminForumSettingsPageTest extends SciProTest {

    private AdminForumSettingsPage page;

    @Before
    public void setUp() throws Exception {
        when(forumMailSettingsService.getSettings()).thenReturn(createSettings());
        page = tester.startPage(AdminForumSettingsPage.class);

    }

    @Test
    public void renders() throws Exception {
        tester.assertRenderedPage(AdminForumSettingsPage.class);
    }

    @Test
    public void form_submit_save_changes() throws Exception {
        String mailServer = submitForm();

        ArgumentCaptor<ForumMailSettings> captor = ArgumentCaptor.forClass(ForumMailSettings.class);
        verify(forumMailSettingsService).save(captor.capture());

        assertEquals(mailServer, captor.getValue().getHost());
    }

    @Test
    public void feedback_when_submitting_form() throws Exception {
        submitForm();
        List<Serializable> messages = tester.getMessages(FeedbackMessage.SUCCESS);
        assertThat(messages, hasItem(containsString(page.getString("saved"))));
    }

    private String submitForm() {
        FormTester form = tester.newFormTester("form");
        String mailServer = "test";
        form.setValue("host", mailServer);
        form.submit();
        return mailServer;
    }

    private ForumMailSettings createSettings() {
        return new ForumMailSettings();
    }


}
