package se.su.dsv.scipro.admin.pages.settings;

import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.markup.html.basic.EnumLabel;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsCollectionContaining.hasItem;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.admin.pages.settings.AdminProjectModuleSettingsPage.*;

public class AdminProjectModuleSettingsPageTest extends SciProTest {

    private ProjectType projectType;

    @Before
    public void setUp() throws Exception {
        projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor");
        List<ProjectType> projectTypes = Collections.singletonList(projectType);
        when(projectTypeService.findAllActive()).thenReturn(projectTypes);
        startPage();
    }

    @Test
    public void renders() throws Exception {
        tester.assertRenderedPage(AdminProjectModuleSettingsPage.class);
    }

    @Test
    public void list_all_project_types() throws Exception {
        tester.assertListView(path(FORM, MODULE_LIST), Arrays.asList(ProjectModule.values()));
    }

    @Test
    public void module_lists_all_project_types() throws Exception {
        tester.assertListView(path(FORM, MODULE_LIST, 0, PROJECT_TYPE_LIST), projectTypeService.findAllActive());
    }

    @Test
    public void lists_modules_as_rows() throws Exception {
        tester.assertListView(path(FORM, AdminProjectModuleSettingsPage.MODULE_LIST), Arrays.asList(ProjectModule.values()));
    }

    @Test
    public void list_module_names() throws Exception {
        String path = path(FORM, MODULE_LIST, 0, NAME);
        tester.assertModelValue(path, ProjectModule.values()[0]);
        tester.assertComponent(path, EnumLabel.class);
    }

    @Test
    public void project_types_acts_as_table_header() throws Exception {
        tester.assertLabel(path(FORM, HEADER_LIST, 0, AdminProjectModuleSettingsPage.HEADER_NAME), projectType.getName());
    }

    @Test
    public void submit_form_saves_modules_to_project_type() throws Exception {
        FormTester formTester = tester.newFormTester(FORM);
        int selectedModule = 0;
        formTester.setValue(path(MODULE_LIST, 0, PROJECT_TYPE_LIST, selectedModule, CHECKBOX), true);
        formTester.submit();
        ArgumentCaptor<ProjectType> captor = ArgumentCaptor.forClass(ProjectType.class);
        verify(projectTypeService).save(captor.capture());
        assertThat(captor.getValue().getProjectModules(), hasItem(ProjectModule.values()[selectedModule]));
    }

    @Test
    public void submit_generates_feedback() throws Exception {
        FormTester formTester = tester.newFormTester(FORM);
        formTester.submit();
        List<Serializable> messages = tester.getMessages(FeedbackMessage.SUCCESS);
        assertThat(messages, hasItem(tester.getLastRenderedPage().getString("submitted")));
    }

    private AdminProjectModuleSettingsPage startPage() {
        return tester.startPage(AdminProjectModuleSettingsPage.class);
    }
}
