package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.model.Model;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.Target;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import static java.util.Arrays.asList;
import static se.su.dsv.scipro.applicationperiod.ApplicationPeriodSupervisorPanel.*;

@RunWith(MockitoJUnitRunner.class)
public class ApplicationPeriodSupervisorPanelTest extends SciProTest {
    public static final long MATCHED_IDEAS = 4L;
    public static final int IDEA_TARGET_NUMBER = 5;
    private ApplicationPeriodSupervisorPanel panel;
    private List<ProjectType> currentTypes;

    @Test
    public void testRenders() throws Exception {
        startPanel(true);
        Assert.assertNotNull("Panel does not render", panel);
    }

    @Test
    public void list_shows_the_current_application_periods_project_types() throws Exception {
        startPanel(true);
        tester.assertVisible(path(panel.getId(), LIST));
        tester.assertListView(path(panel.getId(), LIST), currentTypes);
    }

    @Test
    public void testLevel() throws Exception {
        startPanel(true);
        for (int i = 0; i < currentTypes.size(); i++) {
            tester.assertLabel(path(panel, LIST, i, PROJECT_TYPE), currentTypes.get(i).getName());
        }
    }

    @Test
    public void testTarget() throws Exception {
        startPanel(true);
        for (int i = 0; i < currentTypes.size(); i++) {
            tester.assertLabel(path(panel, LIST, i, TARGET), String.valueOf(IDEA_TARGET_NUMBER));
        }
    }

    @Test
    public void testCurrent() throws Exception {
        startPanel(true);
        for (int i = 0; i < currentTypes.size(); i++) {
            tester.assertLabel(path(panel, LIST, i, CURRENT), String.valueOf(MATCHED_IDEAS));
        }
    }

    @Test
    public void testInfoVisibleWhenEnoughIdeas() throws Exception {
        startPanel(false);
        for (int i = 0; i < currentTypes.size(); i++) {
            tester.assertVisible(path(panel, LIST, i, INFO));
        }
    }

    @Test
    public void testInfoInvisibleWhenEnoughIdeas() throws Exception {
        startPanel(true);
        for (int i = 0; i < currentTypes.size(); i++) {
            tester.assertInvisible(path(panel, LIST, i, INFO));
        }
    }

    private void startPanel(boolean hasEnoughIdeas) {
        ProjectType bachelor = createProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", 1L);
        ProjectType master = createProjectType(DegreeLevel.SECOND_CYCLE, "Master", 2L);

        if (hasEnoughIdeas) {
            Mockito.when(targetService.ideasLeftToCreateToMeetTarget(Matchers.isA(ApplicationPeriod.class), Matchers.isA(User.class), Matchers.isA(ProjectType.class))).thenReturn(0);
        } else {
            Mockito.when(targetService.ideasLeftToCreateToMeetTarget(Matchers.isA(ApplicationPeriod.class), Matchers.isA(User.class), Matchers.eq(bachelor))).thenReturn(1);
            Mockito.when(targetService.ideasLeftToCreateToMeetTarget(Matchers.isA(ApplicationPeriod.class), Matchers.isA(User.class), Matchers.eq(master))).thenReturn(2);
        }

        ApplicationPeriod ap = new ApplicationPeriod();
        ap.setId(1L);
        ap.setProjectTypes(new HashSet<>(asList(bachelor, master)));
        currentTypes = new ArrayList<>(ap.getProjectTypes());

        Target t = new Target(new UserBuilder().create(), ap, bachelor);
        t.setTarget(IDEA_TARGET_NUMBER);

        Mockito.when(targetService.findOne(Matchers.isA(ApplicationPeriod.class), Matchers.isA(User.class), Matchers.isA(ProjectType.class))).thenReturn(t);
        Mockito.when(ideaService.countMatched(Matchers.isA(ApplicationPeriod.class), Matchers.isA(User.class), Matchers.isA(ProjectType.class))).thenReturn(MATCHED_IDEAS);

        Mockito.when(projectTypeService.findAllActive()).thenReturn(asList(bachelor, master));

        panel = tester.startComponentInPage(new ApplicationPeriodSupervisorPanel("panel", Model.of(ap)));
    }

    private ProjectType createProjectType(DegreeLevel degreeLevel, String name, Long id) {
        ProjectType projectType = new ProjectType(degreeLevel, name, "description");
        projectType.setId(id);
        projectType.setDeleted(false);
        return projectType;
    }
}
