package se.su.dsv.scipro.checklists;

import org.apache.wicket.markup.html.form.CheckBoxMultipleChoice;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import static org.mockito.Mockito.verify;
import static se.su.dsv.scipro.checklists.AdminEditChecklistTemplatePage.*;

public class AdminEditChecklistTemplatePageTest extends SciProTest {
    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        setLoggedInAs(user);
    }

    @Test
    public void renders() throws Exception {
        startPage();
    }

    @Test
    public void feedback() throws Exception {
        startPage();
        tester.assertComponent(path(FEEDBACK), FeedbackPanel.class);
    }

    @Test
    public void form() throws Exception {
        startPage();
        tester.assertComponent(path(FORM), AdminEditChecklistTemplatePage.EditForm.class);
    }

    @Test
    public void title() throws Exception {
        startPage();
        tester.assertComponent(path(FORM, TITLE), TextField.class);
    }

    @Test
    public void categories() throws Exception {
        startPage();
        tester.assertComponent(path(FORM, CATEGORIES), CheckBoxMultipleChoice.class);
    }

    @Test
    public void project_types() throws Exception {
        startPage();
        tester.assertComponent(path(FORM, PROJECT_TYPES), CheckBoxMultipleChoice.class);
    }

    @Test
    public void doesnt_save_no_title() throws Exception {
        startPage();
        tester.newFormTester(path(FORM)).submit();
        tester.assertErrorMessages("Could not save checklist template", "Field 'Title:' is required");
    }

    @Test
    public void doesnt_save_short_title() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester(path(FORM));
        formTester.setValue(TITLE, "a");
        formTester.submit();
        tester.assertErrorMessages("Could not save checklist template", "The value of 'Title:' is shorter than the minimum of 3 characters.");
    }

    @Test
    public void saves() throws Exception {
        ChecklistTemplate template = new ChecklistTemplate();
        template.setId(1L);
        Mockito.when(checklistTemplateService.save(Matchers.isA(ChecklistTemplate.class))).thenReturn(template);
        Mockito.when(checklistTemplateService.findOne(Matchers.isA(Long.class))).thenReturn(template);

        startPage();
        FormTester formTester = tester.newFormTester(path(FORM));
        formTester.setValue(TITLE, "A nice title");
        formTester.submit();
        ArgumentCaptor<ChecklistTemplate> captor = ArgumentCaptor.forClass(ChecklistTemplate.class);
        verify(checklistTemplateService).save(captor.capture());
        Assert.assertEquals("A nice title", captor.getValue().getName());
        tester.assertNoErrorMessage();
    }

    @Test
    public void listview() throws Exception {
        startPage();
        tester.assertComponent(path(FORM, "wmc", "questions"), ListView.class);
    }

    private AdminEditChecklistTemplatePage startPage() {
        return tester.startPage(AdminEditChecklistTemplatePage.class);
    }

    @Test
    public void description_field() throws Exception {
        startPage();
        tester.assertComponent(path(FORM, AdminEditChecklistTemplatePage.DESCRIPTION), TextArea.class);
    }
}