package se.su.dsv.scipro.checklists;

import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.checklist.Checklist;
import se.su.dsv.scipro.checklist.ChecklistAnswer;
import se.su.dsv.scipro.checklist.ChecklistQuestion;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

public class ChecklistQuestionPanelTest extends SciProTest {

    public static final ProjectType SOME_PROJECT_TYPE = new ProjectType(DegreeLevel.FIRST_CYCLE, "Some project type", "Some description");
    private ChecklistQuestionPanel panel;
    private Project project;
    private Checklist checklist;
    private User user;
    private ChecklistQuestion question;
    private ChecklistAnswer answer;

    @Before
    public void setUp() throws Exception {
        user = new UserBuilder().create();
        setLoggedInAs(user);
        question = new ChecklistQuestion("What?", 1);
        answer = new ChecklistAnswer(user);
        question.addAnswer(answer);
        project = Project.builder().title("Some title").projectType(SOME_PROJECT_TYPE).build();
        project.addProjectParticipant(user);
        checklist = Checklist.builder().name("Some name").project(project).build();
        checklist.addQuestion(question);
        DomainObjects.injectId(checklist, 1L);
        panel = (ChecklistQuestionPanel) startPanel();
    }

    @Test
    public void renders() throws Exception {
        Assert.assertNotNull(panel);
    }

    @Test
    public void sendsNotificationWhenWholeListIsChecked() throws Exception {
        Mockito.when(activityPlanFacade.isCompletedByUser(user, checklist)).thenReturn(true);

        answerChecklistQuestion();

        Mockito.verify(notificationController).notifyProject(Matchers.eq(project), Matchers.eq(ProjectEvent.Event.CHECKLIST_ANSWERED), Matchers.any(NotificationSource.class));
    }

    @Test
    public void doesNotSendNotificationWhenWholeListIsNotChecked() throws Exception {
        Mockito.when(activityPlanFacade.isCompletedByUser(user, checklist)).thenReturn(false);

        answerChecklistQuestion();

        Mockito.verify(notificationController, Mockito.never()).notifyProject(Matchers.eq(project), Matchers.eq(ProjectEvent.Event.CHECKLIST_ANSWERED), Matchers.any(NotificationSource.class));
    }

    private void answerChecklistQuestion() {
        Mockito.when(checklistQuestionRepo.findOne(question.getId())).thenReturn(question);
        Mockito.when(checklistAnswerService.save(answer)).thenReturn(answer);

        panel = (ChecklistQuestionPanel) startPanel();

        tester.clickLink(path(panel.getId(), "wmc", ChecklistQuestionPanel.QUESTIONS, 0, ChecklistQuestionPanel.ANSWERS, 0, "answer", "answerLink"));
        FormTester formTester = tester.newFormTester(path(panel.getId(), "wmc", ChecklistQuestionPanel.QUESTIONS, 0, ChecklistQuestionPanel.ANSWERS, 0, "answer", "dialog", "body", "form"));//  (panel.getId(), ChecklistQuestionPanel.ANSWER_DIALOG, AnswerDialogPanel.ANSWER_DIALOG, AnswerDialogPanel.CONTENT, AnswerDialogPanel.ANSWER_FORM));
        formTester.setValue("comment", "Comment");

//        tester.clickLink(path(panel.getId(), "wmc", ChecklistQuestionPanel.QUESTIONS, 0, ChecklistQuestionPanel.ANSWERS, 0, "answer", "dialog", "body", "form", "submit"));
        formTester.submit();
    }

    private Panel startPanel() {
        Mockito.when(checklistService.findOne(checklist.getId())).thenReturn(checklist);
        DetachableServiceModel<Checklist> checklistModel = new DetachableServiceModel<>(checklistService, checklist.getId());
        return tester.startComponentInPage(new ChecklistQuestionPanel("panel", new CompoundPropertyModel<>(checklistModel)));
    }
}
