package se.su.dsv.scipro.dataproviders;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.FilteredService;

import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.mockito.Matchers.eq;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class FilteredDataProviderTest {
    @Mock
    private FilteredService<DomainObject, Long, String> filteredService;

    private String filter;
    private FilteredDataProvider<DomainObject,String> provider;

    @Before
    public void setUp() throws Exception {
        filter = "";
        provider = new FilteredDataProvider<>(filteredService, Model.of(filter));
    }

    @Test
    public void size() throws Exception {
        long count = 8L;
        when(filteredService.count(filter)).thenReturn(count);

        assertEquals(count, provider.size());
    }

    @Test
    public void iterator() throws Exception {
        DomainObject domainObject = mock(DomainObject.class);
        when(filteredService.findAll(eq(filter), isA(Pageable.class))).thenReturn(Collections.singletonList(domainObject));

        Iterator<? extends DomainObject> iterator = provider.iterator(0, Integer.MAX_VALUE);
        assertEquals(domainObject, iterator.next());
        assertFalse(iterator.hasNext());
    }
}
