package se.su.dsv.scipro.datatables.target;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.Target;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class EditTargetPanelTest extends SciProTest {

    private ProjectType bachelor;
    private User employee;
    private ApplicationPeriod applicationPeriod;
    private EditTargetPanel panel;

    @Before
    public void setUp() throws Exception {
        bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        applicationPeriod = new ApplicationPeriod();
        applicationPeriod.addProjectType(bachelor);
        employee = User.builder().firstName("Kalle").lastName("Testare").emailAddress("kalle@test.com").build();

        when(targetService.findOne(applicationPeriod, employee, bachelor))
                .thenReturn(new Target(employee, applicationPeriod, bachelor));
    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void changing_value_saves_new_target() throws Exception {
        int newTarget = 42;

        startPanel();
        setNewTarget(newTarget);

        ArgumentCaptor<Target> captor = ArgumentCaptor.forClass(Target.class);
        verify(targetService).save(captor.capture());

        assertEquals(newTarget, captor.getValue().getTarget());
    }

    private void setNewTarget(int newTarget) {
        String path = path(panel, "input");
        tester.getRequest().getPostParameters().setParameterValue(getInputName(path), String.valueOf(newTarget));
        tester.executeAjaxEvent(path, "change");
    }

    private String getInputName(String path) {
        return ((FormComponent<?>) tester.getComponentFromLastRenderedPage(path)).getInputName();
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new EditTargetPanel("id", Model.of(applicationPeriod), Model.of(employee), Model.of(bachelor), true) {
            @Override
            public void update(AjaxRequestTarget ajaxTarget, User e) {}
        });
    }
}
