package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.model.Model;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Date;

@RunWith(MockitoJUnitRunner.class)
public class OverviewSeminarPanelTest extends SciProTest {

    OverviewSeminarPanel panel;

    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().setFirstName("author").setLastName("1").create();
        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(user)
                .build();
        DomainObjects.injectId(project, 1L);

        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setProject(project);
        finalSeminar.setStartDate(new Date());

        Mockito.when(finalSeminarService.findByProject(Matchers.isA(Project.class))).thenReturn(finalSeminar);
        Mockito.when(dateService.getFormat(Matchers.isA(DateStyle.class))).thenReturn("yyyy-MM-dd");

        panel = tester.startComponentInPage(new OverviewSeminarPanel("panel", Model.of(project)));
    }

    @Test
    public void testRenders() throws Exception {
        Assert.assertNotNull("Panel does not render", panel);
    }

    @Test
    public void testNoSeminar() throws Exception {
        tester.assertContains("[" + path(OverviewSeminarPanel.NOSEMINAR) + "]");
    }

    @Test
    public void testCreate() throws Exception {
        tester.assertContains("[" + path(OverviewSeminarPanel.NOSEMINAR, OverviewSeminarPanel.CREATE) + "]");
    }

    @Test
    public void testHas() throws Exception {
        tester.assertContains("[" + path(OverviewSeminarPanel.HASSEMINAR) + "]");
    }

    @Test
    public void testDate() throws Exception {
        tester.assertContains("[" + path(OverviewSeminarPanel.HASSEMINAR, OverviewSeminarPanel.DATE) + "]");
    }

    @Test
    public void testRoom() throws Exception {
        tester.assertContains("[" + path(OverviewSeminarPanel.HASSEMINAR, OverviewSeminarPanel.ROOM) + "]");
    }

    @Test
    public void testView() throws Exception {
        tester.assertContains("[" + path(OverviewSeminarPanel.HASSEMINAR, OverviewSeminarPanel.VIEW) + "]");
    }

    @Override
    protected String path(Object... parts) {
        return super.path(panel.getId(), super.path(parts));
    }
}
