package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.util.Arrays;
import java.util.Collections;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.when;

public class ProjectActiveParticipationListPanelTest extends SciProTest {
    private Project bills;
    private FinalSeminarActiveParticipation billsParticipation;

    @Before
    public void setUp() throws Exception {
        ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        User bob = newUser(1L);
        User bill = newUser(2L);
        Project bobs = Project.builder().title("My project").projectType(bachelor).build();
        bobs.setId(8L);
        bobs.addProjectParticipant(bob);
        bills = Project.builder().title("My project").projectType(bachelor).build();
        bills.addProjectParticipant(bill);
        FinalSeminar finalSeminar = new FinalSeminar(bobs);
        billsParticipation = newActiveParticipation(bill, finalSeminar, bills);
        finalSeminar.addActiveParticipant(billsParticipation);
        finalSeminar.addActiveParticipant(newActiveParticipation(bob, finalSeminar, bobs));
        when(finalSeminarService.findProjectParticipating(eq(bills), any(Pageable.class))).thenReturn(Collections.singleton(finalSeminar));
    }

    @Test
    public void lists_correct_oppositions() throws Exception {
        ProjectActiveParticipationListPanel panel = tester.startComponentInPage(new ProjectActiveParticipationListPanel("id", Model.of(bills)));
        tester.assertListView(path(panel, "active.participations"), Collections.singletonList(billsParticipation));
    }

    private FinalSeminarActiveParticipation newActiveParticipation(final User author, final FinalSeminar finalSeminar, final Project authors) {
        FinalSeminarActiveParticipation finalSeminarActiveParticipation = new FinalSeminarActiveParticipation();
        finalSeminarActiveParticipation.setFinalSeminar(finalSeminar);
        finalSeminarActiveParticipation.setUser(author);
        finalSeminarActiveParticipation.setProject(authors);
        return finalSeminarActiveParticipation;
    }

    private User newUser(final long id) {
        User user = User.builder().firstName("Spongebob").lastName("Squarepants").emailAddress("spongebob@example.com").build();
        user.setId(id);
        return user;
    }
}