package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.model.Model;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.test.SeminarBuilder;

public class SeminarCancelledInfoPanelTest extends SciProTest {

    SeminarCancelledInfoPanel panel;
    FinalSeminar seminar;

    @Before
    public void setUp() throws Exception {
        seminar = new SeminarBuilder().create();
//        seminar.setDeleted(true);
        Mockito.when(finalSeminarService.findOne(Matchers.isA(Long.class))).thenReturn(seminar);
//        startPanel();
    }

    @Test
    public void testRenders() throws Exception {
        seminar.setDeleted(true);
        startPanel();
        Assert.assertNotNull("Does not render", panel);
    }

    @Test
    public void testPanelContainsWMC() throws Exception {
        seminar.setDeleted(true);
        startPanel();
        tester.assertComponent(path(panel.getId(), SeminarCancelledInfoPanel.CANCELLED), WebMarkupContainer.class);
    }

    @Test
    public void testPanelContainsLabel() throws Exception {
        seminar.setDeleted(true);
        startPanel();
        tester.assertComponent(path(panel.getId(), SeminarCancelledInfoPanel.CANCELLED, SeminarCancelledInfoPanel.CANCELLED_INFO), MultiLineLabel.class);
    }

    @Test
    public void testWMCIsVisibleWhenSeminarIsDeleted() throws Exception {
        seminar.setDeleted(true);
        startPanel();
        tester.assertVisible(path(panel.getId(), SeminarCancelledInfoPanel.CANCELLED));
    }

    @Test
    public void testLabelIsVisibleWhenSeminarIsDeleted() throws Exception {
        seminar.setDeleted(true);
        startPanel();
        tester.assertVisible(path(panel.getId(), SeminarCancelledInfoPanel.CANCELLED, SeminarCancelledInfoPanel.CANCELLED_INFO));
    }

    @Test
    public void testWMCIsInvisibleWhenSeminarIsNotDeleted() throws Exception {
        seminar.setDeleted(false);
        startPanel();
        tester.assertInvisible(path(panel.getId(), SeminarCancelledInfoPanel.CANCELLED));
    }

    @Test
    public void testLabelIsInvisibleWhenSeminarIsNotDeleted() throws Exception {
        seminar.setDeleted(false);
        startPanel();
        tester.assertInvisible(path(panel.getId(), SeminarCancelledInfoPanel.CANCELLED, SeminarCancelledInfoPanel.CANCELLED_INFO));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new SeminarCancelledInfoPanel("panel", Model.of(seminar)));
    }
}
