package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeSettings;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;

import java.util.Date;

public class SupervisorFinalSeminarDetailsPageTest extends SciProTest {


    private FinalSeminar finalSeminar;

    @Before
    public void setUp() throws Exception {

        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        ProjectTypeSettings projectTypeSettings = new ProjectTypeSettings();
        projectTypeSettings.setMaxFinalSeminarActiveParticipation(1);
        projectTypeSettings.setMaxOpponentsOnFinalSeminar(1);
        projectType.setProjectTypeSettings(projectTypeSettings);

        User supervisor = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        Project project = Project.builder().title("bla").projectType(projectType).headSupervisor(supervisor).build();
        project.setId(2352350L);

        finalSeminar = new FinalSeminar();
        finalSeminar.setProject(project);
        finalSeminar.setStartDate(new Date());
        DomainObjects.injectId(finalSeminar, 1L);

        Mockito.when(projectService.findOne(Matchers.anyLong())).thenReturn(project);
        Mockito.when(finalSeminarService.findByProject(Matchers.any(Project.class))).thenReturn(finalSeminar);
        Mockito.when(finalSeminarService.findOne(Matchers.anyLong())).thenReturn(finalSeminar);
        Mockito.when(finalSeminarSettingsService.getInstance()).thenReturn(new FinalSeminarSettings());
    }

    @Test
    public void renders() throws Exception {
        tester.startPage(new SupervisorFinalSeminarDetailsPage(Model.of(finalSeminar)));
        tester.assertRenderedPage(SupervisorFinalSeminarDetailsPage.class);
        tester.assertNoErrorMessage();
    }
}
