package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.TreeSet;

import static org.mockito.Mockito.when;

public class SupervisorFinalSeminarPageTest extends PageTest {

    private Project project;
    private User headSupervisor;

    @Before
    public void setUp() throws Exception {
        headSupervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        User student2 = new UserBuilder().setFirstName("author").setLastName("2").create();
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(headSupervisor)
                .build();
        DomainObjects.injectId(project, 1L);
        when(projectService.findOne(Matchers.isA(Long.class))).thenReturn(project);
        when(projectService.isSupervisorOrFollower(Matchers.isA(Project.class), Matchers.isA(User.class))).thenReturn(true);
        when(finalSeminarSettingsService.getInstance()).thenReturn(new FinalSeminarSettings());
    }

    @Test
    public void renders_when_final_seminar_exists() throws Exception {
        createFinalSeminar();
        startPage();
        tester.assertRenderedPage(getPage());
    }

    @Test
    public void render_create_seminar_when_current_user_is_head_supervisor() throws Exception {
        setLoggedInAs(headSupervisor);
        startPage();
        tester.assertComponent("panel", SeminarPanel.class);
    }

    private void createFinalSeminar() {
        FinalSeminar finalSeminar = new FinalSeminar(project);
        when(finalSeminarService.findByProject(project)).thenReturn(finalSeminar);
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(1L));
        return pp;
    }

    @Override
    protected Class<? extends Page> getPage() {
        return SupervisorFinalSeminarPage.class;
    }
}




