package se.su.dsv.scipro.forum.pages.threaded;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.TreeSet;

import static org.mockito.Mockito.*;

public class SupervisorThreadedForumPageTest extends PageTest {

    SupervisorThreadedForumPage page;
    private Project project;

    @Override
    protected Class<? extends Page> getPage() {
        return SupervisorThreadedForumPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Project.class), 1);
        return pageParameters;
    }

    @Before
    public void setUp() throws Exception {
        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");
        User headSupervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(headSupervisor)
                .build();
        DomainObjects.injectId(project, 1L);
        project.setDateCreated(new Date());
        when(projectService.findOne(isA(Long.class))).thenReturn(project);

        when(projectService.isSupervisorOrFollower(isA(Project.class), isA(User.class))).thenReturn(true);
        page = tester.startPage(SupervisorThreadedForumPage.class, getPageParameters());
    }

    @Test
    public void testClickCreateLinkTakesUserToCreatePage() throws Exception {
        tester.assertBookmarkablePageLink(SupervisorThreadedForumPage.CREATE_THREAD_LINK, SupervisorCreateForumThreadPage.class, getPageParameters());
    }

}
