package se.su.dsv.scipro.forum.panels.threaded;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ForumBuilder;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class ForumPostPanelTest extends SciProTest {

    ForumPostPanel panel;
    ForumPost forumPost;

    @Before
    public void setUp() throws Exception {
        forumPost = new ForumBuilder().createPost();
        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");
        when(projectService.getMemberType(isA(Project.class), isA(User.class))).thenReturn(Member.Type.SUPERVISOR);
        startPanel();
    }

    @Test
    public void testRenders() throws Exception {
        assertNotNull("Panel does not render", panel);
    }

    @Test
    public void testCorrectModelValues() throws Exception {
        tester.assertModelValue(path(panel.getId(), ForumPostPanel.POSTED_BY), forumPost.getPostedBy());
        tester.assertModelValue(path(panel.getId(), ForumPostPanel.CONTENT), forumPost.getContent());
        tester.assertModelValue(path(panel.getId(), ForumPostPanel.DATE_CREATED), forumPost.getDateCreated());
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new ForumPostPanel("panel", Model.of(forumPost)));
    }
}
