package se.su.dsv.scipro.forum.panels.threaded;

import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.forum.ProjectForum;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.ForumBuilder;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.anySetOf;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.mockito.internal.verification.VerificationModeFactory.times;

public class SubmitForumThreadPanelTest extends SciProTest {

    SubmitForumThreadPanel panel;
    Project project;
    ProjectThread thread;

    @Before
    public void setUp() throws Exception {
        thread = new ForumBuilder().createThread();
        thread.getForumThread().addPost(new ForumBuilder().createPost());

        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");

        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        User supervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        when(projectService.findOne(isA(Long.class))).thenReturn(project);

        when(projectService.isPartOfProject(isA(Project.class), isA(User.class))).thenReturn(true);
        when(projectForumService.createThread(isA(Project.class), isA(User.class), isA(String.class),
                isA(String.class), anySetOf(FileUpload.class))).thenReturn(thread);
        startPanel();
    }

    @Test
    public void testRenders() throws Exception {
        assert panel != null;
    }

    @Test
    public void testSaveForm() throws Exception {
        User user = new UserBuilder().setFirstName("Kalle").setLastName("Testare").create();
        setLoggedInAs(user);

        FormTester formTester = tester.newFormTester(path(panel.getId(), "form"));
        formTester.setValue("content", "content");
        formTester.setValue("subject", "subject");

        tester.executeAjaxEvent(path(panel.getId(), "form", "postLink"), "onclick");

        tester.assertNoErrorMessage();

        verify(projectForumService, times(1)).createThread(isA(Project.class), isA(User.class), isA(String.class), isA(String.class),
                anySetOf(FileUpload.class));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new SubmitForumThreadPanel<>("panel", new ProjectForum(projectForumService, Model.of(project))));
    }
}
