package se.su.dsv.scipro.group;

import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.supervisor.pages.SupervisorGroupPage;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;

import static org.mockito.Mockito.when;

public class GroupsOverviewPanelTest extends SciProTest {

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void lists_all_groups_that_the_project_is_involved_in() throws Exception {
        mockExistingGroup();
        startPanel();
        tester.assertListView(path("id", "groupList"), Collections.singletonList(getGroup()));
    }

    @Test
    public void group_link() throws Exception {
        mockExistingGroup();
        startPanel();
        tester.assertBookmarkablePageLink(path("id", "groupList", 0, "groupLink"), SupervisorGroupPage.class, mockPP());
    }

    @Test
    public void hide_panel_if_no_groups_are_connected_to_project() throws Exception {
        startPanel();
        tester.assertInvisible("id");
    }

    private PageParameters mockPP() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Group.class), 6L);
        return pageParameters;
    }

    private void mockExistingGroup() {
        GroupService.Filter filter = new GroupService.Filter();
        filter.setProject(ObjectMother.SOME_PROJECT);
        when(groupService.findAll(filter)).thenReturn(Collections.singletonList(getGroup()));
    }

    private Group getGroup() {
        Group group = new Group();
        group.setTitle("group title");
        group.setUser(ObjectMother.SOME_USER);
        group.setId(6L);
        return group;
    }

    private GroupsOverviewPanel startPanel() {
        return tester.startComponentInPage(new GroupsOverviewPanel("id", Model.of(ObjectMother.SOME_PROJECT), new GroupsOverviewPanel.GroupLinkSupplier() {
            @Override
            public BookmarkablePageLink<Void> newGroupLink(String id, PageParameters pp) {
                return new BookmarkablePageLink<>(id, SupervisorGroupPage.class, pp);
            }
        }));
    }
}
