package se.su.dsv.scipro.match;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.util.visit.IVisit;
import org.apache.wicket.util.visit.IVisitor;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

import java.util.ArrayList;

import static java.util.Arrays.asList;

public class AdminIdeaOverviewPanelTest extends SciProTest {

    private AdminIdeaOverviewPanel panel;
    private Idea idea;
    private boolean checkBoxIsAvailable;

    @Before
    public void setUp() throws Exception {
        Mockito.when(applicationPeriodService.findAll()).thenReturn(new ArrayList<ApplicationPeriod>());
    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void click_delete() throws Exception {
        mockIdea();
        startPanel();

        panel.visitChildren(AjaxConfirmationLink.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component deleteLink, IVisit<Object> visit) {
                tester.clickLink(deleteLink.getPageRelativePath());
                Mockito.verify(ideaService).deleteUnmatchedIdea(idea);
            }
        });
    }

    @Test
    public void inactivate_idea_checkbox_is_available() throws Exception {
        mockIdea();
        startPanel();
        Panel dataTable = (Panel) panel.get(AbstractAdminIdeaPanel.DATA_TABLE);
        checkBoxIsAvailable = false;
        dataTable.visitChildren(AjaxCheckBox.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component checkbox, IVisit<Object> visit) {
                checkBoxIsAvailable = true;
            }
        });
        Assert.assertTrue(checkBoxIsAvailable);
    }

    @Test
    public void inactivate_idea() throws Exception {
        mockIdea();
        final User admin = new UserBuilder().setFirstName("Admin").setLastName("User").create();
        setLoggedInAs(admin);
        startPanel();
        Panel dataTable = (Panel) panel.get(AbstractAdminIdeaPanel.DATA_TABLE);
        dataTable.visitChildren(AjaxCheckBox.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component checkbox, IVisit<Object> visit) {
                tester.executeAjaxEvent(checkbox.getPageRelativePath(), "onclick");
                Mockito.verify(ideaService).inactivateIdea(Matchers.eq(idea), Matchers.eq(idea.isInactive()), Matchers.eq(admin));
            }
        });
    }

    private void mockIdea() {
        idea = new Idea();
        Match match = new Match();
        match.setStatus(Idea.Status.UNMATCHED);
        idea.setMatch(match);
        idea.setType(Idea.Type.STUDENT);
        idea.setProjectType(new ProjectType(ProjectType.BACHELOR, "bachelor", "bachelor"));
        Mockito.when(ideaService.findAll(Matchers.any(IdeaService.Filter.class), Matchers.any(Pageable.class))).thenReturn(asList(idea));
        Mockito.when(ideaService.count(Matchers.any(IdeaService.Filter.class))).thenReturn(1L);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new AdminIdeaOverviewPanel("id"));
    }
}
