package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.util.visit.IVisit;
import org.apache.wicket.util.visit.IVisitor;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.test.DomainObjects;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class AdminKeywordCrudPanelTest extends SciProTest {

    AdminKeywordCrudPanel panel;
    FeedbackPanel feedbackPanel;

    @Before
    public void setUp() throws Exception {
        feedbackPanel = new FeedbackPanel("feedbackPanel");
        feedbackPanel.setOutputMarkupId(true);

        Keyword keyword = new Keyword("Test keyword");
        DomainObjects.injectId(keyword, 1L);
        List<Keyword> keywords = Collections.singletonList(keyword);
        Mockito.when(keywordService.findAll(Matchers.isA(Pageable.class))).thenReturn(keywords);
        Mockito.when(keywordService.count()).thenReturn(1L);
        Mockito.when(keywordService.findOne(Matchers.anyLong())).thenReturn(keyword);
        Mockito.when(dateService.getFormat(Matchers.isA(DateStyle.class))).thenReturn("yyyy-mm-dd");

        panel = tester.startComponentInPage(new AdminKeywordCrudPanel("panel"));
    }

    @Test
    public void testRenders() throws Exception {
        Assert.assertNotNull(panel);
    }

    @Test
    public void testClickOnCreateNewLinkOpensDialog() throws Exception {
        tester.clickLink(path(panel.getId(), AdminKeywordCrudPanel.CREATE_NEW_LINK));
        tester.assertComponentOnAjaxResponse(path(panel.getId(), AdminKeywordCrudPanel.DIALOG));
    }

    @Test
    public void testClickOnLazyDeleteCheckBoxRendersWMC() throws Exception {
        panel.visitChildren(AjaxCheckBox.class, new IVisitor<AjaxCheckBox, Object>() {
            @Override
            public void component(AjaxCheckBox component, IVisit<Object> visit) {
                tester.executeAjaxEvent(component.getPageRelativePath(), "onclick");
                ExportableDataPanel dataPanel = (ExportableDataPanel) panel.get(AdminKeywordCrudPanel.DATA_TABLE);
                tester.assertComponentOnAjaxResponse(dataPanel);
                visit.stop();            }
        });

    }

    @Test
    public void testClickOnEditOpensDialog() throws Exception {
        panel.visitChildren(AjaxLink.class, new IVisitor<AjaxLink, Object>() {
            @Override
            public void component(AjaxLink component, IVisit<Object> visit) {
                if(component.getParent().getParent().getId().equals(EDIT_COLUMN_ID)) {
                    tester.executeAjaxEvent(component.getPageRelativePath(), "onclick");
                    tester.assertComponentOnAjaxResponse(path(panel.getId(), AdminKeywordCrudPanel.DIALOG));
                }
            }
        });
    }

    @Test
    public void testClickOnDeleteCallsService() throws Exception {
        panel.visitChildren(AjaxLink.class, new IVisitor<AjaxLink, Object>() {
            @Override
            public void component(AjaxLink component, IVisit<Object> visit) {
                if(component.getParent().getParent().getId().equals(DELETE_COLUMN_ID)) {
                    tester.executeAjaxEvent(component.getPageRelativePath(), "onclick");
                    Mockito.verify(keywordService, Mockito.times(1)).delete(Matchers.isA(Long.class));
                    tester.assertNoErrorMessage();
                }
            }
        });
    }

    static final String EDIT_COLUMN_ID = "4";
    static final String DELETE_COLUMN_ID = "5";

}
