package se.su.dsv.scipro.milestones;

import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.mockito.internal.verification.VerificationModeFactory;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;

public class EditMilestonePhasePanelTest extends SciProTest {

    EditMileStonePhasePanel panel;
    MilestonePhaseTemplate phase;

    @Before
    public void setUp() throws Exception {
        phase = new MilestonePhaseTemplate("title", "desc");
        startPanel(phase);
    }

    @Test
    public void testRenders() throws Exception {
        Assert.assertNotNull(panel);
    }

    @Test
    public void testPanelContainsFeedback() throws Exception {
        tester.assertComponent(path(panel.getId(), EditMileStonePhasePanel.FEEDBACK), FeedbackPanel.class);
    }

    @Test
    public void testPanelContainsForm() throws Exception {
        tester.assertComponent(path(panel.getId(), EditMileStonePhasePanel.EDIT_FORM), Form.class);
    }

    @Test
    public void testFormContainsTitleField() throws Exception {
        tester.assertComponent(formPath(EditMileStonePhasePanel.TITLE), RequiredTextField.class);
        tester.assertModelValue(formPath(EditMileStonePhasePanel.TITLE), phase.getTitle());
    }

    @Test
    public void testFormContainsDescriptionArea() throws Exception {
        tester.assertComponent(formPath(EditMileStonePhasePanel.DESCRIPTION), TextArea.class);
        tester.assertModelValue(formPath(EditMileStonePhasePanel.DESCRIPTION), phase.getDescription());
    }

    @Test
    public void testFormContainsSaveButton() throws Exception {
        tester.assertComponent(formPath(EditMileStonePhasePanel.SAVE), AjaxSubmitLink.class);
    }

    @Test
    public void testSubmitInvalidForm() throws Exception {
        submitForm("", "description");

        Assert.assertFalse(tester.getMessages(FeedbackMessage.ERROR).isEmpty());

        tester.clickLink(formPath(EditMileStonePhasePanel.SAVE));
        tester.assertComponentOnAjaxResponse(path(panel.getId(), EditMileStonePhasePanel.FEEDBACK));
    }

    @Test
    public void testSubmitCorrectForm() throws Exception {
        submitForm("Title", "Description");
        Mockito.verify(mileStonePhaseTemplateService, VerificationModeFactory.times(1)).save(Matchers.isA(MilestonePhaseTemplate.class));
    }

    @Test
    public void testSubmitFormSavesCorrectInformation() throws Exception {
        String titleString = "this is a title";
        submitForm(titleString, "and this is a description");

        ArgumentCaptor<MilestonePhaseTemplate> captor = ArgumentCaptor.forClass(MilestonePhaseTemplate.class);
        Mockito.verify(mileStonePhaseTemplateService).save(captor.capture());

        Assert.assertEquals(titleString, captor.getValue().getTitle());
    }

    private void submitForm(String title, String desc) {
        FormTester formTester = tester.newFormTester(path(panel.getId(), EditMileStonePhasePanel.EDIT_FORM));
        formTester.setValue(EditMileStonePhasePanel.TITLE, title);
        formTester.setValue(EditMileStonePhasePanel.DESCRIPTION, desc);
        formTester.submit();
    }

    private void startPanel(final MilestonePhaseTemplate phase) {
        panel = tester.startComponentInPage(new EditMileStonePhasePanel("panel", Model.of(phase)));
    }

    private String formPath(String... paths) {
        return path(panel.getId(), EditMileStonePhasePanel.EDIT_FORM, path((Object[]) paths));
    }
}
