package se.su.dsv.scipro.notifications.panels;

import org.apache.wicket.markup.html.form.RadioGroup;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.system.User;

import static org.junit.Assert.assertSame;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class NotificationSettingsPanelTest extends SciProTest {

    NotificationSettingsPanel panel;

    @Before
    public void setUp() throws Exception {
        when(userProfileService.findByUser(isA(User.class))).thenReturn(new UserProfile());
    }

    private void start_panel() {
        panel = tester.startComponentInPage(NotificationSettingsPanel.class);
    }

    @Test
    public void test_renders() throws Exception {
        start_panel();
        assert panel != null : "Panel could not be rendered";
    }

    @Test
    public void testClickingRadioUpdatesProfile() throws Exception {
        // given
        boolean mailCompilation = true;

        // when
        start_panel();

        String componentPath = path(panel.getId(), NotificationSettingsPanel.FORM, NotificationSettingsPanel.MAIL_COMPILATION);
        tester.assertComponent(componentPath, RadioGroup.class);
        RadioGroup<?> radioGroup = (RadioGroup<?>) tester.getComponentFromLastRenderedPage(componentPath);

        FormTester formTester = tester.newFormTester(path(panel.getId(), NotificationSettingsPanel.FORM));
        formTester.select(NotificationSettingsPanel.MAIL_COMPILATION, mailCompilation ? 0 : 1);

        radioGroup.onSelectionChanged();

        // then
        ArgumentCaptor<UserProfile> captor = ArgumentCaptor.forClass(UserProfile.class);
        verify(userProfileService).save(captor.capture());

        assertSame("Mail compilation setting is wrong", mailCompilation, captor.getValue().isMailCompilation());
    }
}
