package se.su.dsv.scipro.notifications.settings.panels;

import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.list.ListView;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.dataobject.PeerEvent;
import se.su.dsv.scipro.notifications.settings.service.ReceiverConfigurationService;

import java.util.Arrays;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.eq;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

//import se.su.dsv.scipro.notifications.settings.service.EventConfigurationService;

public class AbstractReceiverConfigurationPanelTest extends SciProTest {

    @Mock ReceiverConfigurationService receiverConfiguration;

    static Notification.Type TYPE;
    static List<Member.Type> MEMBERS;
    static Enum[] EVENTS;

    private AbstractReceiverConfigurationPanel panel;

    @Before
    public void setUp() throws Exception {
        TYPE = Notification.Type.PEER;
        MEMBERS = Arrays.asList(Member.Type.values());
        EVENTS = PeerEvent.Event.values();

        panel = tester.startComponentInPage(new TestPanel("panel"));
    }

    @Test
    public void testRenders() throws Exception {
        assertNotNull("Does not render", panel);
    }

    @Test
    public void testContainsListOfMembers() throws Exception {
        tester.assertComponent(path(panel.getId(), AbstractReceiverConfigurationPanel.MEMBER_LIST), ListView.class);
        ListView<?> listView = (ListView<?>) panel.get(AbstractReceiverConfigurationPanel.MEMBER_LIST);
        assertEquals("Does not list every member", MEMBERS.size(), listView.getViewSize());
    }

    @Test
    public void testContainsAllEvents() throws Exception {
        tester.assertComponent(path(panel.getId(), AbstractReceiverConfigurationPanel.EVENT_LIST), ListView.class);
        ListView<?> listView = (ListView<?>) panel.get(AbstractReceiverConfigurationPanel.EVENT_LIST);
        assertEquals("Does not list every event", EVENTS.length, listView.getViewSize());
    }

    @Test
    public void testEventRowContainsMemberList() throws Exception {
        tester.assertComponent(path(panel.getId(), AbstractReceiverConfigurationPanel.EVENT_LIST, 0, AbstractReceiverConfigurationPanel.CONFIGURATIONS), ListView.class);
    }

    @Test
    public void testCheckboxEveryEventAndMember() throws Exception {
        tester.assertComponent(path(panel.getId(),
                AbstractReceiverConfigurationPanel.EVENT_LIST, 0,
                AbstractReceiverConfigurationPanel.CONFIGURATIONS, 0,
                AbstractReceiverConfigurationPanel.CONFIGURATION), CheckBox.class);
    }

    @Test
    public void testClickingCheckboxCallsService() throws Exception {
        tester.executeAjaxEvent(panel.get(path(
                AbstractReceiverConfigurationPanel.EVENT_LIST, 0,
                AbstractReceiverConfigurationPanel.CONFIGURATIONS, 0,
                AbstractReceiverConfigurationPanel.CONFIGURATION)),
                "onclick");

        verify(receiverConfiguration).setReceiving(eq(TYPE), isA(Enum.class), isA(Member.Type.class), isA(Boolean.class));
    }

    @Test
    public void testCheckboxStartsWithProperValue() throws Exception {
        // given
        boolean setting = true;

        // when
        when(receiverConfiguration.isReceiving(isA(Notification.Type.class), isA(Enum.class), isA(Member.Type.class))).thenReturn(setting, !setting);
        setUp(); // Re-render the panel

        // then
        tester.assertModelValue(path(panel.getId(),
                AbstractReceiverConfigurationPanel.EVENT_LIST, 0,
                AbstractReceiverConfigurationPanel.CONFIGURATIONS, 0,
                AbstractReceiverConfigurationPanel.CONFIGURATION), setting); // First checkbox should be checked
        tester.assertModelValue(path(panel.getId(),
                AbstractReceiverConfigurationPanel.EVENT_LIST, 0,
                AbstractReceiverConfigurationPanel.CONFIGURATIONS, 1,
                AbstractReceiverConfigurationPanel.CONFIGURATION), !setting); // All after should not be checked
    }
}

class TestPanel extends AbstractReceiverConfigurationPanel {

    public TestPanel(String id) {
        super(id, AbstractReceiverConfigurationPanelTest.TYPE, AbstractReceiverConfigurationPanelTest.EVENTS);
    }

    @Override
    protected List<Member.Type> getMemberTypes() {
        return AbstractReceiverConfigurationPanelTest.MEMBERS;
    }
}