package se.su.dsv.scipro.peer;

import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeSettings;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Date;

import static org.mockito.Matchers.anyLong;
import static org.mockito.Mockito.when;

public class AdminPeerReviewPageTest extends SciProTest {

    @Before
    public void setUp() throws Exception {
        ProjectTypeSettings projectTypeSettings = new ProjectTypeSettings();

        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        projectType.setProjectTypeSettings(projectTypeSettings);

        Project project = new Project();
        project.setTitle("My project");
        project.setProjectType(projectType);

        User user = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();

        FileDescription fileDescription = new FileDescription();
        fileDescription.setIdentifier("some_file.pdf");
        fileDescription.setId(1L);

        PeerRequest peerRequest = new PeerRequest();
        peerRequest.setProject(project);
        peerRequest.setRequester(user);
        peerRequest.setFile(fileDescription);

        User reviewerUser = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();

        PeerReview peerReview = new PeerReview();
        peerReview.setPeerRequest(peerRequest);
        peerReview.setDateCreated(new Date());
        peerReview.setReviewer(reviewerUser);
        peerReview.setProject(Project.builder().title("some reviewer project").projectType(projectType).build());

        when(peerReviewService.findOne(anyLong())).thenReturn(peerReview);
        when(performReviewService.getInProgressReview(reviewerUser, project)).thenReturn(Option.<PeerReview>empty());
    }

    @Test
    public void renders() throws Exception {
        startPage();
        tester.assertRenderedPage(AdminPeerReviewPage.class);
    }

    private void startPage() {
        PageParameters parameters = new PageParameters();
        parameters.add(PageParameterKeys.MAP.get(PeerReview.class), 1L);

        tester.startPage(AdminPeerReviewPage.class, parameters);
    }
}
