package se.su.dsv.scipro.peer;

import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.peer.LatestReviewPanel.*;

public class LatestReviewPanelTest extends SciProTest {

    public static final String PANEL = "id";
    public static final boolean DISPLAY_REVIEWS = true;
    public static final boolean NOT_DISPLAY_REVIEWS = false;
    public static final boolean NOT_PUBLIC = false;
    public static final boolean PUBLIC_REVIEWS = true;

    @Test
    public void panel_visible_if_display_is_active_in_settings() throws Exception {
        startPanel(DISPLAY_REVIEWS, PUBLIC_REVIEWS, getReviewList());
        tester.assertVisible(PANEL);
    }

    @Test
    public void not_visible_if_display_is_inactivated_in_settings() throws Exception {
        startPanel(NOT_DISPLAY_REVIEWS, PUBLIC_REVIEWS, getReviewList());
        tester.assertInvisible(PANEL);
    }

    @Test
    public void displays_reviews() throws Exception {
        startPanel(DISPLAY_REVIEWS, PUBLIC_REVIEWS, getReviewList());
        tester.assertListView(path(PANEL, LIST_VIEW), getReviewList());
    }

    @Test
    public void no_reviewers_label_visible_if_no_reviews_are_available() throws Exception {
        startPanel(DISPLAY_REVIEWS, PUBLIC_REVIEWS, new ArrayList<PeerReview>());
        tester.assertVisible(path(PANEL, NO_REVIEWERS));
    }

    @Test
    public void no_reviewers_label_invisible_if_reviews_are_available() throws Exception {
        startPanel(DISPLAY_REVIEWS, PUBLIC_REVIEWS, getReviewList());
        tester.assertInvisible(path(PANEL, NO_REVIEWERS));
    }

    @Test
    public void link_to_review_from_list() throws Exception {
        startPanel(DISPLAY_REVIEWS, PUBLIC_REVIEWS, getReviewList());
        PageParameters parameters = new PageParameters();
        parameters.add(PageParameterKeys.MAP.get(PeerReview.class), getReviewList().get(0).getId());
        tester.assertBookmarkablePageLink(path(PANEL, LIST_VIEW, 0, REVIEW_LINK), ViewPeerReviewPage.class, parameters);
    }

    @Test
    public void link_to_review_disabled_if_not_activated_in_settings() throws Exception {
        startPanel(DISPLAY_REVIEWS, NOT_PUBLIC, getReviewList());
        tester.assertDisabled(path(PANEL, LIST_VIEW, 0, REVIEW_LINK));
    }

    private void startPanel(boolean displayReviews, boolean reviewsActivated, List<PeerReview> reviewList) {
        when(peerReviewService.findAllCompleted(any(Pageable.class))).thenReturn(reviewList);
        when(generalSystemSettingsService.getGeneralSystemSettingsInstance()).thenReturn(createSettings(displayReviews, reviewsActivated));
        tester.startComponentInPage(new LatestReviewPanel(PANEL));
    }

    private GeneralSystemSettings createSettings(boolean displayLatestReviews, boolean reviewsActivated) {
        GeneralSystemSettings settings = new GeneralSystemSettings();
        settings.setPeerDisplayLatestReviews(displayLatestReviews);
        settings.setPublicReviewsActivated(reviewsActivated);
        return settings;
    }

    private List<PeerReview> getReviewList() {
        PeerReview peerReview = new PeerReview();
        peerReview.setId(5L);
        return Collections.singletonList(peerReview);
    }
}
