package se.su.dsv.scipro.profile;

import org.apache.wicket.extensions.markup.html.basic.SmartLinkMultiLineLabel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.Unit;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.Collections;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class DisplayUserProfilePanelTest extends SciProTest {

    DisplayUserProfilePanel panel;
    private User user;

    @Before
    public void setUp() throws Exception {
        Unit unit = new Unit();
        unit.setTitle("DSV IT");

        Language swedish = new Language();
        swedish.setName("Swedish");

        ResearchArea it = new ResearchArea();
        it.setTitle("IT");

        user = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        user.setUnit(unit);
        user.setLanguages(Collections.singleton(swedish));
        user.addResearchArea(it);
        user.addRole(Roles.SUPERVISOR);
        when(userService.findOne(any(Long.class))).thenReturn(user);
        when(userProfileService.findByUser(any(User.class))).thenReturn(new UserProfile());
        panel = tester.startComponentInPage(new DisplayUserProfilePanel("panel", Model.of(user)));
    }

    @Test
    public void testRenders() throws Exception {
        assertNotNull("Panel does not render", panel);
    }

    @Test
    public void testMail() throws Exception {
        tester.assertComponent(path(DisplayUserProfilePanel.MAIL), ExternalLink.class);
    }

    @Test
    public void testSkype() throws Exception {
        tester.assertComponent(path(DisplayUserProfilePanel.SKYPE), Label.class);
    }

    @Test
    public void testPhone() throws Exception {
        tester.assertComponent(path(DisplayUserProfilePanel.PHONE), Label.class);
    }

    @Test
    public void testOther() throws Exception {
        tester.assertComponent(path(DisplayUserProfilePanel.OTHER), SmartLinkMultiLineLabel.class);
    }

    @Test
    public void testEmployee() throws Exception {
        tester.assertContains("["+path(DisplayUserProfilePanel.EMPLOYEE)+"]");
    }

    @Test
    public void testUnit() throws Exception {
        tester.assertLabel(path(DisplayUserProfilePanel.EMPLOYEE,  DisplayUserProfilePanel.UNIT), user.getUnit().getTitle());
    }

    @Test
    public void testAreas() throws Exception {
        tester.assertListView(path(DisplayUserProfilePanel.EMPLOYEE, DisplayUserProfilePanel.AREAS), new ArrayList<>(user.getActiveResearchAreas()));
    }

    @Test
    public void testLanguages() throws Exception {
        tester.assertListView(path(DisplayUserProfilePanel.EMPLOYEE, DisplayUserProfilePanel.LANGUAGES), new ArrayList<>(user.getLanguages()));
    }

    @Override
    protected String path(Object... parts) {
        return super.path(panel.getId(), super.path(parts));
    }
}
