package se.su.dsv.scipro.project.pages;

import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static java.util.Arrays.asList;
import static org.mockito.Mockito.when;

public class PublicActiveThesisProjectListingPageTest extends SciProTest {

    @Before
    public void setUp() {
        User headSupervisor = User.builder().firstName("s.f.name").lastName("s.l.name").emailAddress("s.n@example.com").build();
        headSupervisor.setId(11L);
        User author = User.builder().firstName("a.f.name").lastName("a.l.name").emailAddress("a.n@example.com").build();
        author.setId(22L);
        Project project =
                Project.builder().title("title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).
                        projectStatus(ProjectStatus.ACTIVE).headSupervisor(headSupervisor).build();
        when(projectService.findAll(correctFilter(), correctLimitAndSort())).thenReturn(Collections.singletonList(project));
        tester.startPage(PublicActiveThesisProjectListingPage.class);
    }

    @Test
    public void render() throws Exception {
        tester.assertRenderedPage(PublicActiveThesisProjectListingPage.class);
    }

    private PageRequest correctLimitAndSort() {
        return new PageRequest(0, 20, new Sort(Sort.Direction.DESC, "dateCreated"));
    }

    private ProjectService.Filter correctFilter() {
        ProjectService.Filter filter = new ProjectService.Filter();
        filter.setStatuses(new ArrayList<>(Collections.singletonList(ProjectStatus.ACTIVE)));
        return filter;
    }
}