package se.su.dsv.scipro.project.panels;

import org.apache.wicket.markup.html.form.RadioChoice;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.ObjectMother;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.project.panels.SupervisorEditProjectPanel.*;

public class SupervisorEditProjectPanelTest extends SciProTest {

	private Project project;
    private SupervisorEditProjectPanel panel;

    @Before
    public void setUp() throws Exception {
        project = Project.builder().title("any").projectType(ObjectMother.BACHELOR).build();
		DomainObjects.injectId(project, 1L);
		when(projectService.save(project)).thenReturn(project);
	}

	@Test
	public void render() throws Exception {
		startPanel();
	}

	@Test
	public void change_project_title() throws Exception {
		startPanel();

		String newTitle = "new title";
		submitForm(newTitle);

		Project updatedProject = captureSavedProject();
		assertEquals(newTitle, updatedProject.getTitle());
	}

	private Project captureSavedProject() {
		ArgumentCaptor<Project> projectArgumentCaptor = ArgumentCaptor.forClass(Project.class);
		verify(projectService).save(projectArgumentCaptor.capture());
		return projectArgumentCaptor.getValue();
	}

	@Test
	public void feedback_message_after_save() throws Exception {
		startPanel();
		submitForm("any");
		tester.assertComponent(path("id", FORM, FEEDBACK), FeedbackPanel.class);
	}

	@Test
	public void co_supervisor_deleted() throws Exception {
		User user = User.builder().firstName("").lastName("").emailAddress("A@A.S").build();
		DomainObjects.injectId(user, 1L);
        project.addCoSupervisor(user);

		startPanel();

		tester.executeAjaxEvent(path("id", FORM, CO_SUPERVISORS, "wmc", "users", 0, "remove"), "click");

		submitForm("any");
		Project updatedProject = captureSavedProject();
		assertEquals(0, updatedProject.getCoSupervisors().size());
	}

    @Test
    public void has_radios() throws Exception {
        startPanel();
        tester.assertComponent(path(panel, FORM, STATUS_RADIOS), RadioChoice.class);
    }

    @Test
    public void radios_has_model_value() throws Exception {
        startPanel();
        tester.assertModelValue(path(panel, FORM, STATUS_RADIOS), project.getProjectStatus());
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new SupervisorEditProjectPanel("id", Model.of(project)));
    }

    private void submitForm(String newTitle) {
        FormTester formTester = tester.newFormTester(path("id", FORM));
		formTester.setValue(TITLE, newTitle);
		formTester.submit();
	}
}
