package se.su.dsv.scipro.projectpartner;

import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.test.ObjectMother;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class ProjectPartnerPageTest extends SciProTest {

    @Before
    public void setUp() throws Exception {
        GeneralSystemSettings settings = new GeneralSystemSettings();
        int daysToLive = 3;
        final ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        bachelor.setId(9374L);
        final ProjectType master = new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "Master");
        master.setId(2409875L);
        final List<ProjectType> projectTypes = Arrays.asList(bachelor, master);
        Mockito.when(projectTypeService.findWithModule(ProjectModule.MATCH)).thenReturn(projectTypes);
        settings.setProjectPartnerDaysToLive(daysToLive);
        Mockito.when(generalSystemSettingsService.getGeneralSystemSettingsInstance()).thenReturn(settings);
        Mockito.when(projectPartnerService.getProjectPartnerInSpan(Matchers.eq(daysToLive), Matchers.any(ProjectType.class), Matchers.any(Pageable.class))).thenReturn(Collections.singletonList(new ProjectPartner(ObjectMother.SOME_USER)));
        Mockito.when(projectPartnerService.countProjectPartnerInSpan(Matchers.eq(daysToLive), Matchers.any(ProjectType.class))).thenReturn(1L);
        tester.startPage(ProjectPartnerPage.class);
    }

    @Test
    public void renders() throws Exception {
        tester.assertRenderedPage(ProjectPartnerPage.class);
    }

    @Test
    public void submit_new_ad() throws Exception {
        FormTester formTester = tester.newFormTester("projectPartnerForm");
        formTester.setValue("infotext", "help!");
        formTester.select("projectType", 0);
        formTester.submit();

        Mockito.verify(projectPartnerService).save(Matchers.any(ProjectPartner.class));
    }
}
