package se.su.dsv.scipro.projectstate;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.components.AjaxDropDownChoice;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.icons.ImageObject;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.StateOfMind;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.TreeSet;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class ProjectStatePanelTest extends SciProTest {

    ProjectStatePanel panel;
    Project project;

    @Before
    public void setUp() throws Exception {
        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");

        User user = new UserBuilder().setFirstName("author").setLastName("1").create();
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(user)
                .build();
        DomainObjects.injectId(project, 1L);
    }

    @Test
    public void testRenders() throws Exception {
        startPanel(StateOfMind.FINE, false);
        assertNotNull("Panel does not render", panel);
    }

    @Test
    public void testGreenVisibleWhenGreen() throws Exception {
        startPanel(StateOfMind.FINE, false);
        tester.assertComponent(path(panel.getId(), ProjectStatePanel.GREEN), ImageObject.class);
        tester.assertVisible(path(panel.getId(), ProjectStatePanel.GREEN));
    }

    @Test
    public void testGreenInvisibleWhenNotGreen() throws Exception {
        startPanel(StateOfMind.NEUTRAL, false);
        tester.assertInvisible(path(panel.getId(), ProjectStatePanel.GREEN));
    }

    @Test
    public void testYellowVisibleWhenYellow() throws Exception {
        startPanel(StateOfMind.NEUTRAL, false);
        tester.assertComponent(path(panel.getId(), ProjectStatePanel.YELLOW), ImageObject.class);
        tester.assertVisible(path(panel.getId(), ProjectStatePanel.YELLOW));
    }

    @Test
    public void testYellowInvisibleWhenNotYellow() throws Exception {
        startPanel(StateOfMind.FINE, false);
        tester.assertInvisible(path(panel.getId(), ProjectStatePanel.YELLOW));
    }

    @Test
    public void testRedVisibleWhenRed() throws Exception {
        startPanel(StateOfMind.NEEDHELP, false);
        tester.assertComponent(path(panel.getId(), ProjectStatePanel.RED), ImageObject.class);
        tester.assertVisible(path(panel.getId(), ProjectStatePanel.RED));
    }

    @Test
    public void testRedInvisibleWhenNotRed() throws Exception {
        startPanel(StateOfMind.FINE, false);
        tester.assertInvisible(path(panel.getId(), ProjectStatePanel.RED));
    }

    @Test
    public void testStateVisibleWhenNotStudent() throws Exception {
        startPanel(StateOfMind.FINE, false);
        tester.assertComponent(path(panel.getId(), ProjectStatePanel.STATE), Label.class);
        tester.assertVisible(path(panel.getId(), ProjectStatePanel.STATE));
    }

    @Test
    public void testStateInvisibleWhenStudent() throws Exception {
        startPanel(StateOfMind.FINE, true);
        tester.assertInvisible(path(panel.getId(), ProjectStatePanel.STATE));
    }

    @Test
    public void testUpdatedVisible() throws Exception {
        startPanel(StateOfMind.FINE, false);
        tester.assertVisible(path(panel.getId(), ProjectStatePanel.DATE));
    }

    @Test
    public void testDropDownVisibleWhenStudent() throws Exception {
        startPanel(StateOfMind.FINE, true);
        tester.assertComponent(path(panel.getId(), ProjectStatePanel.DROPDOWN), AjaxDropDownChoice.class);
        tester.assertVisible(path(panel.getId(), ProjectStatePanel.DROPDOWN));
    }

    @Test
    public void testContainsReason() throws Exception {
        startPanel(StateOfMind.NEEDHELP, false);
        tester.assertComponent(path(panel.getId(), ProjectStatePanel.REASON_LABEL), MultiLineLabel.class);
        tester.assertVisible(path(panel.getId(), ProjectStatePanel.REASON_LABEL));
    }

    private void startPanel(StateOfMind stateOfMind, final boolean isAuthor) {
        project.setStateOfMind(stateOfMind);
        panel = tester.startComponentInPage(new ProjectStatePanel("panel", Model.of(project)) {
            @Override
            public boolean isAuthorView() {
                return isAuthor;
            }
        });
    }
}
