package se.su.dsv.scipro.reviewer;

import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.dataobjects.ReviewerThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewing.*;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;

import java.util.Collections;
import java.util.Date;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class ApprovalReviewerPanelTest extends SciProTest {

    private Project project;

    @Before
    public void setUp() throws Exception {
        project = createProject();
        ReviewerApproval approval = createApproval();
        ReviewerThread reviewerThread = new ReviewerThread();
        reviewerThread.setProject(project);
        when(myReviewService.count(any(MyReviewService.Filter.class))).thenReturn(1L);
        when(myReviewService.findAll(any(MyReviewService.Filter.class), any(Pageable.class))).thenReturn(Collections.singletonList(approval));
        when(myReviewService.findApproval(anyLong())).thenReturn(approval);
        when(reviewerDeadlineSettingsService.getInstance()).thenReturn(new ReviewerDeadlineSettings());
        when(finalSeminarApprovalService.findBy(project)).thenReturn(Option.<FinalSeminarApproval>empty());
        when(roughDraftApprovalService.findBy(project)).thenReturn(Option.<RoughDraftApproval>empty());
        setLoggedInAs(project.getReviewer());
        tester.startComponentInPage(FinalSeminarApprovalReviewerPanel.class);
    }

    @Test
    public void make_decision() throws Exception {
        tester.clickLink(path("undecidedApprovals", "body", "rows", 1, "cells", 3, "cell", "link"));
        tester.assertRenderedPage(FinalSeminarApprovalDecisionPage.class);
    }

    private Project createProject() {
        final ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        final User jane = User.builder().firstName("Jane").lastName("Doe").emailAddress("jane@example.com").build();
        final User john = User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build();
        john.setId(23523L);
        final Project project = Project.builder().title("Title").projectType(bachelor).headSupervisor(jane).build();
        project.setId(987912L);
        project.addReviewer(john);
        return project;
    }

    private FinalSeminarApproval createApproval() {
        FinalSeminarApproval finalSeminarApproval = new FinalSeminarApproval(project, mock(FileDescription.class),"test", new Date());
        DomainObjects.injectId(finalSeminarApproval, 978L);
        return finalSeminarApproval;
    }
}
