package se.su.dsv.scipro.reviewer;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.admin.pages.AdminStartPage;
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage;
import se.su.dsv.scipro.forum.dataobjects.ReviewerThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewing.FinalSeminarApproval;
import se.su.dsv.scipro.reviewing.RoughDraftApproval;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Mockito.when;

public class ReviewerInteractionPageTest extends SciProTest {

    @Before
    public void setUp() throws Exception {
        when(finalSeminarApprovalService.findBy(any(Project.class))).thenReturn(Option.<FinalSeminarApproval>empty());
        when(roughDraftApprovalService.findBy(any(Project.class))).thenReturn(Option.<RoughDraftApproval>empty());
    }

    @Test
    public void redirects_to_home_page_if_no_value_found() throws Exception {
        tester.startPage(ReviewerInteractionPage.class);
        tester.assertRenderedPage(AdminStartPage.class);
    }

    @Test
    public void access_denied_if_no_project_is_found() throws Exception {
        startWithParametersAndExpect(AccessDeniedPage.class);
    }

    @Test
    public void access_denied_if_not_reviewer_on_project() throws Exception {
        when(projectService.findOne(anyLong())).thenReturn(ObjectMother.SOME_PROJECT);
        startWithParametersAndExpect(AccessDeniedPage.class);
    }

    @Test
    public void renders_when_reviewer_on_project() throws Exception {
        mockProjectWithReviewerAndThread();
        startWithParametersAndExpect(ReviewerInteractionPage.class);
    }

    private void startWithParametersAndExpect(Class<? extends Page> pageClass) {
        tester.startPage(ReviewerInteractionPage.class, getPageParameters());
        tester.assertRenderedPage(pageClass);
    }

    private void mockProjectWithReviewerAndThread() {
        User reviewer = new UserBuilder().create();
        setLoggedInAs(reviewer);
        ObjectMother.SOME_PROJECT.addReviewer(reviewer);
        when(projectService.findOne(anyLong())).thenReturn(ObjectMother.SOME_PROJECT);
        ReviewerThread reviewerThread = new ReviewerThread();
        reviewerThread.setProject(ObjectMother.SOME_PROJECT);
        when(reviewerInteractionService.getReviewerThread(ObjectMother.SOME_PROJECT)).thenReturn(reviewerThread);
    }

    private PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Project.class), 5L);
        return pageParameters;
    }
}
