package se.su.dsv.scipro.reviewer.timeline;

import org.apache.wicket.model.AbstractReadOnlyModel;
import org.junit.Before;
import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewing.FinalSeminarApproval;
import se.su.dsv.scipro.reviewing.RoughDraftApproval;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import java.util.Collections;
import java.util.Date;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class TimelinePanelTest extends SciProTest {

    private Project project;

    @Before
    public void setUp() throws Exception {
        final ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        project = Project.builder().title("Project").projectType(bachelor).build();

        final ForumPost forumPost = new ForumPost();
        when(reviewerInteractionService.getPosts(project)).thenReturn(Collections.singletonList(forumPost));

        final FinalSeminarApproval finalSeminarApproval = new FinalSeminarApproval(project, createFileDescription(), "comment", new Date());
        when(finalSeminarApprovalService.findBy(project)).thenReturn(Option.apply(finalSeminarApproval));

        final RoughDraftApproval draftApproval = new RoughDraftApproval(project, createFileDescription(), "comment", new Date());
        draftApproval.reject("Really bad", Option.<FileDescription>empty(), Option.<FileDescription>empty());
        draftApproval.addNewThesis(createFileDescription(), "now?", new Date());
        draftApproval.approve("Ok", Option.<FileDescription>empty(), Option.<FileDescription>empty());
        when(roughDraftApprovalService.findBy(project)).thenReturn(Option.apply(draftApproval));
    }

    @Test
    public void renders() throws Exception {
        tester.startComponentInPage(new TimelinePanel("id", new AbstractReadOnlyModel<Project>() {
            @Override
            public Project getObject() {
                return project;
            }
        }));
    }

    private FileDescription createFileDescription() {
        return mock(FileDescription.class);
    }
}