package se.su.dsv.scipro.rwandagrading;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.grading.Grade;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;

import static org.mockito.Mockito.when;

public class DisplayGradePanelTest extends SciProTest {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING);

    DisplayGradePanel panel;
    Project project;
    User user;

    @Before
    public void setUp() throws Exception {
        user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        user.setRoles(new HashSet<>(Collections.singletonList(Roles.SUPERVISOR)));

        project = Project.builder().title("some title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).headSupervisor(user).build();

        setLoggedIn(true);
        setLoggedInAs(user);

        when(gradeService.getGrade(project)).thenReturn(Option.<Grade>empty());
    }

    @Test
    public void display() throws Exception {
        startPanel();
        tester.assertComponent(path(panel.getId(), "grade"), Label.class);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new DisplayGradePanel("panel", Model.of(project)));
    }

}