package se.su.dsv.scipro.rwandagrading;

import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.grading.Grade;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.io.Serializable;
import java.util.*;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsCollectionContaining.hasItem;
import static org.mockito.Matchers.eq;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class GradePanelTest extends SciProTest {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING);

    GradePanel panel;
    Project project;
    User user;

    @Before
    public void setUp() throws Exception {
        user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        user.setRoles(new HashSet<>(Collections.singletonList(Roles.SUPERVISOR)));

        project = Project.builder().title("some title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).headSupervisor(user).build();

        setLoggedIn(true);
        setLoggedInAs(user);
    }

    @Test
    public void set_grade() throws Exception {
        noPreExistingGrade();
        String gradeValue = "a grade";
        reportGrade(gradeValue);
        verify(gradeService).reportGrade(eq(project), eq(user), isA(Date.class), eq(gradeValue));
        List<Serializable> messages = tester.getMessages(FeedbackMessage.SUCCESS);
        assertThat(messages, hasItem("Grade saved"));
    }

    @Test
    public void update_grade() throws Exception {
        String gradeValue = "B";
        alreadyGraded();
        reportGrade(gradeValue);
        verify(gradeService).reportGrade(eq(project), eq(user), isA(Date.class), eq(gradeValue));
    }

    @Test
    public void remove_grade() throws Exception {
        alreadyGraded();
        reportGrade("");
        verify(gradeService).removeGrade(project);
    }

    private void alreadyGraded() {
        Grade grade = new Grade(project);
        when(gradeService.getGrade(project)).thenReturn(Option.apply(grade));
    }

    private void noPreExistingGrade() {
        when(gradeService.getGrade(project)).thenReturn(Option.<Grade>empty());
    }

    private void reportGrade(final String gradeValue) {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel.getId(), "form"));
        formTester.setValue("grade", gradeValue);
        formTester.submit();
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new GradePanel("panel", Model.of(project)));
    }

}