package se.su.dsv.scipro.statistics;

import org.hamcrest.MatcherAssert;
import org.hamcrest.core.StringContains;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import java.util.Arrays;
import java.util.Date;

public class MilestoneFilterDetailsPanelTest extends SciProTest {

    private MilestoneFilterDetailsPanel panel;

    public static final ProjectType SOME_PROJECT_TYPE = new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor");
    public static final ProjectType SOME_OTHER_PROJECT_TYPE = new ProjectType(DegreeLevel.SECOND_CYCLE, "master", "master");

    @Test
    public void filtering_without_project_started_between_includes_not_available_string() throws Exception {
        startPanel(new MileStoneService.Filter());
        assertThatPageContains(panel.getString("notAvailable"));
    }

    @Test
    public void filter_with_project_started_between() throws Exception {
        String dateString = "xxxx-xx-xx";
        Mockito.when(dateService.format(Matchers.any(Date.class), Matchers.eq(DateStyle.DATE))).thenReturn(dateString);
        MileStoneService.Filter filter = new MileStoneService.Filter();
        filter.setProjectStartedAfter(new Date());
        startPanel(filter);
        assertThatPageContains(dateString + " and " + dateString);
    }

    @Test
    public void filter_with_project_types() throws Exception {
        MileStoneService.Filter filter = new MileStoneService.Filter();
        filter.setProjectTypes(Arrays.asList(SOME_PROJECT_TYPE, SOME_OTHER_PROJECT_TYPE));
        startPanel(filter);
        for (ProjectType projectType : filter.getProjectTypes()) {
            assertThatPageContains(projectType.getName());
        }
    }

    private void startPanel(MileStoneService.Filter filter) {
        panel = tester.startComponentInPage(new MilestoneFilterDetailsPanel("id", filter));
    }

    private void assertThatPageContains(String string) {
        String lastResponseAsString = tester.getLastResponseAsString();
        MatcherAssert.assertThat(lastResponseAsString, StringContains.containsString(string));
    }

}
