package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import scala.Option;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.grading.Grade;
import se.su.dsv.scipro.group.Group;
import se.su.dsv.scipro.group.GroupService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class SupervisorProjectDetailsPageTest extends PageTest {

    private Project masterProject;

    @Before
    public void setUp() throws Exception {
        User headSupervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        Project bachelorProject = createProject(headSupervisor, student,
                new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"), 1L);
        masterProject = createProject(headSupervisor, student,
                new ProjectType(DegreeLevel.SECOND_CYCLE, "master", "master"), 3L);
        when(projectService.findOne(isA(Long.class))).thenReturn(bachelorProject);
        when(projectService.isSupervisorOrFollower(isA(Project.class), isA(User.class))).thenReturn(true);
        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");
        FileDescription fileDescription = new FileDescription();
        fileDescription.setMimeType("pdf");
        fileDescription.setId(1L);
        when(finalThesisService.getFinalThesisFileDescription(any(Project.class))).thenReturn(fileDescription);
        when(projectService.getMemberType(any(Project.class), any(User.class))).thenReturn(Member.Type.SUPERVISOR);
        when(finalSeminarService.findProjectOpposing(any(Project.class), any(Pageable.class))).thenReturn(Collections.<FinalSeminar>emptySet());
        when(finalSeminarService.findProjectParticipating(any(Project.class), any(Pageable.class))).thenReturn(Collections.<FinalSeminar>emptySet());
        when(gradeService.getGrade(any(Project.class))).thenReturn(Option.<Grade>empty());
    }

    @Test
    public void final_seminar_panel_is_visible_for_bachelor_project() throws Exception {
        startPage();
        tester.assertVisible(SupervisorProjectDetailsPage.FINAL_SEMINAR_NEW);
    }

    @Test
    public void final_seminar_panel_is_visible_for_master_project() throws Exception {
        when(projectService.findOne(isA(Long.class))).thenReturn(masterProject);
        startPage();
        tester.assertVisible(SupervisorProjectDetailsPage.FINAL_SEMINAR_NEW);
    }

    @Test
    public void group_overview_link_to_supervisor_group_page() throws Exception {
        long id = 55L;
        mockGroup(id);
        when(projectService.findOne(isA(Long.class))).thenReturn(masterProject);
        startPage();
        PageParameters parameters = new PageParameters();
        parameters.add(PageParameterKeys.MAP.get(Group.class), id);
        tester.assertBookmarkablePageLink(path("groups", "groupList", 0, "groupLink"),
                SupervisorGroupPage.class, parameters);
    }

    private void mockGroup(long id) {
        Group group = new Group();
        group.setId(id);
        when(groupService.findAll(any(GroupService.Filter.class))).thenReturn(
                Collections.singletonList(group)
        );
    }

    @Override
    protected Class<? extends Page> getPage() {
        return SupervisorProjectDetailsPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(1L));
        return pp;
    }

    private Project createProject(User headSupervisor, User student, ProjectType projectType, long id) {
        Project project = Project.builder().title("some title").projectType(projectType)
                .headSupervisor(headSupervisor).build();
        DomainObjects.injectId(project, id);
        return project;
    }
}
