package se.su.dsv.scipro.supervisor.panels;

import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.project.ExternalOrganization;
import se.su.dsv.scipro.project.Project;

import static org.mockito.Mockito.verify;
import static se.su.dsv.scipro.supervisor.panels.SupervisorExternalProjectPanel.*;

@RunWith(MockitoJUnitRunner.class)
public class SupervisorExternalProjectPanelTest extends SciProTest {

    private SupervisorExternalProjectPanel panel;

    @Test
    public void testRenders() throws Exception {
        startPanelWithExternalOrganization();
    }

    @Test
    public void testHasExternalCheckbox() throws Exception {
        startPanel();

        tester.assertComponent(path(FORM, EXTERNAL_CHECKBOX), AjaxCheckBox.class);
    }

    @Test
    public void testExternalTextFieldHiddenIfNotExternal() throws Exception {
        startPanelWithoutExternalOrganization();

        tester.assertInvisible(path(FORM, ENCLOSURE, EXTERNAL_FIELD));
    }

    @Test
    public void testExternalTextFieldVisibleIfExternal() throws Exception {
        startPanelWithExternalOrganization();

        tester.assertVisible(path(FORM, ENCLOSURE, EXTERNAL_FIELD));
    }

    @Test
    public void testClickingCheckboxAddsTextFieldToAjax() throws Exception {
        startPanelWithoutExternalOrganization();
        clickCheckBox();

        tester.assertComponentOnAjaxResponse(path(FORM));
    }

    @Test
    public void testSelectingCheckboxShowsTextField() throws Exception {
        startPanelWithoutExternalOrganization();
        clickCheckBox();

        tester.assertVisible(path(FORM, ENCLOSURE, EXTERNAL_FIELD));
    }

    @Test
    public void testDeselectingCheckboxHidesTextField() throws Exception {
        startPanelWithExternalOrganization();
        clickCheckBox();

        tester.assertInvisible(path(FORM, ENCLOSURE, EXTERNAL_FIELD));
    }

    @Test
    public void testSelectingCheckboxSavesProject() throws Exception {
        Project project = startPanelWithoutExternalOrganization();
        clickCheckBox();

        verify(projectService).save(project);
    }

    @Test
    public void testShowsHelpTextWhenLinkedToOrganization() throws Exception {
        startPanelWithExternalOrganization();

        tester.assertComponent(path(FORM, ENCLOSURE, HELP_TEXT), InfoPanel.class);
    }

    @Test
    public void shows_feedback_that_connection_to_external_organization_has_been_removed() throws Exception {
        startPanelWithExternalOrganization();
        clickCheckBox();

        tester.assertFeedback(path(FORM, FEEDBACK), panel.getString("external.organization.removed"));
    }

    @Test
    public void shows_feedback_when_changing_external_organization() throws Exception {
        startPanelWithExternalOrganization();

        FormTester formTester = tester.newFormTester(path(FORM));
        formTester.setValue(super.path(ENCLOSURE, EXTERNAL_FIELD), "New organization");
        formTester.submit();

        tester.assertFeedback(path(FORM, FEEDBACK), panel.getString("external.organization.changed"));
    }

    private void clickCheckBox() {
        FormTester formTester = tester.newFormTester(path(FORM));
        formTester.setValue(EXTERNAL_CHECKBOX, !getCheckBox().getModelObject());
        tester.executeAjaxEvent(path(FORM, EXTERNAL_CHECKBOX), "onclick");
    }

    private AjaxCheckBox getCheckBox() {
        return (AjaxCheckBox) panel.get(super.path(FORM, EXTERNAL_CHECKBOX));
    }

    private void startPanel() {
        startPanel(new Project());
    }

    private void startPanel(final Project project) {
        panel = tester.startComponentInPage(new SupervisorExternalProjectPanel("panel", Model.of(project)));
    }

    @Override
    protected String path(Object... parts) {
        return super.path(panel, super.path(parts));
    }

    private void startPanelWithExternalOrganization() {
        Project project = new Project();
        project.setExternalOrganization(new ExternalOrganization());
        startPanel(project);
    }

    private Project startPanelWithoutExternalOrganization() {
        Project project = new Project();
        project.setExternalOrganization(null);

        startPanel(project);
        return project;
    }
}
