package se.su.dsv.scipro.supervisor.panels;

import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.supervisor.pages.AbstractSupervisorPage;
import se.su.dsv.scipro.system.ProjectModule;

import static org.hamcrest.core.IsNot.not;
import static org.hamcrest.core.StringContains.containsString;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.when;

public class SupervisorTabMenuPanelTest extends SciProTest {

    @Test
    public void ideas_tab_visible_if_project_type_have_match_module() throws Exception {
        when(projectTypeService.projectTypeWithModuleExists(ProjectModule.MATCH)).thenReturn(true);
        startPanel();
        assertVisible("Ideas");
    }

    @Test
    public void ideas_tab_not_visible_if_project_type_does_not_have_match_module() throws Exception {
        when(projectTypeService.projectTypeWithModuleExists(ProjectModule.MATCH)).thenReturn(false);
        startPanel();
        assertNotVisible("Ideas");
    }

    private SupervisorTabMenuPanel startPanel() {
        return tester.startComponentInPage(new SupervisorTabMenuPanel("id", AbstractSupervisorPage.class));
    }

    private String renderedPanel() {
        startPanel();
        return tester.getLastResponseAsString();
    }

    private void assertVisible(String menuChoice) {
        assertThat(renderedPanel(), containsString(menuChoice));
    }

    private void assertNotVisible(String menuChoice) {
        assertThat(renderedPanel(), not(containsString(menuChoice)));
    }
}
