package se.su.dsv.scipro.user;

import org.apache.wicket.Page;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.system.Hash;
import se.su.dsv.scipro.system.Program;
import se.su.dsv.scipro.system.User;

import java.util.Arrays;
import java.util.Collections;

import static org.mockito.Matchers.eq;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class AdminChangePasswordPageTest extends PageTest {

    AdminChangePasswordPage page;

    @Before
    public void setUp() throws Exception {

        User user = new User();
        when(userService.findOne(isA(Long.class))).thenReturn(user);
        user.setId(1L);
        user.setFirstName("John");
        user.setLastName("Rambo");
        when(userService.save(isA(User.class))).thenReturn(user);

        Program program = new Program();
        program.setName("Super program");
        program.setNameEn("Super program");
        program.setCode("sp");
        program.setId(1L);

        when(programService.findAll()).thenReturn(Collections.singletonList(program));
        when(hashService.createForUser(isA(User.class))).thenReturn(Hash.builder().hash("anicehash").user(user).build());
        when(userFacade.createUser(isA(User.class))).thenReturn(user);

        page = tester.startPage(AdminChangePasswordPage.class);
    }


    @Test
    public void change_password() throws Exception {
        FormTester changePassword = tester.newFormTester("form");
        String password = "4veryL0ngsEcr3t";
        changePassword.setValue("password1", password);
        changePassword.setValue("password2", password);
        changePassword.submit();

        verify(passwordFacade).setPassword(isA(User.class), eq(password));
    }

    @Test
    public void passwords_must_match() throws Exception {
        FormTester changePassword = tester.newFormTester("form");
        String password = "4veryL0ngsEcr3t";
        changePassword.setValue("password1", password);
        changePassword.setValue("password2", "foo");
        changePassword.submit();

        tester.assertErrorMessages(page.getString("password2.EqualPasswordInputValidator"));
    }

    @Override
    protected Class<? extends Page> getPage() {
        return AdminChangePasswordPage.class;
    }

}