package se.su.dsv.scipro.user;

import org.apache.wicket.Page;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.system.Hash;
import se.su.dsv.scipro.system.Program;
import se.su.dsv.scipro.system.User;

import java.util.Arrays;
import java.util.Collections;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class AdminEditUserPageTest extends PageTest {

    AdminEditUserPage page;

    @Before
    public void setUp() throws Exception {

        User user = new User();
        when(userService.findOne(isA(Long.class))).thenReturn(user);
        user.setId(1L);
        user.setFirstName("John");
        user.setLastName("Rambo");
        when(userService.save(isA(User.class))).thenReturn(user);

        Program program = new Program();
        program.setName("Super program");
        program.setNameEn("Super program");
        program.setCode("sp");
        program.setId(1L);

        when(programService.findAll()).thenReturn(Collections.singletonList(program));
        when(hashService.createForUser(isA(User.class))).thenReturn(Hash.builder().hash("anicehash").user(user).build());
        when(userFacade.createUser(isA(User.class))).thenReturn(user);

        page = tester.startPage(AdminEditUserPage.class);
    }

    @Test
    public void requires() throws Exception {
        FormTester formTester = tester.newFormTester("form");
        formTester.submit();
        tester.assertErrorMessages("Field 'firstName' is required", "Field 'lastName' is required", "Field 'email' is required");//, "Field 'password' is required");
    }

    @Test
    public void email() throws Exception {
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("firstName", "John");
        formTester.setValue("lastName", "Rambo");
        formTester.setValue("email", "notanemailaddress");
        formTester.submit();
        tester.assertErrorMessages("The value of 'email' is not a valid email address.");
    }

    @Test
    public void unique() throws Exception {
        User existingUserWithSameEmail = new User();
        existingUserWithSameEmail.setId(324L);
        when(userService.findByUsername(isA(String.class))).thenReturn(existingUserWithSameEmail);
        page = tester.startPage(AdminEditUserPage.class);
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("firstName", "John");
        formTester.setValue("lastName", "Rambo");
        formTester.setValue("email", "example@example.com");
        formTester.submit();
        tester.assertErrorMessages(page.getString("email.UniqueValidator"));
    }

    @Test
    public void saves() throws Exception {
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("firstName", "John");
        formTester.setValue("lastName", "Rambo");
        formTester.setValue("email", "example@example.com");
        formTester.submit();
        verify(userFacade).createUser(isA(User.class));
        verify(hashService).createForUser(isA(User.class));
        tester.assertInfoMessages("Saved user John Rambo");
    }

    @Override
    protected Class<? extends Page> getPage() {
        return AdminEditUserPage.class;
    }
}