package se.su.dsv.scipro.user;

import org.apache.wicket.util.tester.FormTester;
import org.hamcrest.CoreMatchers;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;

public class AdminSwitchUserPageTest extends SciProTest {

    public static final long SWITCH_TO_ID = 666L;
    private User user;

    @Before
    public void setUp() throws Exception {
        user = createUser(234L, "Bill", "Gates", "bill.gates@microsoft.com");
        user.addRole(Roles.SYSADMIN);
        setLoggedInAs(user);
    }

    private User createUser(long id, String firstName, String lastName, String emailAddress) {
        User user = new User();
        user.setFirstName(firstName);
        user.setLastName(lastName);
        user.setEmailAddress(emailAddress);
        DomainObjects.injectId(user, id);
        return user;
    }

    @Test
    public void renders() throws Exception {
        tester.startPage(AdminSwitchUserPage.class);
    }

    @Test
    public void submitting_form_switches_to_user() throws Exception {
        User kalle = createUser(SWITCH_TO_ID, "kalle", "anka", "kalle.anka@ankeborg.com");
        mockUser(kalle);
        submitForm();
        Assert.assertThat(SciProSession.get().getUser(), CoreMatchers.is(kalle));
    }

    @Test
    public void submitting_without_selecting_user() throws Exception {
        tester.startPage(AdminSwitchUserPage.class);
        FormTester formTester = tester.newFormTester(AdminSwitchUserPage.FORM);
        formTester.setValue(AdminSwitchUserPage.USER, "");
        formTester.submit();
        Assert.assertThat(SciProSession.get().getUser(), CoreMatchers.is(user));
    }

    private void mockUser(User user) {
        Mockito.when(userService.findOne(SWITCH_TO_ID)).thenReturn(user);
    }

    private void submitForm() {
        tester.startPage(AdminSwitchUserPage.class);
        FormTester formTester = tester.newFormTester(AdminSwitchUserPage.FORM);
        formTester.setValue(AdminSwitchUserPage.USER, "" + SWITCH_TO_ID);
        formTester.submit();
    }
}
